import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import ovi.connector 1.0

import "DownloadTools.js" as Tools

Page {
    id: downloadPage

    property variant region
    anchors.fill: parent

    Item {
        id: internals

        property int _NEEDED_SPACE_MULTIPLIER_: 2

        property double totalSize: region && (region.downloadSize || (mapCatalog.region(region.id) && mapCatalog.region(region.id).downloadSize)) || 0
        property double downloadedSize: 0
        property double timeLeft: 0
        property double freeSpace: 0

        property bool testDownloadInProgress: false
        property bool willNotTestDownload: false
        property bool downloadInProgress: false
        property bool installInProgress: false
        property bool downloadPaused: false
        property bool undeterminedAfterResume: false

        property bool allFinished: false
        property bool navigationAvailable: true

        property bool triggeredDownload: false
        property bool triggeredCancel: false
        property bool triggeredExit: false

        function resetToDefaultAllVariables() {
            internals.totalSize = region && (region.downloadSize || (mapCatalog.region(region.id) && mapCatalog.region(region.id).downloadSize)) || 0;
            internals.downloadedSize = 0;
            internals.timeLeft = 0;
            internals.freeSpace = downloadManager.freeDiskSpace;

            internals.testDownloadInProgress = false;
            internals.willNotTestDownload = false;
            internals.downloadInProgress = false;
            internals.installInProgress = false;
            internals.downloadPaused = false;
            internals.undeterminedAfterResume = false;

            internals.allFinished = false;
            internals.navigationAvailable = true;
        }

        function getUrlToDownload(shortMode) {
            return Tools.getUrlToDownload(shortMode, region);
        }

        function handleDownloadButtonClicked() {
            internals.triggeredDownload = true;
            pageStack.pop(offlineMapsPage);
        }
        function handleTriggeredDownload() {
            if (internals.triggeredDownload) {
                internals.triggeredDownload = false;

                offlineMapsPage.subOfflineMapListResetIndex();

                if (mainscreen.enableStaticODMLAddMode && offlineMapsPage.downloadsStartedInThisCycle == 0) {
                    appState.killDrive();
                    MapsPlugin.clearMemcache();
                    mainscreen.___clearAllStacksAndKeepODML();
                }
                offlineMapsPage.downloadsStartedInThisCycle++;
                downloadManager.download(region.id, internals.getUrlToDownload(), mapCatalog.mapBaseDirectory + "/diskcache" + "/", true);
            }
        }

        function handleCancelButtonClicked() {
            internals.triggeredCancel = true;
            pageStack.pop(offlineMapsPage);
        }
        function handleTriggeredCancel() {
            if (internals.triggeredCancel) {
                internals.triggeredCancel = false;

                offlineMapsPage.subOfflineMapListResetIndex();

                downloadManager.cancel(internals.getUrlToDownload());
                offlineMapsPage.handleOnCompleted();
            }
        }

        function testDownload() {
            internals.testDownloadInProgress = true;
            internals.downloadInProgress = false;
            downloadManager.testDownload(internals.getUrlToDownload(true));
        }

        function handleConfirmPause() {
            if (!internals.downloadPaused) {
                internals.downloadPaused = true;
                downloadManager.suspend();
            } else {
                internals.downloadPaused = false;
                internals.undeterminedAfterResume = true;
                downloadManager.resume();
            }
        }

        function handleConfirmExit() {
            queryDialog.titleText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_confirm_exit_caption");
            queryDialog.message = qsTranslate("Maps", "qtn_maps_more_offlinemaps_confirm_exit_body");
            queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_no");
            queryDialog.acceptButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_yes");
            queryDialog.open();
        }

        function handleOnProgress(url, bytesReceived, bytesTotal, estimatedTime) {
            if (downloadPage.visible) {
                internals.undeterminedAfterResume = false;
                if (bytesTotal) {
                    if (url == internals.getUrlToDownload()) {
                        internals.downloadInProgress = true;

                        internals.downloadedSize = bytesReceived; internals.timeLeft = estimatedTime; internals.totalSize = bytesTotal;
                    } else if (url == internals.getUrlToDownload(true)) {
                        internals.totalSize = bytesTotal;
                        internals.timeLeft = estimatedTime;
                    }
                }
            }
        }
        function handleOnFinished(operation, url) {
            if (downloadPage.visible) {
                if (url == internals.getUrlToDownload() || url == internals.getUrlToDownload(true)) {
                    queryDialog.close();

                    if (internals.downloadInProgress && operation == 2) {
                        internals.installInProgress = true;

                        internals.downloadInProgress = false;
                    } else if (operation == 3) {
                        internals.installInProgress = true;
                    } else if (operation == 4) {
                        internals.allFinished = true;
                        internals.installInProgress = false;
                    } else {
                        internals.testDownloadInProgress = false;
                    }
                }
            }
        }
        function handleOnError(url) {
            if (downloadPage.visible) {
                if (url == internals.getUrlToDownload()) {
                    //pageStack.pop(offlineMapsPage);   // don't pop as some freezes appear
                } else if (url == internals.getUrlToDownload(true)) {
                    internals.willNotTestDownload = true;
                }
            }
        }

        function handleOnIsSuspendedChanged() {
            if (downloadPage.visible) {
                internals.downloadPaused = downloadManager.isSuspended;
            }
        }

        function handleQueryDialogAccept() {
            internals.triggeredExit = true;
            pageStack.pop();
        }
        function handleTriggeredExit() {
            if (internals.triggeredExit) {
                internals.triggeredExit = false;

                offlineMapsPage.saveCurrentProgressInStash(-1);
                downloadManager.cancel(internals.getUrlToDownload());
            }
        }

        function handleQueryDialogReject() {
            queryDialog.close();
            //pageStack.pop();
        }

        function tryToGetCurrentProcessFromStash() {
            if (region) {
                var tmpStash = offlineMapsPage.getCurrentProgressFromStash(region.id);
                if (tmpStash && tmpStash.valid) {
                    internals.totalSize = tmpStash.totalSize;
                    internals.downloadedSize = tmpStash.downloadedSize;
                    internals.timeLeft = tmpStash.timeLeft;
                    internals.downloadInProgress = tmpStash.status == "downloading";
                    internals.installInProgress = tmpStash.status == "installing";
                    internals.undeterminedAfterResume = !(internals.downloadInProgress || internals.installInProgress);
                }
            }
        }
    }

    OdmlHeader {
        id: header

        title: region ? region.localizedName || region.name : ""

        z: 1
    }

    Rectangle {
        anchors.top: header.bottom
        anchors.bottom: parent.bottom
        anchors { left: parent.left; right: parent.right }
        clip: true
        color: "#f0f1f2"

        Flickable {
            anchors.fill: parent

            contentWidth: parent.width
            contentHeight: columnBody.height

            Column {
                id: columnBody

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 16;  color: "#f0f1f2"}
                MapsText {
                    objectName: "downloadStreetMapsTxt"
                    state: "normalWrap"
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    visible: !internals.allFinished
                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_download_street_maps") //Download the latest maps over Wi-Fi and save on data costs when home or abroad.
                }

                MapsText {
                    state: "normalWrap"

                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    visible: !internals.allFinished && !internals.navigationAvailable

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_note_no_navigation_available")
                }

                MapsText {
                    state: "normalWrapRed"

                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_not_enough_space").arg(
                              Tools.formatBytes(internals.freeSpace)).arg(
                              Tools.formatBytes(Math.abs(internals.freeSpace - internals.totalSize * internals._NEEDED_SPACE_MULTIPLIER_)))

                    visible: !internals.allFinished && internals.totalSize * internals._NEEDED_SPACE_MULTIPLIER_ > internals.freeSpace
                }

                Row {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    visible: !downloadManager.hasWifiConnection && !internals.allFinished && !internals.downloadInProgress && !internals.installInProgress

                    Image {
                        id: imageA
                        source: odmlAssetsPath + "images/wifi.png"
                    }
                    MapsText {
                        state: "normalWrap"

                        anchors.left: imageA.right
                        anchors.right: parent.right
                        anchors.leftMargin: 16
                        anchors.rightMargin: 16
                        anchors.verticalCenter: parent.verticalCenter

                        text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_wifi_required")
                    }
                }

                Row {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    visible: (!internals.allFinished && !internals.downloadInProgress && !internals.installInProgress ) && downloadManager.hasWifiConnection

                    Image {
                        id: imageB
                        source: odmlAssetsPath + "images/filesize.png"
                    }
                    MapsText {
                        state: "normalWrap"
                        anchors.verticalCenter: imageB.verticalCenter

                        text: {
                            if (internals.totalSize && !internals.testDownloadInProgress) {
                                return qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_size_megabytes").arg(Tools.formatBytes(/*internals._NEEDED_SPACE_MULTIPLIER_ * */internals.totalSize));
                            } else if (internals.willNotTestDownload && internals.totalSize) {
                                return qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_size_megabytes").arg(Tools.formatBytes(/*internals._NEEDED_SPACE_MULTIPLIER_ * */internals.totalSize));
                            } else if (internals.willNotTestDownload && !internals.totalSize) {
                                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_size_undefined");
                            } else {
                                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_calculating_dots");
                            }
                        }
                    }
                }

                MapsText {
                    state: "normalWrap"

                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    visible: !(internals.downloadedSize || internals.totalSize) && !internals.downloadInProgress && !internals.downloadPaused && !internals.allFinished

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_calculating")
                }

                Rectangle {
                    color: "transparent"
                    height: 50
                    width: parent.width

                    visible: progressBar.visible

                    Column {
                        anchors.fill: parent

                        Row {
                            id: downloadigStatus

                            anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                            visible: internals.downloadedSize && internals.totalSize && internals.downloadInProgress && !internals.downloadPaused && !internals.undeterminedAfterResume && !internals.allFinished && !internals.installInProgress

                            Image {
                                id: imageB1
                                source: odmlAssetsPath + "images/filesize.png"
                            }
                            MapsText {
                                state: "normalWrap"
                                wrapMode: "WrapAnywhere"
                                anchors.verticalCenter: imageB1.verticalCenter
                                visible: internals.downloadedSize && internals.totalSize && internals.downloadInProgress && !internals.downloadPaused && !internals.allFinished && !internals.installInProgress

                                text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_status").arg(
                                          Tools.formatBytes(internals.downloadedSize)).arg(
                                          Tools.formatBytes(internals.totalSize))/*.arg(
                                          Tools.formatMilliseconds(internals.timeLeft))*/
                            }
                        }

                        Item {
                            id: pausedStatus

                            visible: internals.undeterminedAfterResume || (internals.downloadPaused && !internals.allFinished)

                            anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                            MapsText {
                                state: "normalWrap"
                                anchors.top: parent.top
                                anchors.topMargin: 10

                                Behavior on visible { NumberAnimation { duration: 150 } }

                                text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_paused_status")
                            }
                        }
                    }
                }

                MapsText {
                    state: "normalWrap"
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    visible: internals.installInProgress && !internals.allFinished
                    Behavior on visible { NumberAnimation { duration: 150 } }

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_installing_status")
                }

                MapsText {
                    state: "normalWrap"
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    visible: internals.allFinished
                    Behavior on visible { NumberAnimation { duration: 150 } }

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_install_completed")
                }

                ProgressBar {
                    id: progressBar
                    visible: internals.undeterminedAfterResume || ((internals.downloadedSize && internals.totalSize && (internals.downloadInProgress || internals.installInProgress) || internals.downloadPaused) && !internals.allFinished)
                    value: internals.downloadedSize / internals.totalSize * 100
                    indeterminate: cancelButton.visible || internals.downloadPaused || internals.undeterminedAfterResume
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    minimumValue: 0
                    maximumValue: 100
                }
            }
        }
    }

    QueryDialog {
        id: queryDialog
        onAccepted: { internals.handleQueryDialogAccept() }
        onRejected: { internals.handleQueryDialogReject() }
    }

    Connections {
        target: downloadManager
        ignoreUnknownSignals: true

        onProgress: { internals.handleOnProgress(url, bytesReceived, bytesTotal, estimatedTime) }
        onFinished: { internals.handleOnFinished(operation, url) }
        onError: { internals.handleOnError(url) }
        onIsSuspendedChanged: { internals.handleOnIsSuspendedChanged() }
    }

    Timer {
        id: smallWaiterToPreventEndlessTestingDownload
        running: internals.testDownloadInProgress
        interval: 1000 * 5

        onTriggered: {
            internals.willNotTestDownload = true;
            internals.testDownloadInProgress = false;
        }
    }

    tools: ToolBarLayout {
        ToolIcon {
            id:iconToolbarBack
            iconId: "icon-m-toolbar-back"
            anchors.left: parent.left
            onClicked: { pageStack.pop() }
        }
        ToolButton {
            id: downloadButton

            platformStyle: ToolButtonStyle {
                disabledTextColor: pressedTextColor
            }

            objectName: "downloadBtn"
            text: {
                if (downloadManager.hasWifiConnection) {
                    if (downloadManager.requests && region && downloadManager.isInQueue(region.id)) {
                        return qsTranslate("Maps", "qtn_maps_more_in_progress");
                    } else {
                        return qsTranslate("Maps", "qtn_maps_more_download");
                    }
                } else {
                    return qsTranslate("Maps", "qtn_maps_more_download");
                }
            }
            implicitHeight: iconToolbarBack.height - 20
            width: parent.width / 2
            anchors.centerIn: parent
            enabled: (downloadManager.requests || !!downloadManager.requests) && region && !downloadManager.isInQueue(region.id) && downloadManager.hasWifiConnection || false
            visible: !internals.allFinished && !internals.downloadInProgress && !internals.installInProgress && (internals.totalSize * internals._NEEDED_SPACE_MULTIPLIER_ < internals.freeSpace) && (region && downloadManager.currentlyActiveId() == region.id || !downloadManager.isInQueue(region.id))
            onClicked: { internals.handleDownloadButtonClicked() }
        }
        ToolButton {
            id: cancelButton
            objectName: "cancelBtn"
            text: qsTranslate("Maps", "qtn_maps_more_cancel")
            visible: (downloadManager.requests || !!downloadManager.requests) && region && downloadManager.isInQueue(region.id) && downloadManager.currentlyActiveId() != region.id && !internals.installInProgress || false
            anchors.centerIn: parent
            width: parent.width / 2
            implicitHeight: iconToolbarBack.height - 20
            onClicked: { internals.handleCancelButtonClicked() }
        }
        ToolIcon {
            id: pauseButton
            iconId: internals.downloadPaused ? "icon-m-toolbar-refresh1" : "icon-m-toolbar-mediacontrol-pause"
            objectName: "pauseResumeBtn"
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: - width/2
            visible: internals.downloadInProgress && !internals.allFinished && !internals.installInProgress
            implicitHeight: iconToolbarBack.height - 20
            onClicked: { internals.handleConfirmPause() }
        }

        ToolIcon {
            iconId: "icon-m-toolbar-stop"
            objectName: "cancelBtn"
            anchors.verticalCenter: parent.verticalCenter
            anchors.left: pauseButton.right
            visible: internals.downloadInProgress && !internals.allFinished && !internals.installInProgress
            implicitHeight: iconToolbarBack.height - 20
            onClicked: { internals.handleConfirmExit() }
        }

    }

    onStatusChanged: {
        if (status === PageStatus.Inactive) {
            internals.handleTriggeredDownload();
            internals.handleTriggeredCancel();
            internals.handleTriggeredExit();

            downloadPage.region = null;
        }
    }

    onRegionChanged: {
        if (region) {
            internals.resetToDefaultAllVariables();
            internals.downloadPaused = downloadManager.isSuspended && region && downloadManager.currentlyActiveId() == region.id? true : false;
            internals.tryToGetCurrentProcessFromStash();

            if (!internals.downloadInProgress && !downloadManager.currentlyActiveId() && mainscreen.onlineMode && downloadManager.hasWifiConnection) {
                internals.testDownload();
            } else if (region && downloadManager.currentlyActiveId() == region.id) {
                internals.downloadInProgress = true;
                internals.installInProgress = downloadManager.requests[0].type > 2;
            } else {
                internals.willNotTestDownload = true;
            }
        }
    }
}
