import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import "OfflineMapDownloader.js" as Downloader

Item {
    id: mapInstallMonitor

    anchors.fill: parent

    Item {
        id: internals

        property int numberOfSuccessfulDownloadUpdates: 0

        function startTheInstall() {
            mapCatalog.addInstalledRegion(mapInstallMonitor.id, downloadManager.unpackedFileList);

            log.info("*** checking if city index is installed for this region...")
            Downloader.getCityIndexDataByRegionId(mapInstallMonitor.id, mapCatalog.installedMapsVersion, function(result) {
                if (result) {
                    log.info("*** city index for this region is " + result.Id + " :: " + result.Description + " and the install status is :: " + !!cityIndexList.cityIndex(result.Id))
                    if (!cityIndexList.cityIndex(result.Id)) {
                        log.info("*** you dont't have it, will be downloaded and installed...");
                        mapInstallMonitor.cityIndexData = result;
                        Downloader.downloadCityIndexList(result.Link);

                        internals.checkForUpdateFinish();
                    } else {
                        log.info("*** it seems you have it, skipping...");

                        internals.checkForUpdateFinish();
                    }
                } else {
                    log.info("*** nothing found for city index, skipping...");

                    internals.checkForUpdateFinish();
                }
            });
        }

        function checkForUpdateFinish() {
            log.info("@@@@@@@@@@@@@@          current appState is " + appState.applicationState)
            if (appState.applicationState == "update") {
                var tmp, url, id, countOfFake = 0;
                for (var i = 0; i < downloadManager.requests.length; i++) {
                    tmp = downloadManager.requests[i];
                    url = tmp.url;
                    id = tmp.id;
                    if (id <= 0) {
                        log.info("######COUNT OF FAKE ++###### " + url + " :: " + id + " :: " + i);
                        countOfFake++;
                    }
                }
                if (countOfFake == downloadManager.requests.length) {
                    if (internals.numberOfSuccessfulDownloadUpdates > 0) {
                        log.info("*** all updates finished, will reboot now (cityindexes will trigger after reboot)...");

                        errorMessage.text = qsTranslate("Maps", "qtn_maps_more_offlinemaps_restart_button") + "...";
                        errorMessage.show();

                        appState.restart(1);
                    } else {
                        log.info("*** it seems there is no successful downloads so far, so we shall cancel...")
                        lazyCancelUpdater.start();

                    }
                } else {
                    log.info("*** NOT all updates finished, wait some more...");
                }
            }
        }

        Timer {
            id: lazyCancelUpdater
            interval: 1250
            onTriggered: {
                appState.applicationState = "cancel-update";
            }
        }
    }

    property bool triggerUpdate: false
    property int id
    property variant cityIndexData
    property int failedId
    Connections {
        target: downloadManager
        ignoreUnknownSignals: true

        /*
        onProgress: {
            internetSpeedMonitor.someProgress = true;
            //log.trace("## progress .......................................               " + id + " :: " + operation + " :: " + bytesReceived + " :: " + bytesTotal + " :: " + estimatedTime / 1000 + " :: " + (((bytesTotal - bytesReceived) / 1024) / (estimatedTime / 1000)))
        }
        */
        onFinished: {
            if (operation == 2) {
                //
            } else if (operation == 4) {
                if (id && id > 0) {
                    if (id != failedId) {
                        if (appState.applicationState == "update") {
                            internals.numberOfSuccessfulDownloadUpdates++;
                        }

                        mapInstallMonitor.triggerUpdate = true;
                        mapInstallMonitor.id = id;
                        if (!mainscreen.mapCatalogUpdatedAtLeastOnce) {
                            mainscreen.mapCatalogUpdatedAtLeastOnce = true;
                            mapCatalog.updateCatalog();
                        } else {
                            internals.startTheInstall();
                        }
                    } else {
                        log.info("*** skipping " + id + " as it seems it failed some time ago...");

                        failedId = -100;

                        internals.checkForUpdateFinish();
                    }
                } else if (id == -3 && mapInstallMonitor.cityIndexData) {
                    var tmpCityIndexData = mapInstallMonitor.cityIndexData;
                    log.info("*** it seems city index is downloaded and installed, will update the city index list...")
                    cityIndexList.addCityIndex(tmpCityIndexData.Id, tmpCityIndexData.Description, mapCatalog.installedMapsVersion);

                    internals.checkForUpdateFinish();
                }

                errorMessage.everShown = false;
            }

        }
        onError: {
            log.trace("## error .......................................               " + id + " :: " + error + " :: " + url)
            if (error == 2 || error == 3 || error == 4) {
                failedId = id;

                if (id > 0) {
                    if (errorMessage.scale == 0) {
                        errorMessage.timerEnabled = false;
                        errorMessage.text = qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_failed");
                        errorMessage.show();
                        hideErrorMessageSomeTimeAfterLastOpen.restart();
                    }
                }

                if (error == 4) {
                    downloadManager.cancel(url);
                }
            }
            internals.checkForUpdateFinish();
        }
        onRequestsChanged: {
            internals.checkForUpdateFinish();
        }
    }

    Connections {
        target: mapCatalog
        ignoreUnknownSignals: true

        onBussyChanged: {
            if (!mapCatalog.bussy && mapInstallMonitor.triggerUpdate) {
                mapInstallMonitor.triggerUpdate = false;

                internals.startTheInstall();
            }
        }
    }

    Connections {
        target: appState
        ignoreUnknownSignals: true

        onApplicationStateChanged: {
            if (appState.applicationState != "update") {
                internals.numberOfSuccessfulDownloadUpdates = 0;
            }
        }
        onRestartFailed: {
            waitForWhileBeforeRetryToRestart.go(mode);
        }
    }

    Timer {
        id: waitForWhileBeforeRetryToRestart

        property int mode

        interval: 250
        function go(mode) {
            waitForWhileBeforeRetryToRestart.mode = mode;
            waitForWhileBeforeRetryToRestart.start();
        }

        onTriggered: {
            appState.restart(waitForWhileBeforeRetryToRestart.mode);
        }
    }

    /*
    Timer {
        id: internetSpeedMonitor

        property bool someProgress: false

        running: mainscreen.applicationMaximized && !downloadManager.isSuspended && downloadManager.requests.length > 0
        repeat: true
        interval: 1000 * 60 * 1
        onTriggered: {
            if (!internetSpeedMonitor.someProgress) {
                log.info("it seems your internet is too slow or not working....")

                if (!errorMessage.everShown) {
                    //errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_too_slow_connection");
                    //errorMessage.show();
                    errorMessage.everShown = true;
                }
            } else {
                internetSpeedMonitor.someProgress = false;
            }
        }
    }
    */

    Timer {
        id: hideErrorMessageSomeTimeAfterLastOpen

        interval: 1000 * 30

        onTriggered: {
            errorMessage.hide();
        }
    }

    InfoBanner {
        id: errorMessage

        property bool everShown: false

        timerEnabled: true
        timerShowTime: 15 * 1000

        anchors.top: parent.top
        anchors.topMargin: 89

        onScaleChanged: {
            if (errorMessage.scale == 0) {
                errorMessage.timerEnabled = true;
            }
        }
    }
}
