import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0

import "DownloadTools.js" as Tools

CommonItemDelegate {
    id: commonItemDelegate

    property string name

    property bool listenForSuspendedChanges: false

    property bool waiting
    property bool downloading
    property bool installing
    property bool paused
    property bool forcepaused
    property bool failed
    property bool installed

    property string _status

    property real progress
    property real bytesTotal
    property real bytesReceived
    property real estimatedTime

    property bool hasChildren
    property bool error: false

    property string regionId

    signal itemClicked(int index)

    Rectangle {
        id: backgroundRectangle
        anchors.fill: parent
        color: "#f0f1f2"
    }

    MapsText {
        state: "listNormalBold"
        id: countryName
        objectName: "countryNameTxt"
        text: name.indexOf("qtn_") === -1 ? name : qsTranslate("Maps", name);
        elide: Text.ElideMiddle

        width: cloneText.paintedWidth > parent.width - 60? (parent.width - 60) : cloneText.paintedWidth
        anchors {
            left: parent.left
            leftMargin: 20
            verticalCenter: parent.verticalCenter
        }
    }

    MapsText {
        state: "listNormalBold"
        id: cloneText
        visible: false
        text: countryName.text
    }

    MapsText {
        state: "smallLight"
        id: status
        objectName: "statusTxt"
        visible: !installed
        elide: Text.ElideMiddle
        width: text? (parent.width - countryName.width - 40) : 0
        horizontalAlignment: Text.AlignRight

        anchors {
            right: parent.right
            rightMargin: 20
            verticalCenter: parent.verticalCenter
        }
        text: {
            if (forcepaused) {
                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_paused_status");
            } else if (waiting) {
                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_waiting_status");
            } else if (downloading) {
                if (bytesTotal && estimatedTime) {
                    return qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_status").arg(
                        Tools.formatBytes(bytesReceived)).arg(
                        Tools.formatBytes(bytesTotal))/*.arg(
                        Tools.formatMilliseconds(estimatedTime)
                    )*/;
                } else {
                    return qsTranslate("Maps", "qtn_maps_more_offlinemaps_download_starting_status");
                }
            } else if (installing) {
                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_installing_status");
            } else if (paused) {
                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_paused_status");
            } else if (failed) {
                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_paused_status");
            } else {
                return "";
            }
        }

    }

    ProgressBar {
        id: progressBar
        visible: waiting || downloading || installing || forcepaused || paused || failed
        value: progress
        indeterminate: forcepaused || paused || failed || waiting || (installing && !progress)
        anchors {
            left: parent.left
            right: parent.right
            leftMargin: 10
            rightMargin: 10
            bottom: parent.bottom
            bottomMargin: 10
        }

        minimumValue: 0
        maximumValue: 100
    }

    Image {
        id: rightGraphic;
        visible: !installed && !hasChildren ? false : true
        source: odmlAssetsPath + "images/%1.png".arg(installed ? "check" : "arrow_right_dark")

        anchors {
            right: parent.right
            rightMargin: installed ? 15 : 30
            verticalCenter: parent.verticalCenter
        }
    }

    MouseArea {
        id: mouseAreaDelegate
        anchors.fill: backgroundRectangle
        onPressed: { if (!error) backgroundRectangle.color = "#d0d1d2" }
        onReleased: { if (!error) backgroundRectangle.color = "#f0f1f2" }
        onCanceled: { if (!error) backgroundRectangle.color = "#f0f1f2" }
        onClicked: {
            if (!error) {
                itemClicked(index);
            }
        }
    }

    Connections {
        target: downloadManager
        ignoreUnknownSignals: true

        onProgress: {
            if (id == commonItemDelegate.regionId) {
                if (operation === 2) {
                    commonItemDelegate._status = "downloading";
                } else if (operation === 3 || operation === 4) {
                    commonItemDelegate._status = "installing";
                }

                commonItemDelegate.bytesTotal = bytesTotal;
                commonItemDelegate.bytesReceived = bytesReceived;
                commonItemDelegate.progress = (bytesReceived * 100 / bytesTotal);
                commonItemDelegate.estimatedTime = estimatedTime;

                offlineMapsPage.saveCurrentProgressInStash(id, commonItemDelegate._status, commonItemDelegate.bytesTotal, commonItemDelegate.bytesReceived, commonItemDelegate.estimatedTime);
            }
        }
        onFinished: {
            offlineMapsPage.saveCurrentProgressInStash(-1);
        }

        onIsSuspendedChanged: {
            if (commonItemDelegate.listenForSuspendedChanges) {
                if (downloadManager.isSuspended) {
                    commonItemDelegate.forcepaused = true;
                } else {
                    commonItemDelegate.forcepaused = false;
                }
            }
        }
    }

    Component.onCompleted: {
        var tmpStash = offlineMapsPage.getCurrentProgressFromStash(commonItemDelegate.regionId);
        if (tmpStash && tmpStash.valid) {
            commonItemDelegate.bytesTotal = tmpStash.totalSize;
            commonItemDelegate.bytesReceived = tmpStash.downloadedSize;
            commonItemDelegate.progress = (commonItemDelegate.bytesReceived * 100 / commonItemDelegate.bytesTotal);
            commonItemDelegate.estimatedTime = tmpStash.timeLeft;
            commonItemDelegate._status = tmpStash.status;
        }
    }
}
