function pushNextRegionPage(regionHead) {
    var tmp = offlineMapsPage.subOfflineMapListGetInstance(offlineMapsPage.__subList, pageStack);

    tmp.headRegion = regionHead.modelData;

    pageStack.push(tmp);

    delete tmp;

    offlineMapsPage.subOfflineMapListIncreaseIndex();
}

function pushDownloadPage(region) {
    var tmp = offlineMapsPage.downloadPageInstance;
    if (!tmp) {
        tmp = offlineMapsPage.__downloadPageComponent.createObject(offlineMapsPage);
        offlineMapsPage.downloadPageInstance = tmp;
    }

    tmp.region = region.modelData;
    pageStack.push(tmp);

    delete tmp;
}

function pushRemovePage(region) {
    var container = removePage.createObject(pageStack);
    container.region = region.modelData;
    pageStack.push(container);
}

function processItemSelected(item) {
    if (!item.modelData.installed && item.modelData.children && item.modelData.children.length > 0) {
        pushNextRegionPage(item);
    } else if (!item.modelData.installed) {
        pushDownloadPage(item);
    } else {
        pushRemovePage(item);
    }
}

/**
  * aParams:
  *     aParams.type: "contintent" || "country" || installed
  *     aParams.model: the first level model to return
  *     aParams.availableRegions: the list of regions available
  *     aParams.installedRegions: the list of regions installed
  *     aParams.showInstalledRegions: default to false, puts installed regions at start of the given model
  */
function getFirstLevel(aParams) {
    aParams.model.clear();
    if (aParams.modelDownloading) {
        aParams.modelDownloading.clear();
    }

    if (aParams.type === "continent") {
        showContinentLevel(aParams);
    } else if (aParams.type === "installed") {
        showInstalledLevel(aParams);
    } else if (aParams.type === "installing") {
        showInstalledAndInstallingLevel(aParams);
    } else if (aParams.type === "country") {
        showCountryLevel(aParams);
    }
}

function showInstalledAndInstallingLevel(aParams) {
    if (aParams.modelDownloading) {
        _populateListInModel(
            _getListOfInstallingRegions(aParams.queue, aParams.availableRegions),
            aParams.modelDownloading,
            "installing"
        );
    }
    _populateListInModel(
        _getListOfInstalledRegions(aParams.installedRegions),
        aParams.model,
        "installed"
    );
}

function showInstalledLevel(aParams) {
    _populateListInModel(
        _getListOfInstalledRegions(aParams.installedRegions),
        aParams.model,
        "installed"
    );
}

function showContinentLevel(aParams) {
    if (aParams.availableRegions.length === 0) {
        aParams.model.append(_getErrorItem(qsTranslate("Maps", "qtn_maps_more_offlinemaps_nomapsavailable")));
    } else {
        var installedIndex = 0;
        var tmp = [];
        for (var i = 0; i < aParams.availableRegions.length; i++) {
            if (aParams.showInstalledRegions && aParams.availableRegions[i] && aParams.availableRegions[i].installed) {
                tmp.push(mapCatalog.availableRegions[i]);
            } else if(!aParams.availableRegions[i].hasParent) {
                tmp.push(mapCatalog.availableRegions[i]);
            }
        }
        tmp = orderArrayByName(tmp);
        for (i = 0; i < tmp.length; i++) {
            if (aParams.showInstalledRegions && tmp[i] && tmp[i].installed) {
                aParams.model.insert(installedIndex++, {modelData: tmp[i]});
            } else if (!tmp[i].hasParent) {
                aParams.model.insert(aParams.model.count, {modelData: tmp[i]});
            }
        }
    }
}

function showCountryLevel(aParams) {
    var installedIndex = 0, tmpArray = [], i, tmp;
    if (aParams.showInstalledRegions) {
        for (i = 0; i < aParams.installedRegions.length; i++) {
            aParams.model.append({modelData: aParams.installedRegions[i]});
        }
    }
    if (aParams.availableRegions.length === 0) {
        aParams.model.append(_getErrorItem(qsTranslate("Maps", "qtn_maps_more_offlinemaps_nomapsavailable")));
    } else {
        for (i = 0; i < aParams.availableRegions.length; i++) {
            if (!aParams.availableRegions[i].hasParent) {
                for(var c = 0; c < aParams.availableRegions[i].children.length; c++) {
                    tmp = aParams.availableRegions[i].children[c];
                    if (!tmp.installed) {
                        tmpArray[tmpArray.length] = tmp;
                    }
                }
            }
        }
        tmpArray = orderArrayByName(tmpArray);
        for (i = 0; i < tmpArray.length; i++) {
            aParams.model.append({modelData: tmpArray[i]});
        }
    }
}

function _getErrorItem(aMessage) {
    return {
        modelData: {
            name: aMessage,
            installed: false,
            children: [],
            error: true
        }
    }
}

function _getListOfInstallingRegions(installingRegions, availableRegions) {
    var currentItem, currentItemId, data, status;
    var listToApply = {};

    for (var i = 0; i < installingRegions.length; i++) {
        currentItem = installingRegions[i];
        currentItemId = parseInt(currentItem.id, 10);

        if (!listToApply[currentItemId]) {
            if (currentItemId > 0) {
                log.trace("*** adding URL to the list :: " + currentItemId);

                if (i === 0) {
                    if (i === 0 && downloadManager.isSuspended) {
                        status = "paused";
                    } else {
                        if (currentItem.type == 2) {
                            status = "downloading";
                        } else {
                            status = "installing";
                        }
                    }
                } else {
                    status = "waiting";
                }

                data = _getItemFromAvailableRegions(availableRegions, currentItemId);
                if (data) {
                    listToApply[currentItemId] = {
                        url: currentItem.url,
                        id: currentItemId,
                        downloadUrl: data.downloadUrl,
                        localizedName: data.localizedName || data.name,
                        installed: false,
                        status: status,
                        operation: null,
                        bytesTotal: data.bytesTotal,
                        bytesReceived: 0,
                        estimatedTime: 0
                    };
                }

            } else if (!currentItemId) {
                log.trace("*** skipping URL as it's testing download " + currentItem.url + " :: " + currentItem.id);
            } else if (currentItemId == -1 || currentItemId == -2) {
                log.trace("*** skipping URL as it's updating catalog download " + currentItem.url + " :: " + currentItem.id);
            }
        } else {
            //log.trace("*** already in the list, skipping");
        }
    }

    return _objectToArray(listToApply)
}

function _getListOfInstalledRegions(installedRegions) {
    var currentItem, currentItemId;
    var listToApply = {};

    var alreadyInstalledRegions = cloneAndOrderArrayByName(installedRegions);
    for (var i = 0; i < alreadyInstalledRegions.length; i++) {
        currentItem = alreadyInstalledRegions[i];
        currentItemId = parseInt(alreadyInstalledRegions[i].id, 10);

        if (!listToApply[currentItemId]) {
            listToApply[currentItemId] = {
                url: currentItem.url,
                id: currentItemId,
                downloadUrl: currentItem.downloadUrl,
                localizedName: currentItem.localizedName || currentItem.name,
                installed: true,
                status: "finished",
                operation: null,
                bytesTotal: currentItem.bytesTotal,
                bytesReceived: 0,
                estimatedTime: 0
            };
        }
    }

    return _objectToArray(listToApply);
}

function _populateListInModel(list, model, mode) {
    for (var i = 0; i < list.length; i++) {
        model.append({modelData: list[i]});
    }

    if (model.count === 0) {
        if (mode == "installed") {
            model.append(_getErrorItem(qsTranslate("Maps", "qtn_maps_more_offlinemaps_nomapsinstalled")));
        } else if (mode == "available") {
            model.append(_getErrorItem(qsTranslate("Maps", "qtn_maps_more_offlinemaps_nomapsavailable")));
        }
    }
}

function _getItemFromAvailableRegions(availableRegions, id) {
    var data;

    for (var i = 0; i < availableRegions.length; i++) {
        if (availableRegions[i].id == id) {
            data = availableRegions[i];
            break;
        }
    }

    return data;
}

function _sort(a, b) {
    var first = a.localizedName || a.name;
    var second = b.localizedName || b.name;
    if(first < second) {
        return -1;
    }
    if(first > second) {
        return 1;
    }
    return 0;
}

function _objectToArray(object) {
    var result = [];

    for (var i in object) {
        if (object.hasOwnProperty(i)) {
            result.push(object[i]);
        }
    }

    return result;
}

function cloneAndOrderArrayByName(array) {
    var tmpArray = [];

    for (var i = 0; i < array.length; i++) {
        tmpArray.push(array[i]);
    }

    return orderArrayByName(tmpArray);
}

function orderArrayByName(array) {
    var i, j, ttt, tmpNameA, tmpNameB;
    for (i = 0; i < array.length - 1; i++) {
        for (j = i + 1; j < array.length; j++) {
            tmpNameA = array[i].localizedName? array[i].localizedName : array[i].name;
            tmpNameB = array[j].localizedName? array[j].localizedName : array[j].name;
            if (mapCatalog.localeAwareCompare(tmpNameA, tmpNameB) > 0) {
                ttt = array[i];
                array[i] = array[j];
                array[j] = ttt;
            }
        }
    }
    return array;
}
