import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import ovi.connector 1.0

import "OfflineMapLogic.js" as Logic
import "OfflineMapSubOfflineMapList.js" as SubOfflineMapList
import "OfflineMapDownloader.js" as Downloader

Page {
    id: offlineMapsPage
    objectName: "offlineMapsPage"

    property bool alreadyLoaded: false
    property bool hideButtons: false
    property bool triggerResetSignals: false

    property int numberOfSuccessDownloads: 0
    property bool monitorDownloads: false
    property bool preventListeningForRequestChanges: false

    property bool updateMode: appState.applicationState == "update"

    property bool waitingForApplicationModeToChange: false
    property bool waitingForUpdateCatalog: false
    property bool postTriggerHandleOnCompleted: false
    property bool postTriggerSkipUpdate: false
    property bool preventCancelUpdateSignal: false

    property int downloadsStartedInThisCycle: 0

    property bool __bogusUpdater__: true
    property bool __hideMenuToolIconAfterUpdateCancel__: false

    property variant stash
    function saveCurrentProgressInStash(id, status, totalSize, downloadedSize, timeLeft) {
        if (id != -1) {
            offlineMapsPage.stash = {id: id, status: status, totalSize: totalSize, downloadedSize: downloadedSize, timeLeft: timeLeft};
        } else {
            offlineMapsPage.stash = null;
        }
    }
    function getCurrentProgressFromStash(id) {
        var result;

        var tmpStash = offlineMapsPage.stash;
        if (tmpStash && tmpStash.id == id) {
            result = {valid: true, status: tmpStash.status, totalSize: tmpStash.totalSize, downloadedSize: tmpStash.downloadedSize, timeLeft: tmpStash.timeLeft};
        } else {
            result = {valid: false, status: "", totalSize: 0, downloadedSize: 0, timeLeft: 0};
        }

        return result;
    }

    function updateMapCatalog(version) {
        var installedMapVersionBefore = mapCatalog.installedMapsVersion;
        mapCatalog.installedMapsVersion = version;
        if (installedMapVersionBefore != mapCatalog.installedMapsVersion) {
            indicator.visible = true;
            if (mapCatalog.bussy) {
                waitingForUpdateCatalog = true;
            } else {
                mainscreen.mapCatalogUpdatedAtLeastOnce = true;
                mapCatalog.updateCatalog();

                indicator.visible = false;
            }
        }
    }

    function checkForUpdates() {
        var tmp = updatesPageInstance;
        if (!tmp) {
            tmp = updatesPageComponent.createObject(offlineMapsPage);
            updatesPageInstance = tmp;
        }

        offlineMapsPage.pageStack.push(tmp);

        delete tmp;
    }

    function addNewMap() {
        var tmp = mapCatalogInstance;
        if (!tmp) {
            tmp = mapCatalogComponent.createObject(offlineMapsPage);
            mapCatalogInstance = tmp;
        }

        offlineMapsPage.pageStack.push(tmp);

        delete tmp;
    }

    function handleOnCompleted() {
        if (!handleOnCompleteFunctionFilter.running) {
            handleOnCompleteFunctionFilter.start();
        } else {
            //console.log("... just ignore the request as we have it already ...");
        }
    }

    function _proceedhandleOnCompleted() {
        var ensureCatalogDataResult = Downloader.ensureCatalogData();
        if (ensureCatalogDataResult == 1) {
            hideButtons = false;
            loading.visible = false;
            loadingNeedsWiFi.visible = false;
            monitorDownloads = false;
            alreadyLoaded = true;

            Logic.getFirstLevel({
                type: "installing",
                model: firstLevelModel,
                modelDownloading: firstLevelDownloadingModel,
                availableRegions: mapCatalog.availableRegions,
                installedRegions: mapCatalog.installedRegions,
                queue: downloadManager.requests,
                showInstalledRegions: true
            });

            return true;
        } else if (ensureCatalogDataResult == -1) {
            hideButtons = true;
            loading.visible = false;
            loadingNeedsWiFi.visible = true;
            monitorDownloads = false;

            return false;
        } else {
            hideButtons = true;
            loading.visible = true;
            loadingNeedsWiFi.visible = false;
            monitorDownloads = true;

            return false;
        }
    }

    function checkForLeakingDownloads() {
        log.info("######SIZE OF THE QUEUE###### " + downloadManager.requests.length)
        var tmp, url, id;
        for (var i = 0; i < downloadManager.requests.length; i++) {
            tmp = downloadManager.requests[i];
            url = tmp.url;
            id = tmp.id;
            log.trace("######STILL IN THE QUEUE###### " + url + " :: " + id)
            if (id == -1 || id == -2 || id === "") {
                log.info("######REMOVE FROM QUEUE###### " + url + " :: " + id);
                downloadManager.cancel(url);
            }
        }
        log.trace("######SIZE OF THE QUEUE AFTER CLEANUP###### " + downloadManager.requests.length)
    }

    function handleCancelUpdate() {
        waitingForApplicationModeToChange = true;
        indicator.visible = true;
        downloadManager.cancelAll();
        mapCatalog.resetInstanceWithPathSuffix();
        cityIndexList.resetInstanceWithPathSuffix();
        if (appState.applicationState && !mainscreen.allTabsReady) {
            offlineMapsPage.__hideMenuToolIconAfterUpdateCancel__ = true;
        }

        appState.applicationState = "";

        __bogusUpdater__ = !__bogusUpdater__;
        __bogusUpdater__ = !__bogusUpdater__;
    }

    function subOfflineMapListIncreaseIndex() { SubOfflineMapList.subOfflineMapListIncreaseIndex(); }
    function subOfflineMapListDecreaseIndex() { SubOfflineMapList.subOfflineMapListDecreaseIndex(); }
    function subOfflineMapListResetIndex() { SubOfflineMapList.subOfflineMapListResetIndex(); }
    function subOfflineMapListGetInstance(component, parent) { return SubOfflineMapList.subOfflineMapListGetInstance(component, parent); }

    tools: ToolBarLayout {
        ToolIcon {
            id: iconToolbarBack
            visible: !offlineMapsPage.updateMode && !indicator.visible
            enabled: __bogusUpdater__ || (mainscreen.enableStaticODMLAddMode? downloadManager.requests.length == 0 : true)
            opacity: enabled? 1 : 0.4

            anchors.left: parent.left
            objectName: "toolBarIconBackButton"
            iconId: "icon-m-toolbar-back"
            onClicked: { mainscreen.ensureAllPageStacks(); waiterForTimeout.stop(); pageStack.pop(); }
        }
        ToolButton {
            text: qsTranslate("Maps", "qtn_map_apply_run_parameters_cancelbtn")
            implicitHeight: iconToolbarBack.height - 20
            width: parent.width / 2

            objectName: "toolBarUpdateCancel"
            anchors.horizontalCenter: parent.horizontalCenter
            visible: offlineMapsPage.updateMode && !indicator.visible
            onClicked: {
                queryDialog.titleText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_confirm_exit_caption");
                queryDialog.message = qsTranslate("Maps", "qtn_maps_more_offlinemaps_confirm_exit_body");
                queryDialog.acceptButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_yes");
                queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_no");
                queryDialog.mode = "cancel-update";
                queryDialog.open();
            }
        }
        ToolIcon {
            visible: !offlineMapsPage.updateMode && !hideButtons && !indicator.visible && !offlineMapsPage.__hideMenuToolIconAfterUpdateCancel__

            iconId: "icon-m-toolbar-view-menu"
            objectName: "toolBarIconViewButton"
            onClicked: {
                if (triggerResetSignals || mapCatalog.installedRegions.length > 0 && downloadManager.requests.length == 0) {
                    viewMenuBig.open();
                } else {
                    viewMenuSmall.open();
                }
            }
        }
    }

    Menu {
        id: viewMenuBig
        visualParent: mainscreen
        MenuLayout {
            MenuItem {
                text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_addmaps")
                onClicked: { addNewMap() }
            }
            MenuItem {
                text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_checkupdates")
                onClicked: {
                    if (triggerResetSignals || mapCatalog.installedRegions.length > 0 && downloadManager.requests.length == 0) {
                        checkForUpdates();
                    }
                }
            }
            MenuItem {
                text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_delete_all_map_data")
                onClicked: {
                    if (triggerResetSignals || mapCatalog.installedRegions.length > 0 && downloadManager.requests.length == 0) {
                        queryDialog.titleText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_delete_all_map_data_title");
                        queryDialog.message = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_delete_all_map_data_body");
                        queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_no");
                        queryDialog.acceptButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_yes");
                        queryDialog.mode = "delete-all";
                        queryDialog.open();
                    }
                }
            }
        }
    }
    Menu {
        id: viewMenuSmall
        visualParent: mainscreen
        MenuLayout {
            MenuItem {
                text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_addmaps")
                onClicked: { addNewMap() }
            }
        }
    }

    Rectangle {
        anchors.fill: parent

        color: "#f0f1f2"
    }

    OdmlHeader {
        id: header
        title: qsTranslate("Maps", "qtn_maps_more_offlinemaps")

        z: 2
    }

    Flickable {
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: header.bottom
        anchors.bottom: parent.bottom

        contentHeight: firstPageDownloadingList.visible? 75 * firstLevelDownloadingModel.count + 10 + listHeader.height + 75 * firstLevelModel.count + 10 : parent.height
        interactive: firstPageDownloadingList.visible
        clip: true

        Rectangle {
            color: "#f0f1f2"
            anchors.fill: parent

            ListView {
                id: firstPageDownloadingList

                anchors {left: parent.left; right: parent.right; top: parent.top;}
                clip: false
                model: firstLevelDownloadingModel
                interactive: false
                height: 75 * firstLevelDownloadingModel.count + 10

                visible: !indicator.visible && firstLevelDownloadingModel.count > 0

                delegate: OfflineMapListDelegate {
                    id: odmlDelegate
                    objectName:  "odmlDelegate_" + index
                    name: modelData.localizedName || modelData.name

                    listenForSuspendedChanges: true

                    waiting:  modelData.status ==  "waiting"
                    downloading: modelData.status == "downloading"
                    installing: modelData.status == "installing"
                    paused: modelData.status == "paused"
                    failed: modelData.status == "failed"
                    installed: modelData.status == "finished" || modelData.installed? true : false

                    _status: modelData.status

                    regionId: modelData.id

                    progress: {
                        if (modelData.status == "downloading" || modelData.status == "installing") {
                            return modelData.progress || 0;
                        } else if (modelData.status == "waiting") {
                            return 0;
                        } else {
                            return 75;
                        }
                    }
                    bytesTotal: 0
                    bytesReceived: 0
                    estimatedTime: 0

                    error: modelData.error || false
                    hasChildren: modelData.children? (modelData.children.length > 0) : false
                    onItemClicked: Logic.processItemSelected(firstLevelDownloadingModel.get(index));
                }
            }

            ScrollDecorator {
                flickableItem: firstPageDownloadingList
            }

            ListModel { id: firstLevelDownloadingModel }

            OviListHeader {
                id: listHeader
                anchors {
                    left: parent.left;
                    right: parent.right;
                    top: firstPageDownloadingList.bottom
                }
                visible: !loading.visible && !loadingFailed.visible && !indicator.visible && !loadingNeedsWiFi.visible
                headerText: qsTranslate("Maps", "qtn_maps_more_offlinemaps_ondevice");

            }

            Rectangle {
                id: loading
                color: "transparent"
                visible: false
                anchors {left: parent.left; right: parent.right; top: parent.top; topMargin: 10}

                MapsText {
                    state: "listNormal"

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_catalog")
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }

            Rectangle {
                id: loadingFailed
                color: "transparent"
                visible: false
                anchors {left: parent.left; right: parent.right; top: parent.top; topMargin: 10; leftMargin: 10}

                MapsText {
                    state: "normalWrap"

                    anchors.fill: parent

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_failed")
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }

            Rectangle {
                id: loadingNeedsWiFi
                color: "transparent"
                visible: false
                anchors {left: parent.left; right: parent.right; top: parent.top; topMargin: 10; leftMargin: 10}

                MapsText {
                    state: "normalWrap"

                    anchors.fill: parent

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_wifi_required")
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }

            ListView {
                id: firstPageList

                anchors {left: parent.left; right: parent.right; top: listHeader.bottom;}
                clip: !firstPageDownloadingList.visible
                model: firstLevelModel
                interactive: !noMapsInstalledButtons.visible && !firstPageDownloadingList.visible

                height: {
                    return !firstPageDownloadingList.visible? mainscreen.height - listHeader.height - listHeader.height - 100 : 75 * firstLevelModel.count + 10
                }

                visible: !indicator.visible

                delegate: OfflineMapListDelegate {
                    id: odmlDelegate
                    objectName:  "odmlDelegate_" + index
                    name: modelData.localizedName || modelData.name

                    waiting:  modelData.status ==  "waiting"
                    downloading: modelData.status == "downloading"
                    installing: modelData.status == "installing"
                    paused: modelData.status == "paused"
                    failed: modelData.status == "failed"
                    installed: modelData.status == "finished" || modelData.installed? true : false

                    _status: modelData.status? modelData.status : ""

                    regionId: modelData.id? modelData.id : ""

                    progress: {
                        if (modelData.status == "downloading" || modelData.status == "installing") {
                            return modelData.progress || 0;
                        } else if (modelData.status == "waiting") {
                            return 0;
                        } else {
                            return 75;
                        }
                    }
                    bytesTotal: 0
                    bytesReceived: 0
                    estimatedTime: 0

                    error: modelData.error || false
                    hasChildren: modelData.children? (modelData.children.length > 0) : false
                    onItemClicked: Logic.processItemSelected(firstLevelModel.get(index));
                }
            }

            ScrollDecorator {
                flickableItem: firstPageList
            }

            ListModel { id: firstLevelModel }

            Rectangle {
                id: noMapsInstalledButtons
                color: "transparent"
                visible: !hideButtons && appState.applicationState == "normal" && downloadManager.requests && mapCatalog && downloadManager.requests.length == 0 && mapCatalog.installedRegions.length == 0 && !indicator.visible
                anchors {left: parent.left; right: parent.right; top: parent.top; topMargin: 160}

                Button {
                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_addmaps")

                    width: parent.width / 2
                    anchors.centerIn: parent

                    onClicked: { addNewMap() }
                }
            }
        }
    }

    property Component __subList: subList
    Component {
        id: subList
        SubOfflineMapList {
            onChildClicked: Logic.processItemSelected({modelData: child});
        }
    }

    property variant downloadPageInstance
    property Component __downloadPageComponent: downloadPageComponent
    Component {
        id: downloadPageComponent
        DownloadPage {}
    }

    Component {
        id: removePage
        RemovePage {}
    }

    property variant mapCatalogInstance
    Component {
        id: mapCatalogComponent
        MapCatalog {}
    }

    property variant updatesPageInstance
    Component {
        id: updatesPageComponent
        UpdatesPage {}
    }

    QueryDialog {
        id: queryDialog

        property string mode

        onAccepted: {
            if (mode == "delete-all") {
                errorMessage.text = qsTranslate("Maps", "qtn_maps_more_offlinemaps_restart_button") + "...";
                errorMessage.show();

                waitForWhileBeforeRestart.go(2);
            } else if (mode == "cancel-update") {
                preventCancelUpdateSignal = true;
                handleCancelUpdate();
                preventCancelUpdateSignal = false;
            }
        }
        onRejected: {}
    }

    BusyIndicator {
        id: indicator

        z:2

        platformStyle: BusyIndicatorStyle { size: "large" }
        running: visible
        visible: false

        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenterOffset: 60
    }

    InfoBanner {
        id: errorMessage

        property bool everShown: false

        timerEnabled: true
        timerShowTime: 15 * 1000

        anchors.top: parent.top
        anchors.topMargin: 89
    }

    Connections {
        target: mapCatalog
        ignoreUnknownSignals: true

        onBussyChanged: {
            if (waitingForUpdateCatalog && !mapCatalog.bussy) {
                waitingForUpdateCatalog = false;

                if (!postTriggerSkipUpdate) {
                    mainscreen.mapCatalogUpdatedAtLeastOnce = true;
                    mapCatalog.updateCatalog();

                    postTriggerSkipUpdate = false;
                }

                indicator.visible = false;

                if (postTriggerHandleOnCompleted) {
                    offlineMapsPage.handleOnCompleted();
                    postTriggerHandleOnCompleted = false;
                }
            }
        }
    }

    Connections {
        target: downloadManager
        ignoreUnknownSignals: true

        onRequestsChanged: {
            if (!monitorDownloads && !preventListeningForRequestChanges) {
                handleOnCompleted();
            }
        }

        onFinished: {
            offlineMapsPage.triggerResetSignals = true;
            offlineMapsPage.triggerResetSignals = false;

            if (monitorDownloads) {
                if (operation == 4) {
                    offlineMapsPage.numberOfSuccessDownloads++;
                }
                if (offlineMapsPage.numberOfSuccessDownloads == 2) {
                    mapCatalog.installedMapsVersion = Downloader.getCurrentMapVersion();

                    indicator.visible = true;
                    mainscreen.mapCatalogUpdatedAtLeastOnce = true;
                    mapCatalog.updateCatalog();

                    if (!mapCatalog.catalogFilesExist()) {
                        mapCatalog.updateLocalContentFile();
                    }

                    handleOnCompleted();

                    indicator.visible = false;
                    alreadyLoaded = true;
                }

                log.info("finished *******************              " + operation + " :: " + url + " " + id)
            }
        }
        /*
        onProgress: {
            if (monitorDownloads) {
                log.trace("progress *******************              " + operation + " :: " + url + " " + id)
            }
        }
        */
        onError: {
            if (monitorDownloads) {
                log.error("error *******************              " + operation + " :: " + url + " " + id)
                log.warning("retrying ****************************  ")
                log.info("..... will try to download \"" + url + "\" to \"" + mapCatalog.mapLoaderDirectory + "\" ....");
                //downloadManager.download(id, url, mapCatalog.mapLoaderDirectory + "/", false);
                errorMessage.text = qsTranslate("qtn_maps_more_offlinemaps_downloading_failed");
                errorMessage.show();
            }
        }

    }

    Connections {
        target: appState
        ignoreUnknownSignals: true

        onApplicationStateChanged: {
            if (waitingForApplicationModeToChange) {
                if (!updateMode) {
                    if (!mapCatalog.bussy) {
                        offlineMapsPage.handleOnCompleted();
                        indicator.visible = false;
                    } else {
                        waitingForUpdateCatalog = true;
                        postTriggerSkipUpdate = true;
                        postTriggerHandleOnCompleted = true;
                    }
                }

                waitingForApplicationModeToChange = false;
            }

            if (appState.applicationState == "cancel-update") {
                if (!preventCancelUpdateSignal) {
                    handleCancelUpdate();
                }
            }
        }
    }

    onStatusChanged: {
        if (status == PageStatus.Active) {
            if (alreadyLoaded && downloadManager.requests.length > 0) {
                if (!waitingForUpdateCatalog) {
                    offlineMapsPage.handleOnCompleted();
                } else {
                    postTriggerHandleOnCompleted = true;
                }
            } else if (!alreadyLoaded && loadingNeedsWiFi.visible) {
                offlineMapsPage.handleOnCompleted();
            }

            offlineMapsPage.__hideMenuToolIconAfterUpdateCancel__ = false;

        }
    }

    Component.onCompleted: {
        mainscreen.mapCatalogUpdatedAtLeastOnce = true;
        mapCatalog.updateCatalog();

        checkForLeakingDownloads();
        if (handleOnCompleted()) { }
        offlineMapsPage.downloadsStartedInThisCycle = 0;
    }

    Timer {
        id: waitForWhileBeforeRestart

        property int restartMode

        interval: 1

        function go(restartMode) {
            waitForWhileBeforeRestart.restartMode = restartMode;
            waitForWhileBeforeRestart.start();
        }

        onTriggered: {
            appState.restart(waitForWhileBeforeRestart.restartMode);
        }
    }

    Timer {
        id: waiterForTimeout
        running: mainscreen.applicationMaximized && monitorDownloads
        interval: 60 * 1000
        onTriggered: {
            if (monitorDownloads) {
                log.trace("............. TIMEOUT (offline maps) ................")
                loadingFailed.visible = true;
                loading.visible = false;
                downloadManager.cancelAll();
            }
        }
    }

    Timer {
        id: handleOnCompleteFunctionFilter
        interval: 100

        onTriggered: {
            _proceedhandleOnCompleted();
        }
    }
}
