import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import ovi.connector 1.0

import "DownloadTools.js" as Tools

Page {
    property variant region;
    anchors.fill: parent;

    Item {
        id: internals

        property double totalSize: region && (region.downloadSize || (mapCatalog.region(region.id) && mapCatalog.region(region.id).downloadSize)) || 0

        function handleRemoveButtonClicked() {
            queryDialog.titleText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_confirm_remove_caption");
            queryDialog.message = qsTranslate("Maps", "qtn_maps_more_offlinemaps_confirm_remove_body");
            queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_remove_closebnt");
            queryDialog.acceptButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_remove_acceptbnt");
            queryDialog.open();
        }

        function handleQueryDialogAccept() {
            mapCatalog.deleteInstalledRegion(region.id);

            offlineMapsPage.handleOnCompleted();
            pageStack.pop(offlineMapsPage);
        }

        function handleQueryDialogReject() {
            //
        }
    }

    OdmlHeader {
        id: header

        title: region ? region.localizedName : ""
        z: 1
    }

    Rectangle {
        anchors.top: header.bottom
        anchors.bottom: parent.bottom
        anchors { left: parent.left; right: parent.right }
        clip: true
        color: "#eff0f1"

        Flickable {
            anchors.fill: parent

            contentWidth: parent.width
            contentHeight: columnBody.height

            Column {
                id: columnBody

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 16; color: "#eff0f1" }

                MapsText {
                    state: "normalWrap"

                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_available_this_map_offline_use")
                }

                MapsText {
                    state: "normalWrap"
                    font.family: mainscreen.__labelStyle.fontFamily
                    visible: false

                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_megabytes_to_free").arg(
                              internals.totalSize? Tools.formatBytes(internals.totalSize, 0) : qsTranslate("Maps", "qtn_maps_more_offlinemaps_size_undefined"))
                }

                Row {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    Image {
                        id: imageB1
                        source: odmlAssetsPath + "images/filesize.png"
                    }
                    MapsText {
                        state: "normalWrap"
                        wrapMode: "WrapAnywhere"
                        anchors.verticalCenter: imageB1.verticalCenter
                        visible: internals.downloadedSize && internals.totalSize && internals.downloadInProgress && !internals.downloadPaused && !internals.allFinished && !internals.installInProgress

                        text: internals.totalSize? Tools.formatBytes(internals.totalSize, 0) + " " + qsTranslate("Maps", "qtn_maps_more_offlinemaps_megabytes") : qsTranslate("Maps", "qtn_maps_more_offlinemaps_size_undefined")
                    }
                }

                Rectangle {
                    anchors { left: parent.left; right: parent.right }
                    height: 24;
                    color: "#eff0f1"
                }

                Button {
                    visible: false
                    text: qsTranslate("Maps", "qtn_maps_more_remove")
                    anchors {left: parent.left; right: parent.right; leftMargin: parent.width/6; rightMargin: parent.width/6}

                    onClicked: { internals.handleRemoveButtonClicked() }
                }

                Rectangle { anchors { left: parent.left; right: parent.right } height: 16; color: "#eff0f1" }
            }
        }
    }

    QueryDialog {
        id: queryDialog
        onAccepted: { internals.handleQueryDialogAccept() }
        onRejected: { internals.handleQueryDialogReject() }
    }

    tools: ToolBarLayout {
        ToolIcon {
            iconId: "icon-m-toolbar-back"
            anchors.left: parent.left
            onClicked: { pageStack.pop() }
        }
    }

    onRegionChanged: {
        print(region);
    }
}
