import QtQuick 1.0

import com.nokia.meego 1.0

import "OfflineMapLogic.js" as OML

Page {
    id: subPage
    objectName: "countryListPage"
    property variant headRegion
    property variant clickedChild
    signal childClicked(variant child)

    tools: ToolBarLayout {
        ToolIcon {
            anchors.left: parent.left
            objectName: "toolBarIconBackButton";
            iconId: "icon-m-toolbar-back";
            onClicked: {
                offlineMapsPage.subOfflineMapListDecreaseIndex();
                pageStack.pop();
            }
        }
    }

    Rectangle {
        anchors.fill: parent
        color: "#f0f1f2"
    }

    OdmlHeader {
        id: header
        title:  (typeof(region) != "undefined" && region)? region && region.localizedName : ""
    }

    ListView {
        id: childrenList
        objectName: "childrenList"
        anchors {
            top: header.bottom
            bottom: parent.bottom
            left: parent.left
            right: parent.right
            topMargin: 20
        }
        clip: true
        model: listModel

        delegate: OfflineMapListDelegate {
            objectName: "offlineMapListDelegate_" + index
            name: modelData.localizedName || modelData.name
            installed: modelData.installed
            hasChildren: modelData.children.length > 0
            error: modelData.error || false
            onItemClicked: {
                childClicked(listModel.get(index).modelData);
            }
        }
    }

    ScrollDecorator {
        flickableItem: childrenList
    }

    ListModel { id: listModel }

    onHeadRegionChanged: {
        header.title = headRegion.localizedName;
        listModel.clear();

        var tmpArray = OML.cloneAndOrderArrayByName(headRegion.children);
        for (var i = 0; i < tmpArray.length; i++) {
            listModel.append({modelData: tmpArray[i]});
        }

        if(headRegion.hasParent) {
            listModel.insert(0, {
                modelData: {
                     id: headRegion.id,
                     localizedName: qsTranslate("Maps", "qtn_maps_more_offlinemaps_all").arg(header.text),
                     downloadUrl: headRegion.downloadUrl,
                     hasParent: headRegion.hasParent,
                     installed: headRegion.installed,
                     children: []
                }
            });
        }
    }

}
