import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import ovi.connector 1.0

import "DownloadTools.js" as Tools
import "OfflineMapDownloader.js" as OfflineMapDownloader
import "OfflineMapLogic.js" as OML

Page {
    id: updatesPage

    anchors.fill: parent;

    Item {
        id: internals

        property bool updatesAvailable: false
        property bool stillCheckingMode: true
        property bool downloadingMode: false
        property bool downloadingFailed: false
        property double totalSize: 0
        property double timeLeft: 0
        property int numberOfSuccessDownloads: 0
        property string latestVersion
        property bool preventListening: false
        property bool wifiRequiredOfflineMode: false

        function handleDownloadAllButtonClicked() {
            queryDialog.titleText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_update_all_update_title");
            queryDialog.message = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_update_all_update_body");
            queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_no");
            queryDialog.acceptButtonText = qsTranslate("Maps", "qtn_maps_more_offlinemaps_dialog_yes");//789789
            queryDialog.open();
        }

        function handleQueryDialogAccept() {
            log.info("************************* NOT allow GPRS")
            downloadManager.allowGprsConnection(false);

            indicator.visible = true;
            internals.preventListening = true;
            internals.triggerMassUpdate();
            mapCatalog.resetInstanceWithPathSuffix("2");
            cityIndexList.resetInstanceWithPathSuffix("2");
            appState.applicationState = "update";
            offlineMapsPage.handleOnCompleted();

            waiterForTimeout.stop();

            pageStack.pop(offlineMapsPage);
        }

        function handleQueryDialogReject() {
            //
        }

        function triggerMassUpdate() {
            var allChecked = internals.getAllCheckedList();
            for (var i = 0; i < allChecked.length; i++) {
                var urlToDownload = Tools.getUrlToDownload(false, allChecked[i]);
                if (urlToDownload) {
                    log.info("**********  UPDATE       " + allChecked[i].id + " :: " + urlToDownload);
                    downloadManager.download(allChecked[i].id, urlToDownload, mapCatalog.mapBaseDirectory + "/diskcache2" + "/", true);
                }
            }
        }

        function getAllUnCheckedList() {
            var allUnChecked = [];
            var tmp, tmpA;
            tmp = installedRegionsView.model.get(0);
            tmpA = tmp? tmp.modelData : null;
            if (tmpA && tmpA.checked == "unchecked") {
                for (var i = 1; i < installedRegionsView.model.count; i++) {
                    tmp = installedRegionsView.model.get(i);
                    tmpA = tmp.modelData;
                    if (tmpA.checked == "unchecked") {
                        allUnChecked.push(tmpA);
                    }
                }
            }
            return allUnChecked;
        }

        function getAllCheckedList() {
            var allChecked = [];
            var tmp, tmpA;
            for (var i = 0; i < installedRegionsView.model.count; i++) {
                tmp = installedRegionsView.model.get(i);
                tmpA = tmp.modelData;
                if (tmpA.checked == "checked") {
                    allChecked.push(tmpA);
                }
            }
            if (allChecked.length == 1 && allChecked[0].downloadUrl == "all") {
                allChecked = [];
                for (var i = 1; i < installedRegionsView.model.count; i++) {
                    tmp = installedRegionsView.model.get(i);
                    tmpA = tmp.modelData;
                    allChecked.push(tmpA);
                }
            }

            return allChecked;
        }

        function getAllCheckedSize() {
            var allChecked = internals.getAllCheckedList();
            var totalSize = 0;
            for (var i = 0; i < allChecked.length; i++) {
                totalSize += allChecked[i].downloadSize;
            }

            internals.totalSize = totalSize;
        }
    }

    OdmlHeader {
        id: header

        title: qsTranslate("Maps", "qtn_maps_more_updates")

        z: 1
    }

    Rectangle {
        anchors.top: header.bottom
        anchors.bottom: parent.bottom
        anchors { left: parent.left; right: parent.right }
        clip: true
        color: "#eff0f1"

        Flickable {
            anchors.fill: parent

            contentWidth: parent.width
            contentHeight: {
                if (internals.updatesAvailable) {
                    return columnBody.height;
                } else if (internals.stillCheckingMode && !internals.downloadingFailed) {
                    return stillCheckingColumnBody.height;
                } else if (internals.stillCheckingMode && internals.downloadingFailed) {
                    return downloadingColumnBody.height;
                } else if (internals.wifiRequiredOfflineMode) {
                    return cannotCheckWifiRequiredColumnBody.height;
                } else {
                    return 1;
                }
            }

            Column {
                id: stillCheckingColumnBody

                visible: internals.stillCheckingMode /*&& !internals.downloadingMode */&& !internals.downloadingFailed && !internals.wifiRequiredOfflineMode

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 32; color: "#eff0f1" }

                MapsText {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                    width: parent.width
                    platformStyle: LabelStyle { fontPixelSize: 23 }

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_still_checking")
                }

            }

            Column {
                id: cannotCheckWifiRequiredColumnBody

                visible: internals.wifiRequiredOfflineMode

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 32; color: "#eff0f1" }

                MapsText {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                    width: parent.width
                    platformStyle: LabelStyle { fontPixelSize: 23 }

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_wifi_required")
                }

            }

            Column {
                id: downloadingColumnBody

                visible: internals.stillCheckingMode && internals.downloadingMode && internals.downloadingFailed

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 32; color: "#eff0f1" }

                MapsText {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}

                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                    platformStyle: LabelStyle { fontPixelSize: 23 }
                    width: parent.width

                    text: internals.downloadingFailed? qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_failed") : qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_new_configurations")
                }

            }

            Column {
                id: noUpdatesColumnBody

                visible: !internals.updatesAvailable && !internals.stillCheckingMode && !internals.downloadingFailed

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 32; color: "#eff0f1" }

                MapsText {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    width: parent.width

                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                    platformStyle: LabelStyle { fontPixelSize: 23 }

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_update_not_available")
                }

            }

            Column {
                id: columnBody

                visible: internals.updatesAvailable && !internals.stillCheckingMode && !internals.downloadingFailed

                anchors { left: parent.left; right: parent.right }

                spacing: 16

                Rectangle { anchors { left: parent.left; right: parent.right } height: 16; color: "#eff0f1" }

                MapsText {
                    anchors {left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16}
                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                    width: parent.width
                    platformStyle: LabelStyle { fontPixelSize: 23 }

                    text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_update_please_select_what_you_want_to_keep")
                }

                Rectangle {
                    anchors { left: parent.left; right: parent.right }
                    height: 24;
                    visible: !internals.allFinished && !internals.downloadInProgress && !internals.installInProgress;
                    color: "#eff0f1"
                }

                ListModel { id: installedRegionsModel }

                Column {
                    anchors {left: parent.left; right: parent.right; }

                    visible: !indicator.visible

                    Repeater {
                        id: installedRegionsView

                        clip: true
                        model: installedRegionsModel

                        delegate: UpdatesMapListDelegate {
                            name: modelData.name && modelData.name.indexOf("qtn_") === -1 ? modelData.name : qsTranslate("Maps", modelData.name)
                            size: modelData.downloadSize
                            checkState: modelData.checked

                            onItemClicked: {
                                if (index == 0) {
                                    if (checked) {
                                        var tmp, tmpa;
                                        for (var i = 1; i < installedRegionsView.model.count; i++) {
                                            tmp = installedRegionsView.model.get(i);
                                            tmpa = tmp.modelData;
                                            tmpa.checked = "unchecked";
                                            tmp.modelData = tmpa;
                                            installedRegionsView.model.set(i, tmp);
                                        }

                                        tmp = installedRegionsView.model.get(0);
                                        tmpa = tmp.modelData;
                                        tmpa.checked = "checked";
                                        tmp.modelData = tmpa;
                                        installedRegionsView.model.set(0, tmp);
                                    } else {
                                        var tmp, tmpa;
                                        for (var i = 0; i < installedRegionsView.model.count; i++) {
                                            tmp = installedRegionsView.model.get(i);
                                            tmpa = tmp.modelData;
                                            tmpa.checked = "unchecked";
                                            tmp.modelData = tmpa;
                                            installedRegionsView.model.set(i, tmp);
                                        }
                                    }
                                } else {
                                    var tmp, tmpa;
                                    tmp = installedRegionsView.model.get(0);
                                    tmpa = tmp.modelData;
                                    tmpa.checked = "unchecked";
                                    tmp.modelData = tmpa;
                                    installedRegionsView.model.set(0, tmp);

                                    tmp = installedRegionsView.model.get(index);
                                    tmpa = tmp.modelData;
                                    tmpa.checked = checked? "checked" : "unchecked";
                                    tmp.modelData = tmpa;
                                    installedRegionsView.model.set(index, tmp);
                                }

                                internals.getAllCheckedSize()
                            }
                        }
                    }
                }

                Rectangle {
                    anchors { left: parent.left; right: parent.right }
                    height: 24;
                    visible: !internals.allFinished && !internals.downloadInProgress && !internals.installInProgress;
                    color: "#eff0f1"
                }

                Item {
                    height: wifiText.paintedHeight > wifiImage.height ? wifiText.paintedHeight : wifiImage.height
                    anchors { right: parent.right; rightMargin: 16}
                    width: mainscreen.width - 32

                    visible: !downloadManager.hasWifiConnection

                    Image {
                        id: wifiImage
                        anchors.right: parent.right
                        source: odmlAssetsPath + "images/wifi.png"
                    }
                    Text {
                        id: wifiText
                        font.family: mainscreen.__labelStyle.fontFamily

                        text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_wifi_required")
                        anchors {left: parent.left; right: wifiImage.left; leftMargin: 16; rightMargin: 16}
                        wrapMode: Text.Wrap;
                        elide: "ElideNone"
                    }
                }

                Item {
                    height: fileSizeText.paintedHeight > fileSizeImage.height ? fileSizeText.paintedHeight : fileSizeImage.height
                    anchors { right: parent.right; rightMargin: 16}
                    width: mainscreen.width - 32

                    visible: !indicator.visible

                    Image {
                        id: fileSizeImage
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        source: odmlAssetsPath + "images/filesize.png"
                    }
                    MapsText {
                        id: fileSizeText
                        state: "listNormalBold"
                        anchors.right: fileSizeImage.left
                        anchors.verticalCenter: parent.verticalCenter
                        text: {
                            if (internals.totalSize) {
                               return Tools.formatBytes(internals.totalSize) + " MB";
                            } else {
                                return qsTranslate("Maps", "qtn_maps_more_offlinemaps_size_undefined");
                            }
                        }
                    }
                }

                Rectangle { anchors { left: parent.left; right: parent.right } height: 16; visible: !internals.allFinished && !internals.downloadInProgress && !internals.installInProgress; color: "#eff0f1"}


                Rectangle { anchors { left: parent.left; right: parent.right } height: 16; color: "#eff0f1" }
            }
        }
    }

    QueryDialog {
        id: queryDialog
        onAccepted: { internals.handleQueryDialogAccept() }
        onRejected: { internals.handleQueryDialogReject() }
    }

    BusyIndicator {
        id: indicator

        z:2

        platformStyle: BusyIndicatorStyle { size: "large" }
        running: visible
        visible: true

        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenterOffset: 60
    }

    InfoBanner {
        id: errorMessage

        timerEnabled: true
        timerShowTime: 15 * 1000

        anchors.top: parent.top
        anchors.topMargin: 89
    }

    tools: ToolBarLayout {
        ToolIcon {
            id:iconToolbarBack
            iconId: "icon-m-toolbar-back"
            anchors.left: parent.left
            onClicked: {
                waiterForTimeout.stop();

                if (!internals.preventListening) {
                    log.trace("--------- CANCEL ALL ON QUIT AS IT'S STILL DOWNLOADING XMLs");
                    downloadManager.cancelAll();
                }


                pageStack.pop();
            }
            z: 1
        }

        ToolButton {
            text: qsTranslate("Maps", "qtn_maps_more_offlinemaps_update_acceptbnt")

            platformStyle: ToolButtonStyle {
                disabledTextColor: pressedTextColor
            }

            implicitHeight: iconToolbarBack.height - 20
            width: parent.width / 2
            anchors.centerIn: parent
            enabled: downloadManager.hasWifiConnection && internals.totalSize
            visible: internals.totalSize

            onClicked: { internals.handleDownloadAllButtonClicked() }
        }
    }

    function compareMapVersions(latestVersion) {
        if (latestVersion && latestVersion > mapCatalog.installedMapsVersion) {
            internals.downloadingMode = true;
            OfflineMapDownloader.downloadConfigurationXMLs(latestVersion);

            internals.latestVersion = latestVersion;
        } else {
            internals.downloadingMode = false;
            internals.stillCheckingMode = false;
            internals.updatesAvailable = false;
            indicator.visible = false;
        }
    }

    function handleOnCompleted() {
        if (mainscreen.onlineMode) {
            indicator.visible = true;
            internals.wifiRequiredOfflineMode = false;

            log.info("************************* allow GPRS")
            downloadManager.allowGprsConnection(true);

            if (internals.stillCheckingMode) {
                if (mainscreen.onlineMode) {
                    OfflineMapDownloader.getMapVersion(compareMapVersions);
                }
            } else {
                if (mapCatalog.installedRegions.length > 0) {
                    installedRegionsModel.clear();

                    var tmp, totalSize = 0;
                    for (var i=0; i < mapCatalog.installedRegions.length; i++) {
                        tmp = mapCatalog.installedRegions[i];
                        totalSize += tmp.downloadSize;
                    }

                    installedRegionsModel.append({modelData: {id: null, name: qsTranslate("Maps", "qtn_maps_more_offlinemaps_all_maps"), downloadSize: totalSize, downloadUrl: "all", checked: "checked"}});
                    var tmpArray = OML.cloneAndOrderArrayByName(mapCatalog.installedRegions);
                    for (var i = 0; i < tmpArray.length; i++) {
                        tmp = tmpArray[i];
                        installedRegionsModel.append({modelData: {id: tmp.id, name: (tmp.localizedName? tmp.localizedName : tmp.name), downloadSize: tmp.downloadSize, downloadUrl: tmp.downloadUrl, checked: "unchecked"}});

                        appState.processEvents();
                    }

                    internals.getAllCheckedSize();

                    indicator.visible = false;
                } else {
                    errorMessage.text = qsTranslate("Maps", "qtn_maps_more_offlinemaps_downloading_failed");
                    errorMessage.show();

                    internals.downloadingMode = false;
                    internals.stillCheckingMode = false;
                    internals.updatesAvailable = false;
                    indicator.visible = false;
                }
            }
        } else {
            indicator.visible = false;
            internals.wifiRequiredOfflineMode = true;
        }
    }

    Component.onCompleted: {
        internals.preventListening = false;
        handleOnCompleted();
    }

    onStatusChanged: {
        if (status == PageStatus.Inactive) {
            offlineMapsPage.updateMapCatalog("");
            offlineMapsPage.preventListeningForRequestChanges = false;
        } else {
            offlineMapsPage.preventListeningForRequestChanges = true;
        }
    }

    Connections {
        target: downloadManager
        ignoreUnknownSignals: true

        onFinished: {
            if (updatesPage.visible) {
                if (updatesPage.status == PageStatus.Active) {
                    if (!internals.preventListening) {
                        if (operation == 4) {
                            internals.numberOfSuccessDownloads++;
                        }
                        if (internals.numberOfSuccessDownloads == 2) {
                            internals.preventListening = true;
                            internals.stillCheckingMode = false;
                            internals.updatesAvailable = true;
    
                            waiterForTimeout.stop();
    
                            offlineMapsPage.updateMapCatalog(internals.latestVersion);
    
                            handleOnCompleted();
                        }
                    }
                }
            }
        }
        /*
        onProgress: {
            //
        }
        */
        onError: {
            if (updatesPage.visible) {
                if (updatesPage.status == PageStatus.Active) {
                    internals.downloadingFailed = true;
                    indicator.visible = false;
                }
            }
        }

    }

    Timer {
        id: waiterForTimeout
        running: !internals.preventListening && mainscreen.applicationMaximized
        interval: 60 * 1000
        onTriggered: {
            if (!internals.preventListening && updatesPage.status == PageStatus.Active) {
                log.trace("............. TIMEOUT (updates page) ................")
                internals.downloadingFailed = true;
                indicator.visible = false;
                downloadManager.cancelAll();
            }
        }
    }
}
