import QtQuick 1.0

Item {
    id: analyticsComponent
    property variant _analytics: null
    property QtObject _component: null
    property int timeStampStartNavigation: -1
    property int timeStampEndNavigation: -1
    property string agentName
    property string agentVersion
    property bool sessionOpened: false

    property variant queue

    function _addToQueue(mode, message, args) {
        log.trace("--- [NAC] logging QUEUE event mode: " + mode + " message:" + message + " fired; args: " + JSON.stringify(args));

        var tmpQueue = queue;

        if (!tmpQueue) {
            tmpQueue = [];
        }
        tmpQueue.push({mode: mode, message: message, args: args});

        queue = tmpQueue;
    }

    function _dequeueAll() {
        if (_analytics && sessionOpened) {
            dequeueInTimer.start();
        }
    }

    function init() {
        if (analyticsAvailable) {
            _component = Qt.createComponent("AnalyticsServiceMaps.qml");
            _analytics = _component.createObject(analyticsComponent, { "agentName" : agentName, "agentVersion" : agentVersion } );
            sessionOpened = true;

            _dequeueAll();
        }
    }

    function destruct() {
        log.trace("--- [NAC] Analytics.qml: try to close NAC sessions.")
        if (_analytics && sessionOpened) {
            sessionOpened = false;
            _analytics.destruct();
        }
    }

    function logIMEIActivity(message, args) {
        log.trace("--- [NAC] logging IMEI NAC event type Activity: " + message + " fired; args: " + JSON.stringify(args));
        if (_analytics && sessionOpened && _analytics.sessionIMEI) {
            _analytics.sessionIMEI.logEvent(_analytics.sessionIMEI.ActivityLogEvent, message, args);
        } else {
            _addToQueue("IMEIActivity", message, args);
        }
    }

    function logUUIDActivity(message, args) {
        log.trace("--- [NAC] logging UUID NAC event type Activity: " + message + " fired; args: " + JSON.stringify(args));
        if (_analytics && sessionOpened && _analytics.sessionUUID) {
            _analytics.sessionUUID.logEvent(_analytics.sessionUUID.ActivityLogEvent, message, args);
        } else {
            _addToQueue("UUIDActivity", message, args);
        }
    }

    function logIMEIOther(message, args) {
        log.trace("--- [NAC] logging IMEI NAC event type Other: " + message + " fired; args: " + JSON.stringify(args));
        if (_analytics && sessionOpened && _analytics.sessionIMEI) {
            _analytics.sessionIMEI.logEvent(_analytics.sessionIMEI.OtherLogEvent, message, args);
        } else {
            _addToQueue("IMEIOther", message, args);
        }
    }

    function logUUIDOther(message, args) {
        log.trace("--- [NAC] logging UUID NAC event type Other: " + message + " fired; args: " + JSON.stringify(args));
        if (_analytics && sessionOpened && _analytics.sessionUUID) {
            _analytics.sessionUUID.logEvent(_analytics.sessionUUID.OtherLogEvent, message, args);
        } else {
            _addToQueue("UUIDOther", message, args);
        }
    }

    function extractAddress(address){
        var analyticsAddress;

        var addrCityName     = typeof(address.addrCityName)     != "undefined" ? address.addrCityName     : "";
        var addrCountryName  = typeof(address.addrCountryName)  != "undefined" ? address.addrCountryName  : "";
        var addrDistrictName = typeof(address.addrDistrictName) != "undefined" ? address.addrDistrictName : "";
        var addrHouseNumber  = typeof(address.addrHouseNumber)  != "undefined" ? address.addrHouseNumber  : "";
        var addrStreetName   = typeof(address.addrStreetName)   != "undefined" ? address.addrStreetName   : "";
        var addrPostalCode   = typeof(address.addrPostalCode)   != "undefined" ? address.addrPostalCode   : "";

        analyticsAddress = {
            "addrCityName":     addrCityName,
            "addrCountryName":  addrCountryName,
            "addrDistrictName": addrDistrictName,
            "addrHouseNumber":  addrHouseNumber,
            "addrStreetName":   addrStreetName,
            "addrPostalCode":   addrPostalCode
        };
        return analyticsAddress;
    }

    Component.onCompleted: {
        lazyAnalyticsInitialized.start();
    }

    Timer {
        id: dequeueInTimer

        interval: 250 * 2
        repeat: true

        onTriggered: {
            if (_analytics && sessionOpened) {
                var tmpQueue = queue;
                if (tmpQueue && tmpQueue.length > 0) {
                    var tmp = tmpQueue.shift();
                    try {
                        appState.processEvents();
                        analyticsComponent["log" + tmp.mode].apply(null, [tmp.message, tmp.args]);
                        appState.processEvents();
                    } catch (e) {
                        //
                    }

                    queue = tmpQueue;
                } else {
                    dequeueInTimer.stop();
                }

            }
        }
    }

    Timer {
        id: lazyAnalyticsInitialized
        interval: 1000 * 60 * 5
        onTriggered: {
            analyticsComponent.init();
        }
    }
}
