import QtQuick 1.0

/**
 * The CloudAdapter baseclass abstracts the underlying data structure, and allows basic CRUD functionality.
 * A CloudAdapter is supposed to be stateless and RESTful.
 * It works async and on the ConnectedObjects or ConnectedList of its connector.
 */
QtObject {
    id: root

    /// Allow child objects to be declared under a cloudAdapter
    default property alias children: root.myChildren
    property list<QtObject> myChildren

    /**
     * Creates an object part of this connector.
     * Overwrite this functionality inside your Connector to create your object types.
     */
    function createObject(options) {
    }

    /**
     * Lists objects on the cloud adapter.
     * Uses the options to configure the query, eg. by passing a searchString etc.
     * Implementation might differ from adapter to adapter.
     *
     * Will emit a queryFinished signal on the model, with error and results, a list of objects just containing uris.
     *
     * @param variant model The model that did the query.
     * @param variant options The options to configure the query method.
     */
    function query(model, options) {
    }

    /**
     * Reads data from the cloud adapter into the given object.
     * Uses the options to configure eg. the source or partial data interest.
     * Implementation might differ from adapter to adapter.
     *
     * Will change the object contents.
     * Will emit a readFinished signal on the object, with error and the original object.
     *
     * @param variant object The object that will be filled. Contains a uri property.
     * @param variant options The options to configure the read method.
     */
    function read(object, options) {
    }

    /**
     * Creates an object on the cloud adapter from the given object.
     * Uses the options to configure eg. the target or partial data interest.
     * Implementation might differ from adapter to adapter.
     *
     * Will store the object contents in the cloudAdapter.
     * Will emit a createFinished signal on the object, with error and the original object.
     * Will store the generated uri into the object.
     *
     * @param variant object The object that will be created.
     * @param variant options The options to configure the create method.
     */
    function create(object, options) {
    }

    /**
     * Updates an object on the cloud adapter from the given object.
     * Uses the options to configure eg. the target or partial data interest.
     * Implementation might differ from adapter to adapter.
     *
     * Will store the object contents in the cloudAdapter.
     * Will emit a updateFinished signal on the object, with error and the original object.
     *
     * @param variant object The object that will be created. Contains a uri property.
     * @param variant options The options to configure the create method.
     */
    function update(object, options) {
    }

    /**
     * Removes an object from the cloud adapter.
     * Uses the options to configure eg. the target or partial data interest.
     * Implementation might differ from adapter to adapter.
     *
     * Will remove the object contents from the cloudAdapter.
     * Will emit a removeFinished signal on the object, with error and the original object.
     * Will remove the uri from the object.
     *
     * @param variant object The object that will be removed. Contains a uri property.
     * @param variant options The options to configure the remove method.
     */
    function remove(object, options) {
    }
}
