import QtQuick 1.0

ConnectorPage {
    id: flow

    width: parent ? parent.width : 100
    height: parent ? parent.height : 100

    // Mirrors ConnectorPage API
    property variant model
    property variant result

    signal finished(variant error, variant result)


    property alias status: loader.status

    property string uri
    property string action
    property string connectorId
    property string objectType


    property QtObject pageStack

    ConnectorRegistry {
        id: registry
    }

    Loader {
        id: loader
        anchors.fill: parent
        source: ""

        onStatusChanged: {
            if (item) {
                if (model) {
                    item.model = model;
                } else {
                    item.model = registry.createObject(uri, connectorId, objectType);
                }
                item.pageStack = pageStack;
                item.visible = true;

                if (flow._active) {
                    try {
                        item.activating();
                        item.activated();
                    } catch(e) {};
                }
            }
        }
    }

    Connections {
        target: loader.item
        ignoreUnknownSignals: true

        onFinished: {
            flow.finished(error, result);
            flow.result = result;
        }
    }

    Component.onCompleted: configureFlow()
    onUriChanged: configureFlow()
    onActionChanged: configureFlow()
    onConnectorIdChanged: configureFlow()
    onObjectTypeChanged: configureFlow()

    function configureFlow() {
        if (action && (uri || connectorId || objectType)) {
            var view = registry.getViewComponent(uri, action, connectorId, objectType);
            if (view != loader.sourceComponent) {
                loader.sourceComponent = view;
            }
        } else {
            loader.sourceComponent = null;
        }
    }

    property bool _active: false

    onActivating: {
        if (!pageStack) {
            pageStack = flow.parent.parent
        }

        _active = true;

        if (loader.item) {
            loader.item.pageStack = pageStack;
            loader.item.activating();
        }
    }

    onActivated: {
        if (loader.item) {
            loader.item.activated();
        }
    }

    onDeactivating: {
        if (loader.item) {
            loader.item.deactivating();
        }
        _active = false;
    }

    onDeactivated: {
        if (loader.item) {
            loader.item.deactivated();
        }
    }
}
