import QtQuick 1.0
import ovi.connector 1.0
import MapsPlugin 1.0
import "FavoritesHelper.js" as Helper
import "FavoritesCloudAdapter.js" as JS
import "PtSearchHelper.js" as PtHelper

CloudAdapter {
    id: cloudAdapter
    property int uriCount: 0
    property variant tmpGeo: GeoCoordinates {}
    property variant relCenter: GeoCoordinates {}
    property Component objectFactory
    property string syncShareKey: "maps-qml"
    property string favoritesConnectorVersion: appState.mapsVersion

    Component.onCompleted: {
        Helper.initialize();
    }

    TransitDatabase {
        id: transitDb
        property bool busy: false;
    }

    Timer {
        id: ptSearchTimer
        interval: 3*1000
        onTriggered: PtHelper.placeSearchTimedOut();
    }


    function searchForPtPlace(aPlace) {
        PtHelper.addToPtQueue(aPlace);
    }

    function synchronize(f, progressCallback) {
        Helper.synchronizeFavorites(f, progressCallback);
    }

    function setRelativeCenter(lat, lon) {
        Helper.setRelativeCenter(lat, lon);
    }


    function calculateDistance(lat, lon) {
        return Helper.calculateDistance(lat, lon);
    }

    function createObject(uri) {
        return Helper.createObject(uri);
    }


    function addEventListener(el) {
        JS.addEventListener(el);
    }


    function getFromStore(id) {
        return JS.getFromStore(id);
    }


    function removeAll() {
        Helper.addRequest(JS.REQ_REMOVE_ALL);
    }


    function query(model, options) {
        Helper.addRequest(JS.REQ_QUERY, model, options);
    }


    function read(object, options) {
    }


    function create(object, options) {
        Helper.addRequest(JS.REQ_CREATE, object, options);
    }


    function update(object, options) {
        Helper.addRequest(JS.REQ_UPDATE, object, options);
    }


    function remove(object, options) {
        Helper.addRequest(JS.REQ_REMOVE, object, options);
    }


    function getTypeFromCategory(aCategory) {
        return Helper.getTypeFromCategory(aCategory);
    }


    function getCategoryTypeByName(aName) {
        return Helper.getCategoryTypeByName(aName);
    }


    function getCategoryTypeById(categoryNumber) {
        return Helper.getCategoryTypeById(categoryNumber);
    }


    function getIconFromMosCategoryMapping(category){
        return Helper.getIconFromMosCategoryMapping(category);
    }


    function getIconPath(category, mode) {
        return Helper.getIconPath(category, mode);
    }


    function formatAddress(properties) {
        return Helper.formatAddress(properties);
    }

    function getPlaceSubtitle(object) {
        return Helper.getPlaceSubtitle(object);
    }
}
