import QtQuick 1.0
import ovi.connector 1.0

ConnectedList {
    id: model
    cloudAdapter: FavoritesCloudAdapter {}
    queryMode: ConnectedList.Replace

    signal itemCreated(variant object);
    signal itemRemoved(variant object);
    signal itemUpdated(variant object);
    signal modelUpdated(variant object, int action);

    property int _item_created: 0
    property int _item_updated: 1
    property int _item_removed: 2
    property int _query_finished: 3

    Item {
        id: eventHandler
        property variant model: model
    }

    function query(options) {
        doQuery(options);
    }

    function removeAll() {
        cloudAdapter.removeAll();
    }

    onItemCreated: {
        model.append(object);
        modelUpdated(object, _item_created);
    }

    onItemRemoved: {
        var item, found = false;
        for (var i = 0; !found && i < model.count; i++) {
            item = model.get(i);
            if (item.key == object.key) {
                model.remove(i);
                found = true;
            }
        }
        modelUpdated(object, _item_removed);
    }

    onItemUpdated: {
        var found = false;
        for (var i = 0; !found && i < model.count; i++) {
            if (model.get(i).key == object.key) {
                model.remove(i);
                model.insert(i, object);
                found = true;
            }
        }
        modelUpdated(object, _item_updated);
    }

    onQueryFinished: {
        if (!error) {
            model.clear();
            for (var i = 0; i < results.length; i++) {
                model.append(results[i]);
                //appState.processEvents();
            }
        }
        modelUpdated(null, null);
    }

    Component.onCompleted: {
        cloudAdapter.addEventListener({model: model});
    }
}
