import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector.places 1.0

Rectangle {
    id: view

    color:"#f0f0f0"
    anchors.fill: parent
    property variant cloudAdapter: FavoritesCloudAdapter {}
    property variant placesCloudAdapter: PlacesCloudAdapter {}
    property variant showDeleteButton: deleteButton.visible

    property alias name: name.text
    property alias description: description.text

    signal deleteClicked

    function setTitleCursorPosition(pos) {
        name.cursorPosition = pos;
    }

    Flickable {
        id: flickableContent
        clip: true
        anchors.fill:parent
        contentHeight: flickContent.height
        boundsBehavior: Flickable.StopAtBounds

        Column {
            id: flickContent
            width: parent.width
            Rectangle {
                id: headerArea

                width: parent.width
                height: 40
                color: "#E6E8E8"

                Text {
                    id: personalizeHeader
                    objectName: "personalizeHeader"
                    text: qsTranslate("Maps", "qtn_favoritesconnector_favoritesedit_personalize_your_favorite") // Personalize your Favorite

                    color: "#000"
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    font.family: mainscreen.__labelStyle.fontFamily
                    font.letterSpacing: -1
                    anchors.leftMargin: 10
                    font.pixelSize: 20
                }
            }


            Column {
                spacing: 5
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: 10
                    rightMargin: 10
                }
                Rectangle {
                    width: parent.width
                    height: 20
                    color: "#00000000"
                }
                Text {
                    id: sectionAHeader
                    objectName: "sectionAHeader"
                    text: qsTranslate("Maps", "qtn_favoritesconnector_favoritesedit_your_name_for_the_place") // Your name for the place
                    color: "#858686"
                    font.family: mainscreen.__labelStyle.fontFamily
                    font.letterSpacing: -1
                    font.pixelSize: 18
                }

                TextField {
                    id: name
                    objectName: "nameInput"
                    inputMethodHints: Qt.ImhNoPredictiveText
                    width: parent.width
                    cursorPosition: 0
                    onActiveFocusChanged: {
                        if (activeFocus) {
                            cursorPosition = text.length;
                        } else {
                            cursorPosition = 0;
                        }
                    }
                }
                Rectangle {
                    width: parent.width
                    height: 20
                    color: "#00000000"
                }

                Text {
                    id: sectonBHeader
                    objectName: "sectionBHeader"
                    text: qsTranslate("Maps", "qtn_favoritesconnector_new_edit_title") // Your description
                    color: "#858686"
                    font.family: mainscreen.__labelStyle.fontFamily
                    font.letterSpacing: -1
                    anchors.leftMargin: 10
                    font.pixelSize: 18
                }

                TextArea {
                    id: description
                    objectName: "descriptionInput"
                    anchors {
                        left: parent.left
                        right: parent.right
                    }
                    height: Math.max (180, implicitHeight)
                }
            }

            Rectangle {
                width: parent.width
                height: 20
                color: "#00000000"
            }

            Button {
                id: deleteButton
                text: qsTranslate("Maps", "qtn_favoritesconnector_new_edit_delete")
                onClicked: deleteClicked();
                anchors.horizontalCenter: parent.horizontalCenter
            }
            Rectangle {
                width: parent.width
                height: 20
                color: "#00000000"
            }
        }
    }
    ScrollDecorator {
        flickableItem: flickableContent
    }
}
