import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0

Sheet {
    id: favoritesEditSheet

    property bool addMode: false
    property variant pickedObject
    property variant favoriteObj
    signal objectRemoved

    visualParent: mainscreen

    content: FavoritesEdit {
        id: favoritesEdit
        anchors.fill: parent
        showDeleteButton: !addMode

        onDeleteClicked: deleteDialog.open()
    }

    onPickedObjectChanged: {
        fillContent();
    }
    onAccepted: {
        favoriteObj.text = favoritesEdit.name;
        pickedObject.saveInStore(favoritesEdit.name, favoritesEdit.description);
    }

    buttons: Item {
        width: parent.width
        height: 70

        Item {
            width: parent.width - 50
            height: 70
            anchors.centerIn: parent
            Item {
                anchors.fill: parent
                SheetButton {
                    id: rejectButton
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    onClicked: { reject(); }
                    text: qsTranslate("Maps", "qtn_favoritesconnector_favoritesedit_toolbar_cancel")
                }
                SheetButton {
                    id: acceptButton
                    anchors.right: parent.right
                    anchors.verticalCenter: parent.verticalCenter
                    platformStyle: SheetButtonAccentStyle { }
                    onClicked: { accept(); }
                    enabled: new String(favoritesEdit.name).trim()? true : false
                    text: qsTranslate("Maps", "qtn_favoritesconnector_new_edit_save")
                }
            }
        }
    }

    QueryDialog {
        id: deleteDialog
        titleText: qsTranslate("Maps", "qtn_favoritesconnector_new_remove_dialog_title") // Unfavorite?
        message: qsTranslate("Maps", "qtn_favoritesconnector_new_remove_dialog_body") // By removing this Favourite you will lose your custom title and description.

        onAccepted: {
            favoritesEditSheet.close();
            objectRemoved();
        }
        acceptButtonText: qsTranslate("Maps", "qtn_favoritesconnector_new_remove_dialog_yes")    // Yes
        rejectButtonText: qsTranslate("Maps", "qtn_favoritesconnector_new_remove_dialog_no")    // No
    }

    function fillContent() {
        if (pickedObject) {
            favoriteObj = pickedObject.getFavorite();
            if (favoriteObj) {
                favoritesEdit.name = favoriteObj.text;
                favoritesEdit.description = favoriteObj.description;
                favoritesEdit.setTitleCursorPosition(0);
            }
        }
    }
}
