import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.places 1.0

ConnectorView {
    id: picker

    anchors.fill: parent
    model: FavoritesConnectedList {}

    signal picked(string pickedUri, variant pickedObject, variant listModel)

//    property alias showMap: mapButton.mapActive
    property bool refreshInProgress: false
    property bool emptyModel: false
    property variant tmpGeo: GeoCoordinates {}
    property variant favCloudAdapter: FavoritesCloudAdapter {}
    property variant placesCloudAdapter: PlacesCloudAdapter {}

    function _log (message, trace) {
        message = '[favorites-picker] ' + message;

        if (trace) {
            log.trace(message);
        } else {
            log.info(message);
        }
    }

    function updateList() {
        _log('queried to update the list', true);

        model.query({
            latitude: mainscreen.mapSharedData.currentCenter.latitude,
            longitude: mainscreen.mapSharedData.currentCenter.longitude
        });
    }

    function blurMe() {
        preloader.visible = true;
    }

    function distanceToCenter(lat, lon) {
        tmpGeo.latitude = lat;
        tmpGeo.longitude = lon;
        return mapView.center.distance(tmpGeo)
    }

    Place {
        id: placeObj
    }

    function handleModelQueryFinished(aModel) {
//        rootApplicationLevel.benchmark ('handleModelQueryFinished', function () {
            listModel.clear();
            var fav, place, categoryType;
            var places = [];

            if (emptyModel = aModel.count === 0) {
                return;
            }

//            rootApplicationLevel.benchmark ('Update ListModel', function () {
                for (var i = 0, l = aModel.count; i < l; i++) {
                    fav = aModel.get(i);
                    if (fav && fav.getPlace) {
                        place = fav.getPlace();
                        places.push(place);
                        listModel.append({object: place, favorite: fav}); // we need the favorite for the map model
                    }
                }
//            });

            preloader.visible = false;
            places.splice(0, places.length);
//        });
    }

    Connections {
        target: model
        ignoreUnknownSignals: true

        onQueryFinished: {
            handleModelQueryFinished(model);
        }
        onModelUpdated: {
            handleModelQueryFinished(model);
        }
    }

    Rectangle {
        anchors.fill: parent
        color: "#f0f1f2"
        clip: true

        Header {
            id: header
            text: qsTranslate("Maps", "qtn_favoritesconnector_new_title") // Favorites
            textWidth: parent.width - synchButton.width
            z: 1
            dropShadow: false

            Button {
                id: synchButton
                height: 50; width: 50

                anchors {
                    right: parent.right
                    rightMargin: 10
                    verticalCenter: parent.verticalCenter
                }

                Image {
                    anchors.centerIn: parent;
                    source: {
                        var base = favoritesAssetsPath + 'images/layout/icon-sync';
                        if (parent.enabled) {
                            return base + (parent.pressed ? '_light.png' : '_dark.png');
                        } else {
                            return '';
                        }
                    }
                }

                BusyIndicator {
                    id: refreshIndicator
                    anchors.centerIn: parent
                    visible: !synchButton.enabled
                    running: true
                    height: 32; width: 32
                }

                enabled: !rootApplicationLevel.isSynchronizing
                visible: !appState.isEmbarcoVariant
                onClicked: {
                    _log("Synch button pressed", true);
                    rootApplicationLevel.isSynchronizing = true;
                    rootApplicationLevel.synchronizationProgress = 0.0;
                    _log("-- synch button pressed", true);
                    favCloudAdapter.synchronize(function (success) {
                        _log("-- syncing done")
                        rootApplicationLevel.isSynchronizing = false;

                        if (success) {
                            _log('Going to update the list because syncing performed');
                            updateList();
                        } else {
                            _log('Not going to update the list because syncing aborted');
                        }
                    }, function (progress, status) {
                        rootApplicationLevel.synchronizationProgress = progress;
                    });
                }
            }
        }


        ListModel {
            id: listModel
        }

        Rectangle {
            id: noFavourites
            visible: emptyModel
            color: "#f0f1f2"

            anchors {
                top: header.bottom;
                bottom: parent.bottom;
                left: parent.left;
                right: parent.right;
            }

            MapsText {
                id: noFavouritesText
                text: qsTranslate("Maps", "qtn_favoritesconnector_no_favourites_text") // Save places to your favourites by tapping on the
                horizontalAlignment: Text.AlignHCenter
                state: "normalWrap"
                width: mainscreen.width //can be used if CDATA is not used anymore in translation file.
                //CDATA also prevents from aligning it horizontally
                anchors {
                    leftMargin: 16
                    rightMargin: 16
                    horizontalCenter: parent.horizontalCenter;
                    verticalCenter: parent.verticalCenter;
                    verticalCenterOffset: -50
                }
            }

            Image {
                anchors.horizontalCenter: parent.horizontalCenter;
                anchors.top: noFavouritesText.bottom;
                anchors.topMargin: 5;
                source: favoritesAssetsPath + "images/layout/star.svg"
            }
        }

        ListView {
            id: favoritesList
            clip: true

            anchors { 
                top: header.bottom
                bottom: parent.bottom
                right: parent.right
                left: parent.left
            }

            model: listModel
            delegate: PlaceListItem {
                onItemSelected: {
                    picker.picked(place.uri, listModel.get(index).object, listModel);
                }
            }
            highlight: Rectangle { color: "#c0c1c2" }
            focus: true

        }
        ScrollDecorator {
            flickableItem: favoritesList
        }

        Rectangle {
            id: preloader
            objectName: "favPreloader"

            visible: true

            anchors.top: header.bottom
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            anchors.left: parent.left

            color: "black"
            opacity: 0

            Text {
                objectName: "preloaderText"
                text: qsTranslate("Maps", "qtn_favoritesconnector_loading") // loading...
                anchors.centerIn: parent
            }
        }
    }

    Component {
        id: placesViewer
        PlacesViewer {
            object : place
        }
    }
    Place {
        id: place
    }

    Component.onCompleted: {
        preloader.visible = false;
        // _log('going to update list because component completed', true);
        // updateList();
    }
}
