import QtQuick 1.0

Rectangle {
    id: debugOverlayPanel

    visible: mainscreen.enableCompletePositioningDebug

    anchors.left: parent.left
    anchors.right: parent.right
    anchors.leftMargin: 10
    anchors.rightMargin: 10
    anchors.top: parent.top
    anchors.topMargin: 15
    height: parent.height / 2 - 60

    color: "black"
    opacity: 0.5

    property bool freezed: false
    property bool logging: false
    property string buffer: ""
    property int __BUFFER_SIZE__: 10 * 1000

    function log(message) {
        if (visible) {
            var now = new Date();
            if (!freezed) {
                text.text = now.getTime() + ") " + message + "\n" + text.text.substring(0, __BUFFER_SIZE__);
            } else {
                buffer = now.getTime() + ") " + message + "\n" + buffer.substring(0, __BUFFER_SIZE__);
            }

            console.log(now.getTime() + ") " + message);
        }
    }

    Flickable {
        id: panel

        anchors.fill: parent

        contentWidth: parent.width;
        contentHeight: text.paintedHeight
        clip: true
        interactive: true

        Text {
            id: text
            color: "white"

            anchors.left: parent.left
            anchors.right: parent.right

            wrapMode: Text.WrapAnywhere

        }
    }

    Rectangle {
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.rightMargin: 10
        anchors.bottomMargin: 10

        width: 50
        height: 50
        color: freezed? "red" : "green"
        radius: 25

        Text {
            anchors.centerIn: parent

            text: "P"
            color: "white"
            font.pixelSize: 25
        }

        MouseArea {
            anchors.fill: parent

            onClicked: {
                freezed = !freezed;
            }
        }
    }

    Rectangle {
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.rightMargin: 60
        anchors.bottomMargin: 10

        width: 50
        height: 50
        color: logging? "green" : "yellow"
        radius: 25

        Text {
            anchors.centerIn: parent

            text: "L"
            color: "white"
            font.pixelSize: 25
        }

        MouseArea {
            anchors.fill: parent

            onClicked: {
                if (!logging) {
                    appState.startFileLogging();
                } else {
                    appState.stopFileLogging();
                }
                logging = !logging;
            }
        }
    }

    Rectangle {
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        anchors.leftMargin: 10
        anchors.bottomMargin: 10

        width: 50
        height: 50
        color: "red"
        radius: 25

        Text {
            anchors.centerIn: parent

            text: "C"
            color: "white"
            font.pixelSize: 25
        }

        MouseArea {
            anchors.fill: parent

            onClicked: {
                text.text = ""
            }
        }
    }

    Connections {
        target: debugOverlayPanel
        ignoreUnknownSignals: true

        onFreezedChanged: {
            if (!freezed) {
                text.text = buffer + text.text;
                buffer = "";
            }
        }
    }

}
