import QtQuick 1.0

import MapsPlugin 1.0

import "InfoBubbleBehavior.js" as EventHandlers

Item {
    id: infoBubbleBehavior
    objectName: "infoBubbleBehavior"

    property variant mapView
    property variant ui

    property int _DEFAULT_ZOOM_LEVEL_FOR_STREET_VIEW_: 14
    property double _MAXIMUM_DIFFERENCE_BETWEEN_DESTINATION_AND_FINAL_ANIMATION_CENTER_: 0.002

    function showMarker(marker) {
        EventHandlers.handleShowMarker(marker);
    }
    function showContent(content) {
        EventHandlers.handleShowContent(content);
    }
    function showAt(geoCoordinates, content) {
        EventHandlers.handleShowAt(geoCoordinates, content);
    }
    function showFromMap(geoCoordinates, extraData) {
        EventHandlers.handleShowFromMap(geoCoordinates, extraData);
    }
    function resetAll(force) {
        EventHandlers.handleResetall(force);
    }
    function forceResetAll() {
        EventHandlers.handleResetall(true);
    }
    function getListView(data) {
        return EventHandlers.handleGetListView(data);
    }

    Units {
        id: units
    }

    GeoCoordinates {
        id: spareGeoCoordinates
    }

    GeoRect {
        id: spareGeoRect
    }

    ReverseGeoCodeFinder {
        id: reverseGeoCodeFinder

        mapView: infoBubbleBehavior.mapView

        onFindPureDataComplete: {
            EventHandlers.handleReverseGeoCodeFinderFindPureDataCompleted(contextData);
        }
        onFindComplete: {
            EventHandlers.handleReverseGeoCodeFinderFindComplete(id, geoCoordinates, context);
        }
    }

    Component {
        id: spareListView

        ListView {
            anchors.fill: parent

            model: ListModel {
            }
            delegate: Rectangle {
                width: parent.width
                height: 50
                Text {
                    horizontalAlignment: Text.AlignHCenter
                    font.pixelSize: 23
                    width: parent.width
                    wrapMode: Text.WordWrap

                    text: name
                }
            }
        }
    }

    Timer {
        id: smoothTemporaryUiContextApplier
        interval: 250 * 4

        property variant temporaryContext
        property variant reverseGeoCodeFinderMode

        onTriggered: {
            EventHandlers.handleSmoothTemporaryUiContextApplierTriggered();
        }
    }

    Timer {
        id: resetChainOfEventsAfterAnimationDone
        interval: 1

        onTriggered: {
            EventHandlers.handleResetChainOfEventsAfterAnimationDone();
        }
    }

    Timer {
        id: postRotateInfoBubbleFixToScreen
        interval: 250
        repeat: true

        onTriggered: {
            EventHandlers.handlePostRotateInfoBubbleFixToScreen();
        }
    }

    Timer {
        id: unblockInteractivityOfMapInWhile
        interval: 250

        onTriggered: {
            EventHandlers.handleUnblockInteractivityOfMapInWhile();
        }
    }

    Timer {
        id: fakeAnimationDoneOnNoAnimationMoves
        interval: 25

        onTriggered: {
            EventHandlers.handleMapViewAnimationDone();
        }
    }

    Connections {
        target: mapView
        ignoreUnknownSignals: true

        onMouseDown: {
            //console.log("........................... 1")
            EventHandlers.handleMapViewMouseDown();
        }
        onMouseUp: {
            //console.log("........................... 2")
            EventHandlers.handleMapViewMouseUp();
        }
        onMapClicked: {
            //console.log("........................... 3")
            EventHandlers.handleMapViewMapClicked(pixelPos, geoPos, mapObject);
        }
        onMapCurrentLocationClicked: {
            //console.log("........................... 4")
            EventHandlers.handleMapViewMapCurrentLocationClicked(pixelPos, geoPos, mapObject);
        }
        onMapDoubleClicked: {
            //console.log("........................... 5")
            EventHandlers.handleMapViewDoubleClicked();
        }
        onCenterChanged: {
            //console.log("........................... 6")
            EventHandlers.handleMapViewCenterChanged();
        }
        onZoomScaleChanged: {
            //console.log("........................... 7")
            EventHandlers.handleMapViewZoomScaleChanged();
        }
        onOrientationChanged: {
            //console.log("........................... 8")
            EventHandlers.handleMapViewOrientationChanged();
        }
        onFlyToCurrentPosition: {
            //console.log("........................... 9")
            EventHandlers.handleMapViewFlyToCurrentPosition();
        }
        onMapLongPressRectangleCompleted: {
            //console.log("........................... 10")
            EventHandlers.handleMapViewMapLongPressRectangleCompleted(pixelPos, geoPos);
        }
        onAnimationDone: {
            //console.log("........................... 11")
            EventHandlers.handleMapViewAnimationDone();
        }
        onUserInteractionChanged: {
            //console.log("........................... 12")
            EventHandlers.handleMapViewUserInteractionChanged();
        }
        onControlChanged: {
            //console.log("........................... 13")
            EventHandlers.handleMapViewControlChanged();
        }
    }

    Connections {
        target: ui
        ignoreUnknownSignals: true

        onInfoBubbleClicked: {
            EventHandlers.handleUiInfoBubbleClicked();
        }

        onInfoBubbleDoubleClicked: {
            EventHandlers.handleUiInfoBubbleDoubleClicked();
        }

        onInfoBubbleClosed: {
            EventHandlers.handleUiInfoBubbleClosed();
        }

        onInfoBubbleVisibilityChanged: {
            EventHandlers.handleUiInfoBubbleVisibilityChanged();
        }
    }

}
