import QtQuick 1.0
import MapsPlugin 1.0

Rectangle {
    id: longPressRectangle

    width: 200
    height: 200
    radius: 80
    color: {
        var schemeParts = new String(mapView.scheme).split(".");
        return schemeParts[0] == "hybrid" || schemeParts[1] == "night"? "white" : "green";
    }
    smooth: true
    visible: false

    property variant mapView
    signal completed()
    signal canceled()

    Behavior on visible {
        ParallelAnimation {
            PropertyAnimation { target: longPressRectangle; property: "scale"; from: 0; to: 1; duration: 350; }
            PropertyAnimation { target: longPressRectangle; property: "opacity"; from: 0; to: 0.6; duration: 350; }
        }
    }

    Timer {
        id: longPressRectangleTimer

        running: false
        interval: 350
        onTriggered: {
            if (longPressRectangle.visible) {
                longPressRectangle.completed();
            }

            longPressRectangle.visible = false;
        }

    }

    Connections {
        target: mapView
        ignoreUnknownSignals: true

        onMapLongPressed: {
            longPressRectangle.x = pixelPos.x - longPressRectangle.width / 2;
            longPressRectangle.y = pixelPos.y - longPressRectangle.height / 2;
            longPressRectangle.visible = true;
            longPressRectangleTimer.running = true;
        }

        onMapDragStarted: {
        //onUserInteractionChanged: {
            longPressRectangleTimer.running = false;
            longPressRectangle.visible = false;
        }

        onMapReleased: {
            if (visible) {
                visible = false;
                longPressRectangle.canceled();
            }
        }
    }
}
