var ZOOM_STEP_MULTIPLIER = 3;
var PERSPECTIVE_3D = 65;
var PUBLIC_TRANSPORT_MODE_OFF = Map.TRANSIT_MODE_SHOW_STOPS_AND_ACCESSES;
var PUBLIC_TRANSPORT_MODE_ON = Map.TRANSIT_MODE_SHOW_EVERYTHING;

function findRootApplicationLevel(currentLevel) {
    var tmp = currentLevel;
    while (tmp.parent) { tmp = tmp.parent; }
    return tmp;
}

var categoriesMap = null;
function initCategories() {
    var categories = mapView.poiCategories;
    categoriesMap = {};
    for (var i = 0; i < categories.length; i++) {
        categoriesMap[categories[i]] = i;
    }
}
function setCategory(name, enabled) {
    if (!categoriesMap) {
        initCategories();
    }

    mapView.showPoiCategory(categoriesMap[name], enabled);
}

function zoomIn(center, orientation) {
    var zoom = mapView.zoomScale / ZOOM_STEP_MULTIPLIER;

    if (mapView.minZoomScale <= zoom) {
        if (false && mapView.isAnimationInProgress) {
            if (center) {
                mapView.center = center;
            }

            mapView.zoomScale = zoom;
            if (orientation !== undefined) {
                mapView.orientation = orientation;
            }
        } else {
            mapView.moveTo(
                center? center : null,
                Map.ANIMATION_LINEAR,
                zoom,
                orientation !== undefined? orientation : Map.PRESERVE_ORIENTATION,
                Map.PRESERVE_PERSPECTIVE
            );
        }
    }
}

function zoomOut(center) {
    center = null;
    var zoom = mapView.zoomScale * ZOOM_STEP_MULTIPLIER;
    zoom = zoom > mapView.maxZoomScale? mapView.maxZoomScale : zoom;

    if (mapView.maxZoomScale === undefined || zoom <= mapView.maxZoomScale) {
        if (false && mapView.isAnimationInProgress/* || mapView.perspective > 0*/) {
            if (center) {
                mapView.center = center;
            }

            mapView.zoomScale = zoom;
        } else {
            mapView.moveTo(
                center? center : null,
                Map.ANIMATION_LINEAR,
                zoom,
                Map.PRESERVE_ORIENTATION,
                Map.PRESERVE_PERSPECTIVE
            );
        }
    }
}

function zoomTo(zoom, center, instant) {
    if ((mapView.maxZoomScale === undefined || zoom <= mapView.maxZoomScale) && zoom >= 0) {
        if (instant) {
            mapView.moveTo(
                center? center : null,
                Map.ANIMATION_NONE,
                zoom,
                Map.PRESERVE_ORIENTATION,
                Map.PRESERVE_PERSPECTIVE
            );
        } else {
            mapView.moveTo(
                center? center : null,
                Map.ANIMATION_LINEAR,
                zoom,
                Map.PRESERVE_ORIENTATION,
                Map.PRESERVE_PERSPECTIVE
            );
        }
    }
}

function zoomToLevel(level, center, instant) {
    var zoomLevels = mapView.zoomLevels;
    var zoom = zoomLevels[level];
    if (zoom) {
        if (instant) {
            mapView.moveTo(
                center? center : null,
                Map.ANIMATION_NONE,
                zoom,
                Map.PRESERVE_ORIENTATION,
                Map.PRESERVE_PERSPECTIVE
            );
        } else {
            mapView.moveTo(
                center? center : null,
                Map.ANIMATION_LINEAR,
                zoom,
                Map.PRESERVE_ORIENTATION,
                Map.PRESERVE_PERSPECTIVE
            );
        }
    }
}

function getCurrentZoomLevel(customZoomScale) {
    var zoomLevels = mapView.zoomLevels;
    var zoomScale = customZoomScale? customZoomScale : mapView.zoomScale;
    var result = zoomLevels.length;
    for (var i = zoomLevels.length - 1; i >= 0; i--) {
        if (zoomScale <= zoomLevels[i]) {
            result = i;
            break;
        }
    }

    return result;
}

function centerAt(center) {
    mapView.moveTo(
       center,
       Map.ANIMATION_BOW,
       Map.PRESERVE_ZOOM_SCALE,
       Map.PRESERVE_ORIENTATION,
       Map.PRESERVE_PERSPECTIVE
    );
}

function checkIfPointIsVisible(point) {
    var pointXY = mapView.geoToScreen(point, false);
    return pointXY.x > 0 && pointXY.y > 0 && pointXY.x < mapView.width && pointXY.y < mapView.height;
}

function getDistance(pointA, pointB) {
    return Math.sqrt(Math.pow(pointA.x - pointB.x, 2) + Math.pow(pointA.y - pointB.y, 2));
}

function getPixelDistanceBetweenCurrentCenterAndPoint(pixelPos) {
    if (mapView.currentPosition) {
        var currentLocationPixelPos = mapView.geoToScreen(mapView.currentPosition.geoCoordinates, false);
        return MapApi.getDistance(currentLocationPixelPos, pixelPos);
    } else {
        return -1;
    }
}

function getPixelDistanceBetweenPreviousCenterAndPoint(pixelPos) {
    if (mapView.previousPosition) {
        var previousLocationPixelPos = mapView.geoToScreen(mapView.previousPosition.geoCoordinates, false);
        return MapApi.getDistance(previousLocationPixelPos, pixelPos);
    } else {
        return -1;
    }
}

var checkIfClickedOnCurrentPosition = function () {
    var currentCenterClickCircleRadiusInPixels = 35;

    return function (pixelPos) {
        var tmpC = getPixelDistanceBetweenCurrentCenterAndPoint(pixelPos);
        var tmpP = getPixelDistanceBetweenPreviousCenterAndPoint(pixelPos);
        return (tmpC >= 0 && tmpC <= currentCenterClickCircleRadiusInPixels) || (tmpP >= 0 && tmpP <= currentCenterClickCircleRadiusInPixels);
    }
}()

function metersToPixes(meters) {
    return meters * 100 / mapView.zoomScale;
}

function applyPoiCategoryFilter(categories) {
    if (typeof categories === "object") {
        mapView.showPoiCategory(Icon.All, false);
        for (var i = 0; i < categories.length; i++) {
            mapView.showPoiCategory(categories[i], true);
        }
    } else {
        mapView.showPoiCategory(Icon.All, true);
    }
}

var MapLayers = function () {
    var timeMode = "day";
    var mapMode = "pedestrian";
    var mode3d = false;
    var landmarksVisible = true;
    var trafficInfoVisible = false;
    var publicTransportMode = false;
    var publicTransportModePartial = false;
    var latestTrafficFix = null;

    var _init = function () {
        if (mapView) {
            timeMode = mapView.mapSharedData.timeMode;
            mapMode = mapView.mapSharedData.mapMode;
            mode3d = mapView.mapSharedData.mode3d;
            landmarksVisible = mapView.mapSharedData.landmarksVisible;
            trafficInfoVisible = mapView.mapSharedData.trafficInfoVisible;
            publicTransportMode = mapView.mapSharedData.publicTransportMode;
            publicTransportModePartial = mapView.mapSharedData.publicTransportModePartial;
        }
    }

    var _storeMapSettings = function () {
        if (!mapView.dontSaveData) {
            mapView.mapSharedData.timeMode = timeMode;
            mapView.mapSharedData.mapMode = mapMode;
            mapView.mapSharedData.mode3d = mode3d;
            mapView.mapSharedData.landmarksVisible = landmarksVisible;
            mapView.mapSharedData.trafficInfoVisible = trafficInfoVisible;
            mapView.mapSharedData.publicTransportMode = publicTransportMode;
            mapView.mapSharedData.publicTransportModePartial = publicTransportModePartial;
            mapView.mapSharedData.skipPropertiesOnSaveTime = null;
            mapView.mapSharedData.uiChanged();
        }
        mapView.dontSaveData = false;
    }

    var _setSkipPropertiesOnSaveTime = function (skipProperties) {
        skipProperties = skipProperties? skipProperties : null;
        mapView.mapSharedData.skipPropertiesOnSaveTime = skipProperties;
    }

    var _tryToReverseScheme = function (scheme) {
        var tmpMapMode = null;
        var tmpTimeMode = null;
        var tmpPublicTransportMode = null;

        if (scheme) {
            var schemeParts = scheme.split(".");
            if (schemeParts.length == 3) {
                if (schemeParts[0] == "hybrid") {
                    tmpMapMode = "satellite";
                    tmpTimeMode = "day";
                    tmpPublicTransportMode = true;
                } else if (schemeParts[0] == "normal") {
                    tmpMapMode = "pedestrian";
                    tmpTimeMode = (schemeParts[1] == "day")? "day" : "night";
                    tmpPublicTransportMode = true;
                }
            } else if (schemeParts.length == 2) {
                tmpMapMode = schemeParts[0];
                if (tmpMapMode != "pedestrian" && tmpMapMode != "hybrid") {
                    tmpMapMode = (tmpMapMode == "terrain")? "terrain" : "satellite";
                    tmpTimeMode = "day";
                } else if (tmpMapMode == "pedestrian") {
                    tmpTimeMode = (schemeParts[1] == "day")? "day" : "night";
                    tmpPublicTransportMode = false;
                } else if (tmpMapMode == "hybrid") {
                    tmpMapMode = "satellite";
                    tmpTimeMode = "day";
                }
            }
        }

        return {mapMode: tmpMapMode, timeMode: tmpTimeMode, publicTransportMode: tmpPublicTransportMode};
    }

    var _applyMapSettings = function () {
        if (publicTransportMode && mapMode == "pedestrian") {
            mapView.scheme = "normal." + timeMode + ".transit";
        } else if (mapMode == "satellite") {
            if (publicTransportMode || publicTransportModePartial) {
                mapView.scheme = "hybrid.day.transit";
            } else {
                mapView.scheme = "hybrid.day";
            }
        } else if (mapMode != "pedestrian") {
            mapView.scheme = mapMode + "." + "day";
        } else {
            mapView.scheme = mapMode + "." + timeMode;
        }

        mapView.showPedestrianFeature(Map.ALL_PEDESTRIAN_FEATURE, mapMode == "pedestrian");
        if (publicTransportMode && (mapMode == "pedestrian" || mapMode == "satellite")) {
            mapView.transitMode = PUBLIC_TRANSPORT_MODE_ON;
        } else if (publicTransportModePartial && (mapMode == "pedestrian" || mapMode == "terrain" || mapMode == "satellite")) {
            mapView.transitMode = PUBLIC_TRANSPORT_MODE_ON;
        } else {
            mapView.transitMode = PUBLIC_TRANSPORT_MODE_OFF;
        }

        if ((mapView.mainScreen && !mapView.mainScreen.enableMagic3d2d) || !mapView.mainScreen) {
            mapView.perspective = mode3d? PERSPECTIVE_3D : 0;
        }
        mapView.landmarksVisible = landmarksVisible;
        mapView.trafficInfoVisible = trafficInfoVisible;
    }

    var _apply = function () {
        _applyMapSettings();
        _storeMapSettings();
    }

    var _askForTraffic = function () {
        if (!latestTrafficFix || mapView.center.distance(latestTrafficFix) > 5 * 1000) {
            latestTrafficFix = spareGeoCoordinates.createObject(mapView);
            latestTrafficFix.latitude = mapView.center.latitude;
            latestTrafficFix.longitude = mapView.center.longitude;
            trafficObject.requestTrafficAt(mapView.center);
        }
    }

    return {
        mapViewCenterChanged: function () {
            if (this.isTrafficMode()) {
                _askForTraffic();
            }
        },

        init: function () {
            _init();
        },

        reset: function () {
            _init();
            _applyMapSettings();
        },

        tryScheme: function (scheme) {
            var tmp = _tryToReverseScheme(scheme);

            if (
                (tmp.mapMode !== null && tmp.mapMode != mapMode)
                ||
                (tmp.timeMode !== null && tmp.timeMode != timeMode)
                ||
                (tmp.publicTransportMode !== null && tmp.publicTranposrtMode != publicTransportMode)
            ) {
                mapMode = tmp.mapMode !== null? tmp.mapMode : mapMode;
                timeMode = tmp.timeMode !== null? tmp.timeMode : timeMode;
                publicTransportMode = tmp.publicTransportMode !== null? tmp.publicTransportMode : publicTransportMode;

                _apply();
            } else {
                _applyMapSettings();
            }
        },

        skipPropertiesOnSaveTime: function (skipProperties) {
            _setSkipPropertiesOnSaveTime(skipProperties);
        },

        isMapMode: function () {
            _init();
            return mapMode == "pedestrian";
        },
        isSatelliteMode: function () {
            _init();
            return mapMode == "satellite";
        },
        isTerrainMode: function () {
            _init();
            return mapMode == "terrain";
        },
        setMapMode: function () {
            mapMode = "pedestrian";
            _apply();
        },
        setSatelliteMode: function () {
            mapMode = "satellite";
            _apply();
        },
        setTerrainMode: function () {
            _init();
            if (publicTransportMode && mapView.mapSharedData.publicTransportLineHightlighted) {
                publicTransportModePartial = true;
            }
            mapMode = "terrain";

            _apply();
        },

        is3dMode: function () {
            _init();
            return mode3d;
        },
        is2dMode: function () {
            _init();
            return !mode3d;
        },
        set3dMode: function () {
            mode3d = true;
            _apply();
        },
        set2dMode: function () {
            mode3d = false;
            _apply();
        },
        change2d3dMode: function () {
            _init();
            mode3d = !mode3d;
            _apply();
        },
        tryPerspective: function (value) {
            var tmpMode3d = !(value == 0);
            if (tmpMode3d != mode3d) {
                mode3d = tmpMode3d;
                _apply();
            } else if (value != 0 && value != PERSPECTIVE_3D) {
                _applyMapSettings();
            }
        },
        optPerspective: function () {
            mapView.perspective = mode3d? PERSPECTIVE_3D : 0;
        },

        isPublicTransportMode: function () {
            _init();
            return publicTransportMode;
        },
        isPublicTransportModeActive: function () {
            _init();
            return publicTransportMode && (mapMode == "pedestrian" || mapMode == "satellite");
        },
        setPublicTransportModeOn: function() {
            publicTransportMode = true;
            publicTransportModePartial = false;
            _apply();
        },
        setPublicTransportModeOff: function () {
            publicTransportMode = false;
            publicTransportModePartial = false;
            _apply();

            mapView.clearTransitHighlights();
        },
        changePublicTransportMode: function () {
            _init();
            publicTransportMode = !publicTransportMode;
            _apply();

            if (!publicTransportMode) {
                if (!mapView.mapSharedData.publicTransportLineHightlighted) {
                    mapView.clearTransitHighlights();
                } else {
                    publicTransportModePartial = true;
                    _apply();
                }
            }
        },
        tryPublicTransportMode: function (value) {
            if (!publicTransportMode && !publicTransportModePartial) {
                var tmpPublicTransportMode = (value === PUBLIC_TRANSPORT_MODE_ON);
                if (tmpPublicTransportMode) {
                    if (!publicTransportMode && !publicTransportModePartial) {
                        publicTransportMode = true;
                        _apply();
                    }
                }
            }
        },

        isPublicTransportModePartial: function () {
            _init();
            return publicTransportModePartial;
        },
        setPublicTransportModePartialOn: function () {
            publicTransportModePartial = true;
            _apply();
        },
        setPublicTransportModePartialOff: function () {
            publicTransportModePartial = false;
            _apply();

            mapView.clearTransitHighlights();
        },

        isNightMode: function () {
            _init();
            return timeMode == "night";
        },
        isDayMode: function () {
            _init();
            return timeMode == "day";
        },
        setNightMode: function () {
            timeMode = "night";
            _apply();
        },
        setDayMode: function () {
            timeMode = "day";
            _apply();
        },
        changeDayNightMode: function () {
            _init();
            timeMode = timeMode == "day"? "night" : "day";
            _apply();
        },

        isLandmarksMode: function () {
            _init();
            return landmarksVisible;
        },
        setLandmarksMode: function () {
            landmarksVisible = true;
            _apply();
        },
        unsetLandmarksMode: function () {
            landmarksVisible = false;
            _apply();
        },
        changeLandmarksMode: function () {
            _init();
            landmarksVisible = !landmarksVisible;
            _apply();
        },
        tryLandmakrsVisible: function (value) {
            var tmpLandmarksVisible = (value === true);
            if (tmpLandmarksVisible != landmarksVisible) {
                landmarksVisible = tmpLandmarksVisible;
                _apply();
            }
        },

        isTrafficMode: function () {
            _init();
            return trafficInfoVisible;
        },
        setTrafficMode: function () {
            trafficInfoVisible = true;
            _apply();
            _askForTraffic();
        },
        unsetTrafficMode: function () {
            trafficInfoVisible = false;
            _apply();
        },
        changeTrafficMode: function () {
            _init();
            trafficInfoVisible = !trafficInfoVisible;
            _apply();
            if (mapView.trafficInfoVisible) {
                _askForTraffic();
            }
        },
        tryTrafficModeVisible: function (value) {
            var tmpTrafficInfoVisible = (value === true);
            if (tmpTrafficInfoVisible != trafficInfoVisible) {
                trafficInfoVisible = tmpTrafficInfoVisible;
                _apply();
                if (mapView.trafficInfoVisible) {
                    _askForTraffic();
                }
            }
        },
    };
}()
