import QtQuick 1.0
import ovi.connector 1.0
import MapsPlugin 1.0

// MapModel
// The map model contains a list of MapObjects
// If a MapModel is assigned to the model-property of a Map,
// all objects in the model are added to the modelLayer of the map
ConnectedList {
    id: mapModel

    property int _DEFAULT_ICON_WIDTH_: 70
    property int _DEFAULT_ICON_HEIGHT_: 60
    property variant mainScreen: mainscreen

    signal objectsOwnersChanged(variant objectsOwners)
    property string objectsOwnersCSV
    function setObjectsOwner(owner, append) {
        if (append) {
            mapModel.objectsOwnersCSV += (mapModel.objectsOwnersCSV? "," : "") + owner;
        } else {
            mapModel.objectsOwnersCSV = owner;
        }
        mapModel.objectsOwnersChanged(mapModel.getObjectsOwners());
    }
    function getObjectsOwners() {
        if (mapModel.count == 0) {
            mapModel.objectsOwnersCSV = "";
        }

        return mapModel.objectsOwnersCSV.split(",");
    }
    function checkForOwner(ownerToCheck) {
        var tmp = getObjectsOwners();
        var found = false;
        for (var i = 0; i < tmp.length; i++) {
            if (tmp[i] == ownerToCheck) {
                found = true;
                break;
            }
        }

        return found
    }

    property variant pureSelectedObject
    property variant selectedObject
    function selectObject(object, custom) {
        if (object) {
            var selectedObject = null;
            for (var i = 0, l = mapModel.count; i < l; i++) {
                if (mapModel.get(i) == object) {
                    selectedObject = object;
                    break;
                }
            }

            var tmp, dataA, dataB;
            for (var i = 0, l = mapModel.count; i < l; i++) {
                tmp = mapModel.get(i);
                if (tmp && tmp.location && tmp.location.getField && object.location && object.location.getField) {
                    dataA = tmp.location && tmp.location.getField(Location.OTHER_DATA) && JSON.parse(tmp.location.getField(Location.OTHER_DATA));
                    dataB = object.location && object.location.getField(Location.OTHER_DATA) && JSON.parse(object.location.getField(Location.OTHER_DATA));

                    if (dataA.uri == dataB.uri) {
                        selectedObject = tmp;
                        break;
                    }
                }
            }

            if (selectedObject) {
                mapModel.selectedObject = { fake: true };
                mapModel.selectedObject = selectedObject;

                mapModel.markObjectAsOpenerOfInfoBubble(selectedObject, true);
            } else {
                mapModel.pureSelectedObject = { fake: true };
                mapModel.pureSelectedObject = object;

                mapModel.markObjectAsOpenerOfInfoBubble(object, true);
            }
        } else {
            mapModel.markObjectAsOpenerOfInfoBubble(mapModel.selectedObject, false);
            mapModel.markObjectAsOpenerOfInfoBubble(mapModel.pureSelectedObject, false);

            mapModel.selectedObject = { fake: true };
            mapModel.selectedObject = null;
            if (!custom) {
                mapModel.pureSelectedObject = { fake: true };
                mapModel.pureSelectedObject = null;
            }
        }
    }
    function resetAllSelectedObjects() {
        if (mapModel.selectedObject) {
            mapModel.selectedObject.visible = true;
            mapModel.selectedObject = { fake: true };
            mapModel.selectedObject = null;
        }
        if (mapModel.pureSelectedObject) {
            mapModel.pureSelectedObject.visible = true;
            mapModel.pureSelectedObject = { fake: true };
            mapModel.pureSelectedObject = null;
        }
    }

    property Component iconFactory: mapModel.mainScreen.iconFactory
    function getObjectData(mapObject) {
        var result;

        if (mapObject) {
            if (mapObject.data) {
                result = mapObject.data;
            } else if (mapObject.location && mapObject.location.getField(Location.OTHER_DATA)) {
                result = JSON.parse(mapObject.location.getField(Location.OTHER_DATA));
            }
        }

        return result;
    }

    function getObjectDataKey(mapObject, key) {
        var result;

        var data = getObjectData(mapObject);
        if (data) {
            if (data[key]) {
                result = data[key];
            }
        }

        return result;
    }

    function setObjectDataKey(mapObject, key, value) {
        var success;

        var data = getObjectData(mapObject);
        if (data) {
            data[key] = value;

            if (mapObject.data) {
                mapObject.data = data;
            }
            if (mapObject.location && mapObject.location.getField(Location.OTHER_DATA)) {
                mapObject.location.setField(Location.OTHER_DATA, JSON.stringify(data));
            }

            success = true;
        } else {
            success = false;
        }

        return success;
    }

    function markObjectAsOpenerOfInfoBubble(mapObject, opened) {
        setObjectDataKey(mapObject, "infoBubbleOpened", opened);
    }

    function checkIfItemAlreadyExists(uri) {
        var found = false;
        var tmp;
        for (var i = 0, l = mapModel.count; i < l; i++) {
            tmp = mapModel.get(i);
            if (tmp && tmp.uri == uri) {
                found = true;
                break;
            }
        }

        return found;
    }

    function getObjectByPosition(position) {
        var found = null;
        var tmp;

        for (var i = 0, l = mapModel.count; i < l; i++) {
            tmp = mapModel.get(i);

            if (
                tmp && tmp.data && tmp.location && tmp.location.geoCoordinates &&
                tmp.location.geoCoordinates.latitude && tmp.location.geoCoordinates.longitude &&
                tmp.location.geoCoordinates.latitude == position.latitude &&
                tmp.location.geoCoordinates.longitude == position.longitude
            ) {
                found = tmp;
                break;
            }
        }

        return found;
    }

    function appendIcon(latitude, longitude, imageUrl, data, settings) {
        data = data || {};
        settings = settings || {};
        if (!data.uri) {
            data.uri = Math.random() + "." + Math.random();
        }
        data.latitude = latitude;
        data.longitude = longitude;
        if (!settings.width || !settings.height) {
            settings.width = mapModel._DEFAULT_ICON_WIDTH_;
            settings.height = mapModel._DEFAULT_ICON_HEIGHT_;
        }

        var icon = null;

        if (!checkIfItemAlreadyExists(data.uri)) {
            icon = mapModel.iconFactory.createObject(mapModel);
            icon.uri = data.uri;
            icon.location.geoCoordinates.latitude = latitude;
            icon.location.geoCoordinates.longitude = longitude;
            icon.icon.localUrl = imageUrl;
            icon.anchorPoint = settings.anchorPoint || Qt.point(settings.width / 2, settings.height);
            icon.data = data;
            icon.location.setField(Location.OTHER_DATA, JSON.stringify(data));

            mapModel.append(icon);
        }

        return icon;
    }

    function cleanRemove() {
        for (var a = mapModel.count - 1; a >= 0; a--) {
            var item = mapModel.get(a);
            mapModel.remove(a);
            item.destroy();
        }
        mapModel.clear();
    }

    onItemsRemoved: {
        var iObject;
        for (var i = 0; i < removedObjects.length; ++i) {
            iObject = removedObjects[i];
            if (iObject && iObject.uri) {
                if (mapModel.selectedObject && iObject.uri == mapModel.selectedObject.uri) {
                    mapModel.selectedObject = null;
                }
            }
        }
    }
}
