function _ensureModelLayer() {
    var _modelLayer = mapView.modelLayer;
    if (_modelLayer) {
        mapView.removeLayer(_modelLayer);
        mapView.removeLayer(mapView.modelLayer);
        mapView.modelLayer.destroy();
        mapView.modelLayer = null;
        if (_modelLayer) {
            _modelLayer.destroy();
            _modelLayer = null;
        }
    }
    _modelLayer = mapView.modelLayerFactory.createObject(mapView.mainScreen);
    mapView.addLayer(_modelLayer);
    mapView.modelLayer = _modelLayer;
}

function onModelChanged(model) {
    if (model) {
        _ensureModelLayer();

        var modelLayer = mapView.modelLayer;
        for (var i = 0, l = model.count; i < l; i++) {
            if (modelLayer && modelLayer.addMapObject) {
                modelLayer.addMapObject(model.get(i));
                //if (appState && mapView.lazyAppendIconMode) appState.processEvents();
            }
        }
        model.itemsInserted.connect(function (index, count) {
            if (modelLayer && modelLayer.addMapObject) {
                for (var i = 0; i < count; ++i) {
                    modelLayer.addMapObject(model.get(index + i));
                    //if (appState && mapView.lazyAppendIconMode) appState.processEvents();
                }
            }
        });
        model.itemsRemoved.connect(function (index, count, removedObjects) {
            if (modelLayer && modelLayer.removeMapObject) {
                for (var i = 0; i < removedObjects.length; ++i) {
                    if (removedObjects[i]) {
                        var dataFromObject = removedObjects[i].data? removedObjects[i].data : null;
                        var dataFromLocation = removedObjects[i].location && removedObjects[i].location.getField? JSON.parse(removedObjects[i].location.getField(33/*Location.OTHER_DATA*/)) : null;
                        if ((dataFromObject && dataFromObject.infoBubbleOpened) || (dataFromLocation && dataFromLocation.infoBubbleOpened)) {
                            if (typeof(mainscreen) == "object") {
                                //log.trace("\n\n EMIT CLOSE INFOBUBBLE \n\n")
                                mainscreen.mapSharedData.setControl("close-infobubble");
                            }
                        }

                        modelLayer.removeMapObject(removedObjects[i]);
                    }
                }
            }
        });
    }
}

function setDeviceOrientation() {
    if (mainscreen) {
        if (mainscreen.state == "portrait") {
            return Map.PORTRAIT;
        } else if (mainscreen.state == "landscape") {
            return Map.LANDSCAPE;
        } else if (mainscreen.state == "portraitInverted") {
            return Map.PORTRAIT_INVERTED;
        } else if (mainscreen.state == "landscapeInverted") {
            return Map.LANDSCAPE_INVERTED;
        } else {
            return Map.LANDSCAPE;
        }
    } else {
        return Map.LANDSCAPE;
    }
}

function setBehaviors() {
    var behaviorsParts = mapView.behaviors.split(";");
    for (var i = 0; i < behaviorsParts.length; i++) {
        var behaviorKeyValueParts = behaviorsParts[i].split(":");
        if (behaviorKeyValueParts.length == 2) {
            mapView.setBehavior(behaviorKeyValueParts[0], behaviorKeyValueParts[1]);
        }
    }
}

function setStartupCenter() {
    if (mapView.startupCenterLatitude && mapView.startupCenterLongitude) {
        mapView.center.latitude = mapView.startupCenterLatitude;
        mapView.center.longitude = mapView.startupCenterLongitude;

        MapApi.zoomToLevel(mapView.startupZoomLevel? mapView.startupZoomLevel : 8, null, true);
    }
}

function handleOnLoad() {
    setBehaviors();
    setStartupCenter();
    myPositionBar.onLoad();

    if (!mapView.model) {
        if (!mapView.useGlobalShare) {
            mapView.model = mapView.modelFactory.createObject(mapView);
        } else {
            var rootLevel = MapApi.findRootApplicationLevel(mapView);
            if (rootLevel) {
                var currentGlobalSharedMarker = null;
                var children = rootLevel.children;
                var nameOfChild;

                for (var i = 0, l = children.length; i < l; i++) {
                    nameOfChild = children[i].toString().replace(/^([^_]*).*$/, "$1");
                    if (nameOfChild == "MapModelMarker") {
                        currentGlobalSharedMarker = children[i];
                        break;
                    }
                }

                if (currentGlobalSharedMarker) {
                    mapView.model = currentGlobalSharedMarker.mapModel;
                } else {
                    var model = mapView.modelFactory.createObject(rootLevel);
                    var marker = mapView.modelMarkerFactory.createObject(rootLevel);
                    marker.mapModel = model;
                    mapView.model = model;
                }
            }
        }
    }
}

function handleGetBoundingBox(x, y, width, height, angle) {
    x = parseFloat(x? x : 0) + 1;
    y = parseFloat(y? y : 0) + 1;
    width = parseFloat(width? width : mapView.width) - 1;
    height = parseFloat(height? height : mapView.height) - 1;
    angle = parseFloat(angle? angle : mapView.rotation);

    var angleToRadians = angle * Math.PI / 180;

    var radius = Math.sqrt(Math.pow(width, 2) + Math.pow(height, 2)) / 2;

    var xP = x - radius * Math.sin(angleToRadians);
    var yP = y + radius * (Math.cos(angleToRadians) - 1);

    x = xP;
    y = yP;

    var tmpEdgeTopLeft, tmpEdgeTopRight, tmpEdgeBottomRight, tmpEdgeBottomLeft;
    if (MapApi.MapLayers.is2dMode()) {
        tmpEdgeTopLeft = mapView.screenToGeo(Qt.point(x, y), true);
        tmpEdgeTopRight = mapView.screenToGeo(Qt.point(width, x), true);
    } else {
        tmpEdgeTopLeft = mapView.screenToGeo(Qt.point(x, (height) / 3), true);
        tmpEdgeTopRight = mapView.screenToGeo(Qt.point(width, (height) / 3), true);
    }
    tmpEdgeBottomRight = mapView.screenToGeo(Qt.point(width, height), true);
    tmpEdgeBottomLeft = mapView.screenToGeo(Qt.point(x, height), true);

    mapView.mapSharedData.edgeTopLeft = { latitude: tmpEdgeTopLeft.latitude, longitude:tmpEdgeTopLeft.longitude  };
    mapView.mapSharedData.edgeTopRight = { latitude: tmpEdgeTopRight.latitude, longitude:tmpEdgeTopRight.longitude  };
    mapView.mapSharedData.edgeBottomRight = { latitude: tmpEdgeBottomRight.latitude, longitude:tmpEdgeBottomRight.longitude  };
    mapView.mapSharedData.edgeBottomLeft = { latitude: tmpEdgeBottomLeft.latitude, longitude:tmpEdgeBottomLeft.longitude  };

    return {
        edgeTopLeft: { latitude: tmpEdgeTopLeft.latitude, longitude: tmpEdgeTopLeft.longitude },
        edgeTopRight: { latitude: tmpEdgeTopRight.latitude, longitude: tmpEdgeTopRight.longitude },
        edgeBottomRight: { latitude: tmpEdgeBottomRight.latitude, longitude: tmpEdgeBottomRight.longitude },
        edgeBottomLeft: { latitude: tmpEdgeBottomLeft.latitude, longitude: tmpEdgeBottomLeft.longitude }
    }
}
