function init() {
    if (appState.positioningEnabled) {
        myPositionBar.positionProviderStart();
    }
}

function positionProviderStart() {
    if (!internalStateKeeper.positionProviderDisabled) {
        if (!internalStateKeeper.positionProviderEnabled) {
            log.info("position provider enabled set to TRUE " + mapView)
            internalStateKeeper.positionUpdateIndex = 0;

            internalStateKeeper.positionProviderEnabled = true;
            if (!positionProvider.enabled) {
                positionProvider.enabled = true;
            }
        }
    }
}

function positionProviderStop() {
    if (internalStateKeeper.positionProviderEnabled) {
        log.info("position provider enabled set to FALSE " + mapView)

        if (mapView.visible) {
            InternalLib.setCurrentPosition();
        }

        internalStateKeeper.positionProviderEnabled = false;
        if (positionProvider.enabled) {
            positionProvider.enabled = false;
        }
    }
}

function disablePositionProvider() {
    internalStateKeeper.positionProviderDisabled = true;
}

function enablePositionProvider() {
    internalStateKeeper.positionProviderDisabled = false;
}

function myPositionBarClicked() {
    if (myPositionBar.hasPosition && !myPositionBar.attachedToLocation) {
        _flyToCurrentPosition();
    } else if (myPositionBar.hasPosition && myPositionBar.attachedToLocation) {
        if (!mapView.getBehavior(mapView._NO_COMPASS_ALLOWED_)) {
            if (!myPositionBar.compassMode) {
                if (mainscreen.magicCompass) {
                    _turnOnCompass();
                }
            } else {
                if (mainscreen.magicCompass) {
                    _turnOffCompass();
                }
            }
        }
    } else if (!myPositionBar.hasPosition) {
        _haveNoLocation();
    }

    mapView.myPositionBarClicked();
}
function _flyToCurrentPosition(noAttachment, force) {
    if (force || !mapView.getBehavior(mapView._NO_MY_POSITION_CLICK_ACTION_AT_ALL_)) {
        if (force || !mapView.getBehavior(mapView._NO_MY_POSITION_DEFAULT_CLICK_ACTION_)) {
            mapView.flyToCurrentPosition();
            if (myPositionBar.currentPosition) {
                //log.info("jump to current position and attach");
                if (!myPositionBar.previousCompassMode && mapView.orientation == 0) {
                    mapView.moveTo(
                        myPositionBar.currentPosition.geoCoordinates,
                        Map.ANIMATION_BOW,
                        mapView.zoomLevels[14] / 1.5,
                        0,
                        Map.PRESERVE_PERSPECTIVE
                    );

                    internalStateKeeper.flyingToLocation = true;
                    if (mapView.mode == mapView._DEFAULT_MODE_ && !noAttachment) {
                        InternalLib.methodToApply.set("attach-to-location");
                        mapView.status = mapView._DEFAULT_STATUS_;
                        //rightPopup.message = qsTranslate("Maps", "qtn_mapconnector_myposition_flytolocation");
                    }
                } else {
                    mapView.moveTo(
                        mapView.center,
                        Map.ANIMATION_BOW,
                        Map.PRESERVE_ZOOM_SCALE,
                        0,
                        Map.PRESERVE_PERSPECTIVE
                    );
                    myPositionBar.previousCompassMode = false;

                    if (myPositionBar._ALLOW_AUTO_COMPASS_ON_IN_CASE_OR_PRE_PREVIOUS_COMPASS_MODE_) {
                        myPositionBar.autoStartCompassOnAttachToLocation = true;
                    } else {
                        myPositionBar.autoStartCompassOnAttachToLocation = false;
                    }

                    mapView.propagandize("sync-compass", {from: "fly-to-current-position", previousCompassMode: myPositionBar.previousCompassMode, autoStartCompassOnAttachToLocation: myPositionBar.autoStartCompassOnAttachToLocation});
                }
            }
        } else {
            InternalLib.methodToApply.set("attach-to-location");
            mapView.status = mapView._DEFAULT_STATUS_;
            _applyPostEventAttachment();
        }
    }
}
function _flyToPreviousPosition() {
    mapView.flyToCurrentPosition();
    if (myPositionBar.previousPosition) {
        //log.info("jump to previous position");
        mapView.moveTo(
            myPositionBar.previousPosition.geoCoordinates,
            Map.ANIMATION_BOW,
            mapView.zoomLevels[14] / 1.5,
            0,
            Map.PRESERVE_PERSPECTIVE
        );

        internalStateKeeper.flyingToLocation = true;
        if (mapView.mode == mapView._DEFAULT_MODE_) {
            rightPopup.message = qsTranslate("Maps", "qtn_mapconnector_myposition_flytoprevlocation");
        }
    }
}

function _turnOnCompass(dontPropagandize) {
    if (!myPositionBar.compassMode) {
        myPositionBar.previousCompassMode = false;
        myPositionBar.compassMode = true;

        buggy.visible = true;
        background.opacity = 0.1;

        appState.startCompass();

        //rightPopup.message =  qsTranslate("Maps", "qtn_mapconnector_myposition_compasson");

        if (!dontPropagandize) {
            mapView.setControl("emit-turn-on-compass");
            mapView.propagandize("sync-compass", {from: "turn-on-compass", previousCompassMode: myPositionBar.previousCompassMode, autoStartCompassOnAttachToLocation: myPositionBar.autoStartCompassOnAttachToLocation});
        }
    }
}

var compassCalibrationDialogueOpened = false;
function _handleCompass(azimuth, calibrationLevel) {
    if (azimuth) {
        //log.trace(".............COMPASS..............            " + azimuth + " :: " + calibrationLevel)
        if (calibrationLevel < 0.2) {
            if (!compassCalibrationDialogueOpened) {
                queryDialog.titleText = qsTranslate("Maps", "qtn_mapconnector_myposition_setcompass_head");
                queryDialog.message = qsTranslate("Maps", "qtn_mapconnector_myposition_setcompass_body")
                queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_mapconnector_myposition_cancelbtn");
                queryDialog.acceptButtonText = "";
                queryDialog.open();

                queryDialogSelectedCallback = function(label, options) {
                    _turnOffCompass();
                    compassCalibrationDialogueOpened = false;
                }

                log.info("OPEN COMPASS caliBRAtioN")
                compassCalibrationDialogueOpened = true;

                queryDialogSelectedMode = "compass-mode";
            }
        } else {
            if (queryDialogSelectedMode) {
                appState.startVibration(1000 * 3);
                queryDialog.close();
                queryDialogSelectedMode = "";
            }
        }

        mapView.orientation = compassTimeApplier.azimuth;
    }
}

function _turnOffCompass(keepCompassModeInPreviousStage, dontPropagandize) {
    if (myPositionBar.compassMode) {
        //rightPopup.message =  qsTranslate("Maps", "qtn_mapconnector_myposition_compassoff");

        myPositionBar.previousCompassMode = keepCompassModeInPreviousStage? myPositionBar.compassMode : false;
        myPositionBar.compassMode = false;

        appState.stopCompass();

        buggy.visible = false;
        background.opacity = 1;

        compassTimeApplier.reset();

        if (!dontPropagandize) {
            mapView.setControl("emit-turn-off-compass");
            mapView.propagandize("sync-compass", {from: "turn-off-compass", previousCompassMode: myPositionBar.previousCompassMode, autoStartCompassOnAttachToLocation: myPositionBar.autoStartCompassOnAttachToLocation});
        }
    }
}

function conditionalTurnOnOffCompass() {
    if (internalStateKeeper.applicationMaximized) {
        if (myPositionBar.compassMode) {
            appState.startCompass();
        }
    } else {
        if (myPositionBar.compassMode) {
            appState.stopCompass();
        }
    }
}

function _haveNoLocation() {
    //rightPopup.message = "you still have no location"; NOT USED AT THE MOMENT
    if (!myPositionBar.previousPosition) {
        /*
        queryDialog.titleText = qsTranslate("Maps", "qtn_mapconnector_myposition_nopositionhead");
        queryDialog.message = qsTranslate("Maps", "qtn_mapconnector_myposition_noprevpositionbody") + " \n";
        queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_mapconnector_myposition_closebtn");
        queryDialog.open();
        */
    } else {
        _flyToPreviousPosition();
        /*
        queryDialog.titleText = qsTranslate("Maps", "qtn_mapconnector_myposition_nopositionhead");
        queryDialog.message = qsTranslate("Maps", "qtn_mapconnector_myposition_noprevpositionbody") + " \n";
        queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_mapconnector_myposition_cancelbtn");
        queryDialog.acceptButtonText = qsTranslate("Maps", "qtn_mapconnector_myposition_latestbtn");
        queryDialog.open();

        queryDialogSelectedCallback = function(label, options) {
            if (queryDialogSelectedMode == "no-location") {
                _flyToPreviousPosition();
            }
        }
        */
    }
    queryDialogSelectedMode = "no-location";
}

function positionProviderPositionUpdated() {
    mapView.__debugLogger.log(">> " + "position updated :: " + positionProvider.positionMethod);
    //log.info("*** Position provider current mapMatcherMode :: " + positionProvider.mapMatcherMode);
    //log.info("*** Position provider :: position updated in MyPositionBar in MAPS :: " + mapView);
    if (mapView.visible && internalStateKeeper.positionProviderEnabled && !internalStateKeeper.flyingToLocation) {
        //log.info("LOCAITON CHANGED...")
        if (queryDialogSelectedMode == "no-location") {
            queryDialog.close();
            queryDialogSelectedMode = "";
        }

        if (internalStateKeeper.positionUpdateIndex == 0) {
            if (!myPositionBar.attachedToLocation) {
                if (mapView.mode == mapView._NO_POSITION_AT_ALL_) {
                    rightPopup.hideTimeMultiplier = 3;
                    //rightPopup.message = qsTranslate("Maps", "qtn_mapconnector_myposition_locationfirsttime");
                } else {
                    //rightPopup.message = qsTranslate("Maps", "qtn_mapconnector_myposition_jumpbygreendot");
                }
            }
        }
        internalStateKeeper.positionUpdateIndex++;

        //InternalLib.setCurrentPosition(positionProvider.mapMatchedPosition? positionProvider.mapMatchedPosition : positionProvider.devicePosition);
        InternalLib.setCurrentPosition(positionProvider.position);

        if (mapView.mode == mapView._NO_POSITION_AT_ALL_ || !myPositionBar.currentPosition) {
            _flyToCurrentPosition();

            internalStateKeeper.currentPositionVisible = true;
            mapView.mode = mapView._DEFAULT_MODE_;
        } else {
            if (myPositionBar.attachedToLocation) {
                if (!myPositionBar.skipAnimationOnAttachedToLocation) {
                    //mapView.center = myPositionBar.currentPosition.geoCoordinates;
                    mapView.moveTo(
                        myPositionBar.currentPosition.geoCoordinates,
                        Map.ANIMATION_NONE,
                        Map.PRESERVE_ZOOM_SCALE,
                        Map.PRESERVE_ORIENTATION,
                        Map.PRESERVE_PERSPECTIVE
                    );
                }
            } else if (myPositionBar.hasPosition) {
                InternalLib.resetDistancePointer();
            }
        }

        internalStateKeeper.hasLocationFixInTheMoment = true;

        positionMarker.centerChanged();

        _applyFlyToCurrentPositionOnStartup();
    }

    if (positionProvider.positionMethod != PositionProvider.METHOD_NONE) {
        myPositionBar.positionUpdatedWithValidPosition();
    } else {
        myPositionBar.positionUpdatedWithInvalidPosition();
    }
}

function _applyFlyToCurrentPositionOnStartup() {
    if (!mainscreen.forcePreventJumpToCurrentPositionOnStartup) {
        if (internalStateKeeper.positionUpdateIndex == 1) {
            if (!myPositionBar.attachedToLocation) {
                if (mapView.mode == mapView._NO_POSITION_AT_ALL_) {
                } else {
                    if (mapView.mapSharedData.state == "modified") {
                        if (!mapView.getBehavior(mapView._NO_AUTO_GPS_INIT_)) {
                            _flyToCurrentPosition(true);
                            mapView.mapSharedData.state = "afterInitialFly";
                        }
                    }
                }
            }
        }
    }
}

function positionProviderPositionLost(hasValidPosition) {
    mapView.__debugLogger.log(">> " + "position lost with hasValidPosition :: " + hasValidPosition);
    if (!hasValidPosition && internalStateKeeper.positionProviderEnabled) {
        mapView.__debugLogger.log(">> " + "position lost with hasValidPosition :: " + hasValidPosition + " :: enter and mark as has now location in the moment");
        internalStateKeeper.hasLocationFixInTheMoment = false;

        myPositionBar.positionLost();
    }
}

function mapViewOnMapDragStarted() {
    myPositionBar.attachedToLocation = false;
    internalStateKeeper.flyingToLocation = false;
    _turnOffCompass(true);

    InternalLib.methodToApply.reset();
    if (!rightPopup.dontClosePopupOnMouseMove) {
        rightPopup.message = "";
    }
}

function mapViewOnCenterChanged() {
    InternalLib.resetDistancePointer();
}

function mapViewOnMapDragged() {
    mapView.mode = mapView._DEFAULT_MODE_;
}

function mapViewOnMapLongPressed() {
    myPositionBar.attachedToLocation = false;
}

function mapViewOnMapClicked(pixelPos, geoPos) {
    myPositionBar.attachedToLocation = false;
    _turnOffCompass(true);

    InternalLib.methodToApply.reset();
    if (!rightPopup.dontClosePopupOnMouseMove) {
        rightPopup.message = "";
    }
}

function mapViewOnMapDoubleClicked(pixelPos, geoPos) {
    myPositionBar.attachedToLocation = false;
}

function mapViewOnAnimationDone() {
    _applyPostEventAttachment();
}

function mapViewOnPerspectiveChanged() {
    if (mapView.perspective == 0) {
        positionMarker.icon3d = false;
    } else {
        positionMarker.icon3d = true;
    }
}

function mapViewOnZoomScaleScanged() {
    InternalLib.resetDistancePointer();
    if (mapView.perspective == 0) {
        positionMarker.icon3d = false;
    } else {
        positionMarker.icon3d = true;
    }
}

function mapViewOnPreviousPositionChanged() {
    if (queryDialogSelectedMode == "no-location") {
        _haveNoLocation();
    }
}

function mapViewOnVisibleChanged() {
    if (!mapView.visible) {
        if (myPositionBar.compassMode) {
            if (mainscreen.magicCompass) {
                //
            }
        }
    }
}

function compassTimeApplierReset() {
    compassTimeApplier.queue = [];
}

function compassTimeApplierSetToAverage(azimuth) {
    var tmp = compassTimeApplier.queue;
    if (!tmp || Math.abs(compassTimeApplier.azimuth - azimuth) > 300) {
        tmp = [];
    }
    tmp.push(azimuth);
    if (tmp.length > compassTimeApplier._MAXIMUM_QUEUE_COUNT_) {
        tmp.shift();
    }
    compassTimeApplier.queue = tmp;

    var sum = 0;
    for (var i = 0; i < tmp.length; i++) {
        sum += tmp[i];
    }

    var newAverage = sum / tmp.length;

    if (true/*tmp.length == 1 || Math.abs(newAverage - compassTimeApplier.azimuth) < myPositionBar._MAXIMAL_COMPASS_DIFFERENT_ALLOWED_TO_APPLY_*/) {
        compassTimeApplier.azimuth = sum / tmp.length;
    }
}

function mapViewControlChanged(control) {
    if (control == "turn-on-compass") {
        _turnOnCompass(true);
    } else if (control == "turn-off-compass") {
        _turnOffCompass(false, true);
    }
}

function mapViewPropagandaChanged(propaganda) {
    if (propaganda && propaganda.action == "sync-compass") {
        myPositionBar.autoStartCompassOnAttachToLocation = propaganda.autoStartCompassOnAttachToLocation;
        myPositionBar.previousCompassMode = propaganda.previousCompassMode;
    }
}

var queryDialogSelectedCallback = null;
var queryDialogSelectedMode = null;
function _handleQueryCallback(optionText) {
    if (queryDialogSelectedCallback) {
        queryDialogSelectedCallback(optionText);
        queryDialogSelectedCallback = null;
    }
    queryDialogSelectedMode = null;
}
function queryDialogAccepted(optionText) {
    _handleQueryCallback(optionText);
}
function queryDialogRejected(optionText) {
    _handleQueryCallback(optionText);
}

function positionLostApplierOnTriggered() {
    mapView.__debugLogger.log(">> " + "position lost with current hasPosition :: " + myPositionBar.hasPosition + " :: " + myPositionBar.positionType);
    if (myPositionBar.hasPosition) {
        mapView.__debugLogger.log(">> " + "position lost will be applied now for sure");
        internalStateKeeper.positionUpdateIndex = 0;
        InternalLib.setCurrentPosition();

        //rightPopup.message = qsTranslate("Maps", "qtn_mapconnector_myposition_locationlost");
    }
}

function _applyPostEventAttachment() {
    var methodToApply = InternalLib.methodToApply.get();

    if (mapView.status != mapView._INTERRUPTED_ANIMATION_) {
        if (methodToApply) {
            //log.info("method to post-apply is " + methodToApply);
            if (methodToApply == "attach-to-location") {
                //log.info("--- ATTACHED MODE TRUE ---")
                myPositionBar.attachedToLocation = true;
                internalStateKeeper.currentPositionVisible = true;

                if (myPositionBar.autoStartCompassOnAttachToLocation) {
                    _turnOnCompass();
                }

                myPositionBar.autoStartCompassOnAttachToLocation = false;

                //bottomPopup.distance = Units.normalizeDistance(0).readableWithUnits;
            } else {
                //log.info("method not located, skipping \"" + methodToApply + "\"...");
            }
        }

    }

    mapView.status = mapView._DEFAULT_STATUS_;

    internalStateKeeper.flyingToLocation = false;
}
