import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import "MapApi.js" as MapApi
import "MyPositionBar.js" as EventHandlers
import "MyPositionBarTools.js" as InternalLib

Item {
    id: myPositionBar

    property variant mapView
    property variant positionProvider: mainscreen.positionProvider

    property bool attachedToLocation: false
    property bool skipAnimationOnAttachedToLocation: false

    property bool compassMode: false
    property bool previousCompassMode: false
    property bool autoStartCompassOnAttachToLocation: false
    property bool _ALLOW_AUTO_COMPASS_ON_IN_CASE_OR_PRE_PREVIOUS_COMPASS_MODE_: false

    property int _MAXIMAL_COMPASS_DIFFERENT_ALLOWED_TO_APPLY_: 20

    property bool hasPosition: myPositionBar.positionType == mapView._POSITION_TYPE_GPS_ || myPositionBar.positionType == mapView._POSITION_TYPE_WIFI_CELL_
    property variant currentPosition: null
    property variant previousPosition: null

    property variant averageSpeed: positionProvider.averageSpeed

    property alias drawnHaloSize: positionMarker.accuracy

    property variant referenceToMapMatchedPosition: positionProvider.mapMatchedPosition
    property variant referenceToDevicePosition: positionProvider.devicePosition

    /**
      * o NONE - no positioning
      * o GPS - GPS positioning
      * o WIFI_CELL - wifi or cell
      */
    property string positionType: mapView._POSITION_TYPE_NONE_

    property alias positionProviderEnabled: internalStateKeeper.positionProviderEnabled

    property int myBtnSize: 52
    property int myRadius: 20
    property int myBottomPopupExpansionAnimationDuration: 50
    property int myRightPopupExpansionAnimationDuration: 250

    property Item __rightPopup: rightPopup

    signal positionUpdatedWithValidPosition()
    signal positionUpdatedWithInvalidPosition()
    signal positionLost()

    function startPositionProvider() { EventHandlers.positionProviderStart() }
    function stopPositionProvider() { EventHandlers.positionProviderStop() }
    function disablePositionProvider() { EventHandlers.disablePositionProvider() }
    function enablePositionProvider() { EventHandlers.enablePositionProvider() }
    function flyToCurrentPosition() { EventHandlers._flyToCurrentPosition(true, true) }
    function forceChangeAccuracy(accuracy) { positionMarker.changeAccuracy(accuracy) }

    Units { id: units }

    Item {
        id: internalStateKeeper

        property bool flyingToLocation: false
        property bool currentPositionVisible: false
        property int positionUpdateIndex: 0
        property bool hasLocationFixInTheMoment: false
        property bool positionProviderEnabled: false
        property bool positionProviderDisabled: false

        property bool applicationMaximized: true
    }

    Rectangle {
        id: expandableRectangle

        width: myBtnSize + rightPopup.width
        height: myBtnSize
        radius: myRadius

        color: Qt.rgba(0, 0, 0, 0.5)
        border.color: "white"
        border.width: 2

        smooth: true

        anchors.bottom: background.bottom

        Behavior on width { PropertyAnimation { duration: myRightPopupExpansionAnimationDuration } }
    }

    Rectangle {
        id: background

        width: myBtnSize
        height: myBtnSize
        radius: myRadius

        color: "transparent"
        border.color: "white"
        border.width: 2

        smooth: true

        //anchors.bottom: bottomPopup.top
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3

        signal clicked()
        MouseArea {
            anchors.fill: parent
            onClicked: { background.clicked() }
        }

        states: [
            State {
                name: "hidden"
                when: !myPositionBar.hasPosition || myPositionBar.attachedToLocation || internalStateKeeper.flyingToLocation || internalStateKeeper.currentPositionVisible
                /*
                AnchorChanges {
                    target: background
                    anchors.bottom: parent.bottom
                }
                PropertyChanges {
                    target: background
                    anchors.bottomMargin: 0
                }
                */
            }
        ]
        state: "hidden"

        transitions: [
            Transition {
                to: "hidden"
                AnchorAnimation { duration: 50; }
            },
            Transition {
                AnchorAnimation { duration: 50; }
            }
        ]
    }

    Item {
        id: arrow

        rotation: 0
        visible: bottomPopup.visible && (/*myPositionBar.hasPosition && */(myPositionBar.currentPosition || myPositionBar.previousPosition)) && !myPositionBar.attachedToLocation && !internalStateKeeper.flyingToLocation && !internalStateKeeper.currentPositionVisible && !myPositionBar.previousCompassMode && mapView.orientation == 0

        Image {
            id: arrowImage
            source: mapAssetsPath + "images/layout/arrow.png"

            smooth: true

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.top
            anchors.verticalCenterOffset: -15
        }

        anchors.horizontalCenter: background.horizontalCenter
        anchors.verticalCenter: background.verticalCenter
    }

    Item {
        id: dot

        visible: !myPositionBar.compassMode || myPositionBar.previousCompassMode

        clip: true
        width: myBtnSize
        height: myBtnSize
        rotation: -1 * mapView.orientation
        smooth: true

        Image {
            id: dotCenterImage
            objectName: "dotCenterImage"
            source: mapAssetsPath + "images/layout/buttonSymbol.png"
            smooth: true

            states: [
                State {
                    name: "dotD-AttachedToLocation"
                    when: ((!!myPositionBar.currentPosition || !!myPositionBar.previousPosition) && myPositionBar.attachedToLocation) || internalStateKeeper.flyingToLocation || (!bottomPopup.visible && (!!myPositionBar.currentPosition || !!myPositionBar.previousPosition || mapView.orientation != 0))
                    PropertyChanges {
                        target: dotCenterImage
                        anchors.verticalCenterOffset: -70
                        source: mapAssetsPath + "images/layout/buttonSymbol.png"
                    }
                },
                State {
                    name: "dotA-Yellow"
                    when: (mapView.getBehavior(mapView._POSITION_TYPE_REQUIRED_GPS_) && mapView.positionType != mapView._POSITION_TYPE_GPS_) || (!myPositionBar.hasPosition && !internalStateKeeper.flyingToLocation && !!myPositionBar.previousPosition && !myPositionBar.currentPosition)
                    PropertyChanges {
                        target: dotCenterImage
                        anchors.verticalCenterOffset: 70
                        source: mapAssetsPath + "images/layout/buttonSymbol.png"
                    }
                },
                State {
                    name: "dotB-Gray"
                    when: !myPositionBar.hasPosition && !internalStateKeeper.flyingToLocation && !myPositionBar.previousPosition && !myPositionBar.currentPosition
                    PropertyChanges {
                        target: dotCenterImage
                        anchors.verticalCenterOffset: 70
                        source: mapAssetsPath + "images/layout/buttonSymbol_gray.png"
                    }
                },
                State {
                    name: "dotC-Green"
                    when: myPositionBar.hasPosition && !myPositionBar.attachedToLocation && !internalStateKeeper.flyingToLocation
                    PropertyChanges {
                        target: dotCenterImage
                        anchors.verticalCenterOffset: 0
                        source: mapAssetsPath + "images/layout/buttonSymbol.png"
                    }
                }
            ]

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
        }

        anchors.horizontalCenter: background.horizontalCenter
        anchors.verticalCenter: background.verticalCenter
    }

    Rectangle {
        id: buggy

        visible: false

        anchors.horizontalCenter: background.horizontalCenter
        anchors.verticalCenter: background.verticalCenter

        width: background.width
        height: background.height
        radius: background.radius
        color: Qt.rgba(0, 0, 0, 0.01)

        border.color: "white"
        border.width: 2

        smooth: true
    }

    Item {
        id: compass

        visible: myPositionBar.compassMode

        property int rotation: -1 * mapView.orientation

        clip: true
        width: myBtnSize
        height: myBtnSize

        Image {
            id: compassCenterImage
            source: mapAssetsPath + "images/layout/compassNeedleShadow.png"

            smooth: true

            rotation: compass.rotation

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenterOffset: 3
            anchors.verticalCenterOffset: 3
        }
        Image {
            id: compassCenterImageShadow
            source: mapAssetsPath + "images/layout/compassNeedle.png"

            smooth: true

            rotation: compass.rotation

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
        }

        anchors.horizontalCenter: background.horizontalCenter
        anchors.verticalCenter: background.verticalCenter
    }

    Rectangle {
        id: bottomPopup

        visible: distance != "0.0 m" && (/*myPositionBar.hasPosition && */(myPositionBar.currentPosition || myPositionBar.previousPosition)) && !myPositionBar.attachedToLocation && !internalStateKeeper.flyingToLocation && !internalStateKeeper.currentPositionVisible && !myPositionBar.previousCompassMode && mapView.orientation == 0

        property string distance: "0.0 m"

        width: (distanceText.paintedWidth + 15 < myBtnSize)? myBtnSize : distanceText.paintedWidth + 15
        height: distanceText.height + 10

        Behavior on width { PropertyAnimation { duration: myBottomPopupExpansionAnimationDuration } }

        radius: myRadius

        color: Qt.rgba(0, 0, 0, 0.5)
        border.color: "white"
        border.width: 2

        smooth: true

        Text {
            id: distanceText
            objectName: "distanceText"

            text: bottomPopup.distance
            color: "white"
            font.pixelSize: 11

            //anchors.horizontalCenter: parent.horizontalCenter
            //anchors.verticalCenter: parent.verticalCenter
            anchors.centerIn: parent
        }


        //anchors.bottom: parent.bottom
        anchors.bottom: background.top
        anchors.bottomMargin: 3
    }

    Item {
        visible:  true

        width: rightPopup.width
        height: rightPopup.height

        clip: true

        anchors.verticalCenter: background.verticalCenter
        anchors.left: background.right
        anchors.leftMargin: -2

        Behavior on width { PropertyAnimation { duration: myRightPopupExpansionAnimationDuration } }


        Rectangle {
            id: rightPopup

            smooth: true

            property bool dontClosePopupOnMouseMove: false

            visible: rightPopup.message
            onVisibleChanged: {
                if (!visible) {
                    rightPopup.hideTimeMultiplier = 1;
                }
            }

            property int hideTimeMultiplier: 1

            property string message
            onMessageChanged: {
                hideTimer.restart();
            }

            width: {
                var realWidthToApply = (messageText.paintedWidth + 24) * rightPopup.visible;
                var maxWidthToApply = Math.max((mapView.mainScreen? mapView.mainScreen.width : parent.width) - 150, myBtnSize);

                var resultWidth;

                if (maxWidthToApply > realWidthToApply) {
                    resultWidth = realWidthToApply;
                } else {
                    resultWidth = maxWidthToApply;
                }

                return resultWidth;
            }

            height: myPositionBar.myBtnSize

            color: "transparent"

            Text {
                id: messageText

                visible: true

                text: rightPopup.message
                color: "white"

                font.pixelSize: 15

                elide: Text.ElideRight

                //wrapMode: Text.Wrap

                anchors.verticalCenter: rightPopup.verticalCenter
                anchors.left: rightPopup.left
                anchors.leftMargin: 12

                //anchors.centerIn: rightPopup
            }

            Timer {
                id: hideTimer

                running: false
                interval: 1000 * 2.5 * rightPopup.hideTimeMultiplier
                onTriggered: {
                    rightPopup.message = "";
                    hideTimer.running = false;
                    rightPopup.dontClosePopupOnMouseMove = false;
                }
            }
        }
    }

    PositionMarker {
        id: positionMarker

        mapView: myPositionBar.mapView
        animationInterval: 25

        shown: !!(myPositionBar.currentPosition || myPositionBar.previousPosition) && !mapView.getBehavior(mapView._NO_POSITION_MARKER_)
        center: {
            if (myPositionBar.currentPosition) {
                return myPositionBar.currentPosition.geoCoordinates;
            } else if (myPositionBar.previousPosition) {
                return myPositionBar.previousPosition.geoCoordinates;
            } else {
                return null;
            }
        }
        enabled: (!mapView.getBehavior(mapView._POSITION_TYPE_REQUIRED_GPS_) || (mapView.getBehavior(mapView._POSITION_TYPE_REQUIRED_GPS_) && mapView.positionType == mapView._POSITION_TYPE_GPS_)) && !!myPositionBar.currentPosition
        accuracy: normalizePositionProviderSpeedTimer.accuracy
        //icon3d: mapView.perspective != 0
    }

    Timer {
        id: normalizePositionProviderSpeedTimer
        interval: 1000 * 5
        repeat: true
        running: mapView.visible && internalStateKeeper.applicationMaximized

        property int accuracy: 0

        onTriggered: {
            log.info("5 seconds accuracy update... ")
            normalizePositionProviderSpeedTimer.accuracy = positionMarker.enabled && myPositionBar.currentPosition? InternalLib.normalizeHaloAccuracyInMeters(myPositionBar.currentPosition.accuracy) : 0;
        }
    }

    QueryDialog {
        id: queryDialog
        onAccepted: { EventHandlers.queryDialogAccepted("accept") }
        onRejected: { EventHandlers.queryDialogRejected("reject") }
    }

    function positionProviderStart() { EventHandlers.positionProviderStart() }
    function positionProviderStop() { EventHandlers.positionProviderStop() }

    Timer {
        id: positionLostApplier

        interval: 30 * 1000

        running: !internalStateKeeper.hasLocationFixInTheMoment
    }

    Connections {
        target: appState
        ignoreUnknownSignals: true

        onCompassReading: {
            compassTimeApplier.setToAverage(azimuth);
            compassTimeApplier.calibrationLevel = calibrationLevel;
        }

        onMinimized: { }
        onMaximized: { }
        onWindowDeactivated: {
            internalStateKeeper.applicationMaximized = false;
            EventHandlers.conditionalTurnOnOffCompass();
        }
        onWindowActivated: {
            internalStateKeeper.applicationMaximized = true;
            EventHandlers.conditionalTurnOnOffCompass();
        }
    }

    Connections {
        target: background
        ignoreUnknownSignals: true

        onClicked: { EventHandlers.myPositionBarClicked(); }
    }

    Connections {
        target: positionProvider
        ignoreUnknownSignals: true

        onPositionUpdated: { EventHandlers.positionProviderPositionUpdated() }
        onPositionFixChanged: { EventHandlers.positionProviderPositionLost(positionProvider.hasValidPosition) }
    }

    Connections {
        target: positionLostApplier
        ignoreUnknownSignals: true

        onTriggered: { EventHandlers.positionLostApplierOnTriggered() }
    }

//    Connections {
//        target: queryDialog
//        ignoreUnknownSignals: true

//        onSelected: { EventHandlers.queryDialogSelected(optionText, param) }
//    }

    Connections {
        target: mapView
        ignoreUnknownSignals: true

        onAnimationDone: { EventHandlers.mapViewOnAnimationDone() }
        onPerspectiveChanged: { EventHandlers.mapViewOnPerspectiveChanged() }
        onZoomScaleChanged: { EventHandlers.mapViewOnZoomScaleScanged() }
        onCenterChanged: { EventHandlers.mapViewOnCenterChanged() }

        onMapClicked: { EventHandlers.mapViewOnMapClicked(pixelPos, geoPos) }
        onMapDoubleClicked: { EventHandlers.mapViewOnMapDoubleClicked() }
        onMapDragStarted: { EventHandlers.mapViewOnMapDragStarted(pixelPos, geoPos) }
        onMapDragged: { EventHandlers.mapViewOnMapDragged(pixelPos, geoPos) }
        onMapLongPressed: { EventHandlers.mapViewOnMapLongPressed(pixelPos, geoPos) }

        onPreviousPositionChanged: { EventHandlers.mapViewOnPreviousPositionChanged() }

        onVisibleChanged: { EventHandlers.mapViewOnVisibleChanged() }

        onControlChanged: { EventHandlers.mapViewControlChanged(mapView.control) }
        onPropagandaChanged: { EventHandlers.mapViewPropagandaChanged(mapView.propaganda) }
    }

    Connections {
        target: myPositionBar
        ignoreUnknownSignals: true

        onPreviousPositionChanged: {
            InternalLib.resetDistancePointer();
        }
        onCurrentPositionChanged: {
            InternalLib.resetDistancePointer();
        }

        onHasPositionChanged: {
            mapView.__debugLogger.log(">> " + "hasPosition changed to :: " + hasPosition + " :: " + currentPosition + " :: " + previousPosition);
        }
    }

    Connections {
        target: rootApplicationLevel
        ignoreUnknownSignals: true

        onSignInComplete: {
            if (success) {
                EventHandlers.init();
            }
        }
    }

    Connections {
        target: positionMarker
        ignoreUnknownSignals: true

        onAccuracyChanged: {

        }
    }

    function onLoad() { lazyLoader.start() }

    Timer {
        id: lazyLoader
        interval: 1
        repeat: false
        running: false
        onTriggered: {
            if (typeof(rootApplicationLevel) == "object" && !rootApplicationLevel.mapLocked || typeof(rootApplicationLevel) != "object") {
                if (mainscreen.applicationMaximized) {
                    EventHandlers.init();
                } else {
                    log.info("*** POSITION PROVIDER START :: Seems application is minimized already, so will not start position provider...");
                }
            }
        }
    }

    Timer {
        id: compassTimeApplier

        property real azimuth
        property int calibrationLevel

        property variant queue
        property variant _MAXIMUM_QUEUE_COUNT_: 7

        function reset() { EventHandlers.compassTimeApplierReset() }
        function setToAverage(azimuth) { EventHandlers.compassTimeApplierSetToAverage(azimuth) }

        running: mainscreen.magicCompass && myPositionBar.compassMode && mapView.attachedToLocation && mapView.visible && internalStateKeeper.applicationMaximized
        repeat: true
        interval: 25
        onTriggered: { EventHandlers._handleCompass(azimuth, calibrationLevel) }
    }

    /*
    Timer {
        id: __debugger
        interval: 1000
        running: true
        repeat: true
        onTriggered: {
            if (mapView.visible) {
                log.info(("*** " + dotCenterImage.state + " :: " + myPositionBar.hasPosition + " :: " + internalStateKeeper.flyingToLocation + " :: " + myPositionBar.attachedToLocation + " :: " + myPositionBar.currentPosition + " :: " + myPositionBar.previousPosition)
            }
        }
    }
    */

    /*
    Timer {
        id: regularTimer

        interval: 30 * 1000

        running: true
        repeat: false

        onTriggered: {
            log.info("...........................****************************************............................")
            log.info("...........................****************************************............................")
            log.info("...........................****************************************............................")
            if (true || internalStateKeeper.positionProviderEnabled) {
                log.info("...........................STOP............................")
                EventHandlers.positionProviderStop();
            } else {
                log.info("...........................START............................")
                EventHandlers.positionProviderStart();
            }
        }
    }
    */

    /*
    Timer {
        interval: 1000 * 10
        running: true
        repeat: true

        onTriggered: {
            if (mapView.visible) {
                spareGeoCoordinates.latitude = mapView.center.latitude + Math.random() / 100000;
                spareGeoCoordinates.longitude = mapView.center.longitude + Math.random() / 100000;

                console.log(".....................     " + spareGeoCoordinates.latitude + " :: " + spareGeoCoordinates.longitude)

                positionProvider.forcePosition(spareGeoCoordinates, mainscreen.__gps);
            }
        }
    }
    GeoCoordinates {
        id: spareGeoCoordinates
    }
    */
}
