var methodToApply = function () {
    var _method = null;

    return {
        get: function () {
            var tmp = _method;
            _method = null;
            return tmp;
        },
        set: function (method) {
            _method = method;
        },
        reset: function () {
            _method = null;
        }
    }
}()

var stash = function () {
    var _collection = {};

    return {
        get: function (key) {
            return _collection[key] || null;
        },
        set: function (key, value) {
            _collection[key] = value;
        },
        del: function (key) {
            if (_collection[key]) {
                _collection[key] = null;
                delete _collection[key];
            }
        },
        reset: function () {
            _collection = {};
        },

        CURRENT_POSITION: "current_position"
    }
}()

function setCurrentPosition(newPosition) {
    //Log.info(("\n\n\n\n (0)" + myPositionBar.positionType + "\n\n\n\n")
    if (
        newPosition && positionProvider.positionMethod != PositionProvider.METHOD_NONE
        &&
        (
            !mapView.getBehavior(mapView._POSITION_TYPE_REQUIRED_GPS_)
            ||
            mapView.getBehavior(mapView._POSITION_TYPE_REQUIRED_GPS_) && positionProvider.positionMethod == PositionProvider.METHOD_GPS
        )
    ) {
        var position;
        if (newPosition.clone) {
            position = newPosition.clone(mapView);
            mapView.__debugLogger.log(">> newPosition is cloneable " + position)
        } else {
            position = newPosition;
            mapView.__debugLogger.log(">> newPosition is NOT cloneable " + position)
        }

        mapView.__debugLogger.log(">> new position is " + position + " :: " + newPosition)

        if (myPositionBar.currentPosition) {
            if (myPositionBar.previousPosition && !myPositionBar.previousPosition.fake && myPositionBar.previousPosition.destroy) {
                myPositionBar.previousPosition.destroy();
            }
            if (myPositionBar.currentPosition && myPositionBar.currentPosition.clone) {
                myPositionBar.previousPosition = myPositionBar.currentPosition.clone(mapView);
            } else if (myPositionBar.currentPosition) {
                myPositionBar.previousPosition = myPositionBar.currentPosition;
            }
        }
        if (myPositionBar.currentPosition && myPositionBar.currentPosition.destroy) {
            myPositionBar.currentPosition.destroy();
        }

        stash.set(stash.CURRENT_POSITION, position);
        myPositionBar.currentPosition = position;

        myPositionBar.positionType = (positionProvider.positionMethod == PositionProvider.METHOD_GPS || positionProvider.positionMethod == PositionProvider.METHOD_GPS_NETWORK)? mapView._POSITION_TYPE_GPS_ : mapView._POSITION_TYPE_WIFI_CELL_;
        mapView.__debugLogger.log(">> will set new position with position type " + myPositionBar.positionType)
        //log.info("\n\n\n\n (1)" + myPositionBar.positionType + "\n\n\n\n" + newPosition.geoCoordinates.latitude + " :: " + newPosition.geoCoordinates.longitude)
    } else if (myPositionBar.positionType != mapView._POSITION_TYPE_NONE_) {
        //myPositionBar.attachedToLocation = false;
        if (myPositionBar.currentPosition) {
            if (myPositionBar.previousPosition && !myPositionBar.previousPosition.fake && myPositionBar.previousPosition.destroy) {
                myPositionBar.previousPosition.destroy();
            }
            if (myPositionBar.currentPosition && myPositionBar.currentPosition.clone) {
                myPositionBar.previousPosition = myPositionBar.currentPosition.clone(mapView);
            } else if (myPositionBar.currentPosition) {
                myPositionBar.previousPosition = myPositionBar.currentPosition;
            }

        }
        if (myPositionBar.currentPosition && myPositionBar.currentPosition.destroy) {
            myPositionBar.currentPosition.destroy();
        }

        stash.del(stash.CURRENT_POSITION);
        myPositionBar.currentPosition = null;

        myPositionBar.positionType = mapView._POSITION_TYPE_NONE_;
        mapView.__debugLogger.log(">> will set new position with position type " + myPositionBar.positionType)
        //log.info("\n\n\n\n (2)" + myPositionBar.positionType + "\n\n\n\n")
    }

    //log.info(("\n\n\n\n (3)" + myPositionBar.positionType + " :: " + myPositionBar.currentPosition + " :: " + myPositionBar.previousPosition + "\n\n\n\n")

    myPositionBar.currentPositionChanged();
    myPositionBar.previousPositionChanged();
}

var normalizeHaloAccuracyInMeters = function () {
    var toleranceDeltaInMeters = 15;
    var minAccuracyInMeters = 15;
    var maxAccuracyInMeters = 1000;
    var minAccuracyInPixels = 30;

    var _prevAcceptedAccuracy = 0;

    var _getFix = function (accuracy) {
        return (minAccuracyInMeters <= accuracy && maxAccuracyInMeters >= accuracy)? accuracy : 0;
    }

    return function (accuracy) {
        var delta = Math.abs(accuracy - _prevAcceptedAccuracy);
        if ((delta > toleranceDeltaInMeters) || (delta == 0)) {
            _prevAcceptedAccuracy = accuracy;
            return _getFix(accuracy);
        } else if (_prevAcceptedAccuracy) {
            var result = _getFix(_prevAcceptedAccuracy);
            _prevAcceptedAccuracy = accuracy;
            return result;
        } else {
            return 0;
        }
    }
}()

function resetDistancePointer() {
    if (myPositionBar.currentPosition || myPositionBar.previousPosition) {
        var position = myPositionBar.currentPosition? myPositionBar.currentPosition : myPositionBar.previousPosition;
        arrow.rotation = mapView.center.heading(position.geoCoordinates) - mapView.orientation;
        bottomPopup.distance = units.normalizeDistance(
            mapView.center.distance(position.geoCoordinates)
        ).readableWithUnits;
        internalStateKeeper.currentPositionVisible = MapApi.checkIfPointIsVisible(position.geoCoordinates);
    }
}
