var initialize;
var terminate;
var setCenter;
var getCenter;
var set3dIcon;
var scale3dIcon;
var setIconEnabled;
var drawMarker;
var hideMarker;
var drawCircle;
var hideCircle;
var setHaloRadius;
var getHaloRadius;
var checkVisibileHaloOrNot;

(function () {

    // private
    var _points = [];
    var _map = mapView;
    var _polygon = null;
    var _marker = null;
    var _markerShouldBeVisible = false;
    var _iconEnabled = false;
    var _icon3d = false;
    var _layer = null;
    var _initialized = false;
    var _pointNumber = 40;
    var _radius = 0;
    var _center = null;
    var _R = 6371; // earth radius
    var _tempPolygonHidden = false;
    var _tempPolygonHiddenStatus = false;

    var _marker2dGreenSvg = '<svg width="100%" height="100%" viewBox="0 0 40 40" enable-background="new 0 0 40 40" xml:space="preserve"><circle cx="20" cy="20" r="18" stroke="black" stroke-width="4" fill="white"/><circle cx="20" cy="20" r="12" fill="green"/><path fill="#FFFFFF" d="M20,10c5.606,0,10.151,4.545,10.151,10.15c0,5.606-4.545,10.153-10.151,10.153 c-5.605,0-10.151-4.546-10.151-10.153C10.001,14.394,14.393,10,20,10 M20,8.485C13.636,8.485,8.486,13.636,8.486,20 c0,6.363,5.15,11.667,11.666,11.667c6.363,0,11.666-5.151,11.666-11.667C31.667,13.636,26.514,8.485,20,8.485L20,8.485z"/></svg>';
    var _marker2dGreySvg  = '<svg width="100%" height="100%" viewBox="0 0 40 40" enable-background="new 0 0 40 40" xml:space="preserve"><circle cx="20" cy="20" r="18" stroke="black" stroke-width="4" fill="white"/><circle cx="20" cy="20" r="12" fill="grey"/><path fill="#FFFFFF" d="M20,10c5.606,0,10.151,4.545,10.151,10.15c0,5.606-4.545,10.153-10.151,10.153 c-5.605,0-10.151-4.546-10.151-10.153C10.001,14.394,14.393,10,20,10 M20,8.485C13.636,8.485,8.486,13.636,8.486,20 c0,6.363,5.15,11.667,11.666,11.667c6.363,0,11.666-5.151,11.666-11.667C31.667,13.636,26.514,8.485,20,8.485L20,8.485z"/></svg>';
    var _marker3dGreenSvg = '<svg width="100%" height="100%" viewBox="0 0 40 40" enable-background="new 0 0 40 40" xml:space="preserve"><ellipse cx="20" cy="20" rx="18" ry="9" stroke="black" stroke-width="4" fill="white"/><ellipse cx="20" cy="20" rx="12" ry="5" fill="green"/><path fill="#FFFFFF" d="M20,10c5.606,0,10.151,4.545,10.151,10.15c0,5.606-4.545,10.153-10.151,10.153 c-5.605,0-10.151-4.546-10.151-10.153C10.001,14.394,14.393,10,20,10 M20,8.485C13.636,8.485,8.486,13.636,8.486,20 c0,6.363,5.15,11.667,11.666,11.667c6.363,0,11.666-5.151,11.666-11.667C31.667,13.636,26.514,8.485,20,8.485L20,8.485z"/></svg>';
    var _marker3dGreySvg  = '<svg width="100%" height="100%" viewBox="0 0 40 40" enable-background="new 0 0 40 40" xml:space="preserve"><ellipse cx="20" cy="20" rx="18" ry="9" stroke="black" stroke-width="4" fill="white"/><ellipse cx="20" cy="20" rx="12" ry="5" fill="grey"/><path fill="#FFFFFF" d="M20,10c5.606,0,10.151,4.545,10.151,10.15c0,5.606-4.545,10.153-10.151,10.153 c-5.605,0-10.151-4.546-10.151-10.153C10.001,14.394,14.393,10,20,10 M20,8.485C13.636,8.485,8.486,13.636,8.486,20 c0,6.363,5.15,11.667,11.666,11.667c6.363,0,11.666-5.151,11.666-11.667C31.667,13.636,26.514,8.485,20,8.485L20,8.485z"/></svg>';

    // math shortcurs
    var PI = Math.PI;
    var PI2 = Math.PI * 2;
    var asin = Math.asin;
    var sin = Math.sin;
    var cos = Math.cos;
    var rad = Math.PI / 180;
    var diff = rad * (360 / _pointNumber);

    setCenter = function (center) {
        if (center) {
            _center = center;

            if (_initialized && _marker) {
                var loc = _marker.location;
                loc.geoCoordinates = _center;
                _marker.location = loc;

                if (_radius > 0) {
                    drawCircle(_radius);
                }

                if (_markerShouldBeVisible) {
                    drawMarker();
                }
            }
        }
    }

    getCenter = function () {
        return _center;
    }

    set3dIcon = function (flag) {
        _icon3d = flag;

        if (_initialized && _marker)
            _marker.icon = _getIcon(_icon3d, _iconEnabled);
    }

    scale3dIcon = function () {
        if (_icon3d && _center) {
            var pixels = mapView.geoToScreen(_center, false);
            var delta = pixels.y / (mapView.height * 0.7);
            /*
            _marker.icon.size.width = 10;
            _marker.icon.size.height = 10;
            */
        }
    }

    setIconEnabled = function (flag) {
        _iconEnabled = flag;

        if (_initialized && _marker)
            _marker.icon = _getIcon(_icon3d, _iconEnabled);
    }

    drawMarker = function () {
        if (_initialized && _center && !_marker.visible) {
            _marker.visible = true;
        } else {
            _markerShouldBeVisible = true;
        }
    }

    hideMarker = function () {
        if (_initialized && _marker)
            _marker.visible = false;
    }

    drawCircle = function (radius) {
        if (positionMarker.shown) {
            if (!_iconEnabled || !mapView.visible || mapView.currentPosition && !MapApi.checkIfPointIsVisible(mapView.currentPosition.geoCoordinates)) {
                if (_polygon.visible) {
                     _polygon.visible = false;
                }
            } else {
                if (_initialized) {
                    if (checkVisibileHaloOrNot()) return;
                    _tempPolygonHiddenStatus = false;

                    _radius = radius;
                    _calculatePoints(radius);
                    _polygon.points = _points;

                    if (!_polygon.visible) {
                         _polygon.visible = true;
                    }

                    //_polygon.fillColor = _iconEnabled? Qt.rgba( 0.0, 1.0, 0.0, 0.2 ) : Qt.rgba( 1.0, 0.8, 0.1, 0.2 );
                    //_polygon.lineColor = _iconEnabled? Qt.rgba( 0.0, 1.0, 0.0, 0.3 ) : Qt.rgba( 1.0, 0.8, 0.1, 0.3 );
                    _polygon.fillColor = _iconEnabled? Qt.rgba( 0.23, 0.53, 0.04, 0.2 ) : Qt.rgba( 1.0, 0.8, 0.1, 0.2 );
                    _polygon.lineColor = _iconEnabled? Qt.rgba( 0.23, 0.53, 0.04, 0.3 ) : Qt.rgba( 1.0, 0.8, 0.1, 0.3 );
                }
            }
        }
    }

    hideCircle = function () {
        if (_initialized) {
            _polygon.visible = false;
        }
    }

    initialize = function () {
        // polygon
        _layer = _createPluginObject("MapLayer", _map, null);
        _polygon = _createPluginObject("MapPolygon", _map, null);
        _polygon.fillColor = _iconEnabled? Qt.rgba( 0.0, 1.0, 0.0, 0.2 ) : Qt.rgba( 1.0, 0.8, 0.1, 0.2 );
        _polygon.lineColor = _iconEnabled? Qt.rgba( 0.0, 1.0, 0.0, 0.3 ) : Qt.rgba( 1.0, 0.8, 0.1, 0.3 );
        _polygon.lineWidth = 3;
        _polygon.visible = false;
        _layer.addMapObject(_polygon);

        // marker
        var location = _createPluginObject("Location");
        location.geoCoordinates = mapView.center;
        _marker = _createPluginObject("MapIcon");
        _marker.icon = _getIcon(false, true);
        _marker.visible = false;
        _marker.location = location;
        _layer.addMapObject(_marker);
        _map.addLayer(_layer);

        for ( var i = 0; i < _pointNumber; i++ ) {
            _points[i] = _createPluginObject("GeoCoordinates");
        }

        _initialized = true;

        setIconEnabled(positionMarker.enabled);
    }

    terminate = function () {
        _layer.removeMapObject(_polygon);
        _layer.removeMapObject(_marker);
        _map.removeLayer(_layer);
        _points.slice(0, _points.length);
        _layer = null;
        _polygon = null;
        _initialized = false;
        _marker = null;
    }

    setHaloRadius = function (radius) {
        _radius = radius;
    }

    getHaloRadius = function () {
        return _radius;
    }

    checkVisibileHaloOrNot = function() {
        if (mapView.perspective > 0 && _center) {
            var currentPositionCenter = mapView.geoToScreen(_center, true);
            if (currentPositionCenter && currentPositionCenter.y <= mapView.height / 3) {
                _tempPolygonHidden = true;
            } else {
                _tempPolygonHidden = false;
            }
        } else {
            _tempPolygonHidden = false;
        }

        if (_tempPolygonHidden && _polygon.visible) {
            _tempPolygonHiddenStatus = true;

            _polygon.visible = false;

            return true;
        } else {
            if (!_tempPolygonHidden && !_polygon.visible && _tempPolygonHiddenStatus) {
                _polygon.visible = true;
            }

            return false;
        }
    }

    function _calculatePoints(dist) {
        if (_center) {
            var clat = _center.latitude * rad;
            var clon = _center.longitude * rad;
            var angle = 0.0;
            var geo, x, y, i = 0;

            dist /= 1000; // convert form meters to km
            dist /= _R;   // divide by eart radius

            for ( ; i < _pointNumber; i++, angle += diff ) {
                y = asin(sin(clat) * cos(dist) + cos(clat) * sin(dist) * cos(angle));
                x = ((clon - asin(sin(angle) * sin(dist) / cos(clat)) + PI) % (PI2)) - PI;

                var newLat = y * 180 / PI;
                var newLon = x * 180 / PI;
                if (!isNaN(newLat) && !isNaN(newLon)) {
                    _points[i].latitude  = y * 180 / PI;
                    _points[i].longitude = x * 180 / PI;
                }
            }
        }
    }

    function _createPluginObject(objectType) {
      return Qt.createQmlObject("import MapsPlugin 1.0; " + objectType + "{}", _map, null);
    }

    function _getIcon(mode3d, enabled) {
        var icon = _createPluginObject("Icon");

        if (!mode3d) {
            //icon.svgContent = ( enabled ? _marker2dGreenSvg : _marker2dGreySvg );
            icon.localUrl = mapAssetsPath + (enabled ? "images/layout/pos_accurate.png" : "images/layout/pos_lost.png");
        } else {
            //icon.svgContent = ( enabled ? _marker3dGreenSvg : _marker3dGreySvg );
            icon.localUrl = mapAssetsPath + (enabled ? "images/layout/pos_accurate_3d.png" : "images/layout/pos_lost_3d.png");
        }

        return icon;
    }
}())
