import QtQuick 1.0
import MapsPlugin 1.0
import "PositionMarker.js" as Marker
import "MapApi.js" as MapApi

Item {
    id: positionMarker

    property variant mapView
    property variant center: null
    property bool shown: false
    property bool icon3d: false
    property bool enabled: false
    property real accuracy: 0.0
    property real minRadius: 10
    property real maxRadius: 1000
    property real jumpMultiplier: 12.0
    property alias animationInterval: loop.interval    

    onCenterChanged: {
        if (center) {
            Marker.setCenter(center);
        }
    }

    onShownChanged: {
        if (shown) {
            var cradius = Marker.getHaloRadius();
            if (cradius > minRadius) {
                Marker.drawCircle(cradius);
            }

            Marker.drawMarker();
        } else {
            Marker.hideMarker();
            Marker.hideCircle();
            loop.stop();
        }
    }

    onIcon3dChanged: {
        Marker.set3dIcon(icon3d);
    }

    onEnabledChanged: {
        Marker.setIconEnabled(enabled);
    }

    function changeAccuracy(accuracy) {
        Marker.drawCircle(accuracy);
    }

    onAccuracyChanged: {
        if (accuracy == 0 || accuracy > maxRadius || accuracy < minRadius) {
            Marker.setHaloRadius(0);
            Marker.hideCircle();
            loop.stop();
            return;
        }

        var cradius = Marker.getHaloRadius();

        loop.radius = accuracy;
        loop.increase = cradius < accuracy;
        loop.diff = (loop.increase ? accuracy : cradius) / jumpMultiplier;
        loop.restart();
    }

    Timer {
        id: loop
        interval: 33
        repeat: true

        property variant center
        property bool increase: true        
        property real radius: 0
        property real diff: 0

        onTriggered: {
            var diff = loop.diff;
            var currentRadius = Marker.getHaloRadius();

            if (loop.increase && currentRadius  < radius) {
                Marker.drawCircle(currentRadius + diff);
            } else if (!loop.increase && currentRadius  > radius) {
                Marker.drawCircle(currentRadius - diff);
            } else {
                loop.stop();
            }
        }        
    }

    Connections {
        target: mapView
        ignoreUnknownSignals: true
        onCenterChanged: { Marker.checkVisibileHaloOrNot() }
    }

    Component.onCompleted: {
        Marker.initialize();
    }
}
