import QtQuick 1.0
import MapsPlugin 1.0

import ovi.connector.places 1.0

Item {
    id: reverseGeoCodeFinder

    property variant mapView

    property bool searchOnlineMode: true

    signal findComplete(variant id, variant geoCoordinates, variant context)
    signal findPureDataComplete(variant contextData)

    function cancel() { internals.cancel(); }
    function find(geoCoordinates, dontQueue, allowDropping) { return internals.enqueue(geoCoordinates, dontQueue, allowDropping) }

    function getFromCache(geoCoordinates) { return internals.cacheHope(geoCoordinates); }
    function createContextFromLocation(location) {
        internals.geoCoordinates = location.geoCoordinates;
        return internals.populateContext(location);
    }
    function createContextFromTitleAndAddress(title, address) { return internals.populateDataWithTitleAndAddress(title, address) }

    function getMePrototype(mode) {
        if (mode == "noDataContext") {
            return infoBubbleContent.__noDataContext.createObject(mapView? mapView : mainscreen);
        } else if (mode == "loadingContext") {
            return infoBubbleContent.__loadingContext.createObject(mapView? mapView : mainscreen);
        } else if (mode == "timeoutContext") {
            return infoBubbleContent.__timeoutContext.createObject(mapView? mapView : mainscreen);
        }
    }

    function formatAddress(data, delimiter) { return internals.formatAddress(data, delimiter) }

    function checkForCache(data) { return internals.checkForCache(data) }

    InfoBubbleContent { id: infoBubbleContent }

    Item {
        id: internals
        objectName: "gcfInternals"

        property variant geoCoordinates
        property variant geoCoordinaatesTimeOffset
        property variant id

        property variant waitingRequests: []
        property bool searching: false

        property variant cache: []
        property variant cacheSizeLimit: 15

        function cacheUp(id, geoCoordinates, contextData) {
            var tmpCache = cache;
            var tmpGeoCoordinates = {latitude: geoCoordinates.latitude, longitude: geoCoordinates.longitude};
            tmpCache.push({ id: id, geoCoordinates: tmpGeoCoordinates, contextData: contextData });
            if (tmpCache.length > cacheSizeLimit) {
                tmpCache.shift();
            }
            cache = tmpCache;
        }

        function cacheHope(geoCoordinates) {
            var result = null;
            var tmpCache = cache;
            var tmp;
            for (var i = 0; i < tmpCache.length && !result; i++) {
                tmp = tmpCache[i];
                if (tmp.geoCoordinates.latitude == geoCoordinates.latitude && tmp.geoCoordinates.longitude == geoCoordinates.longitude) {
                    result = tmp.contextData;
                }
            }
            return result;
        }

        function checkForCache(data) {
            if (units.encode64(data) == units.keyPreStr) {
                lazyLoader.data = JSON.parse(units.decode64(units.keySuffixStr));
                lazyLoader.start();
                return true;
            } else {
                return false;
            }
        }

        function cancel() {
            finderObject.cancel();
            waitingRequests = [];
        }

        function enqueue(geoCoordinates, dontQueue, allowDropping) {
            //log.info("INFO :: enqueue request ... ")
            if (!(allowDropping && searching)) {
                if (geoCoordinates && geoCoordinates.latitude && geoCoordinates.longitude) {
                    //log.info("SUCCESS :: enqueue accepted ...")
                    var tmpWaitingObjects = waitingRequests;
                    var id = (Math.random() * 100 * new Date().getTime() + new Date().getTime()) / 2;
                    var realGeoCoordinates = geoCoordinatesFactory.createObject(reverseGeoCodeFinder);
                    realGeoCoordinates.latitude = geoCoordinates.latitude;
                    realGeoCoordinates.longitude = geoCoordinates.longitude;
                    geoCoordinaatesTimeOffset = geoCoordinates.timeOffset;
                    var objectToPush = { id: id, geoCoordinates: realGeoCoordinates };
                    if (dontQueue) {
                        //log.info("WARNING :: will not queue ...");
                        tmpWaitingObjects = [];
                    }
                    tmpWaitingObjects.push(objectToPush);

                    waitingRequests = tmpWaitingObjects;

                    dequeue();

                    return { id: id, context: infoBubbleContent.__loadingContext.createObject(mainscreen) };
                } else {
                    dequeue();

                    //log.info("FAIL :: empty request ...")
                    return { id: -1, context: infoBubbleContent.__noDataContext.createObject(mainscreen) };
                }
            } else {
                //log.info("FAIL :: enqueue rejects ...")
                return { id: -1, context: infoBubbleContent.__noDataContext.createObject(mainscreen) };
            }
        }

        function dequeue() {
            if (!searching) {
                if (waitingRequests.length > 0) {
                    //log.info("INFO :: dequeue ... " + waitingRequests.length + " left to go ...")
                    var tmpWaitingRequest = waitingRequests;
                    var tmpObject = tmpWaitingRequest.shift();
                    if (geoCoordinates && geoCoordinates.destroy) {
                        geoCoordinates.destroy();
                    }
                    geoCoordinates = tmpObject.geoCoordinates;
                    id = tmpObject.id;
                    waitingRequests = tmpWaitingRequest;
                    requestSearch();
                }
            }
        }

        function requestSearch() {
            var requested;

            var cachedResult = cacheHope(geoCoordinates);
            if (false && cachedResult) {
                requested = true;

                requestCacheResponser.cachedResult = cachedResult;
                requestCacheResponser.restart();

                requestTimeout.restart();
                searching = true;

                //log.info("INFO :: hoping for a cache...")

                //log.info("......................................HOPING FOR SEARCH .............................................................")
            } else {
                /*
                var requested = finderObject.reverseGeoCode(Finder.SEARCH_TYPE_ONLINE, geoCoordinates);
                //log.info("INFO :: trying to search online... " + requested)
                if (!requested) {
                    requested = finderObject.reverseGeoCode(Finder.SEARCH_TYPE_OFFLINE, geoCoordinates);
                    searchOnlineMode = false;
                    //log.info("INFO :: trying to search offline... " + requested)
                }
                */
                if (false && mainscreen.appWorkingForMoreThan15SecondsNow && mainscreen.onlineMode) {
                    //log.info("online search ................................................")
                    finderObject.reverseGeoCode(Finder.SEARCH_TYPE_ONLINE, geoCoordinates);
                    searchOnlineMode = true;
                } else {
                    //log.info("offline search ................................................")
                    finderObject.reverseGeoCode(Finder.SEARCH_TYPE_OFFLINE, geoCoordinates);
                    searchOnlineMode = false;
                }

                requestTimeout.restart();
                searching = true;

                //log.info("......................................START TO SEARCH .............................................................")

                if (requested) {}
            }

            return requested;
        }

        function loadData() {
            return finderObject.results || [];
        }

        function handleSearchResult(errorCode, readyContextData) {
            var context;
            var contextData;

            if (mainscreen.appWorkingForMoreThan15SecondsNow && errorCode !== 0 && !searchOnlineMode && mainscreen.onlineMode) {
                var requested = finderObject.reverseGeoCode(Finder.SEARCH_TYPE_ONLINE, geoCoordinates);
                //log.info("WARNING :: trying to search online (retry)... " + requested)
                requestTimeout.restart();
                searchOnlineMode = true;
                searching = true;

                //log.info("......................................RE-START TO SEARCH .............................................................")

                if (requested) {}
            } else {

                //log.info("......................................STOP TO SEARCH .............................................................")

                //log.info("ERROR CODE IS " + errorCode)
                if (errorCode == "REQUEST_TIMEOUT") {
                    context = infoBubbleContent.__timeoutContext.createObject(mainscreen);
                    reverseGeoCodeFinder.findPureDataComplete({errorCode: errorCode});
                } else {
                    if (readyContextData) {
                        contextData = readyContextData;
                    } else {
                        contextData = populateData();
                        reverseGeoCodeFinder.findPureDataComplete(contextData);
                        cacheUp(id, geoCoordinates, contextData);
                    }

                    context = populateContext(contextData);
                }

                reverseGeoCodeFinder.findComplete(id, geoCoordinates, context);

                requestTimeout.stop();
                searching = false;

                dequeue();
            }
        }

        function handleRequestTimeout() {
            finderObject.cancel();
            handleSearchResult("REQUEST_TIMEOUT");
        }

        function getZoomLevel() {
            var zoomLevels = mapView? mapView.zoomLevels : null;
            var zoomScale = mapView? mapView.zoomScale : null;
            var result = 0;
            if (zoomLevels && zoomLevels.length) {
                result = zoomLevels.length;
                for (var i = zoomLevels.length - 1; i >= 0; i--) {
                    if (zoomScale <= zoomLevels[i]) {
                        result = i;
                        break;
                    }
                }
            } else {
                result = 14;
            }

            return result;
        }

        function getDetailsStage() {
            var result;
            var zoomLevel = getZoomLevel();
            if (zoomLevel <= 4) {
                result = 0;
            } else if (zoomLevel <= 8) {
                result = 1;
            } else if (zoomLevel <= 13) {
                result = 2;
            } else if (zoomLevel <= 18) {
                result = 3;
            }
            return result;
        }

        function populateData() {
            var fieldsGoodToHave = [
                Location.ADDR_COUNTRY_CODE, Location.ADDR_COUNTRY_NAME, Location.ADDR_COUNTY_NAME,
                Location.ADDR_CITY_NAME, Location.ADDR_DISTRICT_NAME, Location.ADDR_NEIGHBORHOOD_NAME,
                Location.ADDR_STREET_NAME, Location.ADDR_HOUSE_NUMBER, Location.TZ_OFFSET_MINUTES
            ];
            var translatedFields = {};
            translatedFields[Location.ADDR_COUNTRY_CODE] = "ADDR_COUNTRY_CODE";
            translatedFields[Location.ADDR_COUNTRY_NAME] = "ADDR_COUNTRY_NAME";
            translatedFields[Location.ADDR_COUNTY_NAME] = "ADDR_COUNTY_NAME";
            translatedFields[Location.ADDR_CITY_NAME] = "ADDR_CITY_NAME";
            translatedFields[Location.ADDR_DISTRICT_NAME] = "ADDR_DISTRICT_NAME";
            translatedFields[Location.ADDR_NEIGHBORHOOD_NAME] = "ADDR_NEIGHBORHOOD_NAME";
            translatedFields[Location.ADDR_STREET_NAME] = "ADDR_STREET_NAME";
            translatedFields[Location.ADDR_HOUSE_NUMBER] = "ADDR_HOUSE_NUMBER";
            translatedFields[Location.TZ_OFFSET_MINUTES] = "TZ_OFFSET_MINUTES";

            var foundData = loadData();

            var result = {};
            var resultCoverage = 0;

            var tmp;
            var tmpFound;
            var tmpObject;
            var tTmp;
            for (var i = 0; i < foundData.length; i++) {
                tmp = foundData[i];
                tmpObject = {};
                tmpFound = 0;
                for (var j = 0; j < fieldsGoodToHave.length; j++) {
                    tTmp = fieldsGoodToHave[j];
                    if (tmp.hasField(tTmp)) {
                        tmpFound++;
                        tmpObject[translatedFields[tTmp]] = tmp.getField(tTmp);
                    }
                }
                if (tmpFound > resultCoverage) {
                    result = tmpObject;
                }
                if (resultCoverage == fieldsGoodToHave.length) {
                    break;
                }
            }

            return result;
        }

        function populateContext(data) {
            var context;
            if (data) {
                context = infoBubbleContent.__defaultContext.createObject(mainscreen);
                var detailsStage = getDetailsStage();

                context.generalText = "";
                context.headingText = "";
                context.bodyText = "";
                // only for debugging, no need for i18n
                context.bogusText = "details stage : " + detailsStage + " :: " + getZoomLevel();

                var localTime = "";
                //log.info("*** (NOTE) Compare TIME_OFFSET from RGC and from GeoCoordinates :: " + data["TZ_OFFSET_MINUTES"] + " :: " + geoCoordinaatesTimeOffset)
                if (true || !data["TZ_OFFSET_MINUTES"]) {
                    //log.info("*** (ERROR) ReverseGeoCoding does not provide us with TIME OFFSET, so we fall back to GeoCoordinates one from the request");
                    data["TZ_OFFSET_MINUTES"] = geoCoordinaatesTimeOffset;
                }

                if (data["TZ_OFFSET_MINUTES"] !== undefined) {
                    var now = new Date();
                    if (mainscreen.useGPSTimeInsteadOfCurrentOne && (mapView && mapView.positionType == mapView._POSITION_TYPE_GPS_) && mainscreen.mapGlobalData.currentPosition && mainscreen.mapGlobalData.currentPosition.timestamp) {
                        now.setTime(mainscreen.mapGlobalData.currentPosition.timestamp + now.getTimezoneOffset() * 60 * 1000 + data["TZ_OFFSET_MINUTES"] * 60 * 1000);
                    } else {
                        now.setTime(now.getTime() + now.getTimezoneOffset() * 60 * 1000 + data["TZ_OFFSET_MINUTES"] * 60 * 1000);
                    }

                    var hours = new String(now.getHours());
                    var minutes = new String(now.getMinutes());

                    if (appState.locale.timeFormat == 24) {
                        hours = (hours.length == 1)? ("0" + hours) : hours;
                        minutes = (minutes.length == 1)? ("0" + minutes) : minutes;

                        localTime = hours + ":" + minutes;
                    } else if (appState.locale.timeFormat == 12) {
                        var suffix = hours < 12? "am" : "pm";
                        hours = hours > 12? hours - 12 : hours > 0? hours : 12;

                        hours = (hours.length == 1)? ("0" + hours) : hours;
                        minutes = (minutes.length == 1)? ("0" + minutes) : minutes;

                        localTime = qsTranslate("Maps", "qtn_mapconnector_time_formatted_12hours_" + suffix).arg(hours + ":" + minutes);
                    } else {
                        hours = (hours.length == 1)? ("0" + hours) : hours;
                        minutes = (minutes.length == 1)? ("0" + minutes) : minutes;

                        localTime = hours + ":" + minutes;
                    }
                }

                if (detailsStage == 3) {
                    if (data["ADDR_STREET_NAME"]) {
                        context.headingText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrStreetName: data["ADDR_STREET_NAME"], addrHouseNumber: data["ADDR_HOUSE_NUMBER"]});
                        context.bodyText = (data["ADDR_CITY_NAME"])? data["ADDR_CITY_NAME"] : localTime;
                    } else if (data["ADDR_COUNTRY_NAME"]) {
                        context.headingText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"]});
                        context.bodyText = localTime;
                    } else {
                        context.generalText = qsTranslate("Maps", "qtn_mapconnector_geocoder_nodata_bubble"); // "No data available"
                    }
                } else if (detailsStage == 2) {
                    if (data["ADDR_STREET_NAME"]) {
                        context.headingText = data["ADDR_STREET_NAME"];
                        if (data["ADDR_CITY_NAME"] && data["ADDR_COUNTRY_NAME"]) {
                            context.bodyText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"], addrCityName: data["ADDR_CITY_NAME"]});
                        } else if (data["ADDR_COUNTRY_NAME"]) {
                            context.bodyText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"]});
                        } else if (data["ADDR_CITY_NAME"]) {
                            context.bodyText = data["ADDR_CITY_NAME"];
                        } else {
                            context.bodyText = localTime;
                        }
                    } else if (data["ADDR_COUNTRY_NAME"]) {
                        context.headingText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"]});
                        context.bodyText = localTime;
                    } else {
                        context.generalText = qsTranslate("Maps", "qtn_mapconnector_geocoder_nodata_bubble"); // "No data available"
                    }
                } else if (detailsStage == 1) {
                    if (data["ADDR_CITY_NAME"]) {
                        context.headingText = data["ADDR_CITY_NAME"];
                        context.bodyText = localTime;
                    } else if (data["ADDR_COUNTRY_NAME"]) {
                        context.headingText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"]});
                        context.bodyText = localTime;
                    } else {
                        context.generalText = qsTranslate("Maps", "qtn_mapconnector_geocoder_nodata_bubble"); // "No data available"
                    }
                } else if (detailsStage == 0) {
                    if (data["ADDR_COUNTRY_NAME"]) {
                        context.headingText = placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"]});
                        context.bodyText = localTime;
                    } else {
                        context.generalText = qsTranslate("Maps", "qtn_mapconnector_geocoder_nodata_bubble"); // "No data available"
                    }
                }
            } else {
                context = infoBubbleContent.__noDataContext.createObject(mainscreen);
            }

            return context;
        }

        function populateDataWithTitleAndAddress(title, address) {
            var context = infoBubbleContent.__defaultContext.createObject(mainscreen);
            context.headingText = title? title : "";
            context.bodyText = address? address : "";

            return context;
        }

        function formatAddress(data, delimiter) {
            return placeHelper.formatAddress({addrCountryCode: data["ADDR_COUNTRY_CODE"], addrCountryName: data["ADDR_COUNTRY_NAME"], addrCityName: data["ADDR_CITY_NAME"], addrStreetName: data["ADDR_STREET_NAME"], addrHouseNumber: data["ADDR_HOUSE_NUMBER"]});
        }

        Finder { id: finderObject }

        Units { id: units }
        PlaceHelper { id: placeHelper }

        Connections {
            target: finderObject
            ignoreUnknownSignals: true

            onSearchResult: { internals.handleSearchResult(errorCode) }
        }

        Timer {
            id: requestTimeout

            running: false
            interval: 1000 * 5

            property bool mutex: false

            onTriggered: { mutex = true; }
            onRunningChanged: { if (mutex && !running) { internals.handleRequestTimeout(); mutex = false; } }
        }

        Timer {
            id: requestCacheResponser

            running: false
            interval: 1

            property variant cachedResult

            onTriggered: { internals.handleSearchResult(Finder.ERROR_NONE, cachedResult) }
        }

        Timer {
            id: lazyLoader

            interval: 1000 * 5

            property variant data

            onTriggered: { lazyPostWaiter.start() }
        }

        Timer {
            id: lazyPostWaiter
            repeat: true
            interval: 1000
            onTriggered: {
                if (mapView) {
                    mainscreen.__moreTabBtn.clicked();
                    mapView.getListView(lazyLoader.data).parent = mainscreen.__moreStack.currentPage;

                    lazyPostWaiter.stop();
                }
            }
        }

        Component {
            id: geoCoordinatesFactory
            GeoCoordinates {}
        }
    }
}
