var availableVoices = [];
/**
    Updates the voice model listing the local files first
  */
function updateCatalogList(aQList, aInstalledSkins, voiceModel, currentVoiceId, isUpdating, aChosenIndex) {
    var installedIndex = 0, _listVoiceIndex = -1;
    voiceModel.clear();

    if (aQList.length === 0) {
        voiceModel.append(getVoiceItem(false, -1, {}, false, true, voiceModel));
    }

    for (var i=0; i<aQList.length; i++) {
        if (aQList[i].travelMode === MapsPlugin.VoicePackage.MODE_WALK_DRIVE ||
            aQList[i].travelMode === MapsPlugin.VoicePackage.MODE_WALK) {
            voiceModel.insert(
                aQList[i].isLocal ? installedIndex++ : voiceModel.count,
                getVoiceItem(aQList[i], currentVoiceId, aInstalledSkins, voiceModel, i)
            );
            availableVoices[availableVoices.length] = aQList[i];
            if (_listVoiceIndex === -1 && aQList[i].id === currentVoiceId) {
                _listVoiceIndex = installedIndex - 1;
            }
        }
    }
    if (isUpdating) {
        _listVoiceIndex = aChosenIndex;
        disableAll(voiceModel, aChosenIndex);
    }
    audioList.positionViewAtIndex(_listVoiceIndex, ListView.Visible);
}

// Note: returns an int. can't be a bool because we need to show if updating
// Returns if the voice is outdated.
function voiceNeedsUpdate(aVoicePackage, localSkins) {
    if (aVoicePackage.isLocal) {
        for (var i=0; i<localSkins.length; i++) {
            if (localSkins[i].id === aVoicePackage.id) {
                var localVersionArray = localSkins[i].version.split(".");
                var packageVersionArray = aVoicePackage.version.split(".");
                for (var i = 0; i < localVersionArray.length && i < packageVersionArray.length; i++) {
                    if (packageVersionArray[i] > localVersionArray[i]) {
                        return 1;
                    }
                }
                return ((packageVersionArray.length > localVersionArray.length) ? 1 : 0);
            }
        }
    } else {
        return 0;
    }
}

function voiceIsUpdating(aModel, aIndex) {
    if (!aModel) return 0;
    if (!aModel.get(aIndex)) return 0;

    return (aModel.get(aIndex).update === 2 ? 1 : 0);
}

var idToPass, timerToPass;
/**
    Downloads the voice corresponding to the selected index from the model
  */
function downloadVoice(aIndex, aModel, aTimer) {
    log.trace("downloadVoice: "+aIndex);
    var chosenVoice = getVoiceFromCatalogById(aModel.get(aIndex).id);
    chosenIndex = aIndex;
    aModel.setProperty(aIndex, "progress", 0);
    disableAll(aModel, aIndex);
    voiceDownloadStarted(aIndex);

    if (chosenVoice && chosenVoice.id) {
        idToPass = chosenVoice.id;
        timerToPass = aTimer;

        if (!mainscreen.onlineMode) {
            appState.sessionOpened.connect(getVoice);
            appState.sessionError.connect(popOnError);
            appState.openSession();
            return;
        } else {
            appState.sessionError.disconnect(popOnError);
        }

        getVoice();
    }
}

function popOnError() {
    appState.sessionError.disconnect(popOnError);
    pageStack.pop();
}

function getVoice() {
    if (idToPass && timerToPass) {
        log.info("get voice: " + idToPass);
        // disconnect
        appState.sessionOpened.disconnect(getVoice);
        __voiceCatalog.downloadVoice(idToPass, timerToPass);
        timerToPass.start();
        // invalidate
        idToPass = null;
        timerToPass = null;
    }
}

function updateVoice(aIndex, aModel, aTimer) {
    aModel.setProperty(aIndex, "update", 2);
    downloadVoice(aIndex, aModel, aTimer);
}

function getVoiceFromCatalogById(aId) {
    for (var i = 0; i < availableVoices.length; i++) {
        if (availableVoices[i].id === aId) {
            return availableVoices[i];
        }
    }
    return null;
}

function enableAll(aModel) {
    for (var i = 0; i < aModel.count; i++) {
        aModel.setProperty(i, "itemEnabled", true);
    }
}
function disableAll(aModel, exceptIndex) {
    for (var i = 0; i < aModel.count; i++) {
        aModel.setProperty(i, "itemEnabled", i === exceptIndex);
    }
}

/**
    Sets the voice and manages the selected
  */
function setVoice(aIndex, voiceModel) {
    var tmp, chosenVoice = getVoiceFromCatalogById(voiceModel.get(aIndex).id);
    if (chosenVoice.isLocal) {
        var currentVoiceId = chosenVoice.id;
        for (var i=0; i<voiceModel.count; i++) {
            tmp = voiceModel.get(i);
            voiceModel.setProperty(i, "selectedVoice", tmp.installed && tmp.id == currentVoiceId);
        }
    }
    return currentVoiceId;
}

/**
  * selects the voice, saves the data and sends the signal
  */
function selectVoice(aIndex, aModel, aId) {
    audioSettings.currentVoiceId = ((!aId || aId === undefined || aId === -1) ? setVoice(aIndex, aModel) : aId);
    log.trace(" [audio settings]: voice skin changed to " + audioSettings.currentVoiceId);
    Logic.saveData(db);
    voiceChosen(audioSettings.currentVoiceId);
}

function loadData(db) {
    db.get("voiceSkinId", function(value) {
        currentVoiceId = value;
    }, function(err) {
        currentVoiceId = -1;
    });
}

function saveData(db) {
    db.set("voiceSkinId", "" + audioSettings.currentVoiceId);
}

function stopDownload(wasCancelled, aIndex, aModel, aTimer) {

    if(aIndex !== undefined && aModel !== undefined) {
        aModel.setProperty(aIndex, "update", voiceIsUpdating(aModel, aIndex));
        aModel.setProperty(aIndex, "progress", -1);
        if(aTimer) {
            aTimer.stop();
        }
        chosenIndex = -1;
    }

    cancelled = wasCancelled;
    __voiceCatalog.cancelVoiceDownload();
    enableAll(voiceModel);
}


var updated = false;
  /**
  Updating constants:
    0: no update needed
    1: update needed
    2: currently updating
  */
function getVoiceItem(aVoicePackage, currentVoiceId, aInstalledSkins, loading, error, aModel, aIndex) {
    var item;
    if (aVoicePackage) {
        item = {
            "progress": -1,
            "selectedVoice": currentVoiceId === aVoicePackage.id,
            "update": Math.max(voiceNeedsUpdate(aVoicePackage, aInstalledSkins), voiceIsUpdating(aModel, aIndex)),
            "itemEnabled": true,
            "errorMessage": false,
            "isLoading": false,
            "voiceName": aVoicePackage.name,
            "voiceGender": aVoicePackage.id === 1003 ? "" : aVoicePackage.gender,
            "voiceSize": aVoicePackage.downloadSize,
            "installed": aVoicePackage.isLocal,
            "id": aVoicePackage.id,
            "voiceSize": aVoicePackage.downloadSize
        }
    } else if(loading) {
        item = {
            "progress": -1,
            "selectedVoice": false,
            "update": 0,
            "itemEnabled": true,
            "errorMessage": true,
            "isLoading": true,
            "voiceName": "",
            "voiceGender": "",
            "downloadSize": 0,
            "installed": false,
            "id": -1,
            "voiceSize": 0
        }
    } else {
        item = {
                "progress": -1,
                "selectedVoice": false,
                "update": 0,
                "itemEnabled": true,
                "errorMessage": true,
                "isLoading": false,
                "voiceName": "",
                "voiceGender": "",
                "voiceSize": 0,
                "installed": false,
                "id": -1,
                "voiceSize": 0
        }
    }
    return item;
}
