// -*- mode: js2; espresso-indent-level: 4; -*-
// GuidanceViewer.js
// Stateful logic
var destinationReached = false;
var guidanceZoomScale;
var guidanceOrientation;
var operationToApplyOnAnimationDone = "";
var simulationOn = false;

function onRouteCalculationDone(errorCode, violatedOptions) {
    guidanceZoomScale = guidanceZoomScale || guidancePage.__defaultGuidanceZoomScale;

    if (errorCode === 0) {
        if (guidanceRoutingDialog.status == DialogStatus.Opening) {
            guidanceRoutingDialog.status = DialogStatus.Open;
        }
        guidanceRoutingDialog.closeComplete();
        waitForGPSTimer.stop();

        map.showRouteOnMap();
        map.showWaypointIcons();

        var currentCenterToFlyTo = null;
        if (map.currentPosition) {
            currentCenterToFlyTo = map.currentPosition.geoCoordinates.clone(guidancePage);
        } else if (map.previousPosition) {
            currentCenterToFlyTo = map.previousPosition.geoCoordinates.clone(guidancePage);
        }

        var tmpOrientation = Map.PRESERVE_ORIENTATION;
        var tmpManeuvers;
        var tmpRoute = map.route;
        if (tmpRoute) {
            tmpManeuvers = tmpRoute.maneuvers;
            if (tmpManeuvers && tmpManeuvers[0]) {
                tmpOrientation = tmpManeuvers[0].mapOrientation;
                if (tmpOrientation) {
                    map.orientation = tmpOrientation;
                } else {
                    tmpOrientation = Map.PRESERVE_ORIENTATION;
                }
            }
        }

        map.moveTo(
            currentCenterToFlyTo,
            Map.ANIMATION_NONE,
            guidanceZoomScale,
            tmpOrientation,
            Map.PRESERVE_PERSPECTIVE
        );

        lazyGuidanceStartTimer.start();

    } else {
        guidanceRoutingDialog.openWithContent(
            qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route_error_title"), // Error
            RoutingUtils.getErrorText(errorCode),
            qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route_error_okay"), // Okay
            true    // Force finished signal
        );
    }
}

function startGuidance() {
    if (true || appState.isDevice) {
        if (map.route.__calculationResult) {
            var cloningShat = map.route.__calculationResult.clone(guidancePage);
            guidance.navigateRoute(cloningShat);
        }
    } else {
        map.disablePositionProvider();
        if (map.route.__calculationResult) {
            var cloningShat = map.route.__calculationResult.clone(guidancePage);
            guidance.simulate(cloningShat, 10);
        }
    }
    guidanceToolbar.toolbarTimer.start();
    var tmpOrientation = Map.PRESERVE_ORIENTATION;
    if (map.route.__calculationResult.maneuvers[0]) {
        var firstMan = map.route.__calculationResult.maneuvers[0];
        //guidance.mapUpdateMode = Guidance.MAP_UPDATE_POSITION_ANIMATION;
        guidanceHeader.updateHeader(firstMan, firstMan.distanceFromStart);
        tmpOrientation = firstMan.mapOrientation;
    }

    setMapFollowsPosition(true, false, tmpOrientation);
}

function setMapFollowsPosition(enable, resetZoomLevel, forceOrientation) {
    if (enable) {
        var currentCenterToFlyTo = null;
        if (map.currentPosition && map.currentPosition.geoCoordinates) {
            currentCenterToFlyTo = map.currentPosition.geoCoordinates;
        } else if (map.previousPosition && map.previousPosition.geoCoordinates) {
            currentCenterToFlyTo = map.previousPosition.geoCoordinates;
        }

        if (resetZoomLevel) {
            guidanceZoomScale = guidancePage.__defaultGuidanceZoomScale;
        }

        var orientation = guidanceOrientation;
        if (forceOrientation !== undefined) {
            orientation = forceOrientation;
        }

        operationToApplyOnAnimationDone = "attachToLocation";

        map.moveTo(
            currentCenterToFlyTo,
            Map.ANIMATION_NONE,
            map.zoomLevels[14] / 1.5,//guidanceZoomScale,
            orientation,
            Map.PRESERVE_PERSPECTIVE
        );
    } else {
        map.attachedToLocation = false;
        map.skipAnimationOnAttachedToLocation = false;

        guidanceZoomScale = map.zoomScale;
        guidanceOrientation = map.orientation;
    }

    guidance.mapUpdateMode = getGuidanceUpdateMode(enable);
}

function getGuidanceUpdateMode(followMode) {
    if (followMode) {
        if (mainscreen.walkGuidanceUpdateMode == "roadview") {
            return mainscreen.magicCompass && mainscreen.useMagicCompassInGuidanceInstanadOfMOS? Guidance.MAP_UPDATE_OVERVIEW : Guidance.MAP_UPDATE_ROADVIEW;
        } else if (walkGuidanceUpdateMode == "position-animation") {
            return mainscreen.magicCompass && mainscreen.useMagicCompassInGuidanceInstanadOfMOS? Guidance.MAP_UPDATE_OVERVIEW : Guidance.MAP_UPDATE_POSITION_ANIMATION;
        } else {
            return mainscreen.magicCompass && mainscreen.useMagicCompassInGuidanceInstanadOfMOS? Guidance.MAP_UPDATE_OVERVIEW : Guidance.MAP_UPDATE_OVERVIEW;
        }
    } else {
        return mainscreen.magicCompass && mainscreen.useMagicCompassInGuidanceInstanadOfMOS? Guidance.MAP_UPDATE_OVERVIEW : Guidance.MAP_UPDATE_OVERVIEW;
    }
}

function handleAnimationDone() {
    if (operationToApplyOnAnimationDone == "attachToLocation") {
        map.attachedToLocation = true;
        map.skipAnimationOnAttachedToLocation = true;
    }
    operationToApplyOnAnimationDone = "";
}

function startCompass() {
    if (mainscreen.magicCompass && mainscreen.useMagicCompassInGuidanceInstanadOfMOS) {
        appState.startCompass();
        compassStarted = true;
    }
}

function stopCompass() {
    if (mainscreen.magicCompass && compassStarted) {
        appState.stopCompass();
        compassStarted = false;
    }
}

function endGuidance(finishedSignalPrefix) {
    stopCompass();

    if (guidance.running) {
        guidance.stop();
    }

    //map.removeWaypointIcons();
    map.removeRouteFromMap();
    if (destinationReached) {
        destinationReached = false;
        guidancePage.finished(null, "done" + (finishedSignalPrefix? finishedSignalPrefix : ""));
    } else {
        if (typeof(analyticsComponent) != "undefined") {
            var currentTimeStamp = new Date();
            var guidanceDuration;
            analyticsComponent.timeStampEndNavigation = currentTimeStamp.getTime();
            guidanceDuration = analyticsComponent.timeStampEndNavigation - analyticsComponent.timeStampStartNavigation;
            analyticsComponent.logIMEIActivity("EndNavigation", { duration: guidanceDuration/1000 + " sec", reason: "abortedNavigation" } );
        }
        guidancePage.finished(null, "stop" + (finishedSignalPrefix? finishedSignalPrefix : ""));
    }
}

function doLogStartNavigation(caption, body) {
    var start = guidancePage.model.waypoints[0];
    var dest = guidancePage.model.waypoints[guidancePage.model.waypoints.length - 1];
    var startAddress = typeof(start.properties) == "object" ? JSON.stringify(analyticsComponent.extractAddress(start.properties)) : caption ? caption : "N/A";
    var destAddress  = typeof(dest.properties)  == "object" ? JSON.stringify(analyticsComponent.extractAddress(dest.properties))  : "N/A";
    var currentTimeStamp = new Date();
    analyticsComponent.timeStampStartNavigation = currentTimeStamp.getTime();
    analyticsComponent.logIMEIActivity("StartNavigation", {
        type: guidancePage.model.mode.toUpperCase(),
        startAddress: startAddress,
        destinationAddress: destAddress,
        placeID: (dest && dest.properties && typeof(dest.properties.placesId) != "undefined") ? dest.properties.placesId : "N/A"
    });
}

function startNavigation() {
    map.attachedToLocation = true;
    destinationReached = false;

    startCompass();

    var lat, lon, start;
    if (guidancePage.model) {
        start = guidancePage.model.waypoints[0];
    }
    if (start.properties) {
        lat = start.properties.geoLatitude;
        lon = start.properties.geoLongitude;
    } else {
        lat = start.latitude;
        lon = start.longitude;
    }

    //analyticsComponent.extractAddress is necessary to not send geo coordinates with Tracking due to legal issues
    //Addresses are being stringified as the won't be accepted in as objects by the NAC backend ///789789
    if (typeof (analyticsComponent) != "undefined") {
        if (typeof(start.properties) == "object") {
            doLogStartNavigation();
        } else {
            finder.actionToPerform = "logAnalyticsStartFromCurrentPosition";
            finder.find({latitude: lat, longitude: lon})
        }
    }

    if (map.currentPosition && map.positionType == map._POSITION_TYPE_GPS_ && map.currentPosition && map.currentPosition.geoCoordinates) {
        var curPos = mainscreen.mapGlobalData.currentPosition.geoCoordinates;
        map.currentPositionChanged.disconnect(startNavigation);

        if (Math.abs(lat - curPos.latitude) > 0.0001 || Math.abs(lon - curPos.longitude) > 0.0001) {
            var wps = guidancePage.model.waypoints;
            wps[0] = {latitude: curPos.latitude, longitude: curPos.longitude};
            guidancePage.model.waypoints = wps;
            guidancePage.model.autoMode = false;

            guidanceRoutingDialog.openWithContent(
                "",
                qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route_with_gps"), // "Updating Route with your GPS position..."
                qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route_cancel") // Cancel
            );
            waitForGPSTimer.stop();
            guidancePage.model.calculate();
        } else {
            if (guidancePage.model.calculated) {
                Logic.onRouteCalculationDone(0, 0);
            } else {
                guidanceRoutingDialog.openWithContent(
                    "",
                    qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route"), // "Updating Route..."
                    qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route_cancel") // Cancel
                );
                waitForGPSTimer.stop();
                guidancePage.model.autoMode = false;
                guidancePage.model.calculate();
            }
        }
    } else {
        guidanceRoutingDialog.openWithContent(
            "",
            qsTranslate("Maps", "qtn_navigationconnector_guidance_waiting_for_gps"), // "Waiting for GPS..."
            qsTranslate("Maps", "qtn_navigationconnector_guidance_updating_route_cancel"), // Cancel
            true,
            true
        );
        waitForGPSTimer.start();
        map.currentPositionChanged.connect(startNavigation);
    }
}

function loadSettings() {
    var tmpVoiceSkin = -1;
    guidanceViewMenu.backlightChecked = mainscreen.backlightsAlwaysOn;
    appState.disableScreenSaver(guidanceViewMenu.backlightChecked);
    db.get("voiceSkinId", function(skinId) {
        guidanceViewMenu.audioEnabled = skinId > 0;
        tmpVoiceSkin = skinId;
        if (guidanceViewMenu.audioChecked && guidanceViewMenu.audioEnabled) {
            guidance.voiceSkinId = skinId;
        } else {
            guidance.voiceSkinId = 0;
        }
    }, function(error) {
        log.error(" Unable to get voice skin id");
        guidanceViewMenu.audioChecked = false;
        guidanceViewMenu.audioEnabled = false;
    });

    db.get("audioCommandsOn", function(value) {
        guidanceViewMenu.audioChecked = (value === "true");
        if (tmpVoiceSkin > 0 && guidanceViewMenu.audioChecked) {
            guidance.voiceSkinId = tmpVoiceSkin;
        }
    }, function(error) {
        log.error("Unable to get voice commands on");
        guidanceViewMenu.audioChecked = false;
        guidanceViewMenu.audioEnabled = (tmpVoiceSkin > 0) || false;
        db.save("audioCommandsOn", false);
    });
}

function showOverview(aShow) {
    if (aShow) {
        if (compassStarted) {
            stopCompass();
            resumeCompass = true;
        }

        setMapFollowsPosition(false);
        map.zoomToRoute(true);
    } else {
        setMapFollowsPosition(true);

        if (resumeCompass) {
            startCompass();

            resumeCompass = false;
        }
    }
}
