import QtQuick 1.0

Item {
    id: self

    property string dbName: "NokiaMapsDataBase"
    property string dbVersion: "1.0"
    property string dbComment: "Settings storage for navigation connector"
    property int    dbLimit: 1000000
    property bool   dbReady: false

    Item {
        id: internals
        objectName: "internals"

        function _getDbConnection() {
            var db = openDatabaseSync(self.dbName, self.dbVersion, self.dbComment, self.dbLimit);

            _ensureDatabaseIsNotChanged(db);

            return db;
        }

        function _ensureDatabaseIsNotChanged(db) {
            try {
                db.transaction(function (tx) {
                    tx.executeSql("select key, value from Settings limit 1");
                });
            } catch (e) {
                try {
                    db.transaction(function (tx) {
                        tx.executeSql("drop table if exists Settings");
                        tx.executeSql("create table Settings (key text unique, value text)");
                    });
                } catch (e) {
                    log.error("cannot create Settings sqllite table, aborting...");
                }
            }
        }

        function _get(key, callbackSuccess, callbackFail) {
            var result;

            var db = _getDbConnection();

            try {
                db.transaction(function (tx) {
                    var queryResult = tx.executeSql("select value from Settings where key=?", [key]);
                    if (queryResult.rows.length > 0 && queryResult.rows.item(0).value) {
                        callbackSuccess(queryResult.rows.item(0).value);
                    } else {
                        callbackFail(queryResult);
                    }
                });
                result = true;
            } catch (e) {
                result = false
            }

            return result;
        }

        function _set(key, value) {
            var result;

            if (key !== undefined && value !== undefined) {
                var db = _getDbConnection();

                try {
                    db.transaction(function (tx) {
                        var ifExists = tx.executeSql("select count(key) key_count from Settings where key=?", [key]);
                        if (ifExists.rows.length > 0 && ifExists.rows.item(0).key_count == 1) {
                            tx.executeSql("update Settings set value=? where key=?", [value, key]);
                        } else {
                            tx.executeSql("insert into Settings values (?, ?)", [key, value]);
                        }
                    });

                    result = true;
                } catch (e) {
                    result = false
                }
            } else {
                result = false;
            }
            return result;
        }

        function _remove(key) {
            var result;

            var db = _getDbConnection();

            try {
                db.transaction(function (tx) {
                    tx.executeSql("delete from Settings where key=?", [key]);
                });
                result = true;
            } catch (e) {
                result = false
            }

            return result;
        }

    }

    function set(key, value) {
        internals._set(key, value);
    }

    function get(key, callbackSuccess, callbackFail) {
        internals._get(key, callbackSuccess, callbackFail);
    }

    function remove(key) {
        internals._remove(key);
    }
}
