var MODE_WALK = 0,
    MODE_PT = 1,
    MODE_DRIVE = 2;
var calculatedRoutes = [null, null, null];

function handleFindPureDataCompleted(contextData, geoPos) {
    if (!contextData.errorCode) {
        var tmpPlace = Qt.createQmlObject("import ovi.connector.places 1.0; Place {}", routePlanner);
        tmpPlace.properties = {
            title: placeHelper.formatAddress({
                addrHouseNumber: contextData.ADDR_HOUSE_NUMBER,
                addrCityName: contextData.ADDR_CITY_NAME,
                addrStreetName: contextData.ADDR_STREET_NAME,
                addrCountryCode: contextData.ADDR_COUNTRY_CODE
            }),
            geoLatitude: geoPos.latitude,
            geoLongitude: geoPos.longitude,
            addrCityName: contextData.ADDR_CITY_NAME,
            addrCountryCode: contextData.ADDR_COUNTRY_CODE,
            addrCountryName: contextData.ADDR_COUNTRY_NAME,
            addrDistrictName: contextData.ADDR_DISTRICT_NAME,
            addrHouseNumber: contextData.ADDR_HOUSE_NUMBER,
            addrStreetName: contextData.ADDR_STREET_NAME,
            tzOffsetMinutes: contextData.TZ_OFFSET_MINUTES,
            addrPostalCode: "",
            phoneNumber: "",
            url: "",
            email: "",
        };

        var tmpWaypoints = routePlanner.model.waypoints;
        tmpWaypoints[longPressGeoFinder.targetToUpdate] = tmpPlace;
        routePlanner.model.waypoints = tmpWaypoints;
    } else {
        log.info("no result so far...")
    }

    longPressGeoFinder.busy = false;
}

function handleMapLongPressRectangleCompleted(geoPos) {
    map.removeRouteFromMap();
    var tmpWaypoints = routePlanner.model.waypoints;
    tmpWaypoints[routePlanner.lastSelectedState == "setStart"? 0 : 1] = geoPos;
    routePlanner.model.waypoints = tmpWaypoints;
    //__updateRoute();
    fakeLazyRouteUpdater.start();

    populateCurrentlySelectedIcon({fake: true});

    longPressGeoFinder.geoPos = geoPos;
    longPressGeoFinder.targetToUpdate = routePlanner.lastSelectedState == "setStart"? 0 : 1;
    longPressGeoFinder.find(geoPos, true);
    longPressGeoFinder.busy = true;

    routePlanner.showInfoBubbleFromDestination = routePlanner.lastSelectedState == "setDestination";

    map.setControl("emit-forget-selected-marker");
    map.model.selectObject(null);
}

function populateCurrentlySelectedIcon(mapObject) {
    if (mapObject) {
        if (routePlanner.lastSelectedState == "setStart") {
            if (map.currentlySelectedIconA) {
                map.currentlySelectedIconA.visible = true;
                //map.currentlySelectedIconA.destroy();
                map.currentlySelectedIconA = null;
            }
            if (mapObject.clone) {
                map.currentlySelectedIconA = mapObject.clone(map);
                map.currentlySelectedIconAInitialVisibility = mapObject.visible;
            }
        } else {
            if (map.currentlySelectedIconB) {
                map.currentlySelectedIconB.visible = true;
                //map.currentlySelectedIconB.destroy();
                map.currentlySelectedIconB = null;
            }
            if (mapObject.clone) {
                map.currentlySelectedIconB = mapObject.clone(map);
                map.currentlySelectedIconBInitialVisibility = mapObject.visible;
            }
        }
    }
}

function handleMapObjectSelected(mapObject) {
    if (mapObject) {
        map.setControl("emit-forget-selected-marker");

        var mapObjectData = map.model.getObjectData(mapObject),
            wps = routePlanner.model.waypoints,
            index,
            tmp,
            tmpWaypoints = routePlanner.model.waypoints
        ;

        map.model.selectedObject = null;

        map.closeInfoBubbleByForce();

        // Create place object from map data
        if (mapObject.type == MapObject.TRANSITSTOP) {
            createPtPlace(mapObject);
        } else if (mapObject.type == "longPress") {
            handleMapLongPressRectangleCompleted(mapObject.position);
        } else if (mapObjectData) {
            if (!mapObjectData.isWaypoint) {
                populateCurrentlySelectedIcon(mapObject);

                if (!routePlanner.skipMapObjectSelected) {
                    createPlaceFromMapIcon(mapObjectData);
                } else {
                    routePlanner.skipMapObjectSelected = false;
                }
            }

            if (mapObjectData.index === 0 || mapObjectData.index === 1) {
                routePlanner.showInfoBubbleFromDestination = mapObjectData.index == 1;
            } else {
                routePlanner.showInfoBubbleFromDestination = routePlanner.lastSelectedState == "setDestination";
            }

            var wps = routePlanner.model.waypoints;

            if (wps[mapObjectData.index]) {
                routePlanner.indexOfFlagToHide = mapObjectData.index;
                createPreviewInfoBubble({
                    latitude: mapObjectData.latitude,
                    longitude: mapObjectData.longitude
                });
            }
        }
    }
}

function changeDestinationToPlace() {
    log.info(" found nearby place. changing destination to place");
    map.removeRouteFromMap();
    routePlanner.__onPlacePicked(placeObj, routePlanner.lastSelectedState);
}

function createPlaceFromMapIcon(aData) {
    var props = {
        placesId: aData.placeId,
        title: aData.title,
        address: aData.address,
        geoLatitude: aData.latitude,
        geoLongitude: aData.longitude
    };
    placeObj.properties = props;
    placeObj.uri = aData.placeId || "";
    changeDestinationToPlace();
}

function createPtPlace(aMapTransitStopObj) {
    populateCurrentlySelectedIcon({fake: true});

    var station = aMapTransitStopObj.transitStopInfo;
    var props = {
        title: "" + station.officialName,
        geoLatitude: station.position.latitude,
        geoLongitude: station.position.longitude
    };
    placeObj.properties = props;
    if (station.placesId !== "") {
        var data = map.model.getObjectData(aMapTransitStopObj);
        if (data) {
            data.title = props.title;
            createPlaceFromMapIcon(data);
        } else {
            placeObj.publictransport = {
                systems: __getSystemDetails(station.systems),
                lines: __getLineDetails(station.lines),
                linetypes: station.lineTypes,  // the bitwise combination of TransitStopInfo::TransitTypes.
                operatingHours: station.operatingHours,
                parkingSize: station.parkingSize,
                parkingHours: station.parkingHours
            }
            changeDestinationToPlace();
        }
    } else {
        placeObj.publictransport = {
            systems: __getSystemDetails(station.systems),
            lines: __getLineDetails(station.lines),
            linetypes: station.lineTypes,  // the bitwise combination of TransitStopInfo::TransitTypes.
            operatingHours: station.operatingHours,
            parkingSize: station.parkingSize,
            parkingHours: station.parkingHours
        }
        changeDestinationToPlace();
    }
}


function __getLineDetails(lines){
    var results = [], tmp;
    for (var i = 0; i < lines.length; i++) {
        results.push(transitDb.getLineInfo(lines[i]));
    }
    return results;
}

function __getSystemDetails(systems){
    var results = [], tmp, details;
    for (var i = 0; i < systems.length; i++) {
        tmp = transitDb.getSystemInfo(systems[i]);
        details = {
            id: tmp.id.stringId,
            objectName: tmp.objectName,
            systemOfficialName: tmp.systemOfficialName,
            systemInformalName: tmp.systemInformalName,
            systemWebsiteUrl: tmp.systemWebsiteUrl,
            companyOfficialName: tmp.companyOfficialName,
            companyShortName: tmp.companyShortName,
            companyWebsiteUrl: tmp.companyWebsiteUrl,
            companyRoutePlannerUrl: tmp.companyRoutePlannerUrl,
            companyScheduleUrl: tmp.companyScheduleUrl,
            companyPhone: tmp.companyPhone,
            systemLogo: tmp.systemLogo ? {
                 svgContent: tmp.systemLogo.svgContent,
                 localUrl: tmp.systemLogo.localUrl,
                 isValid: tmp.systemLogo.isValid,
                 size: tmp.systemLogo.size,
                 imageUrl: tmp.systemLogo.imageUrl
            } : null,
            systemAccessLogo: tmp.systemAccessLogo ? {
                svgContent: tmp.systemAccessLogo.svgContent,
                localUrl: tmp.systemAccessLogo.localUrl,
                isValid: tmp.systemAccessLogo.isValid,
                size: tmp.systemAccessLogo.size,
                imageUrl: tmp.systemAccessLogo.imageUrl
            } : null,
        };
        results.push(details);
    }

    return results;
}

function handleInfoBubbleModeChange(aMode) {
    if (aMode !== routePlanner.model.mode) {
        routePlanner.model.autoMode = false;
        routePlanner.model.mode = aMode;

        var index = getModeIndex(aMode);
        if(index !== -1 && calculatedRoutes[index] !== null) {
            routePlanner.model.__calculationResult = calculatedRoutes[index].route ?
                        calculatedRoutes[index].route.clone(mainscreen || routePlanner.model.parent || routePlanner.model) : null;
            routePlanner.model.maneuvers = calculatedRoutes[index].maneuvers;
            routePlanner.__routingError = calculatedRoutes[index].error;
            routePlanner.indexOfFlagToHide = routePlanner.lastSelectedState == "setStart"? 0 : 1

            if (routePlanner.__routingError === Router.ERROR_NONE) {
                map.showRouteOnMap();
                if(routeErrorBanner.opacity !== 0) {
                    routeErrorBanner.hide();
                }
            } else {
                routeErrorBanner.text = RoutingUtils.getErrorText(routePlanner.__routingError);
                routeErrorBanner.show();
            }
        } else {
            routePlanner.model.cancelCalculation();
            routePlanner.model.calculate();
        }
    }
}
function stashRoute(aRoute, aErrorCode) {
    if (!aRoute) {
        return;
    }
    var index = getModeIndex(aRoute.mode);
    if (index !== -1) {
        calculatedRoutes[index] = {
            "route": aRoute.__calculationResult ? aRoute.__calculationResult.clone(mainscreen || aRoute.parent || aRoute) : null,
            "error": aErrorCode,
            "maneuvers": aRoute.__calculationResult ? aRoute.__calculationResult.clone(mainscreen || aRoute.parent || aRoute).maneuvers : null,
        };
    }
}

function getModeIndex(aMode) {
    var index = -1;
    if (aMode === "walk") {
        index = MODE_WALK;
    } else if (aMode === "pt") {
        index = MODE_PT;
    } else if (aMode === "drive") {
        index = MODE_DRIVE;
    }
    return index;
}

function clearStashedRoutes() {
    log.info("Clear stashed routes");

    if (calculatedRoutes[MODE_WALK]) {
        log.trace("Clear walk");
        if (calculatedRoutes[MODE_WALK].route) {
            calculatedRoutes[MODE_WALK].route.destroy();
        }
    }
    if (calculatedRoutes[MODE_PT]) {
        log.trace("Clear pt");
        if (calculatedRoutes[MODE_PT].route) {
            calculatedRoutes[MODE_PT].route.destroy();
        }
    }
    if (calculatedRoutes[MODE_DRIVE]) {
        log.trace("Clear drive");
        if (calculatedRoutes[MODE_DRIVE].route) {
            calculatedRoutes[MODE_DRIVE].route.destroy();
        }
    }

    calculatedRoutes = [null, null, null];
}

function __getWaypointDescription(wp, indexToAutoReverseGeoCode) {
    var result;

    if (wp && wp.fromGps) {
        return map.currentPosition? qsTranslate("Maps", "qtn_navigationconnector_routeplanner_currentposition") : qsTranslate("Maps", "qtn_navigationconnector_routeplanner_previousposition");
    } else if (wp && wp.name) {
        result = wp.name;
    } else if (wp && wp.properties && wp.properties.title) {
        result = wp.properties.title;
    } else {
        result = qsTranslate("Maps", "qtn_navigationconnector_routeplanner_unknownposition");
        if (indexToAutoReverseGeoCode === 0) {
            lazyLoaderWaiter.indexToFix = 0;
        } else if (indexToAutoReverseGeoCode === 1) {
            lazyLoaderWaiter.otherIndexToFix = 1;
        }
    }

    return result;
}

function __getWaypointDescriptionForDefaultPositions() {
    if (map.currentPosition) {
        return qsTranslate("Maps", "qtn_navigationconnector_routeplanner_currentposition");
    } else if (map.previousPosition) {
        return qsTranslate("Maps", "qtn_navigationconnector_routeplanner_previousposition");
    } else {
        return qsTranslate("Maps", "qtn_navigationconnector_routeplanner_waiting_for_position");
    }
}
