import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.places 1.0

import "RoutePlanner.js" as PlannerLogic
import "../RoutingUtils.js" as RoutingUtils
import "../ui"
import "../"

ConnectorPage {
    id: routePlanner
    signal forward(variant error, variant route)
    property variant mainScreen: parent
    property string searchString
    property bool waitingForCurrentPosition: false
    property int fieldWidth: (parent.width - 64 - swapButton.width) / 2
    property bool showInfoBubbleFromDestination: true
    property string lastSelectedState: "setDestination"
    property int defaultCurrentPositionIndex: 0
    property int __routingError;
    property bool everPickedCompleted: false
    property bool evetPickedCanceled: false

    property int indexOfFlagToHide: -1
    property int __origZoomLevel: -1

    property variant latestPickedDestinationUri
    property variant latestPickedDestinationProperties

    property bool preventAutoRootCreation: false

    anchors.fill: parent

    Header {
        id: header

        RoutePlannerInput {
            id: fromField
            objectName: "fromTextField"

            anchors { left: parent.left;  leftMargin: 16; verticalCenter: parent.verticalCenter; }
            defaultWidth: fieldWidth
            highlightField: lastSelectedState == "setStart"

            targetToUpdate: 0
            placeholderText: qsTranslate("Maps", "qtn_navigationconnector_routeplanner_watermark_setstart")
            isCurrentPosition: model && ((model.waypoints[0] && model.waypoints[0].fromGps) || (!model.waypoints[0] && defaultCurrentPositionIndex === 0))

            text: {
                if (model && model.waypoints[0]) {
                    return PlannerLogic.__getWaypointDescription(model.waypoints[0], 0);
                } else if (defaultCurrentPositionIndex === 0) {
                    return PlannerLogic.__getWaypointDescriptionForDefaultPositions();
                } else {
                    return "";
                }
            }

            onInputClicked: {
                routePlanner.state = "setStart";
                routePlanner.lastSelectedState = "setStart";
            }
        }

        Button {
            id: swapButton

            anchors { verticalCenter: parent.verticalCenter; left: fromField.right; leftMargin: 16 }
            height: 48
            width: 48

            Image {
                anchors.centerIn: parent;
                source: navigationAssetsPath + "images/change_order_" + (swapButton.pressed ? "light" : "dark") + ".png";
            }

            onClicked: {
                if (model) {
                    var wp = model.waypoints;
                    defaultCurrentPositionIndex = (defaultCurrentPositionIndex === 0 ? 1 : 0);
                    model.waypoints = [wp[1], wp[0]];
                    routePlanner.__updateRoute();
                }
            }

            enabled: fromField.text != qsTranslate("Maps", "qtn_navigationconnector_routeplanner_waiting_for_position")
        }

        RoutePlannerInput {
            id: toField;
            targetToUpdate: 1

            highlightField: !fromField.highlightField
            defaultWidth: fieldWidth
            anchors { right: parent.right; rightMargin: 16; verticalCenter: parent.verticalCenter; }

            placeholderText: qsTranslate("Maps", "qtn_navigationconnector_routeplanner_watermark_setdestination")
            isCurrentPosition: model && ((model.waypoints[1] && model.waypoints[1].fromGps) || (!model.waypoints[1] && defaultCurrentPositionIndex === 1))

            text: {
                if (model && model.waypoints[1]) {
                    return PlannerLogic.__getWaypointDescription(model.waypoints[1], 1);
                } else if (defaultCurrentPositionIndex === 1) {
                    return PlannerLogic.__getWaypointDescriptionForDefaultPositions();
                } else {
                    return "";
                }
            }

            onInputClicked: {
                routePlanner.state = "setDestination";
                routePlanner.lastSelectedState = "setDestination";
            }
        }

        z: 1
    }

    property RouteViewer __map: map
    RouteViewer {
        id: map
        objectName: "routeViewerMap"

        anchors { top: header.bottom; left: parent.left; right: parent.right; bottom: parent.bottom; }

        mainScreen: routePlanner.mainScreen
        behaviors: "_NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_:true;_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_:true;_NO_INFO_BUBBLE_CLICK_EFFECT_:true;"

        onMapLongPressRectangleCompleted: { PlannerLogic.handleMapLongPressRectangleCompleted(geoPos) }
        onMapObjectsSelected: { PlannerLogic.handleMapObjectSelected(mapObject) }
        onMapDragStarted: { map.currentlySelectedIconAInitialVisibility = true; __map.currentlySelectedIconBInitialVisibility = true; map.unhideWaypointIcons(null); }
        onMapClicked: { map.unhideWaypointIcons(null); }
    }

    Connections {
        target: map.model
        ignoreUnknownSignals: true

        onPureSelectedObjectChanged: {
            if (!map.model.selectedObject) {
                if (!map.isAnimationInProgress) {
                    map.unhideWaypointIcons(null);
                }
            }
        }

        onSelectedObjectChanged: {
            if (!map.model.selectedObject) {
                if (!map.isAnimationInProgress) {
                    map.unhideWaypointIcons(null);
                }
            }
        }
    }

    TransitDatabase {
        id: transitDb
    }

    Place {
        id: placeObj
    }

    property ReverseGeoCodeFinder __longPressGeoFinder: longPressGeoFinder
    ReverseGeoCodeFinder {
        id: longPressGeoFinder
        property variant geoPos
        property int targetToUpdate: 1
        property bool busy: false
        mapView: map
        onFindPureDataComplete: { PlannerLogic.handleFindPureDataCompleted(contextData, geoPos) }
    }

    property Component __bubbleComponent: bubbleComponent;
    Component {
        id: bubbleComponent;
        PreviewInfoBubble {
            id: bubble;
            route: routePlanner.model;
            start: !routePlanner.showInfoBubbleFromDestination
            destText: routePlanner.showInfoBubbleFromDestination ? toField.text : fromField.text
            isLandscape: routePlanner.width > routePlanner.height;
            transportMode: routePlanner.model.mode
            routingError: routePlanner.__routingError !== Router.ERROR_NONE
            onGoClicked: {
                result = routePlanner.model;
                routePlanner.__origZoomLevel = __map.callMapApi("getCurrentZoomLevel");
                forward(null, routePlanner.model);
            }
            onTransportModeChanged:  {
                PlannerLogic.handleInfoBubbleModeChange(transportMode);
            }
        }
    }

    Component {
        id: placePicker
        PlacesPicker {
            id: picker

            disableLongPress: true
            disablePublicTransport: true
            pickFromMapAllowed: false

            tools: ToolBarLayout {
                ToolIcon {
                    iconId: "icon-m-toolbar-back"
                    anchors.left: parent.left
                    onClicked: {
                        pageStack.pop(null,true);
                    }
                }
            }
            focusedAtStart: true
            popOnCancel: true

            showGPS: lastSelectedState == "setStart" && !!(map.currentPosition || map.previousPosition)
            showPickFromMap: true
            prefilledString: searchString;
            preventPrefilledStringAutoSearch: true
            onPicked: {
                if (!map.model.selectedObject) {
                    defaultCurrentPositionIndex = 0;
                }

                routePlanner.everPickedCompleted = true;
                routePlanner.evetPickedCanceled = false;
                PlannerLogic.populateCurrentlySelectedIcon({fake: true});
                PlannerLogic.populateCurrentlySelectedIcon(map.model.selectedObject);
                routePlanner.skipMapObjectSelected = true;
                routePlanner.__onPlacePicked(pickedObject, routePlanner.state);
            }

            onPickCenter: {
                preventAutoRootCreation = true;
                pageStack.pop(routePlanner, true);
                PlannerLogic.handleMapLongPressRectangleCompleted(map.center);
                preventAutoRootCreation = false;
            }

            onLensClickedAndMapVisible: {
                pageStack.pop(routePlanner, true);
            }

            onStatusChanged: {
                 if (status == PageStatus.Deactivating) {
                     __map.model.resetAllSelectedObjects();

                     if (pickedObject === "") {
                         routePlanner.evetPickedCanceled = true;
                         routePlanner.__onPlacePickingCancelled(cancelMode, input.text);
                     } else if (status == PageStatus.Deactivating && pickedObject) {
                         routePlanner.everPickedCompleted = true;
                         routePlanner.evetPickedCanceled = false;
                     }
                 } else if (status == PageStatus.Deactivated) {
                     picker.destroy();
                 }
             }
         }
     }

    property bool skipMapObjectSelected: false

    Connections {
        target: routePlanner
        ignoreUnknownSignals: true

        onModelChanged: {
            map.removeRouteFromMap();
            map.removeWaypointIcons();

            map.route = model;

            model.calculationDone.connect(__onCalculationDone);
            if (!model.calculated) {
                fakeLazyRouteUpdater.start();
            }
        }
    }

    Timer {
        id: fakeLazyRouteUpdater
        interval: 250
        onTriggered: {
            if (mainscreen.forcePrevetAutoModeInRoute) {
                mainscreen.forcePrevetAutoModeInRoute = false;
            }
            __updateRoute();
        }
    }

    Connections {
        target: map
        ignoreUnknownSignals: true

        onCurrentPositionChanged: {
            if (routePlanner.status == PageStatus.Active && routePlanner.waitingForCurrentPosition) {
                if (!model.calculated) {
                    __updateRoute();
                }
            }
            routePlanner.waitingForCurrentPosition = false;
        }
        onPreviousPositionChanged: {
            if (routePlanner.status == PageStatus.Active && routePlanner.waitingForCurrentPosition) {
                if (!model.calculated) {
                    __updateRoute();
                }
            }
            routePlanner.waitingForCurrentPosition = false;
        }
    }

    function __clonePlace(place) {
        return place.properties.geoLatitude ? {
              name: place.properties.title,
              latitude: parseFloat(place.properties.geoLatitude),
              longitude: parseFloat(place.properties.geoLongitude)
            } : {
              name: place.properties.title,
              latitude: parseFloat(place.details.location.geoCoordinates.latitude),
              longitude: parseFloat(place.details.location.geoCoordinates.longitude)
            }
    }

    function __onPlacePicked(place, aState) {
        pageStack.pop(routePlanner, true);

        if (aState === "setDestination") {
            routePlanner.latestPickedDestinationUri = place.uri;
            routePlanner.latestPickedDestinationProperties = place.properties;
        }

        aState = aState || state;
        var tmpWaypoints = routePlanner.model.waypoints;

        var placeCloned = place ? __clonePlace(place) : place;

        if (aState === "setStart") {
            tmpWaypoints[0] = placeCloned;
        } else if (aState === "setDestination") {
            tmpWaypoints[1] = placeCloned;
        }
        routePlanner.model.waypoints = tmpWaypoints;

        fakeLazyRouteUpdater.start();
        state = "";
    }

    function __onPlacePickingCancelled(cancelMode, searchText) {
        pageStack.pop(routePlanner, true);
        //map.model.clear();
        state = "";
        PlannerLogic.clearStashedRoutes();
        if (cancelMode === "enter" && searchText === "") {
            map.removeRouteFromMap();
            map.removeWaypointIcons();
            routePlanner.model.__reset(true);
        }
    }

    function __fixModelWayPointsWithCurrentPosition(aIndexToFix) {
        var wps = model.waypoints;
        if (wps[0] || wps[1]) {
            var indexToFix = aIndexToFix || 0;
            var otherIndex = (indexToFix == 0 ? 1 : 0);

            defaultCurrentPositionIndex = aIndexToFix;

            if (!wps[indexToFix]) {
                if (!wps[indexToFix] && (map.currentPosition || map.previousPosition)) {
                    var currentGeoCoordinates = map.currentPosition? map.currentPosition.geoCoordinates : map.previousPosition.geoCoordinates;
                    wps[indexToFix] = { latitude: currentGeoCoordinates.latitude, longitude: currentGeoCoordinates.longitude, fromGps: true };
                }

                model.waypoints = wps;
            }
        }

        return wps;
    }

    property Timer __lazyLoaderWaiter: lazyLoaderWaiter
    Timer {
        id: lazyLoaderWaiter

        property int indexToFix: -1
        property int otherIndexToFix: -1
        property int _retries: 0

        interval: 250
        repeat: true
        running: indexToFix != -1 || otherIndexToFix != -1

        function getIndexToFix() {
            var result = -1;

            if (indexToFix != -1) {
                result = indexToFix;
                indexToFix = otherIndexToFix;
                otherIndexToFix = -1;
            } else {
                result = otherIndexToFix;
                otherIndexToFix = -1;
            }

            return result;
        }

        onTriggered: {
            if (!longPressGeoFinder.busy) {
                var toFix = getIndexToFix();
                if (toFix != -1) {
                    var wps = model.waypoints;
                    longPressGeoFinder.geoPos = wps[toFix];
                    longPressGeoFinder.targetToUpdate = toFix;
                    longPressGeoFinder.find(wps[toFix], true);
                    longPressGeoFinder.busy = true;
                }

                _retries = 0;
            } else if (++_retries > 50) {
                _retries = 0;
            }
        }
    }

    function hideFlagAndShowInfoBubble(wps, openInfoBubble, indexToMoveTo) {
        if (indexToMoveTo === undefined) {
            indexToMoveTo = routePlanner.showInfoBubbleFromDestination? 1 : 0;
        }

        var pos;

        if (wps[indexToMoveTo].latitude && wps[indexToMoveTo].longitude) {
            pos = {
                latitude: parseFloat(wps[indexToMoveTo].latitude),
                longitude: parseFloat(wps[indexToMoveTo].longitude)
            };
        } else if (wps[indexToMoveTo].properties && wps[indexToMoveTo].properties.geoLatitude && wps[indexToMoveTo].properties.geoLongitude) {
            pos = {
                latitude: parseFloat(wps[indexToMoveTo].properties.geoLatitude),
                longitude: parseFloat(wps[indexToMoveTo].properties.geoLongitude)
            };
        } else {
            pos = {
                latitude: parseFloat(wps[indexToMoveTo].details.location.geoCoordinates.latitude),
                longitude: parseFloat(wps[indexToMoveTo].details.location.geoCoordinates.longitude)
            };
        }

        if (openInfoBubble) {
            routePlanner.indexOfFlagToHide = indexToMoveTo;

            if (__origZoomLevel > 0) {
                 __map.callMapApi("zoomToLevel", [__origZoomLevel, pos, true]);
                 __origZoomLevel = -1;
            }

            createPreviewInfoBubble(pos);
        }
    }

    function __updateRoute(dontOpenInfoBubble) {
        log.trace(">> updating route")

        PlannerLogic.clearStashedRoutes();

        model.clearRoute();

        routePlanner.__routingError = Router.ERROR_NONE;

        var wps = __fixModelWayPointsWithCurrentPosition(defaultCurrentPositionIndex);

        map.showWaypointIcons();

        if (wps[0] && wps[1]) {
            var indexToMoveTo = routePlanner.showInfoBubbleFromDestination? 1 : 0;
            hideFlagAndShowInfoBubble(wps, !dontOpenInfoBubble);
            model.autoMode = model.isAutoModeRecommended();
            model.calculate();
        } else {
            if (wps[0] || wps[1]) {
                routePlanner.waitingForCurrentPosition = true;
                log.info("not all set, waiting for the current position....")
            }
        }

        log.trace("<< updating route")
    }

    function __onCalculationDone(errorCode, violatedOptions) {
        routePlanner.__routingError = errorCode;
        PlannerLogic.stashRoute(map.route, errorCode);
        if (errorCode === Router.ERROR_NONE) {
            map.showRouteOnMap();
        } else if(status === PageStatus.Active) {
            routeErrorBanner.text = RoutingUtils.getErrorText(errorCode);
            routeErrorBanner.show();
            map.removeRouteFromMap();
        }
    }

    InfoBanner {
        id: routeErrorBanner
        timerEnabled: true
        timerShowTime: 3 * 1000
        width: parent.width - 2*routeErrorBanner.leftMargin

        anchors.top: header.bottom
        anchors.topMargin: 10
    }

    PlaceHelper { id: placeHelper }

    function formatAddress(properties) {
        return placeHelper.formatAddress(properties);
    }

    function createPreviewInfoBubble(infoBubbleCenterAt) {
        map.showInfoBubbleWithContentAt(infoBubbleCenterAt, bubbleComponent.createObject(routePlanner));
        if (routePlanner.indexOfFlagToHide >= 0) {
            map.hideWaypointIcons(routePlanner.indexOfFlagToHide);
        }
    }

    function populateCurrentlySelectedIcon(object) {
        PlannerLogic.populateCurrentlySelectedIcon(object);
    }

    function resetStartToCurrentPosition(clearAll, oldDest) {
        waitingForCurrentPosition = false;
        defaultCurrentPositionIndex = 0;
        if(!clearAll) {
            var tmpWaypoints = routePlanner.model.waypoints;
            tmpWaypoints[1] = oldDest;
            routePlanner.model.waypoints = tmpWaypoints;
            fakeLazyRouteUpdater.start();
            __map.attachedToLocation = false;
        }
        routePlanner.lastSelectedState = "setDestination";

    }

    states: [
        // [ACE]: we are taking care of the visibility of the objects during the transition animation sequence.
        State {
            name: ""
        },

        State {
            name: "setDestination"
        },
        State {
            name: "setStart"
        }
    ]

    transitions:  Transition {
        SequentialAnimation {
            ScriptAction {
                script: {
                    if (state === "setStart") {
                        // to field was just clicked
                        toField.visible = false;
                        fromField.visible = true;
                        swapButton.visible = false;
                    } else if (state === "setDestination") {
                        // from field was just clicked
                        toField.visible = true;
                        fromField.visible = false;
                        swapButton.visible = false;
                    } else if (state === "" && lastSelectedState === "setStart") {
                        // last selected state was the to field and we are returning from a places picker pop
                        toField.visible = false;
                        fromField.visible = true;
                        swapButton.visible = false;
                    } else if (state === "" && lastSelectedState === "setDestination") {
                        // last selected state was the from field and we are returning from a places picker pop
                        toField.visible = true;
                        fromField.visible = false;
                        swapButton.visible = false;
                    }
                }
            }

            PropertyAnimation {
                target: ((state == "setStart" || lastSelectedState == "setStart") ? fromField : toField);
                property: "width";
                to: (state == "" ? fieldWidth : parent.width - 32);
                from: (state == "" ? parent.width - 32 : fieldWidth);
                duration: 150
            }

            ScriptAction {
                script: {
                    if (state === "setStart") {
                        searchString = fromField.isCurrentPosition ? "" : fromField.text;
                        pageStack.push(placePicker, null, true);
                    } else if (state === "setDestination") {
                        searchString = toField.isCurrentPosition ? "" : toField.text;
                        pageStack.push(placePicker, null, true);
                    } else if (state === "") {
                        toField.visible = true;
                        fromField.visible = true;
                        swapButton.visible = true;
                    }
                }
            }
        }
    }
}
