import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0

import "../ui"
import ".."

ConnectorPage {
    id: routePreview
    property variant mainScreen: parent
    signal forward(variant error, variant route)

    property bool dontRemoveMap: false

    property alias mapFocus: header.isMapView // so we know if the map is focused. Used to toggle between map and list

    tools: ToolBarLayout {
        ToolIcon {
            iconId: "icon-m-toolbar-back"
            objectName: "toolBarIconBackButton"
            onClicked: {
                routePlannerFlow.previewJumpToMode = true;
                pageStack.pop(null, true);
            }
        }
        ToolButton {
            id: startButton

            platformStyle: ToolButtonStyle {
                disabledTextColor: pressedTextColor
            }

            objectName: "toolBarIconStartButton"
            text: qsTranslate("Maps", "qtn_navigationconnector_preview_button_start")

            onClicked: {
                forward(header.error, model);
            }
            visible: !!(model && !!model.calculated && model.mode !== "pt")
            enabled: model && model.waypoints && model.waypoints.length == 2 && model.waypoints[1] && !model.waypoints[1].fromGps? true : false
            anchors.centerIn: parent
        }
    }

    RouteViewer {
        id: map
        anchors {top: header.bottom; bottom: parent.bottom; left: parent.left; right: parent.right}
        route: routePreview.model
        behaviors: "_NO_LONG_PRESS_AT_ALL_:true;_NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_:true;_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_:true;_NO_INFO_BUBBLE_CLICK_EFFECT_:true;_NO_COMPASS_ALLOWED_:true;"
        model: map.modelFactory.createObject(routePreview)
        visible: true || mapFocus
        mainScreen: mainscreen
        onDeviceOrientationModeChanged: {
            if (route && route.calculated) {
                map.updateMap();
                map.zoomToRoute(false);
            }
        }
        onCurrentPositionChanged: {
            log.info("*** Position provider :: position updated RoutePreview");
            if (state === "waitingForPosition") {
                state = "";
                calculateRoute();
            }
        }
        onVisibleChanged: {
            if (map.visible) {
                if (route && route.calculated) {
                    map.zoomToRoute(true);
                }
            }
        }
    }

    RouteManeuversList {
        id: list
        anchors { top: header.bottom; bottom: parent.bottom; left: parent.left; right: parent.right }
        visible: !mapFocus
    }

    RouteHeader {
       id: header;
       mapView: map
       route: routePreview.model
       state: routePreview.state
       showMapListToggle: true
       onClicked: {
           if (header.isMapView) {
               focusRoutePreview();
           }
       }
    }

    SettingsPersistence { id: db }

    function loadSettings() {
        db.get("audioCommandsOn", function(value) {
           viewMenu.audioChecked = (value === "true");
           db.get("voiceSkinId", function(skinId) {
                      viewMenu.audioEnabled = skinId > 0;
                  }, function(error) {
                      log.info("Unable to get voice skin");
                  });

       }, function(error) {
           log.info("Unable to get voice commands on");
       });
    }

    onStatusChanged: {
        if (status == PageStatus.Active) {   // activated
            if (model.calculated) {
                showRoutePreview();
            } else {
                model.calculationDone.connect(onCalculationDone);
                calculateRoute();
            }
            dontRemoveMap = false;
        } else if (status == PageStatus.Inactive) {    //Inactive
            if (!dontRemoveMap) {
                map.removeRouteFromMap();
                map.destroy();
            }
        }
    }

    function calculateRoute() {
        map.showWaypointIcons();
        map.moveToDestination();

        if (model.waypoints[0]) {
            state = "calculatingRoute"
            model.calculate();
        } else {
            if (map.positionType == map._POSITION_TYPE_GPS_) {
                var wps = model.waypoints;
                wps[0] = {latitude: map.currentPosition.geoCoordinates.latitude, longitude: map.currentPosition.geoCoordinates.longitude};
                model.waypoints = wps;
                state = "calculatingRoute"
                model.calculate();
            } else {
                state = "waitingForPosition";
            }
        }
    }

    function focusRoutePreview() {
        map.showRouteOnMap();
        map.zoomToRoute(true);
        map.showWaypointIcons();
    }

    function showRoutePreview() {
        state = "";

        // map
        focusRoutePreview();

        // list
        list.fillList(model);
    }

    function showRoutingError(errorCode) {
        header.error = errorCode;
    }

    function onCalculationDone(errorCode, violatedOptions) {
        state = "";
        if (errorCode === Router.ERROR_NONE) {
            showRoutePreview();
        } else {
            showRoutingError(errorCode);
        }
    }
}
