import QtQuick 1.0

import MapsPlugin 1.0

import com.nokia.meego 1.0

import ovi.connector.map 1.0
import ovi.connector.places 1.0

import "../ManeuverConstants.js" as ManeuverConstants

// Guidance header
Header {

    property variant guidance
    property bool showRepeatButton: false

    signal repeatButtonClicked

    id: header

    function showJustStarted() {
        maneuverDistance.visible = false;

        maneuverIcon.source = navigationAssetsPath + "images/maneuver_icons/meegotouch-seattle-maps-maneuver-icon-" +
                1 + ".png";
        streetLabel.text = qsTranslate("Maps", "qtn_navigationconnector_guidanceheader_defaultstarttext");
    }

    function updateHeader(aManeuver, aDistance, aSpeed) {
        if (aManeuver) {
            var streetNameText = aManeuver.nextStreetName;

            if (!streetNameText && aManeuver.roadElements && aManeuver.roadElements[0] &&
                    aManeuver.roadElements[0].formOfWay === RoadElement.FOW_PEDESTRIAN_WALKWAY) {
                streetNameText = qsTranslate("Maps", "qtn_navigationconnector_guidanceheader_streetname_pedestrian_walkway"); // Footpath
            }

            streetLabel.text = streetNameText || qsTranslate("Maps", "qtn_navigationconnector_guidanceheader_no_streetname"); // Unnamed street
            maneuverIcon.source = ManeuverConstants.MANEUVER_ICON_INDEXES[aManeuver.icon] !== undefined ?
                    navigationAssetsPath + "images/maneuver_icons/meegotouch-seattle-maps-maneuver-icon-%1.png".arg(
                        ManeuverConstants.MANEUVER_ICON_INDEXES[aManeuver.icon]) :
                    "";
        }

        maneuverDistance.visible = !(!aDistance || aDistance <= 0);
        maneuverDistance.text =
            !aDistance || aDistance <= 0 ? "" : units.normalizeDistance(aDistance).readableWithUnits;

    }

    /**
      * aDestination - {Place} the destination of the route
      */
    function showDestinationReached(aDestination) {
        maneuverDistance.visible = false;
        maneuverIcon.source = navigationAssetsPath + "images/maneuver_icons/meegotouch-seattle-maps-maneuver-icon-" +
                ManeuverConstants.MANEUVER_ICON_INDEXES[Maneuver.ICON_END] + ".png";
        var destText = qsTranslate("Maps", "qtn_navigationconnector_guidanceheader_defaultdestinationtext");
        if (aDestination && aDestination.properties) {
            destText = (properties.addrStreetName || properties.street ||  "") + " "  + (properties.addrHouseNumber || properties.houseNumber || "");
        }
        streetLabel.text = destText;
    }

    function showRouteUpdated() {
        streetLabel.text = qsTranslate("Maps", "qtn_navigationconnector_guidanceheader_reroutetext");
    }

    Units { id: units }

    Item {
        id: bigButtonContent
        anchors { leftMargin: 10; rightMargin: 10; fill: parent; }
        z: 2

        Item {
            id: maneuver

            anchors {
                left: parent.left;
                leftMargin: 15;
                top: parent.top;
                topMargin: 8;
                bottom: parent.bottom;
            }
            width: maneuverIcon.width + 22;
            height: maneuverDistance.y + maneuverDistance.height

            // Maneuver arrow
            Image {
                id: maneuverIcon
                objectName: "maneuverIconImage"
                anchors {
                    left: parent.left;
                    top: parent.top;
                    topMargin: (maneuverDistance.text === "" ? maneuverIcon.height*.2 : -2)
                }
                fillMode: Image.PreserveAspectCrop
                height: 50
                width: 50
                smooth: true
            }

            // Maneuver distance
            Text {
                id: maneuverDistance
                objectName: "maneuverDistanceText"
                font.pixelSize: 18;
                color: bigButton.pressed ? "#FFF" : "#000";
                anchors {
                    left: maneuver.left
                    top: maneuverIcon.bottom
                    topMargin: -2
                }
            }
        }

        // CurrentStreet
        Text {
            id: streetLabel
            objectName: "currentStreetLabel"
            font.pixelSize: 28
            color: bigButton.pressed ? "#FFF" : "#000"
            width: bigButton.width
            font.bold: true
            elide: Text.ElideRight
            anchors {
                left: maneuver.right;
                leftMargin: 10;
                right: parent.right;
                rightMargin: repeatImg.visible ? repeatImg.width : 0;
                verticalCenter: parent.verticalCenter;
            }
        }

    }

    Button {
        id: bigButton
        visible: showRepeatButton
        anchors { fill: parent; leftMargin: 10; rightMargin: 10; topMargin: 5; bottomMargin: 5; }
        onClicked: repeatButtonClicked();
        z: 1

        // The repeat button
        Image {
            id: repeatImg;
            anchors { right: parent.right; rightMargin: 10; verticalCenter: parent.verticalCenter; }
            source: navigationAssetsPath + "images/meegotouch-seattle-maps-guidance-voice-command.png";
        }
    }

    Component.onCompleted: {
        ManeuverConstants.createManeuverIconIndexMap();
    }
}
