import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0

import "../RoutingUtils.js" as RoutingUtils

Item {
    id: toolbarLayout;
    anchors.fill: parent;

    property string mode: "buttons"
    property variant guidance;
    property bool isLandscape: true;
    property bool isInOverviewMode: false
    property int buttonLayoutTimeout: 5000;
    property variant toolbar;
    property alias toolbarTimer: buttonLayoutTimer
    property variant _walkingSpeed: 4  // 4km/h === 10m/9sec

    signal stopClicked()
    signal overviewClicked()
    signal buttonLayoutTimeoutReached()
    signal viewMenuClicked()
    signal backClicked()

    property variant __distObj: guidance.destinationDistance >= 0 ? units.normalizeDistance(guidance.destinationDistance) : null
    property variant __timeObj: guidance.destinationDistance >= 0 ? units.normalizeDuration((guidance.destinationDistance)) : null
    property variant __speedObj

    function updateGuidance(aGuidance, aSpeed) {
        guidance = aGuidance;
        __distObj = guidance.destinationDistance >= 0 ? units.normalizeDistance(guidance.destinationDistance) : null;
        __timeObj = guidance.destinationDistance >= 0 ? units.normalizeDuration((guidance.destinationDistance)) : null
//        __timeObj = guidance.timeToArrival >= 0 ? units.normalizeDuration(guidance.timeToArrival) : null;
        __speedObj = units.normalizeSpeed(aSpeed);
    }

    onModeChanged: {
        if (mode == "finished") {
            buttonLayoutTimer.stop();

            dashboardButtonLayout.visible = false;
            dashboardInfoLayout.visible = false;
            distanceFinished.visible = true;
        } else if (mode == "buttons") {
            buttonLayoutTimer.restart()
        } else {
            buttonLayoutTimer.stop();
        }

        toolbar.setTools(getLayout(mode));
    }

    Timer {
        id: buttonLayoutTimer;
        interval:  buttonLayoutTimeout;
        onTriggered: buttonLayoutTimeoutReached();
    }

    ToolBarLayout {
        id: distanceFinished
        objectName: "distanceFinished"
        visible: false

        ToolIcon {
            iconId: "icon-m-toolbar-back"
            objectName: "backTabBtn"

            onClicked: { backClicked() }
        }
    }

    ToolBarLayout {
        id: dashboardButtonLayout
        objectName: "dashboardButtonLayout"
        visible: false

        ToolButtonRow {
            ToolButton {
                id: stopBtn
                objectName: "stopBtn"
                text: qsTranslate("Maps", "qtn_navigationconnector_guidancetoolbar_stopbutton")
                onClicked: stopClicked()
            }
            ToolButton {
                id: overviewBtn
                objectName: "overviewBtn"
                Image {
                    anchors.centerIn: parent;
                    source: navigationAssetsPath +
                                (isInOverviewMode ? "images/toolbar_bt_zoom.png": "images/toolbar_bt_overview.png");
                }
                onClicked: overviewClicked()
            }
        }
        ToolIcon {
            iconId: "icon-m-toolbar-view-menu"
            objectName: "toolBarIconViewButton"
            onClicked: viewMenuClicked()
        }
    }

    ToolBarLayout {
        id: dashboardInfoLayout
        objectName: "dashboardInfoLayout"
        visible: false

        Row {
            anchors.fill: parent

            Item {
                id: dtd
                anchors {
                    left: parent.left
                    top: parent.top
                    bottom: parent.bottom
                }
                width: parent.width/3
                Item {
                    height: Math.max(distImg.height, distText.height)
                    width: distImg.width + distText.width
                    anchors.centerIn: parent
                    Image {
                        id: distImg;
                        source: navigationAssetsPath + "images/distance_left.png";
                        anchors.left: parent.left
                    }
                    Item {
                        id: distText
                        anchors.left: distImg.right;
                        anchors.leftMargin: 5;
                        anchors.bottom: distImg.bottom;
                        width: distNbr.paintedWidth + distUnit.paintedWidth + distUnit.anchors.leftMargin
                        MapsText {
                            id: distNbr;
                            objectName: "distTxt";
                            text: __distObj ? ("~" + __distObj.readableWithoutUnits) : "";
                            platformStyle: LabelStyle {fontPixelSize: 36}
                            anchors.baseline: parent.bottom;
                        }
                        MapsText {
                            id: distUnit;
                            objectName: "distUnit";
                            text: __distObj ? "" + __distObj.unit : "";
                            platformStyle: LabelStyle {fontPixelSize: 21}
                            anchors {
                                left: distNbr.right
                                leftMargin: 5
                                baseline: parent.bottom
                            }
                        }
                    }
                }
            }
            Item {
                anchors {
                    top: parent.top
                    bottom: parent.bottom
                }
                width: parent.width/3
                Item {
                    height: Math.max(ttaImg.height, ttaText.height)
                    width: ttaImg.width + ttaText.width
                    anchors.centerIn: parent

                    Image {
                        id: ttaImg;
                        source: navigationAssetsPath + "images/tta.png";
                        anchors.left: parent.left
                    }
                    Item {
                        id: ttaText;
                        anchors.left: ttaImg.right;
                        anchors.leftMargin: 5;
                        anchors.bottom: ttaImg.bottom;
                        width: ttaNbr.paintedWidth + ttaUnit.paintedWidth + ttaUnit.anchors.leftMargin
                        MapsText {
                            id: ttaNbr;
                            objectName: "ttaTxt";
                            text: __timeObj ? ("~" + __timeObj.readableWithoutUnits) : "";
                            platformStyle: LabelStyle { fontPixelSize: 36 }
                            anchors.baseline: parent.bottom;
                        }
                        MapsText {
                            id: ttaUnit;
                            objectName: "ttaUnit";
                            text: __timeObj ? "" +  __timeObj.unit : "";
                            anchors {
                                left: ttaNbr.right;
                                leftMargin: 5;
                                baseline: ttaNbr.baseline;
                            }
                            platformStyle: LabelStyle { fontPixelSize: 21 }
                        }
                    }
                }
            }
            Item {
                anchors {
                    top: parent.top
                    bottom: parent.bottom
                    right: parent.right
                }
                width: parent.width/3
                Item {
                    height: Math.max(speedImg.height, speedText.height)
                    width: speedImg.width + speedText.width
                    anchors.centerIn: parent
                    Image {
                        id: speedImg;
                        source: navigationAssetsPath + "images/speed.png";
                        anchors.left: parent.left
                    }
                    Item {
                        id: speedText
                        anchors.left: speedImg.right;
                        anchors.leftMargin: 5;
                        anchors.bottom: speedImg.bottom;
                        width: speedNbr.paintedWidth + speedUnit.paintedWidth + speedUnit.anchors.leftMargin
                        MapsText {
                            id: speedNbr;
                            objectName: "speedTxt";
                            text: __speedObj ? __speedObj.readableWithoutUnits : "";
                            platformStyle: LabelStyle { fontPixelSize: 36 }
                            anchors.baseline: parent.bottom;
                        }
                        MapsText {
                            id: speedUnit;
                            objectName: "speedUnit";
                            text: __speedObj ? "" +  __speedObj.unit : "";
                            anchors {
                                left: speedNbr.right;
                                leftMargin: 5;
                                baseline: speedNbr.baseline;
                            }
                            platformStyle: LabelStyle { fontPixelSize: 21 }
                        }
                    }
                }
            }
        }
        MouseArea {
            anchors.fill: parent;
            onClicked: {
                mode = "buttons";
            }
        }
        /*
        Rectangle {
            height: parent.height;
            color: "#00000000"
            anchors {
                left: parent.left;
                leftMargin: toolbarLayout.isLandscape ? 230 : 75;
//                right: tta.left;
                verticalCenter: tta.verticalCenter
            }

            Image {
                id: distImg;
                source: navigationAssetsPath + "images/distance_left.png";
                anchors.verticalCenter: parent.verticalCenter;
                anchors.horizontalCenter: parent.horizontalCenter;
                anchors.horizontalCenterOffset: ttaImg.anchors.horizontalCenterOffset-((distNbr.width+distUnit.width)/4);
            }
            Item {
                id: distText
                anchors.left: distImg.right;
                anchors.leftMargin: 5;
                anchors.bottom: distImg.bottom;
                Text {
                    id: distNbr;
                    objectName: "distTxt";
                    text: __distObj ? __distObj.readableWithoutUnits : "";
                    font.pixelSize: 36;
                    anchors.baseline: parent.bottom;
                    color: "#282828"
                }
                Text {
                    id: distUnit;
                    objectName: "distUnit";
                    text: __distObj ? "" + __distObj.unit : "";
                    anchors.left: distNbr.right;
                    anchors.leftMargin: 5
                    font.pixelSize: 21;
                    font.weight: Font.Light;
                    anchors.baseline: parent.bottom;
                    color: "#505050"
                }
            }
        }
        Rectangle {
            id: tta;
            height: parent.height;
            width: toolbarLayout.width*0.25;
            anchors.horizontalCenter: parent.horizontalCenter;
            anchors.verticalCenter: parent.verticalCenter;
            anchors.verticalCenterOffset: 5;
            color: "#00000000";
            Image {
                id: ttaImg;
                anchors.verticalCenter: parent.verticalCenter;
                anchors.horizontalCenter: parent.horizontalCenter;
                anchors.horizontalCenterOffset: -1*(ttaNbr.width + ttaUnit.width)/2;
                source: navigationAssetsPath + "images/tta.png";
            }
            Item {
                id: ttaText;
                anchors.left: ttaImg.right;
                anchors.leftMargin: 5;
                anchors.bottom: ttaImg.bottom;

                Text {
                    id: ttaNbr;
                    objectName: "ttaTxt";
                    text: __timeObj ? __timeObj.readableWithoutUnits : "";
                    font.pixelSize: 36;
                    anchors.baseline: parent.bottom;
                    color: "#282828"
                }
                Text {
                    id: ttaUnit;
                    objectName: "ttaUnit";
                    text: __timeObj ? "" +  __timeObj.unit : "";
                    anchors.left: ttaNbr.right;
                    anchors.leftMargin: 5;
                    font.pixelSize: 21;
                    font.weight: Font.Light;
                    anchors.baseline: ttaNbr.baseline;
                    color: "#505050"
                }
            }
        }

        Rectangle {
            height: parent.height;
            color: "#00000000"
            anchors {
//                left: tta.right;
                verticalCenter: tta.verticalCenter
                right: parent.right
                rightMargin: toolbarLayout.isLandscape ? 285 : 110
            }
            Image {
                id: speedImg;
                source: navigationAssetsPath + "images/speed.png";
                anchors.verticalCenter: parent.verticalCenter;
                anchors.horizontalCenter: parent.horizontalCenter;
//                anchors.horizontalCenterOffset: ttaImg.anchors.horizontalCenterOffset-(speedTxt.width/4);
            }

            Item {
                anchors.left: speedImg.right;
                anchors.leftMargin: 5;
                anchors.bottom: speedImg.bottom;
                Text {
                    id: speedNbr;
                    objectName: "speedTxt";
                    text: __speedObj ? __speedObj.readableWithoutUnits : "";
                    font.pixelSize: 36;
                    anchors.baseline: parent.bottom;
                    color: "#282828"
                }
                Text {
                    id: speedUnit;
                    objectName: "speedUnit";
                    text: __speedObj ? "" +  __speedObj.unit : "";
                    anchors.left: speedNbr.right;
                    anchors.leftMargin: 5;
                    font.pixelSize: 21;
                    font.weight: Font.Light;
                    anchors.baseline: speedNbr.baseline;
                    color: "#505050"
                }
            }
        }

        MouseArea {
            anchors.fill: parent;
            onClicked: {
                mode = "buttons";
            }
        }
        */
    }

    Units { id: units }

    /**
      * getLayout, types: "info", "buttons", "finished"
      */
    function getLayout(type) {
        if (type == "info") {
            return dashboardInfoLayout;
        } else if (type == "buttons") {
            return dashboardButtonLayout;
        } else {
            return distanceFinished;
        }
    }
}
