import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.places 1.0

PlacesCommonItemDelegate {

    property alias maneuverIcon: maneuverIconImage.source
    property alias actionText: directionId.text
    property alias destinationText: streetNameTextId.text
    property alias distanceText: distanceTextId.text
    property alias distanceVisible: distanceTextId.visible
    property alias color: background.color
    property alias maneuverIconObject: maneuverIcon
    property bool formatNoActionText: true
    property int extraIconIndent: 0

    height: background.height

    Rectangle {
        id: background
        anchors { left: parent.left; right: parent.right }
        height: backgroundContent.height + 36

        color: "#f0f1f2"
        Rectangle {
            id: backgroundContent
            color: "#00000000"


            height: Math.max(maneuverIcon.height, content.height)
            anchors { left: parent.left; right: parent.right; rightMargin: 16; leftMargin: 16 + extraIconIndent; verticalCenter: parent.verticalCenter }

            Item {
                id: maneuverIcon
                objectName: "maneuverIconImage"
                anchors {
                    left: parent.left
                    top: parent.top
                }
                height: 64; width: 64
                Image {
                    id: maneuverIconImage
                    height: parent.height
                    fillMode: "PreserveAspectFit"
                    anchors.centerIn: parent
                }
            }

            Item {
                id: content

                anchors {
                    left: maneuverIcon.right
                    leftMargin: 16
                    top: parent.top
                    topMargin: 6
                    bottomMargin: 6
                    right: parent.right
                }
                height: directionId.paintedHeight +
                        streetNameTextId.paintedHeight +
                        content.anchors.topMargin +
                        content.anchors.bottomMargin

                MapsText {
                    id: directionId
                    objectName: "directionText"
                    anchors {
                        left: parent.left
                        right: distanceTextId.left
                        rightMargin: 30
                    }
                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                    platformStyle: LabelStyle { textColor: directionId.secondaryColor }
                }

                MapsText {
                    id: streetNameTextId
                    objectName: "steetNameText"

                    anchors {
                        left: directionId.left
                        top: directionId.bottom
                        right: parent.right
                        rightMargin: 10
                    }
                    platformStyle: LabelStyle {fontPixelSize: 26}
                    font.weight: Font.Bold
                    wrapMode: Text.Wrap;
                    elide: "ElideNone"
                }

                MapsText {
                    id: distanceTextId
                    objectName: "distanceText"
                    state: "smallerText"
                    anchors {
                        right:  parent.right
                        top: parent.top
                        topMargin: 4
                    }
                }
            }
        }
    }

    states: [
        State {
            name: "no_action"
            when: formatNoActionText && actionText.length === 0

            PropertyChanges {
                target: streetNameTextId
                font {
                    pixelSize: 22
                    weight: Font.Light
                    letterSpacing: -1
                }
                verticalAlignment: Text.AlignVCenter
            }

            AnchorChanges {
                target: streetNameTextId
                anchors {
                    top: content.top
                    bottom: content.bottom
                }
            }
        },
        State {
            name: "no_action_centered"
            when: actionText.length === 0

            PropertyChanges {
                target: streetNameTextId
                verticalAlignment: Text.AlignVCenter
            }

            AnchorChanges {
                target: streetNameTextId
                anchors {
                    top: content.top
                    bottom: content.bottom
                }
            }
        }
    ]

}
