import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0 as MapsPlugin

import ovi.connector.map 1.0

import "../RoutingUtils.js" as RoutingUtils

Item {
    id: previewBubble;

    signal goClicked;
    signal repaintRequested;

    property variant route;
    property bool isLandscape: false; // This should be set by whoever is calling this object
    property string transportMode
    property bool start: false
    property int routingError: 0
    property alias destText: destTitle.text

    property int maxWidth: 610

    width: isLandscape ? 510 : 342;
    height: isLandscape ? 160 : 220;
    anchors.bottom: parent.bottom;
    anchors.bottomMargin: 16;

    onIsLandscapeChanged: { repaintRequested() }

    Item {
        id: flagAndInfoItem
        height: 51;
        anchors {
            left: parent.left;
            bottom: isLandscape ? segButton.top : segButton.top;
            right: parent.right;
            leftMargin: 16;
            rightMargin: 16;
            bottomMargin: isLandscape ? 26 : 26;
        }
        Image {
            id: flagImage;
            objectName: "flagImage";
            source: navigationAssetsPath + "images/waypoint_marker_%1.png".arg(start ? "start" : "destination");
            height: 40;
            width: 40;
            anchors.verticalCenter: parent.verticalCenter;
        }
        Column {
            id: textCol;
            width: flagAndInfoItem.width - flagImage.width - 12;
            anchors.left: flagImage.right;
            anchors.leftMargin: 12;
            Text {
                id: destTitle
                objectName: "destTitle"
                width: parent.width
                color: "#FFF"
                font.pixelSize: 26
                font.weight: Font.Bold
                elide: Text.ElideRight
            }
            Text {
                id: routeInfo
                objectName: "routeInfo"
                text: __getRouteInfo(route)
                color: "#FFF"
                font.pixelSize: 22
                elide: Text.ElideRight
                width: parent.width
            }
        }

    }
    Rectangle {
        height: 1;
        color: "#A1A1A1";
        opacity: isLandscape ? 0.5 : 0.5;
        anchors.top: flagAndInfoItem.bottom;
        anchors.topMargin: 10;
        anchors.left: parent.left;
        anchors.right: parent.right;
    }

    ButtonRow {
        id: segButton;
        anchors.left: parent.left
        anchors.leftMargin: isLandscape? 16 : 16
        anchors.bottomMargin: isLandscape ? 0 : 12;
        anchors.bottom: isLandscape ? parent.bottom : goBtn.top;
        height: 51;
        width: isLandscape ? 222 : previewBubble.width - 32;
        platformStyle: ButtonStyle { inverted: true;}

        Button {
            id: walkBtn;
            objectName: "walkBtn";
            checked: previewBubble.transportMode === "walk";
            Image {
                anchors.fill: parent;
                fillMode: Image.PreserveAspectFit;
                source: navigationAssetsPath + "images/meegotouch-seattle-maps-directions-infobubble-icon-walk" +
                        ( walkBtn.enabled ? (walkBtn.pressed ? "-pressed" : "") : "-disabled") + ".png"
                visible: !walkBusy.visible
            }
            BusyIndicator {
                id: walkBusy
                anchors.centerIn: parent
                running: true
                platformStyle: BusyIndicatorStyle { inverted: true }
                visible: !parent.enabled && previewBubble.transportMode === "walk"
            }
            onClicked: previewBubble.transportMode = "walk";
        }
        Button {
            id: ptBtn;
            objectName: "ptBtn";
            checked: previewBubble.transportMode === "pt"
            Image {
                anchors.fill: parent;
                fillMode: Image.PreserveAspectFit;
                source: navigationAssetsPath + "images/meegotouch-seattle-maps-directions-infobubble-icon-pt" +
                        ( ptBtn.enabled ? (ptBtn.pressed ? "-pressed" : "") : "-disabled") + ".png"
                visible: !ptBusy.visible
            }
            BusyIndicator {
                id: ptBusy
                anchors.centerIn: parent
                running: true
                platformStyle: BusyIndicatorStyle { inverted: true }
                visible: !parent.enabled && previewBubble.transportMode === "pt"
            }
            onClicked: previewBubble.transportMode = "pt";
        }
        Button {
            id: driveBtn;
            objectName: "driveBtn";
            checked: previewBubble.transportMode === "drive"
            Image {
                anchors.fill: parent;
                fillMode: Image.PreserveAspectFit;
                source: navigationAssetsPath + "images/meegotouch-seattle-maps-directions-infobubble-icon-drive" +
                        ( driveBtn.enabled ? (driveBtn.pressed ? "-pressed" : "") : "-disabled") + ".png"
                visible: !driveBusy.visible
            }
            BusyIndicator {
                id: driveBusy
                anchors.centerIn: parent
                running: true
                platformStyle: BusyIndicatorStyle { inverted: true }
                visible: !parent.enabled && previewBubble.transportMode === "drive"
            }

            onClicked: previewBubble.transportMode = "drive";
        }
    }

    Button {
        id: goBtn;
        objectName: "goBtn";
        platformStyle: ButtonStyle {inverted: true}
        anchors.right: parent.right;
        anchors.rightMargin: 16;
        anchors.bottom: parent.bottom;
        text: qsTranslate("Maps", "qtn_navigationconnector_infobubble_go");
        width: isLandscape ? previewBubble.width - segButton.width - 32 * 2 : previewBubble.width - 32;
        onClicked: goClicked();
        enabled: true
    }

    Units{ id: units }

    onRouteChanged: {
        if (route.mode === "walk") {
            walkBtn.checked = true;
        } else if (route.mode === "pt") {
            ptBtn.checked = true;
        } else if (route.mode === "drive") {
            driveBtn.checked = true;
        }
    }

    function __getRouteInfo(route) {
        if (route && route.length > 0) {
            return "~%1 | ~%2".arg(
                units.normalizeDistance(route.length).readableWithUnits).arg(
                units.normalizeDuration(route.duration).readableWithUnits
            );
        }

        return "";
    }

    states: [
        State {
            name: "calculating_route"
            when: routingError === 0 && !route.calculated
            PropertyChanges {
                // show spinner?
                target: driveBtn
                enabled: false
            }
            PropertyChanges {
                target: ptBtn
                enabled: false
            }
            PropertyChanges {
                target: walkBtn
                enabled: false
            }

            PropertyChanges {
                target: goBtn
                enabled: false
            }
            PropertyChanges {
                target: routeInfo
                text: ""
            }
        },
        State {
            name: "routing_error"
            when: routingError !== 0
            PropertyChanges {
                target: routeInfo
                text: qsTranslate("Maps", "qtn_navigationconnector_routeplanner_route_error_notsupported")
            }
            PropertyChanges {
                target: goBtn
                enabled: false
            }
        }
    ]
}
