import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector.map 1.0
import ovi.connector.places 1.0

import "RouteManeuversList.js" as RouteManeuversListLogic
import "../ManeuverConstants.js" as ManeuverConstants

Column {
    property variant maneuver
    property bool expanded: true
    property bool __showPtDetails: expanded && maneuver.action !== 17
    property bool __showWalkDetails: expanded && maneuver.action === 17
    property int subManeuverIndentAmount: 10
    property string pointHilightColor: "#e7e8e9"

    TransitDatabase {
        id: transitDb
    }

    Units { id: units }

    ManeuverDelegate {
        id: ptTitleStation
        visible: false
        formatNoActionText: false
        maneuverIcon: RouteManeuversListLogic.getPtStationIcon(maneuver, transitDb)
        destinationText: maneuver.departureStation && maneuver.departureStation.name
        color: pointHilightColor
        partial: false
    }

    ManeuverDelegate {
        id: ptActionSummary
        visible: false
        showSeparator: false
        maneuverIcon: maneuver.type === "pt" && RouteManeuversListLogic.getPtSystemIcon(maneuver)
        actionText: qsTranslate("Maps", "qtn_navigationconnector_ptmaneuverdelegate_takeline").arg(maneuver.lineName)
        destinationText: {
            if(maneuver.destination !== "")
                return qsTranslate("Maps", "qtn_navigationconnector_ptmaneuverdelegate_towards").arg(maneuver.destination)
        }
    }

    Repeater {
        id: partPtList

        anchors {left: parent.left; right: parent.right}
        visible: true

        delegate: PtManeuverStopDelegate {
            isFirst: index === 0
            isLast: index === partPtList.count - 1
            destinationText: model.name
            lineColor: maneuver.primaryLineColor
            distanceText: (isFirst || isLast ?
                               (isFirst ? qsTranslate("Maps", 'qtn_navigationconnector_maneuverlist_stops').arg(maneuver.numStops + 1) :
                                units.normalizeDuration(maneuver.travelTime).readableWithUnits) :
                           "")
            extraIconIndent: subManeuverIndentAmount
            showSeparator: isFirst
            // Uncomment this to show the animation and add a mouse area that sets expanded = !expanded when clicked
            //            show: __showPtDetails
        }
    }

    ManeuverDelegate {
        id: ptEndStation
        visible: false
        formatNoActionText: false
        maneuverIcon: RouteManeuversListLogic.getPtStationIcon(maneuver, transitDb, true)
        destinationText: maneuver.arrivalStation && maneuver.arrivalStation.name
        color: pointHilightColor
    }

    Repeater {
        id: partWalkList

        anchors {left: parent.left; right: parent.right}
        visible: true
        delegate: WalkDriveManeuverDelegate {
            maneuver: __enrichManeuver(model, index)
            extraIconIndent: subManeuverIndentAmount
            color: RouteManeuversListLogic.useHighlightColor(maneuver, index) ? pointHilightColor : "#00000000"
            partial: RouteManeuversListLogic.showPartialWalkSeparator(partWalkList.model, index)
            showSeparator: RouteManeuversListLogic.showWalkSeparator(partWalkList.model, index)
//            visible: (model.action !== 0 && index !== 0)
            // Uncomment this to show the animation and add a mouse area that sets expanded = !expanded when clicked
            // show: __showWalkDetails
        }
    }

    states: [
        State {
            name: "pt"
            when: maneuver.action !== 17
            PropertyChanges {
                target: ptTitleStation
                visible: true
            }
            PropertyChanges {
                target: ptActionSummary
                visible: true
            }
            PropertyChanges {
                target: ptEndStation
                visible: true && ptEndStation.destinationText !== ptEndStation.destinationText
            }
        }
    ]

    function __enrichManeuver(aManeuver, listIndex) {
        return RouteManeuversListLogic.enrichManeuverItem(aManeuver, ManeuverConstants.MANEUVER_ICON_INDEXES,
               ManeuverConstants.getDescription,
               function (distance) {
                  return (aManeuver.action === 100 ? "" : qsTranslate("Maps", 'qtn_navigationconnector_maneuverdistance_prefix')+' ') + units.normalizeDistance(distance).readableWithUnits;
               },
               listIndex,
              !(listIndex === 0 && aManeuver.action === 0) && aManeuver.icon !== MapsPlugin.Maneuver.ICON_END && aManeuver.action !== 100
        );
    }

    Component.onCompleted: {
        ManeuverConstants.createManeuverIconIndexMap();
        if (state === "pt") {
            partPtList.model = maneuver.stationStops;
        } else {
            partWalkList.model = maneuver.walkManeuvers;
        }
    }
}
