import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0 as MapsPlugin

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.places 1.0

import "../RoutingUtils.js" as RoutingUtils

Header {
    id: header;

    signal clicked
    property alias isMapView: mapToggle.mapActive
    property variant mapView
    property variant route // use this to set the information in the header.
    property int error: 0 // Router.ERROR_XYZ indicating a routing error
    property bool showMapListToggle: false

    Units{ id: units }
    dropShadow: mapToggle.mapActive

    // [ace]: including mouse area here so it doesn't eat the button clicked if the map/list toggle is there
    MouseArea {
        anchors.fill: parent
        onClicked: header.clicked()
    }

    Image {
        id: headerImage;
        source: model ? __getModeIcon(model) : ""
        fillMode: Image.PreserveAspectFit
        height: sourceSize.height
        width: sourceSize.width
        smooth: true
        anchors {
            left: parent.left
            leftMargin: 16
            top: parent.top
            topMargin: 8
        }
    }

    Column {
        anchors.left: headerImage.right
        anchors.leftMargin: 16
        anchors.right: mapToggle.left
        anchors.rightMargin: 15
        anchors.verticalCenter: parent.verticalCenter

        Item {
            anchors.left: parent.left
            anchors.right: parent.right
            height: headerLine1to.height

            MapsText {
                id: headerLine1to
                text: qsTranslate("Maps", "qtn_navigationconnector_preview_to")
                platformStyle: LabelStyle { fontPixelSize: 26}
                anchors.left: parent.left
            }
            MapsText {
                id: headerLine1dest
                objectName: "routeOverviewDestination"
                platformStyle: LabelStyle {fontPixelSize: 26 }
                font.weight: Font.Bold
                anchors.left: headerLine1to.right
                anchors.leftMargin: 10
                anchors.right: parent.right
                text: route && route.waypoints ?
                            __getDestinationText(route.waypoints[route.waypoints.length - 1]) :
                            qsTranslate("Maps", "qtn_navigationconnector_preview_loading")
            }
        }
        MapsText {
            id: headerLine2
            objectName: "routeOverviewDistanceTime"
            text: header.error === 0 ? __getRouteInfo(model) : RoutingUtils.getErrorText(header.error)
            anchors {
                left: parent.left
                right: parent.right
            }
            platformStyle: LabelStyle {textColor: headerLine2.secondaryColor}
        }
    }

    MapListToggle {
        id: mapToggle;
        anchors.right: parent.right;
        anchors.rightMargin: 15;
        anchors.verticalCenter: parent.verticalCenter;
        visible: showMapListToggle;
    }

    ReverseGeoCodeFinder {
        id: finder

        mapView: header.mapView

        onFindPureDataComplete: {
            headerLine1dest.text = contextData["ADDR_CITY_NAME"] +
                    (contextData["ADDR_STREET_NAME"]?
                         (
                             " " + contextData["ADDR_STREET_NAME"] +
                             (contextData["ADDR_HOUSE_NUMBER"]?
                                  (" " + contextData["ADDR_HOUSE_NUMBER"])
                                  :
                                  ""
                              )
                         )
                         :
                         ""
                     );
        }
    }

    function __getDestinationText(dest) {
        if (!dest) {
            return  qsTranslate("Maps", "qtn_navigationconnector_preview_loading");
        }

        var result = dest && (dest.name || (dest.properties && dest.properties.title));
        var toSearch = false;
        if (!result && dest.latitude && dest.longitude) {
            finder.find({ latitude: dest.latitude, longitude: dest.longitude }, true);
            toSearch = true;
        }
        headerLine1dest.text = result || (toSearch? qsTranslate("Maps", "qtn_navigationconnector_preview_loading") : "");
    }

    function __getModeIcon(route) {
        if (route) {
            var routeMode = route.mode;
            if (routeMode === "walk" || routeMode === "drive" || routeMode === "pt") {
                return navigationAssetsPath + "images/maps-icon-"+ routeMode + "-headerbar.png";
            }
        }

        return "";
    }

    function __getRouteInfo(route) {
        var routeMode = route && route.calculated ? route.mode : "";

        var infoValue = "";
        if (routeMode === "walk" || routeMode === "pt" || routeMode === "drive") {
            infoValue = "~%1 | ~%2".arg(
                units.normalizeDistance(route.length).readableWithUnits).arg(
                units.normalizeDuration(route.duration).readableWithUnits
            );
        }
        return infoValue;
    }

    onRouteChanged: {
        delayedStarted.start();
    }

    Timer {
        id: delayedStarted
        running: false
        interval: 1000 * 3

        onTriggered: {
            if (route) {
                __getDestinationText(route.waypoints[route.waypoints.length - 1]);
            } else {
                headerLine1dest.text = "";
            }
        }
    }

    states: [
        State {
            name: "waitingForPosition"
            PropertyChanges {
                target: headerLine2
                text: qsTranslate("Maps", "qtn_navigationconnector_preview_waiting_for_gps")
            }
        },
        State {
            name: "calculatingRoute"
            PropertyChanges {
                target: headerLine2
                text: qsTranslate("Maps", "qtn_navigationconnector_preview_calculating_route")
            }
        }
    ]
}
