// -*- mode: js2; espresso-indent-level: 4; -*-

var maneuverIconBasePath = navigationAssetsPath + "images/maneuver_icons/meegotouch-seattle-maps-maneuver-icon-%1.png";
var smallManeuverIconBasePath = navigationAssetsPath + "images/maneuver_icons/small/meegotouch-seattle-maps-maneuver-icon-%1-small.png";

function enrichManeuverItem (item, indices, descriptionfn, unitsfn, listIndex, useSmall) {
    var index, copy = {};

    for (index in item) {
        if (item.hasOwnProperty(index)) {
            copy[index] = item[index];
        }
    }

    if(item.action === 100) {
        copy.iconSource = indices[item.icon] !== undefined ? (useSmall ? smallManeuverIconBasePath : maneuverIconBasePath).arg(indices[item.icon]) : '';
        copy.actionText = qsTranslate("Maps", "qtn_navigationconnector_routemaneuverlist_walkfor").arg(unitsfn(item.length));
    } else {
        var isStartFlag = (item.action === 0 && listIndex === 0);
        copy.roadElementsFormOfWay = "";
        if (!copy.streetName) {
            for (var i = 0; i < copy.roadElements.length; i++) {
                copy.roadElementsFormOfWay += copy.roadElements[i].formOfWay + ",";
            }
        }
        copy.iconSource = indices[item.icon] !== undefined ? (useSmall ? smallManeuverIconBasePath : maneuverIconBasePath).arg(indices[item.icon]) : '';
        copy.actionText = isStartFlag ? "" : descriptionfn(item);
        copy.distanceText = isStartFlag ? "" : unitsfn(item.distanceFromPreviousManeuver);
    }
    return copy;
}

function fillList (model, indices, descriptionfn, unitsfn, route) {
    model.clear();
    if (route.mode === "pt") {
        var ptManeuvers = RoutingUtils.compressPtManeuvers(route);
        ptManeuvers.forEach(function (item, index) {
            var modelItem = item;
            modelItem.index = index;
            model.append(modelItem);
        });
    } else {
        for (var i = 0, l = route.maneuvers.length; i < l; i++) {
            var modelItem = enrichManeuverItem(route.maneuvers[i], indices, descriptionfn, unitsfn);
            modelItem.index = i;
            model.append(modelItem);
        };
    }
}


function getPtGeneralLineIconByType(aType) {
    var type;
    switch (aType) {
        case TransitLineInfo.TRANSIT_TYPE_BUS_PUBLIC:
        case TransitLineInfo.TRANSIT_TYPE_BUS_TOURISTIC:
        case TransitLineInfo.TRANSIT_TYPE_BUS_INTERCITY:
        case TransitLineInfo.TRANSIT_TYPE_BUS_EXPRESS:
            type = "bus";
            break;
        case TransitLineInfo.TRANSIT_TYPE_RAIL_METRO:
            type = "rail_citymetro";
            break;
        case TransitLineInfo.TRANSIT_TYPE_RAIL_LIGHT:
            type = "rail_light";
            break;
        case TransitLineInfo.TRANSIT_TYPE_RAIL_REGIONAL:
            type = "rail_regional";
            break;
        case TransitLineInfo.TRANSIT_TYPE_TRAIN_REGIONAL:
        case TransitLineInfo.TRANSIT_TYPE_TRAIN_INTERCITY:
        case TransitLineInfo.TRANSIT_TYPE_TRAIN_HIGH_SPEED :
            type = "train";
            break;
        case TransitLineInfo.TRANSIT_TYPE_MONORAIL:
            type = "rail_regional";
            break;
        case TransitLineInfo.TRANSIT_TYPE_AERIAL:
        case TransitLineInfo.TRANSIT_TYPE_INCLINED:
        case TransitLineInfo.TRANSIT_TYPE_WATER:
            type = "rail_regional";
            break;
    }

    if (type) {
        return placesAssetsPath + "images/categories/mobile_list_public_transport_" + type + ".png";
    }
    return "";
};

// endStation is optional. If not set to true, will get the icon of the departure station
function getPtStationIcon(aManeuver, aTransitDB, endStation) {
    if(!aManeuver || aManeuver.type !== "pt") {
        return;
    }
    var station = endStation ? aManeuver.arrivalStation.id : aManeuver.departureStation.id;

    var info = aTransitDB.getStopInfo(station);

    if(info && info.systems.length === 1) {
        return getPtSystemIcon(aManeuver);
    }
    return mapAssetsPath + "images/categories/mobile_list_transport.png"

}

function getPtSystemIcon(aManeuver) {
    if(!aManeuver) {
        return;
    }

    if(aManeuver.systemLogo && aManeuver.systemLogo.imageUrl !== "") {
        return aManeuver.systemLogo.imageUrl;
    }

    return getPtGeneralLineIconByType(aManeuver.transitType);
}

function showWalkSeparator(model, index) {
    var prevMan = model.get(index-1);
    var maneuver = model.get(index);
    return (prevMan.action === 100) || ManeuverConstants.MANEUVER_ACTIONS[maneuver.action] === ManeuverConstants.ACTION_END;
}

function showPartialWalkSeparator(model, index) {
    var maneuver = model.get(index);
    return !(index === 0 || maneuver.action === 100 || ManeuverConstants.MANEUVER_ACTIONS[maneuver.action] === ManeuverConstants.ACTION_END);
            // Only whole maneuver if walk summary that is not in the first walk segment AND is not the last maneuver
}
function useHighlightColor(aManeuver, index) {
     return (aManeuver.action === 0 && index === 0) ||
            ManeuverConstants.MANEUVER_ACTIONS[aManeuver.action] === ManeuverConstants.ACTION_END;
}
