import QtQuick 1.0

import com.nokia.meego 1.0

Item {
    id: root
    signal inputClicked;

    property int defaultWidth;
    property bool highlightField: false;
    property bool isCurrentPosition: false;
    property int targetToUpdate

    property alias placeholderText: routeInputField.placeholderText
    property alias text: routeInputField.text

    width: visible ? defaultWidth : 0
    height: routeInputField.height

    TextField {
        id: routeInputField

        inputMethodHints: Qt.ImhNoPredictiveText
        cursorPosition: 0

        width: parent.width

        TextFieldStyle { id: defaultStyle; }

        platformStyle: TextFieldStyle {
            paddingLeft: supplement.width + 20;
            background: (root.highlightField ? defaultStyle.backgroundSelected : defaultStyle.background)
        }

        Item {
            id: supplement
            objectName: "supplement"

            width: visible ? myPositionImage.width : 0;
            height: parent.height;
            anchors {left: parent.left; leftMargin: visible ? 16 : 0; verticalCenter: parent.verticalCenter;}
            visible: routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_currentposition") ||
                routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_previousposition") ||
                routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_waiting_for_position")

            BusyIndicator {
                id: spinner
                running: true
                anchors.centerIn: parent
                platformStyle: BusyIndicatorStyle { size: "small" }
                visible: longPressGeoFinder.busy && longPressGeoFinder.targetToUpdate === root.targetToUpdate &&
                    routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_waiting_for_position")
            }

            Image {
                id: myPositionImage;
                objectName: "myPositionImageFrom";
                smooth: true
                anchors.centerIn: parent;
                visible: (
                     routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_currentposition") ||
                     routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_previousposition")
                 ) ||
                 routeInputField.text == qsTranslate("Maps", "qtn_navigationconnector_routeplanner_waiting_for_position") &&
                 (!(longPressGeoFinder.busy && longPressGeoFinder.targetToUpdate === root.targetToUpdate))

                source: {
                    if (map.currentPosition) {
                        return navigationAssetsPath + "images/my_position_placeholder_small" + "" + ".png";
                    } else if (map.previousPosition) {
                        return navigationAssetsPath + "images/my_position_placeholder_small" + "_none" + ".png";
                    } else {
                        return navigationAssetsPath + "images/my_position_placeholder_small" + "_not_present" + ".png";
                    }
                }
                onSourceChanged: { routeInputField.cursorPosition = 0; }
            }
        }

        onTextChanged: { routeInputField.cursorPosition = 0; }
        onActiveFocusChanged: { if (!activeFocus) { routeInputField.cursorPosition = 0; } }
        Component.onCompleted: { routeInputField.cursorPosition = 0; }
    }

    MouseArea {
        anchors.fill: parent
        onClicked: inputClicked();
    }
}
