import QtQuick 1.0
import ovi.connector.map 1.0
import MapsPlugin 1.0

import ".."

MapViewer {
    id: routeViewer

    useGlobalShare: true

    property QtObject __mapRoute: MapRoute {}
    property QtObject route

    property variant currentlySelectedIconA
    property bool currentlySelectedIconAInitialVisibility
    property variant currentlySelectedIconB
    property bool currentlySelectedIconBInitialVisibility

    property variant __waypointIcons // internal property to store the waypoint icons
    property variant __mapBorder // internal property to store the map border ()

    property bool runInGuidance: false
    property bool flagsHidden: false
    property bool sharedPublicTransportMode
    property string latestCustomSchemeSet

    property Component __waypointIconFactory: Component {
        MapIcon {
            id: mapIcon
            property string type: "stopover"
            property variant geoCoordinates
            anchorPoint: Qt.point(12, 57)

            onGeoCoordinatesChanged: {
                location.geoCoordinates.latitude = geoCoordinates.latitude;
                location.geoCoordinates.longitude = geoCoordinates.longitude;
            }

            onTypeChanged: {
                icon.localUrl = type ? navigationAssetsPath + "/images/waypoint_marker_" + type + ".png" : "";

            }

            icon: Icon {}

            location: Location {
                geoCoordinates: GeoCoordinates {}
            }
        }
    }

    function showRouteOnMap() {
        if (route && route.calculated) {
            __mapRoute.color = _getRouteColor();
            __mapRoute.route = routeViewer.route.__calculationResult.clone(mainscreen);
            if (!routeViewer.containRoute(routeViewer.__mapRoute)) {
                routeViewer.addRoute(routeViewer.__mapRoute);
            }
            setRouteModeMapScheme();
        }
    }

    function removeRouteFromMap() {
        if (route && route.calculated) {
            routeViewer.removeRoute(routeViewer.__mapRoute);
        }
    }

    function zoomToRoute(animate) {
        if (routeViewer.width - __mapBorder.left - __mapBorder.right > 0 && routeViewer.height - __mapBorder.top - __mapBorder.bottom > 0) {
            var rect = Qt.rect(
                __mapBorder.left,
                __mapBorder.top,
                routeViewer.width - __mapBorder.left - __mapBorder.right,
                routeViewer.height - __mapBorder.top - __mapBorder.bottom
            );
            var routeGeoBoundingBox = Qt.createQmlObject("import MapsPlugin 1.0; GeoRect {}", mainscreen, null);
            routeGeoBoundingBox.top = routeViewer.route.__calculationResult.boundingBox.top;
            routeGeoBoundingBox.bottom = routeViewer.route.__calculationResult.boundingBox.bottom;
            routeGeoBoundingBox.left = routeViewer.route.__calculationResult.boundingBox.left;
            routeGeoBoundingBox.right = routeViewer.route.__calculationResult.boundingBox.right;

            if (routeViewer.route.waypoints.length > 1) {
                var wps = [
                    {
                        latitude: routeViewer.route.waypoints[0].latitude || routeViewer.route.waypoints[0].properties.geoLatitude,
                        longitude: routeViewer.route.waypoints[0].longitude || routeViewer.route.waypoints[0].properties.geoLongitude
                    },
                    {
                        latitude: routeViewer.route.waypoints[1].latitude || routeViewer.route.waypoints[1].properties.geoLatitude,
                        longitude: routeViewer.route.waypoints[1].longitude || routeViewer.route.waypoints[1].properties.geoLongitude
                    }
                ];

                routeGeoBoundingBox.top = Math.max(
                    routeViewer.route.__calculationResult.boundingBox.top,
                    Math.max(wps[0].latitude, wps[1].latitude)
                );
                routeGeoBoundingBox.bottom = Math.min(
                    routeViewer.route.__calculationResult.boundingBox.bottom,
                    Math.min(wps[0].latitude, wps[1].latitude)
                );
                routeGeoBoundingBox.left = Math.min(
                    routeViewer.route.__calculationResult.boundingBox.left,
                    Math.min(wps[0].longitude, wps[1].longitude)
                );
                routeGeoBoundingBox.right = Math.max(
                    routeViewer.route.__calculationResult.boundingBox.right,
                    Math.max(wps[0].longitude, wps[1].longitude)
                );
            }

            if (routeViewer.route.__calculationResult) {
                routeViewer.orientation = 0;
                routeViewer.moveToRect(
                    (routeGeoBoundingBox.isValid ? routeGeoBoundingBox : routeViewer.route.__calculationResult.boundingBox),
                    rect,
                    animate ? Map.ANIMATION_LINEAR : Map.ANIMATION_NONE
                );
            }
        }
    }
    function hideWaypointIcons(index) {
        var tmp;
        for (var i=0; i<routeViewer.model.count; i++) {
            tmp = routeViewer.model.get(i);
            if (tmp && tmp.data && tmp.data.isWaypoint == true && (index === null || tmp.data.index == index)) {
                tmp.visible = false;
            }
        }
        routeViewer.flagsHidden = true;
        routeViewer.updateMap();
    }
    function unhideWaypointIcons(index) {
        if (routeViewer.flagsHidden) {
            var tmp;
            for (var i=0; i<routeViewer.model.count; i++) {
                tmp = routeViewer.model.get(i);
                if (tmp && tmp.data && tmp.data.isWaypoint == true && (index === null || tmp.data.index == index)) {
                    tmp.visible = true;
                }
            }
            routeViewer.flagsHidden = false;

            if (routeViewer.currentlySelectedIconA) {
                routeViewer.currentlySelectedIconA.visible = false;
            }
            if (routeViewer.currentlySelectedIconB) {
                routeViewer.currentlySelectedIconB.visible = false;
            }

            routeViewer.updateMap();
        }
    }
    function removeWaypointIcons() {
        var tmp;
        if (routeViewer.model) {
            for (var i=0; i<routeViewer.model.count; i++) {
                tmp = routeViewer.model.get(i);
                if (tmp && tmp.data && tmp.data.isWaypoint == true) {
                    routeViewer.model.remove(i--);
                }
            }
        }
        __waypointIcons = [];
    }

    function showWaypointIcons() {
        removeWaypointIcons();

        var iconUrl;
        var wps = route.waypoints;

        if (wps[0] || wps[1]) {
            var wpIcons = __waypointIcons || [];
            for (var i = 0, len = wps.length; i < len; i++) {
                if (wps[i]) {
                    iconUrl = navigationAssetsPath + "/images/waypoint_marker_" + (i == 0 ? "start" : i == wps.length - 1 ? "destination" : "stopover" ) + ".png";
                    routeViewer.model.appendIcon(
                        wps[i].latitude || wps[i].properties.geoLatitude,
                        wps[i].longitude || wps[i].properties.geoLongitude,
                        iconUrl,
                        { isWaypoint: true, index: i },
                        { anchorPoint: Qt.point(12, 57) }
                    );
                }

            }

            routeViewer.model.setObjectsOwner("route", true);

            if (routeViewer.currentlySelectedIconA) {
                routeViewer.currentlySelectedIconA.visible = false;
            }
            if (routeViewer.currentlySelectedIconB) {
                routeViewer.currentlySelectedIconB.visible = false;
            }

            __waypointIcons = wpIcons;
        }
    }

    function moveToDestination(animate) {
        if(route) {
            var dest = route.waypoints[route.waypoints.length - 1];
            if (dest) {
                var center = Qt.createQmlObject("import MapsPlugin 1.0; GeoCoordinates {}", routeViewer);
                if (dest.properties) {
                    center.latitude = dest.properties.geoLatitude;
                    center.longitude = dest.properties.geoLongitude;
                } else {
                    center.latitude = dest.latitude;
                    center.longitude = dest.longitude;
                }

                routeViewer.moveTo(
                    center,
                    animate ? Map.ANIMATION_LINEAR : Map.ANIMATION_NONE,
                    Map.PRESERVE_SCALE,
                    Map.PRESERVE_ORIENTATION,
                    Map.PRESERVE_PERSPECTIVE
                );
            }
        }
    }

    // Sets the map scheme according to the map. If aType is default, it turns off PT route scheme and ignores the current route mode
    function setRouteModeMapScheme(aType) {
        if (runInGuidance) return;

        var publicTransportModeToApply;

        if (aType === "default") {
            if (sharedPublicTransportMode) {
                publicTransportModeToApply = "On";
            } else {
                publicTransportModeToApply = "Off"
            }
        } else if (route && route.calculated) {
            if (route.mode === "pt") {
                publicTransportModeToApply = "On";
            } else {
                if (sharedPublicTransportMode) {
                    publicTransportModeToApply = "On";
                } else {
                    publicTransportModeToApply = "Off";
                }
            }
        }

        if (routeViewer.latestCustomSchemeSet && routeViewer.latestCustomSchemeSet != routeViewer.scheme) {
            if (!(route.mode === "pt" && publicTransportModeToApply == "On")) {
                return;
            }
        }

        if (publicTransportModeToApply) {
            routeViewer.callMapLayersApi("setPublicTransportMode" + publicTransportModeToApply, null);
            if (route.mode === "pt" && publicTransportModeToApply == "On") {
                routeViewer.callMapLayersApi("skipPropertiesOnSaveTime", [["publicTransportMode"]]);
                routeViewer.setBehavior(routeViewer._NO_PUBLIC_TRANSPORT_MODE_, true);
            } else {
                routeViewer.callMapLayersApi("skipPropertiesOnSaveTime", [[]]);
                routeViewer.setBehavior(routeViewer._NO_PUBLIC_TRANSPORT_MODE_, false);
            }

            routeViewer.latestCustomSchemeSet = routeViewer.scheme;

            routeViewer.setControl("emit-ui-changed");
        }
    }

    function _getRouteColor() {
        var schemeParts = new String(routeViewer.scheme).split(".");
        return schemeParts[0] == "hybrid" || schemeParts[1] == "night"? "#F1F1F1" : route.color;
    }

    onRouteChanged: {
        if (route && route.calculated) {
            __mapRoute.color = _getRouteColor();
            __mapRoute.route = routeViewer.route.__calculationResult;
            setRouteModeMapScheme();
        } else {
            removeRouteFromMap();
        }
    }

    onVisibleChanged: {
        if (routeViewer.visible == true) {
            if (route && route.calculated && route.mode !== "pt") {
                sharedPublicTransportMode = routeViewer.callMapLayersApi("isPublicTransportModeActive")
            }
        }
    }

    onSchemeChanged: {
        if (routeViewer.visible == true && route && route.calculated && __mapRoute) {
            __mapRoute.color = _getRouteColor();
        }
    }

    Component.onCompleted: {
        routeViewer.__mapBorder = {left: 70, right: 70, top: 70, bottom: 70};
        stash = {};
    }
}
