import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0
import ovi.connector.places 1.0

PlacesCommonItemDelegate {
    id: flexDelegate
    objectName: "voiceCatalogItem" + index
    opacity: isEnabled ? 1 : 0.5

    signal voiceItemChosen (int index);
    signal downloadVoiceClicked (int index);
    signal updateVoiceClicked (int index);
    signal cancelDownloadClicked ();

    property int offset_left: 10
    property int offset_top: 25
    property int offset_right: 20
    property int downloadProgress: -1
    property int downloadSize: 0

    property int updateVoice: 0
    property int _updateNotNeeded: 0
    property int _updateNeeded: 1
    property int _updating: 2

    property bool isInstalled: false
    property bool radioChecked
    property bool isEnabled: false
    property bool error: false
    property bool loading: false
    property bool isDownloading: downloadProgress !== -1

    property string name
    property string gender
    property string backgroundColorRegular: "#f0f1f2"
    property string backgroundColorHighlight: "#d0d1d2"

    property variant queryDialog

    Rectangle {
        anchors.fill: parent;
        color: backgroundColorRegular

        BusyIndicator {
            anchors.centerIn: parent
            running: true
            visible: loading
        }

        // The name of the audio file
        MapsText {
            id: textLine1
            objectName: "nameTxt"
            text: name
            elide: Text.ElideRight
            anchors {
                left: parent.left;
                leftMargin: offset_left;
                top: parent.top;
                topMargin: offset_top;
                right: actionComponent.left
                rightMargin: offset_right
            }
            state: "listNormalBold"

        }

        // The voice gender
        MapsText {
            id: textLine2
            objectName: "genderTxt"
            text: gender === "" ? "" :
                gender === "f" ?
                (qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_female")).toLowerCase() :
                (qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_male")).toLowerCase()
            color: secondaryColor
            visible: !progressBar.visible && !error
            anchors { left: parent.left; leftMargin: offset_left; top: textLine1.bottom; topMargin: 5}
        }

        ProgressBar {
            id: progressBar
            objectName: "progessBar"
            visible: isDownloading && isEnabled
            minimumValue: 0
            maximumValue: 100
            value: downloadProgress
            indeterminate: value === 0 || value === 100
            anchors {
                left: parent.left
                leftMargin: offset_left
                right: actionComponent.left
                rightMargin: offset_right
                top: textLine1.bottom;
                topMargin: 15
            }
        }

        // This holds all the components on the right: the update icon, radio button, download image, cancel image
        Item {
            id: actionComponent
            width: Math.max(radioButton.width, downloadCancelImage.width) + (updateImage.visible ? updateImage.width : 0)
            anchors {
                right: parent.right
                rightMargin: 20
                top: parent.top
                bottom: parent.bottom
            }

            // If installed, show this button.
            RadioButton {
                id: radioButton
                objectName: "installBtn"
                visible: isInstalled && downloadProgress === -1 && !error
                checked: radioChecked
                enabled: isEnabled
                    anchors {
                        verticalCenter: parent.verticalCenter;
                        horizontalCenter: parent.horizontalCenter
                        horizontalCenterOffset: updateImage.visible ? updateImage.width/2 : 0
                    }
            }

            Image {
                id: downloadCancelImage
                property bool pressed: mouseArea.pressed
                visible:  (!isInstalled || (updateVoice !== _updateNotNeeded && isDownloading)) && !error && !loading
                source: navigationAssetsPath + "images/meegotouch-maps-voice-catalog-" +
                        ((isDownloading || updateVoice === _updating) ? "cancel" : "download") +
                        (pressed ? "-pressed" : "") +
                        ".png"
                anchors.verticalCenter: parent.verticalCenter
            }
            // The update button
            Image {
                id: updateImage
                property bool pressed: mouseArea.pressed
                visible: isInstalled && !error && updateVoice === _updateNeeded && !isDownloading
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: radioButton.left
                    rightMargin: 10
                }
                source: navigationAssetsPath + "images/meegotouch-maps-voice-catalog-update.png"
            }
        }

        // The mouse area that controls the radio button and the background highlighting
        MouseArea {
            id: mouseArea

            anchors.fill: parent
            enabled: isEnabled
            onPressed: { parent.color = backgroundColorHighlight; radioButton.pressed = true; }
            onReleased: { parent.color = backgroundColorRegular; radioButton.pressed = false; }
            onCanceled: { parent.color = backgroundColorRegular; radioButton.pressed = false; }
            onClicked: {
                if (error || loading){
                    return;
                } else if (updateVoice === _updateNeeded && downloadProgress === -1) {
                    queryDialog.index = index;

                    queryDialog.open();
                } else if (downloadCancelImage.visible) {
                    if (downloadProgress === -1) {
                        downloadVoiceClicked(index);
                    } else {
                        cancelDownloadClicked();
                    }
                } else {
                    voiceItemChosen(index);
                }
            }
        }
    }

    states: [
        State {
            name: "no_gender"
            when: gender === ""

            PropertyChanges {
                target: textLine1
                verticalAlignment: Text.AlignVCenter
            }

            AnchorChanges {
                target: textLine1
                anchors {
                    verticalCenter: parent.verticalCenter
                }
            }
        }
    ]
}

