import QtQuick 1.0
import MapsPlugin 1.0

ManeuverDelegate {
    property variant maneuver
    property bool show
    property int origHeight

    maneuverIcon: maneuver.iconSource
    actionText: maneuver.actionText
    destinationText: __getStreetNameText(maneuver)
    distanceText: maneuver.distanceText
    distanceVisible: !!distanceFromPreviousManeuver
    formatNoActionText: !(maneuver.action === 0 && index === 0)

    function __concatListIfValid(aArray) {
        var tmp,
            retval = ""
        ;
        for (var i=0; i<aArray.length; i++) {
            tmp = aArray[i];
            if(tmp && tmp.length > 0) {
                retval += (retval.length === 0 ? tmp : ", " + tmp);
            }
        }
        return retval;
    }

    function __getStreetNameText(aManeuver) {
        var result = "";
        switch(aManeuver.action) {
        case 100:
            result = qsTranslate("Maps", "qtn_navigationconnector_walkdrivedelegate_to").arg(aManeuver.destination);
            break;
        case MapsPlugin.Maneuver.ACTION_ENTER_HIGHWAY:
        case MapsPlugin.Maneuver.ACTION_ENTER_HIGHWAY_FROM_LEFT:
        case MapsPlugin.Maneuver.ACTION_ENTER_HIGHWAY_FROM_RIGHT:
            var subInfo = aManeuver.nextRouteName || aManeuver.routeName || "";
            result += aManeuver.nextStreetName || aManeuver.signpost.exitDirections || aManeuver.signpost.exitText || "";
            result += (subInfo !== "" ? (result.length === 0 ? subInfo : " (" + subInfo + ")") : "");
            break;

        case MapsPlugin.Maneuver.ACTION_CONTINUE_HIGHWAY:
        case MapsPlugin.Maneuver.ACTION_CHANGE_HIGHWAY:
            var subInfo = aManeuver.nextRouteName || aManeuver.routeName;
            result += aManeuver.signpost.exitDirections || aManeuver.signpost.exitText || "";
            result += (result.length === 0 ? aManeuver.nextStreetName || aManeuver.exitNumber || "" : "");
            result += (subInfo !== "" ? (result.length === 0 ? subInfo : " (" + subInfo + ")") : "");
            break;
        case MapsPlugin.Maneuver.ACTION_LEAVE_HIGHWAY:
            var subInfo = __concatListIfValid([aManeuver.signpost.exitText, aManeuver.signpost.exitNumber, aManeuver.nextRouteName])
            result += aManeuver.signpost.exitDirections || "";
            result += (result.length === 0 ? aManeuver.nextStreetName : "");
            result += (subInfo !== "" ? (result.length === 0 ? subInfo : " (" + subInfo + ")") : "");
            break;
        default:
            result = index === 0 ? aManeuver.streetName : aManeuver.nextStreetName;
            if (!result) {
                var tmpResult = [];
                var tmpString = "";
                var tmpSplits = aManeuver.roadElementsFormOfWay.split(",");
                for (var i = 0; i < tmpSplits.length; i++) {
                    if (tmpSplits[i]) {
                        var strToAdd;
                        switch (parseFloat(tmpSplits[i])) {
                            case RoadElement.FOW_UNDEFINED:
                                strToAdd = qsTranslate("Maps", "qtn_navigationconnector_maneuver_formofway_FOW_UNDEFINED")
                                break;
                            case RoadElement.FOW_PEDESTRIAN_WALKWAY:
                                strToAdd = qsTranslate("Maps", "qtn_navigationconnector_maneuver_formofway_FOW_PEDESTRIAN_WALKWAY")
                                break;
                            default:
                                strToAdd = qsTranslate("Maps", "qtn_navigationconnector_maneuver_formofway_UNKNOWN");
                                break;

                        }
                        if(tmpString.indexOf(strToAdd) === -1) {
                            tmpResult.push(strToAdd);
                            tmpString += strToAdd;
                        }
                    }
                }
                result = tmpResult.join(qsTranslate("Maps", "qtn_navigationconnector_walkdrivedelegate_or"));
            }
            break;
        }
        return result || qsTranslate("Maps", "qtn_navigationconnector_maneuver_formofway_UNKNOWN");
    }

    Component.onCompleted: {
//        print(JSON.stringify(maneuver));
        origHeight = height;
    }

//    ParallelAnimation {
//        id: animationShow
//        NumberAnimation { target: parent; property: "height"; from: 0; to: origHeight; duration: 200; easing.type: Easing.OutQuad}
//        NumberAnimation { target: parent; property: "opacity"; from: 0; to: 1; duration: 200 }
//    }

//    ParallelAnimation {
//        id: animationHide
//        NumberAnimation { target: parent; property: "height"; from: origHeight; to: 0; duration: 200; easing.type: Easing.OutQuad}
//        NumberAnimation { target: parent; property: "opacity"; from: 1; to: 0; duration: 100 }
//    }

    onShowChanged: {
        if(show) {
            animationShow.start();
        } else {
            animationHide.start();
        }
    }
}
