import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import "ui"

Page {
    id: categoryTreeId
    anchors.fill: parent

    signal finished

    property variant mainScreen: parent
    property variant currentCategory
    property variant searchResultModel
    property variant pickedObject
    property variant model
    property bool showSearchResults: false
    property bool refreshSearchResults: false
    property int baseDepth

    property int _timerCount: 0
    property int _maxCategoryQueryTries: 1// 10 // 30 seconds (10 tries x 3 second timer) of waiting for catalog to be downloaded

    property variant instanceOfPlaceCategoryNode: null

    property bool initialized: false

    ListModel {
        id: categoryOrder
        objectName: "categoryOrder"
    }

    PlacesConnectedList {
        id: placesList
        onQueryFinished: {
            categoryTreeId.showSearchResults = true;
            categoryTreeId.searchResultModel = model;
            catList.fillResultList(categoryTreeId.searchResultModel, categoryTreeId.lastClickedCat.name);
        }
    }

    Component {
        id: subCategoryPage

        SubCategoryPage {
            property variant info
            property bool alreadyLoaded: false
            pageStack: pageStack

            onStatusChanged: {
                if (status == PageStatus.Activating) {
                    if (!alreadyLoaded && info) {
                        alreadyLoaded = true;
                        fillListWithCategoryInfo(info);
                        delete info;
                    }
                } else if (status == PageStatus.Inactive) {

                }
            }
            onNextCategoryClicked: {
                pushNextCategoryPage(index);
            }
            onCatPlaceClicked: {
                showCategoryPlace(place, list);
            }
        }
    }

    Component {
        id: factoryPlaceCategories
        PlaceCategoryNode { }
    }

    BusyIndicator {
        z:2
        id: indicator
        platformStyle: BusyIndicatorStyle { size: "large" }
        running:  true
        anchors.centerIn: catList
    }

    InfoBanner {
        z: 2
        id: errorMessage;
        timerEnabled: true;
        timerShowTime: 3*1000;
        //anchors.centerIn: catList; [ace] This is creating an infinte binding loop!!!!!!!! We shouldn't change the anchors for InfoBanners anyway
    }

    CategoryList {
        id: catList
        mainScreen: categoryTreeId.mainScreen
        anchors.fill: parent;
        currentCategory: categoryTreeId.currentCategory
        onSubCategoryClicked: {
            pushNextCategoryPage(index);
        }
    }

    Timer {
        id: categoryQueryTimer
        interval: 3 * 1000
        repeat: true
        onTriggered: {
            _timerCount++;
            queryCatalogReady();
        }
    }

    onStatusChanged: {
        if (status === PageStatus.Activating) {
            //[ace]: IMPORTANT!!! This updates the category order when going back and forth between pages in the tree
            // It is needed to keep the tree and pages in sync. Not checking every time the page is activated results
            // in the tree and pages not being related.
            categoryTreeId.updateCategoryOrder(categoryTreeId.pageStack.depth);
        } else if (status === PageStatus.Deactivating) {
            placesList.cancelQuery();
        }
    }

    // When page is complete, load the data, add it to the category order and load the first category.
    Component.onCompleted: {
        errorMessage.hide();
        /* // Fetching categorylist from Places Server
        catList.fillCategoryList(parseData());
        var lang = appState.locale.displayName.replace(/_/g,"-");
        PCA.request(placesURI+"categories/find-places/grouped",requestDone,requestError,{headers:{"Accept-Language":lang}}); */

        queryCatalogReady();
    }

    function refreshPlacesList() {
        placesList.clear();
        placesList.query({
            // searchTerm: lastClickedCat.name,
            searchCategory: lastClickedCat.categoryId,
            location: location,
            parent: catList
        });
    }

    function selectPlace(index) {
        categoryTreeId.pickedObject = placesList.get(index);
        finished();
    }

    function getNextCategoryInfo(index) {
        var valid = false;
        var nextCategory;
        var lastClickedCat;
        var catChildren = categoryTreeId.currentCategory["childCategories"];
        if (catChildren.get) {
            lastClickedCat = catChildren.get(index);
            if (catChildren.get(index)["childCategories"].count) {
                categoryTreeId.currentCategory = catChildren.get(index);
                nextCategory = catChildren.get(index);
                valid = true;
            }
        } else if (catChildren[index]) {
            lastClickedCat = catChildren[index];
            if (catChildren[index]["childCategories"].length > 0) {
                categoryTreeId.currentCategory = catChildren[index];
                nextCategory = catChildren[index];
                valid = true;
            }
        }
        return {
            category: nextCategory,
            lastLeaf: !valid,
            name: lastClickedCat.name,
            categoryId: lastClickedCat.categoryId || 0
        }
    }

    function showCategoryPlace(aPlace, list) {
        categoryTreeId.pickedObject = aPlace;
        categoryTreeId.model = list;
        finished();
    }

    function pushNextCategoryPage(index) {
        var info = getNextCategoryInfo(index);

        categoryTreeId.pageStack.push(subCategoryPage, {
            info: info,
            category: info.category,
            categoryName: info.name,
            categoryId: info.categoryId
        });

        log.info("pushing new category page");
    }

    function queryCatalogReady() {
        if (instanceOfPlaceCategoryNode) {
            instanceOfPlaceCategoryNode.destroy();
        }
        instanceOfPlaceCategoryNode = factoryPlaceCategories.createObject(categoryTreeId);
        if (instanceOfPlaceCategoryNode && instanceOfPlaceCategoryNode.children && instanceOfPlaceCategoryNode.children.length > 0) {
            categoryQueryTimer.stop();
            requestDone();
        } else if (_timerCount >= _maxCategoryQueryTries) {
            categoryQueryTimer.stop();
            requestError();
        } else {
            if (!categoryQueryTimer.running) {
                categoryQueryTimer.start();
            }

        }
    }

    function prepareMosCategories(placeCategoryNode, level) {
        var info = {name: placeCategoryNode.name || qsTranslate("Maps", "qtn_placesconnector_categories"), categoryId: placeCategoryNode.iconId, childCategories:[]};
        var children = placeCategoryNode.children;
        var length = children.length;
        for (var i = 0; i < length; i++) {
            var child = children[i];
            info.childCategories.push(prepareMosCategories(child, level + 1));
        }
        return info;
    }

    function requestDone() {
        indicator.running = indicator.visible = false;
        var data = prepareMosCategories(instanceOfPlaceCategoryNode);
        categoryTreeId.currentCategory = data;
        categoryOrder.append(currentCategory);
        catList.fillCategoryList(categoryTreeId.currentCategory);
    }

    function requestError() {
        // TODO: Add more Error Messages
        indicator.running = indicator.visible = false;
        //errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_cloud_noconnection");
        //errorMessage.show();
        catList.showErrorConnectionText(qsTranslate("Maps", "qtn_placesconnector_categories"), qsTranslate("Maps", "qtn_placesconnector_cloud_noconnection"));
    }

    /**
     * Updates the current category and category order list based on the page depth
     * changed to accommodate back navigation from the toolbar.
     */
    function updateCategoryOrder(aPageDepth) {
        var target = aPageDepth - baseDepth;

        var i = categoryOrder.count-1;
        while (categoryOrder.count > 0 && target < i) {
            categoryOrder.remove(i--);
        }
        categoryTreeId.currentCategory = categoryOrder.get(categoryOrder.count-1);
    }

    function parseData(aData){
        // var data = JsPP.ovi.places.core.categories.categoryMap;
        var data = typeof aData!== "undefined" && aData.categories.group;
        var head = {
            name: qsTranslate("Maps", "qtn_placesconnector_categories"),
            categoryId: -1,
            childCategories: []
        }

        for (var a=0,al=data.length;a<al;a++) {
            var childList = [];
            for (var b=1,bl=data[a].category.length;b<bl;b++){
                childList.push({ name: data[a].category[b].displayName, categoryId: JsPP.ovi.places.core.categories.getSearchCategory(data[a].category[b].name) ,childCategories: []});
            }

            head.childCategories.push({ name: data[a].groupingCategory.displayName, categoryId: -1, childCategories: childList});
        }
        return head;
    }
}
