import QtQuick 1.0

import ovi.connector 1.0
import ovi.connector.favorites 1.0

import "PlaceStaticHelper.js" as JsPlaceStaticHelper

ConnectedObject {
    id: self

    //cloudAdapter: PlacesCloudAdapter {}
    //property variant favoritesCloudAdapter: FavoritesCloudAdapter {}

    cloudAdapter: mainscreen.placesCloudAdapter
    property variant favoritesCloudAdapter: mainscreen.favoritesCloudAdapter

    property bool   isFavorite : !!(typeof(favorite) === 'object' && favorite.key)
    property bool   isRecommendation : false
    property string placeId
    property string name

    property variant categories: QtObject {}
    // TODO temporary placeholder properties
    property variant properties: QtObject {}
    property variant iconPath: /*QtObject {}*/ ""

    property variant details: QtObject {}
    property variant reviews: QtObject {}

    property variant recommendations: QtObject {}
    property variant publictransport: QtObject {}
    property variant favorite: QtObject {}

//    property QtObject details: QtObject {
//        property bool isPremium: false
//    }
//    property QtObject reviews: QtObject{}

    function rate(stars) {
        adapter.rate(stars);
    }

    function isReady() {
        return true;
    }

    function getFuzzyKey() {
        return "fuzzy-" + properties.geoLatitude + "," + properties.geoLongitude;
    }

    function getFavorite() {
        var favorite = favoritesCloudAdapter.getFromStore(placeId ? placeId : getFuzzyKey());

        return favorite? favorite : null;
    }

    function removeFromStore() {
        var favorite = favoritesCloudAdapter.getFromStore(placeId ? placeId : getFuzzyKey());
        if (favorite) {
            var removalFunction = function () {
                isFavorite = false;
                favorite.removeFinished.disconnect(removalFunction);
            }
            favorite.removeFinished.connect(removalFunction);
            favorite.erase({});
        }
    }

    function saveInStore(modifiedName, description) {
        var favorite = favoritesCloudAdapter.getFromStore(placeId ? placeId : getFuzzyKey());
        if (!favorite) {
            favorite = favoritesCloudAdapter.createObject(Math.random());
        }

        favorite.text = modifiedName || properties.title || ""; // Set initial name to original name, by friendly to other implementations

        favorite.place = self;
        favorite.key = placeId ? placeId : getFuzzyKey();
        favorite.cloudAdapter = favoritesCloudAdapter;
        favorite.description = description || "";
        var updaterFunction = function () {
            isFavorite = true;
            favorite.updateFinished.disconnect(updaterFunction);
        }

        favorite.updateFinished.connect(updaterFunction);
        favorite.write({});

        self.name = modifiedName || properties.title || "";
        if (self.properties && self.properties.title) {
            self.properties.title = modifiedName || properties.title || "";
        }

        mainscreen.refreshConnectedLists();
    }

    function restoreIconPathFromCategories () {
        var category = details.categories.category, categoryName;
        if (category.length === undefined) {
            category = [category];
        }

        categoryName = category[0] ? category[0].name : undefined;

        iconPath = JsPlaceStaticHelper.getIconSourceFromCategoryNameStatic(categoryName, 0, isFavorite, "list", mapAssetsPath);
    }

    onIsFavoriteChanged: {
        var tmp = iconPath;
        if (properties.isPt) {
            if (!isFavorite && publictransport) {
                iconPath = cloudAdapter.getPtIcon(self, "list");
            } else {
                iconPath = JsPlaceStaticHelper.getIconSourceFromCategoryNameStatic("public-transport", 0, isFavorite, "list", mapAssetsPath);
            }

        } else if (!tmp || tmp === "") {
            var cat = categories.length > 0 ? categories[0].name : (details && details.categories) ? details.categories.category : "";
            iconPath = JsPlaceStaticHelper.getIconSourceFromCategoryNameStatic(cat, 0, isFavorite, "list", mapAssetsPath);
        } else if (tmp !== "" && tmp.replace) {
            tmp = tmp.replace(/_fav|_pp/g, "");
            tmp = tmp.substring(0, tmp.lastIndexOf(".")) + (isFavorite ? "_fav" : (properties.typeNumeric&2?"_pp":"")) + ".png";
            iconPath = tmp;
        }

        favorite = getFavorite();
    }
}
