// -*- mode: js2; espresso-indent-level: 2; -*-

.pragma library

Qt.include("PlaceHelper.js")

var triggerReloadRecommendations = false;

var mainscreen, object, guidesModel, galleryModel, reviewsModel, recommendationsModel, viewerInfoPopup, qsTranslate, view, log, PageStatus, ready = false;

function setLocalVariables() {
    mainscreen = arguments[0];
    object = arguments[1];
    guidesModel = arguments[2];
    galleryModel = arguments[3];
    reviewsModel = arguments[4];
    recommendationsModel = arguments[5];
    viewerInfoPopup = arguments[6];
    qsTranslate = arguments[7];
    view = arguments[8];
    log = arguments[9];
    PageStatus = arguments[10];

    initLibrary(null, mainscreen, null, qsTranslate);

    ready = true;
}

function loadRecommendations(model, object, errorCallback, queryMore) {
    model.queryFailed.connect(errorCallback);

    var boundingBox = {};

    if (object.properties.geoLatitude && object.properties.geoLongitude) {
        log.trace("using the spare map to generate bounding box");
        mainscreen.spareMap.center.latitude = object.properties.geoLatitude;
        mainscreen.spareMap.center.longitude = object.properties.geoLongitude;
        boundingBox = mainscreen.spareMap.getMapBoundingBox(0, -150, mainscreen.width, mainscreen.height + 150)
    } else {
        log.trace("using the current active map to generate bounding box");
        boundingBox = mainscreen.mapGlobalData.activeMap?
            mainscreen.mapGlobalData.activeMap.getMapBoundingBox(0, -150, mainscreen.mapGlobalData.activeMap.width, mainscreen.mapGlobalData.activeMap.height + 150)
            :
            {
                edgeTopLeft: { latitude: 0, longitude: 0 },
                edgeTopRight: { latitude: 0, longitude: 0 },
                edgeBottomRight: { latitude: 0, longitude: 0 },
                edgeBottomLeft: { latitude: 0, longitude: 0 }
             }
        ;
    }

    log.trace("the bounding box is: " + JSON.stringify(boundingBox));

    model.query({
        type: "RECO",
        latitude: object.properties.geoLatitude,
        longitude: object.properties.geoLongitude,
        radius: 3,
        total: queryMore ? 25 : 3,
        pid: object.properties.placesId,
        boundingBox: {
            topLeft: { latitude: boundingBox.edgeTopLeft.latitude, longitude: boundingBox.edgeTopLeft.longitude },
            bottomRight: { latitude: boundingBox.edgeBottomRight.latitude, longitude: boundingBox.edgeBottomRight.longitude}
        },
        useBoundingBox: mainscreen.enableRecommendationsBoundingBox && mainscreen.mapGlobalData.activeMap
    });

}

function preClearEverything() {
    mainscreen.clearModel(guidesModel)
    mainscreen.clearModel(galleryModel);
    mainscreen.clearModel(reviewsModel);
    mainscreen.clearModel(recommendationsModel);
}

function loadPlaceData(analyticsComponent) {
    logPlacePageDetails(analyticsComponent);
    preClearEverything();

    // recommendationsModel does automagically clean itself upon loading new data

    if (object.properties.placesId) {
        object.readFinished.connect(onReadDetails);
        object.read({ loadDetails: true, errBack: abortSearch });
    }

    reloadRecommendations();
}

function reloadRecommendations() {
    if (view.recommendationsLeft > 0 && object.properties.geoLatitude && object.properties.geoLongitude) {
        loadRecommendations(recommendationsModel, object, abortSearch, !object.properties.placesId);
    }
}

function logPlacePageDetails(analyticsComponent) {
    var requester = "search";

    if (object.isFavorite == true) {
        requester = "favourite";
    } else if (object.categories.name === "public-transport") {
        requester = "public-transport";
    } else if (object.isRecommendation) {
        requester = "recommendation";
    } else if (typeof(object.properties.placeId) == "undefined" && typeof(object.properties.typeNumeric) == "undefined") {
        requester = "address";
    }//else is search

    if (typeof (analyticsComponent) != "undefined") {
        analyticsComponent.logIMEIActivity("POIView", { Requester: requester } );
        var placeId = returnExist(object, "properties", "placesId");
        if (placeId) analyticsComponent.logUUIDOther("POIView", { PlaceId: placeId } );
    //    log.info("--- --- Search properties: " + JSON.stringify(object.properties));
    //    log.info("--- --- Search properties type: " + typeof(object.properties));
    //    log.info("--- --- Search categories: " + JSON.stringify(object.categories));
    }
}

function disconnectAllSignals() {
    object.readFinished.disconnect(onReadDetails);
    object.readFinished.disconnect(onReadReviews);
}

function splat(data) {
    if (typeof data.length == "undefined" && typeof data[0] == "undefined") {
        data = [data];
    }
    return data;
}

function abortSearch(err) {
    if (err) {
        log.info("Abort search: "+err); // Error might not be JSON, dont stringify it!
        if (err.statusText) viewerInfoPopup.message = err.statusText;
    }
}

function returnExist() {
    var obj = arguments[0];
    if (typeof obj == "undefined") {
        return "";
    }
    for (var i = 1, il = arguments.length; i < il; i++) {
        if (typeof obj[arguments[i]] == "undefined") {
            return "";
        } else {
            obj = obj[arguments[i]];
        }
    }
    return obj;
}

function onReadDetails(err, obj) {
    log.trace("onReadDetails done");

    var media = returnExist(obj,"details","adPlaceContent", "mediaTypes","adPlaceMedia");
    var provider = returnExist(obj, "details","adPlaceContent", "provider");
    if (media) {
        media=splat(media);
        for (var a in media) if (media.hasOwnProperty(a)){
            if (media[a].mediaType=="IMAGE") galleryModel.append({URL: media[a].mediaUrl, adContent: true, provider: provider });
        }
    }

    var guides = returnExist(obj,"details","premiumContent","version","content");
    if (guides){
        guides=splat(guides);
        for (var a in guides) if(guides.hasOwnProperty(a)){
            var description = guides[a].description || guides[a]["short-description"];
            if (description) guidesModel.append({icon: guides[a].providerIconUrl, caption:guides[a].providerDisplayName, description:description});
            if (guides[a].media){
                var media = splat(guides[a].media);
                for (var img in media) if (media.hasOwnProperty(img)){
                    galleryModel.append({URL:media[img].content, provider: guides[a].providerDisplayName})
                }

            }

        }
    }

    obj.readFinished.disconnect(onReadDetails);
    obj.readFinished.connect(onReadReviews);
    obj.read({ loadReviews: true, errBack: abortSearch });
}

function onReadReviews(err, obj) {
    var caption;

    var reviews = returnExist(obj,"reviews","review");
    if (reviews) {
        reviews = splat(reviews);
        for (var a in reviews) {
            if (reviews.hasOwnProperty(a)) {
                caption = reviews[a].creationDate? Qt.formatDate(new Date(reviews[a].creationDate.match(/(.*)T/)[1].replace(/-/g, "/")), Qt.DefaultLocaleLongDate) : "";
                reviewsModel.append({
                    icon: reviews[a].vendorIconUrl || "",
                    caption: caption,
                    description:reviews[a].description,
                    url:reviews[a].originatorUrl
                });
            }
        }
    }

    obj.readFinished.disconnect(onReadReviews);
}

function getCategoryIcon(object) {
    var icon =  getIconSourceFromId(
            object && getCategory(object.categories) && getCategory(object.categories).id,
            object && object.properties && object.properties.typeNumeric,
            object && object.isFavorite,
            "list"
        );
    if (icon.indexOf("default") !== -1) {
        icon = getIconSourceFromCategoryName(
                returnExist(
                    splat(returnExist(object, "categories") ||
                          returnExist(object, "details", "categories", "category"))[0], "name"),
                          object && object.properties && object.properties.typeNumeric,
                          object && object.isFavorite,
                          "list"
            );
    }

    return icon;
}



function createSubPage(container, index, item, view, newPlace) {
    container.recommendationsLeft = view.recommendationsLeft - 1;

    var place = recommendationsModel.get(index);
    var placeClone = newPlace.createObject(container);
    var json = JSON.stringify(place.properties);
    var json_cats = JSON.stringify(place.categories);
    placeClone.properties = JSON.parse(json);
    placeClone.categories = JSON.parse(json_cats);
    placeClone.uri = place.properties.placesId;
    placeClone.placeId = place.properties.placesId;
    placeClone.name = place.name;
    placeClone.iconPath = item.iconPath;
    placeClone.isRecommendation = item.isRecommendation;

    container.object = placeClone;
    //container.navigateTo.connect(view.navigateTo);
    container.favoriteChanged.connect(view.favoriteChanged);
    return container;
}


function pageStatusChanged() {
    if (view.status === PageStatus.Activating) object.favorite = object.getFavorite();
    if (view.status === PageStatus.Deactivating) disconnectAllSignals();
}

function getPlaceTitle(object) {
    return returnExist(object, "favorite", "text") || returnExist(object, "properties", "title") || returnExist(object, "details", "names", "defaultName", "name") || "";
}

function getPlaceFavoriteDescription(object) {
    return returnExist(object, "favorite", "description") || "";
}

function getPlacePhone(object) {
    return returnExist(object,"details","contact","phone") || returnExist(object,"properties","phoneNumber") || "";
}

function getPlaceWebsite(object) {
    return returnExist(object,"details","contact","website") || "";
}
function getPlaceEmail(object) {
    return returnExist(object,"details","contact","email")|| "";
}
function getPlaceAddress(object) {
    if (object) {
        if(object.categories && object.categories.name === "public-transport") {
            return getPlaceSubtitle(object);
        }

        return formatAddress(object.properties) ||
            formatAddress(returnExist(object,"details","location","address"))
    }
    return ""
}

function getPlaceProvider(object) {
    return returnExist(object, "details", "provider");
}


function getPlaceAdditionalInformation(object) {
    if (object && object.details && object.details.a_id)
        return returnExist(object,"details","adPlaceContent","descriptions","description","localizedDescription")
    else
        return ""
}

function getPlaceOpeningHours(object) {
    if (object && object.details && object.details.a_id) {
        var openingHours = returnExist(object,"details","adPlaceContent","businessHours","openingSchedule","openingHours");
        if (!openingHours)
            return "";

        var result = [];
        for (var i in openingHours) {
            if (openingHours.hasOwnProperty(i)) {
                var item = openingHours[i];
                result.push(item.dayOfWeek + " " +item.openFrom+ " - "+item.openTo);
            }
        }
        return result.join("<br>");

    } else {
        return "";
    }
}


function getPlaceAnnualClosing(object) {
    if (object && object.details && object.details.a_id)
        return returnExist(object,"details","adPlaceContent","businessHours","annualclosingsnotes","annualclosingsnote","_v");
    else
        return ""
}

function getPlacePaymentMethods(object) {
    if (object && object.details && object.details.a_id) {
        var paymentMethods = returnExist(object,"details","adPlaceContent","paymentMethods","paymentMetod");
        if (!paymentMethods) return "";
            paymentMethods = splat(paymentMethods);

        var result = [];

        for (var i in paymentMethods) {
            if (paymentMethods.hasOwnProperty(i)) {
                result.push(paymentMethods[i].name);
            }
        }

        return result.join("<br>");

    } else {
        return "";
    }
}

function getPlaceRating(object) {
    var ratings = returnExist(object, "details", "averageRatings","averageRating");
    if (ratings) {
        ratings = splat(ratings);
        for (var i = 0, il = ratings.length; i < il; i++) {
            if (ratings[i].ratingType == "OVERALL") {
                return { count: ratings[i].ratingCount, value: Math.round(ratings[i].rating) };
            }
        }
    }
    return { value: 0, count: 0 };
}

function getPlaceCategoryName(object) {
    var cat = returnExist(object, "details", "categories", "category");
    if (cat) {
        cat = splat(cat);
        return cat[0]? cat[0].displayName : "";
    }
    // REMOVED AS NEEDED 
    /*else if (returnExist(object, "categories", "name") === "public-transport") {
        return qsTranslate("Maps", "qtn_placesconnector_viewer_category_pt");
    }*/
    if (returnExist(object,"properties","placesId")) return " ";

    return "";
}
