import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0

import "PlacesCloudAdapter.js" as PCA
import "PlaceViewerLogic.js" as Logic
import "ui"

Sheet {
    id: placesAddAsPlaceSheet

    property variant pickedObject

    visualParent: mainscreen

    Item {
        id: internals

        property variant placesCategoriesData: null

        property string categoryButtonFieldKey: ""
        property string noaAccountId: "your@email.com"

        function handleDoOpen() {
        }
        function handleDoClose() {
        }

        function fillContent() {
            if (pickedObject) {
                if (!internals.placesCategoriesData) {
                    var lang = appState.locale.displayName.replace(/_/g,"-");
                    PCA.request(placesURI + "categories/find-places/grouped", internals.requestDone, internals.requestError, { headers: { "Accept-Language":lang } } );
                }

                nameInputField.text = pickedObject? Logic.getPlaceTitle(pickedObject).substring(0,90) + (pickedObject && Logic.getPlaceAddress(pickedObject).length > 90 ? "..." : "") : "";
                regionInputField.text = "";
                if (pickedObject.properties) {
                    streetInputField.text = pickedObject.properties.addrStreetName? pickedObject.properties.addrStreetName : "";
                    numberInputField.text = pickedObject.properties.addrHouseNumber? pickedObject.properties.addrHouseNumber : "";
                    zipInputField.text = "";
                    cityInputField.text = pickedObject.properties.addrCityName? pickedObject.properties.addrCityName : "";
                    regionInputField.text = pickedObject.properties.addrDistrictName? pickedObject.properties.addrDistrictName : "";
                }
                additionalInfoInputField.text = "";
                internals.categoryButtonFieldKey = "";
                categoryButtonField.text = "";

                waitSomeBeforeUpdateTheMinimap.start();
            }
        }

        function requestDone(response) {
            if (response && response.categories && response.categories.category) {
                internals.placesCategoriesData = response.categories.category;

                for (var i = 0; i < response.categories.category.length; i++) {
                    categorySelectionDialog.model.append({
                        name: response.categories.category[i].displayName,
                        displayName: response.categories.category[i].displayName,
                        pureName: response.categories.category[i].name,
                        language: response.categories.category[i].language
                    });
                }
            }
        }

        function requestError(response) {
            internals.placesCategoriesData = null;
        }

        function checkSaveConditions() {
            var success = true;

            if (success && nameInputField.withDefaultValue) {
                queryDialog.titleText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_error_name_required_caption");
                queryDialog.message = qsTranslate("Maps", "qtn_placesconnector_new_place_save_error_name_required");
                queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_error_name_required_close");
                queryDialog.open();

                success = false;

                placesAddAsPlaceSheet.open();
                internals.handleDoOpen();
            }
            if (success && categoryButtonField.withDefaultValue) {
                queryDialog.titleText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_error_categorey_required_caption");
                queryDialog.message = qsTranslate("Maps", "qtn_placesconnector_new_place_save_error_category_required");
                queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_error_category_required_close");
                queryDialog.open();

                success = false;

                placesAddAsPlaceSheet.open();
                internals.handleDoOpen();
            }

            return success;
        }

        function doSavePlaceAsPlace() {
            var jsonObject = {
                "place": {
                    "categories": {
                        "category": [
                            {
                                "name": internals.categoryButtonFieldKey
                            }
                        ]
                    },
                    "contact": {
                        "phone": Logic.getPlacePhone(pickedObject),
                        "email": Logic.getPlaceEmail(pickedObject),
                        "website": Logic.getPlaceWebsite(pickedObject)
                    },
                    "location": {
                        "address": {
                            "countryCode": pickedObject.properties.addrCountryCode? pickedObject.properties.addrCountryCode : "",
                            "street": streetInputField.text,
                            "houseNumber": isNaN(numberInputField.text)? 10 : numberInputField.text,
                            "additionalInfo": additionalInfoInputField.text
                        },
                        "geoCoordinates": {
                            "latitude": pickedObject.properties.geoLatitude,
                            "longitude": pickedObject.properties.geoLongitude
                        }
                    },
                    "names": {
                        "defaultName": {
                            "language": appState.locale.displayName.replace(/_/g,"-"),
                            "name": nameInputField.text
                        }
                    }
                }
            };

            PCA.submit(
                placesURI,
                //"http://uc-stable-qa.places.public.devbln.europe.nokia.com/rest/v1/places/",
                //"http://places.maps.ovi.com/rest/v1/places/",
                function (response) {
                    if (response && (response.status == 201 || response.status == 303)) {
                        queryDialog.titleText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_success_caption");
                        queryDialog.message = qsTranslate("Maps", "qtn_placesconnector_new_place_save_success_body");
                        queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_success_close");
                        queryDialog.open();

                        internals.handleDoClose();
                    } else {
                        queryDialog.titleText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_failed_caption");
                        queryDialog.message = qsTranslate("Maps", "qtn_placesconnector_new_place_save_failed_body");
                        queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_failed_close");
                        queryDialog.open();

                        placesAddAsPlaceSheet.open();
                        internals.handleDoOpen();
                    }
                },
                function (error) {
                    queryDialog.titleText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_failed_caption");
                    queryDialog.message = qsTranslate("Maps", "qtn_placesconnector_new_place_save_failed_body");
                    queryDialog.rejectButtonText = qsTranslate("Maps", "qtn_placesconnector_new_place_save_failed_close");
                    queryDialog.open();

                    placesAddAsPlaceSheet.open();
                    internals.handleDoOpen();
                },
                {
                    headers: {"X-Ovi-NcimUser": "your@email.com", "Content-type":"application/json"},
                    data: JSON.stringify(jsonObject)
                    //data: '{   "place":{      "categories":{         "category":[            {               "name":"accommodation"            }         ]      },      "contact":{         "phone":"+49-211-7182974",         "email":"mail@nokia.com",         "website":"http://www.onet.pl/"      },      "location":{         "address":{            "countryCode":"DEU",            "street":"Genjxkwrgmaftageau",            "houseNumber":158,            "additionalInfo":"AdditionalInfo"         },         "geoCoordinates":{            "latitude": 53.2,            "longitude":51.1         }      },      "names":{         "defaultName":{            "language":"de",            "name":"testtest"         }      }   }}'
                }
            );
        }

        function handleSaveClicked() {
            var result;

            if (internals.checkSaveConditions()) {
                internals.doSavePlaceAsPlace();

                result = true;
            } else {
                result = false;
            }

            return result;
        }
    }

    acceptButtonText: qsTranslate("Maps", "qtn_placesconnector_new_place_save")
    rejectButtonText: qsTranslate("Maps", "qtn_placesconnector_new_place_cancel")

    content: Rectangle {
        color:"#f0f0f0"
        anchors.fill: parent

        Flickable {
            id: flickableContent
            clip: true
            anchors.fill:parent
            contentHeight: flickContent.height
            boundsBehavior: Flickable.StopAtBounds

            Column {
                id: flickContent
                width: parent.width
                spacing: 10

                Rectangle {
                    width: parent.width
                    height: 40
                    color: "transparent"

                    Text {
                        text: qsTranslate("Maps", "qtn_placesconnector_new_place_add_as_header")

                        color: "#000"
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        font.family: mainscreen.__labelStyle.fontFamily
                        font.letterSpacing: -1
                        font.pixelSize: 23
                    }
                }

                MapPreviewMini {
                    id: mapPreviewMini

                    object: placesAddAsPlaceSheet.pickedObject

                    height: 130
                    width: parent.width

                    z: 1

                    clickingOnMapEnabled: false

                    Timer {
                        id: waitSomeBeforeUpdateTheMinimap
                        interval: 250
                        onTriggered: { mapPreviewMini.update(); }
                    }
                }

                Rectangle {
                    width: parent.width
                    height: 30
                    color: "transparent"

                    Text {
                        text: qsTranslate("Maps", "qtn_placesconnector_new_place_name_and_category")

                        color: "#000"
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        font.family: mainscreen.__labelStyle.fontFamily
                        font.letterSpacing: -1
                        font.pixelSize: 20
                    }
                }

                TextFieldWithDefaultValue {
                    id: nameInputField
                    objectName: "nameInputField"

                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10

                    defaultValue: qsTranslate("Maps", "qtn_placesconnector_new_place_name")

                }

                ButtonFieldWithDefaultValue {
                    id: categoryButtonField
                    objectName: "categoryButtonField"

                    anchors.horizontalCenter: parent.horizontalCenter

                    defaultValue: {
                        var original = qsTranslate("Maps", "qtn_placesconnector_new_place_category_default");
                        return original.charAt(0).toUpperCase() + original.substring(1);
                    }

                    onClicked: {
                        categorySelectionDialog.open();
                    }
                }

                Rectangle {
                    width: parent.width
                    height: 30
                    color: "transparent"

                    Text {
                        text: qsTranslate("Maps", "qtn_placesconnector_new_place_address")

                        color: "#000"
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        font.family: mainscreen.__labelStyle.fontFamily
                        font.letterSpacing: -1
                        font.pixelSize: 20
                    }
                }

                Row {
                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10

                    TextFieldWithDefaultValue {
                        id: streetInputField
                        objectName: "streetInputField"

                        //anchors.left: parent.left
                        //anchors.right: numberInputField.left
                        width: parent.width - numberInputField.width

                        defaultValue: qsTranslate("Maps", "qtn_placesconnector_new_place_street")
                    }
                    TextFieldWithDefaultValue {
                        id: numberInputField
                        objectName: "numberInputField"

                        //anchors.right: parent.right
                        width: 150

                        defaultValue: qsTranslate("Maps", "qtn_placesconnector_new_place_number")
                    }
                }

                TextFieldWithDefaultValue {
                    id: zipInputField
                    objectName: "zipInputField"

                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10

                    defaultValue: qsTranslate("Maps", "qtn_placesconnector_new_place_zip")
                }

                TextFieldWithDefaultValue {
                    id: cityInputField
                    objectName: "nameInputField"

                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10

                    defaultValue: qsTranslate("Maps", "qtn_placesconnector_new_place_city")
                }

                TextFieldWithDefaultValue {
                    id: regionInputField
                    objectName: "nameInputField"

                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10

                    defaultValue: qsTranslate("Maps", "qtn_placesconnector_new_place_region")
                }

                Rectangle {
                    width: parent.width
                    height: 30
                    color: "transparent"

                    Text {
                        text: qsTranslate("Maps", "qtn_placesconnector_new_place_additional_info")

                        color: "#000"
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        font.family: mainscreen.__labelStyle.fontFamily
                        font.letterSpacing: -1
                        font.pixelSize: 20
                    }
                }

                TextFieldWithDefaultValue {
                    id: additionalInfoInputField
                    objectName: "nameInputField"

                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10
                }

                Rectangle {
                    width: parent.width
                    height: 100
                    color: "#00000000"
                }
            }
        }
        ScrollDecorator {
            flickableItem: flickableContent
        }
    }

    SelectionDialog {
        id: categorySelectionDialog

        titleText: {
            var original = qsTranslate("Maps", "qtn_placesconnector_new_place_category_default");
            return original.charAt(0).toUpperCase() + original.substring(1);
        }

        model: ListModel { }

        onAccepted: {
            categoryButtonField.text = categorySelectionDialog.model.get(categorySelectionDialog.selectedIndex).displayName;
            internals.categoryButtonFieldKey = categorySelectionDialog.model.get(categorySelectionDialog.selectedIndex).pureName;
        }
        onRejected: {
            var original = qsTranslate("Maps", "qtn_placesconnector_new_place_category_default");
            categoryButtonField.text = original.charAt(0).toUpperCase() + original.substring(1);
            categorySelectionDialog.selectedIndex = -1;
        }
    }

    QueryDialog {
        id: queryDialog
    }

    onPickedObjectChanged: {
        internals.fillContent();
    }
    onAccepted: {
        if (internals.handleSaveClicked()) {
        }
        internals.handleDoClose();
    }
    onRejected: {
        internals.handleDoClose();
    }
    Component.onCompleted: {
        //internals.fillContent();
    }

    function doOpen() {
        internals.handleDoOpen();
        open();

        if (rootApplicationLevel && rootApplicationLevel.sso) {
            if (!rootApplicationLevel.sso.noaAccountId) {
                var sync = function (success) {
                    rootApplicationLevel.sso.signInComplete.disconnect(sync);

                    if (success) {
                        internals.noaAccountId = rootApplicationLevel.sso.noaAccountId;
                    } else {
                    }
                };

                rootApplicationLevel.sso.signInComplete.connect(sync);
                rootApplicationLevel.sso.initialize();
            } else {
                internals.noaAccountId = rootApplicationLevel.sso.noaAccountId;
            }
        }

    }

}
