// -*- mode: js2; espresso-indent-level: 4; -*-
// PlaceCloudAdapter.js
//
// A fake server implmentation
.pragma library

var mainscreen;
var log;

/**
 * This function helps store mainscreen reference so you dont need to pass it
 * in every instance of XHR request. This is needed because of using a
 * .pragma library that cannot access QML objects directly 
 */

function setMainScreenAndLog(screen,aLog){
    mainscreen = screen;
    log = aLog;
}

function evaluateRecommendationsUri (uri, options) {
    var recommendationsUri = uri+'%{latitude},%{longitude}/%{radius}/%{total}/core/no-personal/*',
        recommendationsBBUri = uri+'%{latitude1},%{longitude1}/%{latitude2},%{longitude2}/%{total}/core/no-personal/*',
        returnUri;

    ['latitude', 'longitude', 'radius', 'total'].forEach(function(value) {
      if(options[value] === undefined) {
        throw new Error('options must contain at least latitude, longitude, radius, total');
      }
    });

    if (options.useBoundingBox && options.boundingBox) {
        returnUri = recommendationsBBUri.
            replace('%{latitude1}', Math.max(options.boundingBox.topLeft.latitude, options.boundingBox.bottomRight.latitude)).
            replace('%{longitude1}', Math.min(options.boundingBox.topLeft.longitude, options.boundingBox.bottomRight.longitude)).
            replace('%{latitude2}', Math.min(options.boundingBox.topLeft.latitude, options.boundingBox.bottomRight.latitude)).
            replace('%{longitude2}', Math.max(options.boundingBox.topLeft.longitude, options.boundingBox.bottomRight.longitude));
    } else {
        returnUri = recommendationsUri.
            replace('%{latitude}', options.latitude).
            replace('%{longitude}', options.longitude);
    }

    returnUri=returnUri.
        replace('%{radius}', options.radius).
        replace('%{total}', options.total);

    if (options.pid) {
        returnUri += '?pid='+options.pid;
    }

    return returnUri;
}

function createTimeOutTimer(xhr, callback) {
    var component = Qt.createComponent('ui/Timeout.qml');
    var timer = component.createObject(null);
    timer.interval = 15 * 1000;  // timeout: 15 seconds

    timer.running = false;
    timer.timerSignal.connect(function () {
        if (typeof callback === 'function') {
            callback();
        }
    });

    return timer;
}

//var xhr;

function submit(url, callback, errback, options) {
    callback = (typeof(callback) == "function")? callback : function () {};
    errback = (typeof(errback) == "function")? errback : function () {};

    if (!mainscreen.onlineMode) {
        log.info("Application is in OFFLINE mode, no request is sent");
        errback({status: "503", reason: "APPLICATION_OFFLINE"});
        return;
    }
    var xhr = new XMLHttpRequest();

    var http_state = true;

    var timer = createTimeOutTimer(xhr, function () {
        log.warning("Connection timed out !", "PlacesCloudAdapter.js");
        http_state = false;
        xhr.abort();
        errback({status: "408", reason: "SERVER_TIMEOUT"});
    });

    xhr.open("POST", url, true);

    if (options && options.headers !== undefined) {
        for (var index in options.headers) {
            xhr.setRequestHeader(index, options.headers[index]);
        }
    }

    xhr.onreadystatechange = function () {
        if (xhr.readyState === 4 && http_state) {
            timer.stop();
            if (xhr.status === 204) {
                log.info("Cloudadapter: No search results found in this area.")
                callback({results:[]});
            } else if (xhr.responseText === "" ) {
                log.warning("Cloudadapter: Empty reply from server, but *NO* 204 error (no results found) from the server. ");
                callback({results:[], status: xhr.status});
            } else {
                var parsedData;
                try {
                    parsedData = JSON.parse(xhr.responseText);
                } catch (e) {
                    parsedData = {result: [], status: xhr.status};
                }

                callback(parsedData);
            }
        } else if ( xhr.readyState === 3 && xhr.status.toString().charAt(0) !== '2' ) {
            // they send us 303...
            /*
            log.warning("Cloudadapter: XHR error occured in state " + xhr.readyState + " : " + xhr.responseText)
            timer.stop();

            http_state = false;

            if (errback) {
                errback({status: "408", reason: "SERVER_TIMEOUT"});
            } else {
                log.error("Cloud adapter error is uncatched :" + JSON.stringify(xhr) + ". Provide a error callback when using search.");
            }
            */
        }
    };

    log.info('PlacesCloudAdapter: Going to query ' + url + " :: " + options.data);

    xhr.send(options && options.data? options.data : null);
    timer.start();
}

function request(url, callback, errback, options) {
    callback = (typeof(callback) == "function")? callback : function () {};
    errback = (typeof(errback) == "function")? errback : function () {};

//    if(!xhr)
//
    if (!mainscreen.onlineMode) {
        log.info("Application is in OFFLINE mode, no request is sent");
        errback({status: "503", reason: "APPLICATION_OFFLINE"});
        return;
    }
    var xhr = new XMLHttpRequest();

    var http_state= true;

    /*
    if(xhr.readyState > 0 && xhr.readyState < 4){
        log.info("PCA: interrupting old xhr request : "+xhr.readyState);
        xhr.abort(); // new request while old is pending, so interrupt without notifying user
    }
    */

    var timer = createTimeOutTimer(xhr, function () {
        log.warning("Connection timed out !","PlacesCloudAdapter.js");
        http_state = false;
        xhr.abort();
        errback({status: "408", reason: "SERVER_TIMEOUT"});  // Server timeout. handle separately, that's no reals server response or response error in http status
    });

    //xhr.onreadystatechange = undefined;
    xhr.open("GET", url, true);

    // DEBUG: alternative test request:
        // test for illegal coordinates => 400 bad request
        //xhr.open("GET", 'http://places.maps.ovi.com/rest/v1/recommendations/nearby/-100,13.23/3/25/core/no-personal/*', true);
        // log.info("################### FIXME");
        // test for south pole => 204 empty content (obviously)
        // xhr.open("GET", 'http://places.maps.ovi.com/rest/v1/recommendations/nearby/37.0625,-95.677068/3/25/core/no-personal/*', true);

    if (options && options.headers !== undefined) {
        for (var index in options.headers) {
            xhr.setRequestHeader(index, options.headers[index]);
        }
    }

    xhr.onreadystatechange = function () {
//         log.info("################### FIXME");
//         log.info('readyState: '+xhr.readyState);
//         log.info('status: '+xhr.status);
//         log.info('url: '+url);
//         log.info('statusText: '+xhr.statusText);
//         log.info('responseText: '+xhr.responseText);
//         log.info("State: "+http_state);
         //log.info(JSON.stringify(xhr.getAllResponseHeaders()));

        // This function is a closure storing the state of the running request
        // in the variable of the same name (state). If - through checking the
        // status code during request phase [3] - the state should be false in
        // phase [4], the success callback argument never gets executed; an
        // exception gets thrown during phase [3] *instead* so the caller can
        // handle it. For the special case of HTTP status code 204 returned, the
        // callback gets called with an empty object.
        //
        // For all state/readyState definitions see: http://www.w3.org/TR/XMLHttpRequest/#states
        // For all http status codes see: http://de.wikipedia.org/wiki/HTTP-Statuscode
        //
        // e.g. for recommendations, see
        // http://hudson.places.devbln.europe.nokia.com/hudson/view/Recommendations/job/recommendations-nightly-build/site/user/nearby_api.html
        if (xhr.readyState === 4 && http_state) {
            timer.stop();
            if (xhr.status === 204) {
                log.info("Cloudadapter: No search results found in this area.")
                callback({results:[] });
            } else if (xhr.responseText === "" ) {  // known cases: xhr.abort() is called or phone is in offline mode
                log.warning("Cloudadapter: Empty reply from server, but *NO* 204 error (no results found) from recommendations server. ");
                callback({results:[] });
            } else {
                callback(JSON.parse(xhr.responseText));
            }
        } else if ( xhr.readyState === 3 && xhr.status.toString().charAt(0) !== '2' ) {
            log.warning("Cloudadapter: XHR error occured in state "+xhr.readyState+" : "+xhr.responseText)
            timer.stop();

            http_state = false; // prevent processing state 4, if error occurs in state <4

            if (errback) {
                // if(xhr.responseText)  // show detailed error if available
                //    errback(xhr.responseText); // DONT JSON.parse response as this might not be a JSON at all!!
                // else   // should not happen, at least not seen up to now
                    errback({status: "408", reason: "SERVER_TIMEOUT"});  // no details known
            } else {
                log.error("Cloud adapter error is uncatched :"+JSON.stringify(xhr)+". Provide a error callback when using search.");
            }
        }
    };

    log.info('PlacesCloudAdapter: Going to query ' + url);

    xhr.send();
    timer.start();
}

function search (uri, callback, errback, options) {
    if (options) {
        var URL = [];
        if (options.searchTerm) {
            URL.push("q=" + options.searchTerm);
            URL.push("vi=where");
        }
        if (options.searchCategory) URL.push("ta=" + options.searchCategory);
        URL.push("lat=" + options.location.latitude);
        URL.push("lon=" + options.location.longitude);
        URL.push("dv=pca");
        URL.push("to=10");

        URL = uri + "?" + URL.join("&");

        // Using "pca" for "place cloud adapter" as DV parameter. This was aligned with Bernd. [Mkr]
        // See Ticket SLVD-414
        // Compare http://wikis.in.nokia.com/LocalSearch/NokiaSearchProtocol -- akahl

        if (options.enableDidYouMean) {
            URL += "&dym=" + 1;
        }

        request(URL, callback, errback, {});
    }
}

function getSuggestions (uri, callback, errback, options) {
    if(!options){
        log.warning("Suggestions: Sorry, you need to provide a position")
    }
    var URL = uri+"?q=" + options.searchTerm + "&vi=where&lat="+options.location.latitude+"&lon="+options.location.longitude+"&lh=1&to=6";
    request(URL, callback, errback, {});
}

function getRecommendations (uri, callback, errback, options) {
    request(evaluateRecommendationsUri(uri, options), callback, errback, {
            headers: { 'X-Ovi-User': 'test', 'X-User-Agent': 'Maps-QML-SR1' }  // todo: implement headers for recommendation requests. See reco spec [Mkr]
        });
}
