import QtQuick 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.favorites 1.0

/// Put data storage in js, to have an app-wide singleton, this is a fake server
import "PlacesCloudAdapter.js" as JsCloudAdapter
import "PlaceHelper.js" as JsPlaceHelper
import "PublicTransport.js" as PublicTransport

CloudAdapter {
    id: placesCloudAdapter

    property Component objectFactory
    property int count: 0
    property variant _options

    TransitDatabase {
        id: transitDb
    }

    GeoCoordinates {
        id: transitCoord
    }

    PlaceCategoryNode {
        id: placeCategories
    }

    Finder {
        id: finder

        onSearchResult: {
            if (_options.type === "SUGGESTIONS") {
                var res = []
                for (var i = 0, il = results.length; i < il; i++) {
                    res.push(results[i].getField(Location.PLACE_NAME));
                }

                model.suggestionCompleted(res);
                return;
            }

            var objects = [];

            for (var i = 0, il = results.length; i < il; i++) {
                if (results[i].getField(Location.LOCATION_META) === results[i].LOCATION_META_NO_LOCATION_BUT_WITH_DIDYOUMEAN_GUESS) {
                    model.queryFinished(null, objects, model);  // just to finish the call corectly, but it's empty and has no use right now

                    if (_options.parent && _options.parent.didYouMeanCallback) {
                        // TODO: Missing Did you mean translations !!
                        _options.parent.didYouMeanCallback({
                            properties: {
                                preTitle: qsTranslate("Maps", "qtn_placesconnector_did_you_mean") + " ",
                                title: results[i].getField(Location.PLACE_NAME)
                            }
                        });
                    }
                }
            }

            for (var i = 0, j = 0, il = results.length; i < il; i++) {
                if (results[i].getField(Location.LOCATION_META) === results[i].LOCATION_META_SEARCH_CENTER) continue;
                if (results[i].getField(Location.LOCATION_META) === results[i].LOCATION_META_NO_LOCATION_BUT_WITH_DIDYOUMEAN_GUESS) continue;

                var jsplace = results[i];
                var uri = ++count; //TODO! create a better uri incl placeId
                var obj = placesCloudAdapter.createObject(uri);
                var objProperties = {
                    title: jsplace.getField(Location.PLACE_NAME) || jsplace.getField(Location.ADDR_STREET_NAME) || jsplace.getField(Location.ADDR_CITY_NAME) || jsplace.getField(Location.ADDR_COUNTRY_NAME) || jsplace.getField(Location.ADDR_COUNTRY_CODE),
                    placesId: jsplace.getField(Location.PLACE_PREMIUM_NODE_ID),

                    contact: jsplace.getField(Location.PLACE_PHONE_NUMBER),

                    geoLatitude: jsplace.geoCoordinates.latitude,
                    geoLongitude: jsplace.geoCoordinates.longitude,
                    geoDistance: JsPlaceHelper.computeDistance(
                        _options.location,
                        {latitude: jsplace.geoCoordinates.latitude, longitude: jsplace.geoCoordinates.longitude}
                    ),

                    addrCityName: jsplace.getField(Location.ADDR_CITY_NAME),
                    addrCountryCode: jsplace.getField(Location.ADDR_COUNTRY_CODE),
                    addrCountryName: jsplace.getField(Location.ADDR_COUNTRY_NAME),
                    addrDistrictName: jsplace.getField(Location.ADDR_DISTRICT_NAME),
                    addrHouseNumber: jsplace.getField(Location.ADDR_HOUSE_NUMBER),
                    addrPostalCode: jsplace.getField(Location.ADDR_POSTAL_CODE),
                    addrStreetName: jsplace.getField(Location.ADDR_STREET_NAME),

                    phoneNumber:  jsplace.getField(Location.PLACE_PHONE_NUMBER),
                    url:  jsplace.getField(Location.PLACE_URL),
                    typeNumeric: jsplace.getField(Location.PLACE_TYPE)
                };

                obj.placeId = objProperties.placesId;
                obj.categories = [{ id: jsplace.getField(Location.PLACE_CATEGORY_ID) }];
                obj.favorite = obj.getFavorite();
                obj.isFavorite = !!obj.favorite;
                obj.iconPath = JsPlaceHelper.getIconSourceFromId(
                    JsPlaceHelper.getCategory(obj.categories).id,
                    objProperties.typeNumeric,
                    obj.isFavorite,
                    "list"
                );

                if (JsPlaceHelper.getCategory(obj.categories).id == 284) {
                    log.info("pre-formatting title and details of the object as it's address");

                    var useCityInPredefinedAddress = true;

                    var rebuildTitleObject = { addrCountryCode: objProperties.addrCountryCode };
                    if (objProperties.addrStreetName) {
                        rebuildTitleObject.addrStreetName = objProperties.addrStreetName;
                        if (objProperties.addrHouseNumber) {
                            rebuildTitleObject.addrHouseNumber = objProperties.addrHouseNumber;
                        }
                    } else if (objProperties.addrCityName) {
                        rebuildTitleObject.addrCityName = objProperties.addrCityName;
                        if (objProperties.addrPostalCode) {
                            rebuildTitleObject.addrPostalCode = objProperties.addrPostalCode;
                        }

                        useCityInPredefinedAddress = false;
                    } else if (objProperties.addrCountryName) {
                        rebuildTitleObject.addrCountryName = objProperties.addrCountryName;
                        if (objProperties.addrPostalCode) {
                            rebuildTitleObject.addrPostalCode = objProperties.addrPostalCode;
                        }

                        useCityInPredefinedAddress = false;
                    }

                    var rebuildAddressObject = { addrCountryCode: objProperties.addrCountryCode };
                    if (useCityInPredefinedAddress) {
                        rebuildAddressObject.addrCityName = objProperties.addrCityName;
                        if (objProperties.addrPostalCode) {
                            rebuildAddressObject.addrPostalCode = objProperties.addrPostalCode;
                        }
                    }
                    rebuildAddressObject.addrCountryName = objProperties.addrCountryName;

                    objProperties.title = JsPlaceHelper.formatAddress(rebuildTitleObject);
                    objProperties.predefinedAddress = JsPlaceHelper.formatAddress(rebuildAddressObject);
                }

                obj.properties = objProperties;

                objects.push(obj);
            }

            if (mainscreen.onlineMode) {
                model.queryFinished(null, objects, model);  // attentions: needs to be called before "searchComplete"

                if (_options.parent && _options.parent.searchComplete) {
                    _options.parent.searchComplete();
                }
            } else {
                failStartOnOfflineReverseGeoCoding.options = _options;
                failStartOnOfflineReverseGeoCoding.start();

                reverseGeoCodeFinder.objects = objects;
                for (var i = 0; i < objects.length; i++) {
                    reverseGeoCodeFinder.request(i);
                }
            }
        }
    }

    ReverseGeoCodeFinder {
        id: reverseGeoCodeFinder

        property variant requestMap
        property variant objects

        function request(itemIndex) {
            var tmpRequestMap = reverseGeoCodeFinder.requestMap;
            if (!tmpRequestMap) {
                tmpRequestMap = {};
            }

            tmpRequestMap[reverseGeoCodeFinder.find({ latitude: objects[itemIndex].properties.geoLatitude, longitude: objects[itemIndex].properties.geoLongitude }).id] = itemIndex;

            reverseGeoCodeFinder.requestMap = tmpRequestMap;
        }

        onFindComplete: {
            var tmpRequestMap = reverseGeoCodeFinder.requestMap;
            if (tmpRequestMap) {
                var tmpIndex = tmpRequestMap[id];
                if (tmpIndex !== null && tmpIndex !== undefined) {
                    var tmpObjects = reverseGeoCodeFinder.objects;

                    if (tmpObjects) {
                        var tmpObj = tmpObjects[tmpIndex];
                        var tmpProperties = tmpObj.properties;

                        if (context.headingText && context.bodyText) {
                            tmpProperties.predefinedAddress = context.headingText + ", " + context.bodyText;
                        }

                        tmpObj.properties = tmpProperties;
                        tmpObjects[tmpIndex] = tmpObj;
                        reverseGeoCodeFinder.objects = tmpObjects;
                    }
                }
            }
        }
    }

    Timer {
        id: failStartOnOfflineReverseGeoCoding
        interval: 1250

        property variant options

        onTriggered: {
            var tmpObjects = reverseGeoCodeFinder.objects;
            if (tmpObjects) {

                model.queryFinished(null, tmpObjects, model);
            }
            if (options.parent && options.parent.searchComplete) {
                options.parent.searchComplete();
            }

            reverseGeoCodeFinder.cancel();
        }
    }

    Timer {
        id: lazyRetrySearchWithoutDYM
        interval: 250

        onTriggered: {
            finder.placeGeoCode(_options.searchTerm, null,  Finder.PLACE_SEARCH_WHERE, _options.location, 50000, 20, false, 0);
        }
    }

    Component.onCompleted: {
        JsPlaceHelper.initLibrary(mapAssetsPath, mainscreen, null, qsTranslate);
        JsCloudAdapter.setMainScreenAndLog(mainscreen,log);
    }

    function createObject(uri) {
        if (!objectFactory) {
            objectFactory = Qt.createComponent("Place.qml");
        }
        var object = objectFactory.createObject(placesCloudAdapter);
        object.uri = uri;
        return object;
    }

    function fillObject(json, obj) {
        obj.details.isPremium = json.details.isPremium;
    }

    function read (place, options) {
        if (options.loadDetails || options.loadReviews) {
            if (!place.properties.placesId) {
                log.info("place id was not found!");
            } else {
                var lang = appState.locale.displayName.replace(/_/g,"-");
                JsCloudAdapter.request(
                    placesURI + place.properties.placesId + (options.loadReviews ? "/reviews" : ""),
                    function (response) {
                        //log.info("READ response: "+JSON.stringify(response));
                        if (options.loadReviews) {
                            place.reviews = response.reviews;
                        } else {
                            place.details = response.place;
                        }

                        place.readFinished(null, place);
                    },
                    options.errBack,
                    { headers: { "Accept-Language":lang } }
                );
            }
        } else {
            // nothing to do..
            readFinished(null, place);
        }
    }

    function cancel() {
        finder.cancel();
        if (typeof searchInput != "undefined") {
            searchInput.placeholderText = watermark;
        }
    }

    function query(model, options) {
        if (options.type) {
            switch (options.type) {
                case "RECO": queryRecommendations(model, options); break;
                case "PT": queryPublicTransport(model, options); break;
                case "SUGGESTIONS": querySuggestions(model,options); break;
                case "CANCEL": cancel(); break; // A workaround to not exposed functions for PlacesConnectedList
            }
        } else {
            querySearch(model, options);
        }
    }

    function findCategoryNode(node,id){
        if (node.iconId == id) return node;
        for (var a in node.children) if (node.children.hasOwnProperty(a)) {
            var ret = findCategoryNode(node.children[a],id);
            if (ret) return ret;

        }
        return null;
    }

    function querySuggestions(model, options){
        _options = options;
        finder.cancel();

        if (options.searchTerm) {
            log.trace("Suggestion term: "+options.searchTerm);
        }

        if (mainscreen.onlineMode && mainscreen.useKudzoPlacesBackend) {
            searchWithKudzo(options, 0, 1);
        } else {
            finder.placeGeoCode(options.searchTerm, null,  Finder.PLACE_SEARCH_WHERE, options.location, 50000, 20, true, 0);
        }
    }

    function querySearch(model, options) {
        var uri = searchURI;
        _options = options;

        if(options.searchTerm)
            log.trace("Search term: "+options.searchTerm);

        finder.cancel();
        if (options.searchCategory) {
            finder.placeCategoryGeoCode(null, findCategoryNode(placeCategories,options.searchCategory), Finder.PLACE_SEARCH_WHERE, options.location, 50000, 20, false, 0);
        } else {
            if (options.enableDidYouMean) {
                if (mainscreen.onlineMode && mainscreen.useKudzoPlacesBackend) {
                    searchWithKudzo(options, 1);
                } else {
                    finder.placeGeoCode(options.searchTerm, null,  Finder.PLACE_SEARCH_WHERE, options.location, 50000, 20, false, 1);
                }
            } else {
                if (mainscreen.onlineMode && mainscreen.useKudzoPlacesBackend) {
                    searchWithKudzo(options, 0);
                } else {
                    finder.placeGeoCode(options.searchTerm, null,  Finder.PLACE_SEARCH_WHERE, options.location, 50000, 20, false, 0);
                }
            }
        }
    }

    function searchWithKudzo(options, dym, suggestions) {
        var lang = appState.locale.displayName.replace(/_/g,"-");
        JsCloudAdapter.request(
            "http://slo.nose.svc.ovi.com/json?" +
                "q=" + escape(options.searchTerm) +
                "&lat=" + options.location.latitude +
                "&lon=" + options.location.longitude +
                "&vi=where" +
                //"&dv=M6" +
                "&dv=pca" +
                "&la=" + lang +
                "&uu=" + (rootApplicationLevel && rootApplicationLevel.sso && rootApplicationLevel.sso.noaAccountId? rootApplicationLevel.sso.noaAccountId : "anonymous-maps-user") +
                "&dym=" + dym +
                (suggestions? ("&lh=" + suggestions) : "")
            ,
            function (result) {
                if (_options.type === "SUGGESTIONS") {
                    var res = [];
                    for (var i = 0, il = result.length; i < il; i++) {
                        res.push(result[i]);
                    }

                    model.suggestionCompleted(res);
                    return;
                }

                var results = result.results || [];
                var objects = [];

                for (var i = 0, il = results.length; i < il; i++) {
                    if (results[i].type === "DID_YOU_MEAN_SEARCH") {
                        model.queryFinished(null, objects, model);

                        if (_options.parent && _options.parent.didYouMeanCallback) {
                            _options.parent.didYouMeanCallback({
                                properties: {
                                    preTitle: results[i].properties.preTitle + " ",
                                    title: results[i].properties.title
                                }
                            });
                        }
                    }
                }

                for (var i = 0, j = 0, il = results.length; i < il; i++) {
                    if (results[i].type) continue;

                    var jsplace = results[i];
                    var uri = ++count;
                    var obj = placesCloudAdapter.createObject(uri);
                    obj.properties = jsplace.properties;
                    obj.properties.title = jsplace.properties.title || jsplace.properties.addrStreetName || jsplace.properties.addrCityName || jsplace.properties.addrCountryName || jsplace.properties.addrCountryCode;

                    obj.placeId = obj.properties.placesId? obj.properties.placesId : "";

                    obj.categories = jsplace.categories;
                    obj.favorite = obj.getFavorite();
                    obj.isFavorite = !!obj.favorite;
                    obj.iconPath = JsPlaceHelper.getIconSourceFromId(JsPlaceHelper.getCategory(obj.categories).id, obj.properties.typeNumeric, obj.isFavorite, "list");
                    objects.push(obj);
                }

                model.queryFinished(null, objects, model);

                if (_options.parent && _options.parent.searchComplete) {
                    _options.parent.searchComplete();
                }

            },
            function (error) {

            },
            { }
        );
    }

    function getPtIcon(obj, mode) {
        var systems = obj.publictransport.systems;
        var linetypes = obj.publictransport.linetypes;

        if (!obj.isFavorite) {   // If it is a favorite, then the icon is always a transit icon with a star
            if (systems && systems.length === 1 && systems[0].systemLogo && systems[0].systemLogo.imageUrl) {
                // only one system and the system has an icon
                return systems[0].systemLogo.imageUrl;
            } else if ((linetypes & (linetypes - 1)) === 0) {
                // only one line type: show the mode of transport
                var icon = PublicTransport.getPtCategoryIconFromTransportStop(obj.publictransport, mode);

                if (icon) {
                    return icon;
                }
            }
        }
        return JsPlaceHelper.getIconSourceFromCategoryName("public-transport", {}, obj.isFavorite, mode);
    }

    function queryPublicTransport(model, options) {
        log.info("loading pt stations");

        if (!options.latitude || !options.longitude) {
            log.warning("Query pt stations: wrong parameters");
            // todo: error case ?
        }

        var handleSearchDone = function () {
            var objects = [];
            var stops = _getSortedByDistanceArray(transitDb.stops, { latitude: options.latitude, longitude: options.longitude });
            log.trace("PT search done with : " + stops.length + " results.");

            for (var i = 0; i < stops.length && i <  options.total; i++) {  // map results to places objects
                var station = stops[i].item;
                var obj = placesCloudAdapter.createObject(station.placesId);

                var ptObj = getPtInfoFromStop(station, {latitude: options.latitude, longitude: options.longitude});
                obj.properties = ptObj.properties;
                obj.publictransport = ptObj.publictransport;
                obj.categories = ptObj.categories;
                obj.favorite = obj.getFavorite();
                obj.isFavorite = !!obj.favorite;
                obj.iconPath = getPtIcon(obj, "list");
                objects.push(obj);
            }

            model.queryFinished(null, objects, model);
            transitDb.stopSearchDone.disconnect(handleSearchDone);
        }

        transitDb.stopSearchDone.connect(handleSearchDone);
        transitCoord.latitude = options.latitude;
        transitCoord.longitude = options.longitude;
        log.info("PT radius: " + (radius * 1000 || 3000));
        transitDb.searchStop(transitCoord, radius * 1000 || 3000);
    }

    function getPtInfoFromStop(aStation, fromPos) {
        return PublicTransport.createPtPlaceFromStopInfo(aStation, fromPos)
    }

    /**
      * Returns an array of objects {
      *     distance: the distance between the given position and the position of the item,
      *     item: the item in the original array
      * }
      */
    function _getSortedByDistanceArray(aResults, aStartPos) {
        var tmpArray = [];
        for (var i = 0; i < aResults.length; i++) {
            tmpArray.push({
                distance: JsPlaceHelper.computeDistance(
                    {latitude: aStartPos.latitude, longitude: aStartPos.longitude},
                    {latitude: aResults[i].position.latitude, longitude: aResults[i].position.longitude}
                ),
                item: aResults[i]
            });
        }

        tmpArray.sort(_distanceSort);
        return tmpArray;
    }

    function _distanceSort(a, b) {
        var pos1 = a.distance || (a.properties && a.properties.geoDistance);
        var pos2 = b.distance || (b.properties && b.properties.geoDistance);
        if (pos1 < pos2) {
            return -1;
        }
        if (pos1 > pos2) {
            return 1;
        }
        return 0;
    }

    function queryRecommendations(model, options) {
        log.trace("loading recommendations");

        if (!options.latitude || !options.longitude) {
            log.info("Recommendations: wrong parameters");
            // todo: error case ?
        }
            
        options.radius = options.radius || 3;
        options.total = options.total || 10;

        if (options.useBoundingBox && options.boundingBox) {
            // check if bb is valid for backend
            var dist =  JsPlaceHelper.computeDistance(
                     {latitude: options.boundingBox.topLeft.latitude, longitude: options.boundingBox.topLeft.longitude},
                     {latitude: options.boundingBox.bottomRight.latitude, longitude: options.boundingBox.bottomRight.longitude}
            );

            if(dist > 10000){  // backend can handle max of 10km
                options.radius = 10; // use maximum radius
                options.useBoundingBox = false;
            }

            log.info("Bounding box distance: "+dist)
        }

        JsCloudAdapter.getRecommendations(
             recommendationsURI, // from QSettings in PlacesConnector.cpp
             function(resultlist) {
                if (resultlist.recommendations)
                    resultlist = resultlist.recommendations.nearby;
                else
                    resultlist = [];

                var results = [];

                for (var i = 0; i < resultlist.length; i++) {
                    var jsPlace = resultlist[i];
                    //log.info("**** BEFORE: "+JSON.stringify(jsPlace));
                    var obj = placesCloudAdapter.createObject(jsPlace.place.a_id);

                    //if the category is only 1 there is no category[0] - but directly here
                    obj.categories =  jsPlace.place.categories.category[0] || jsPlace.place.categories.category;  // if there are more categories, spec says to take the first one

                    obj.name = jsPlace.place.names.defaultName.name;

                    obj.properties = {
                        title: jsPlace.place.names.defaultName.name,
                        placesId: jsPlace.place.a_id,

                        contact: jsPlace.place.contact,

                        geoDistance: jsPlace.distance*1000,  // result is in km, show in meters
                        geoLatitude: jsPlace.place.location.geoCoordinates.latitude,
                        geoLongitude: jsPlace.place.location.geoCoordinates.longitude,

                        addrCityName: jsPlace.place.location.address.city,
                        addrCountryCode: jsPlace.place.location.address.countryCode3L,
                        addrCountryName: jsPlace.place.location.address.countryCode3L, // todo: no country name ehre ?
                        addrDistrictName: jsPlace.place.location.addressdistrict,
                        addrHouseNumber: jsPlace.place.location.address.houseNumber,
                        addrPostalCode: jsPlace.place.location.address.zipCode,
                        addrStreetName: jsPlace.place.location.address.street,

                        phoneNumber: jsPlace.place.contact ? jsPlace.place.contact.phone:"",
                        url: jsPlace.place.contact ? jsPlace.place.contact.website : "",
                        email: jsPlace.place.contact ? jsPlace.place.contact.email : "",
                    };

                    obj.placeId = obj.properties.placesId;
                    // the API returns an empty string (!) if there
                    obj.recommendations = typeof jsPlace.place.averageRatings === 'object' ? {
                        ratingCount: jsPlace.place.averageRatings.averageRating.ratingCount,
                        ratingType: jsPlace.place.averageRatings.averageRating.ratingType,
                        rating: jsPlace.place.averageRatings.averageRating.rating
                    } : {};
                    obj.favorite = obj.getFavorite();
                    obj.isFavorite = !!obj.favorite;
                    obj.isRecommendation = true;

                    obj.iconPath = JsPlaceHelper.getIconSourceFromCategoryName((obj.categories ? obj.categories.name : undefined), obj.properties.typeNumeric, obj.isFavorite, "list");

                    //log.info("**** AFTER: "+JSON.stringify(obj));

                    results.push(obj);
                }

                results.sort(_distanceSort); // sort the results array by distance
                model.queryFinished(null, results, model);

             }, function(error) {
                model.queryFinished(error, null, null);
             }, options
         );
    }

}
