import QtQuick 1.0

import ovi.connector 1.0

// PlacesConnectedList
ConnectedList {
    id: model

    cloudAdapter: PlacesCloudAdapter {}

    signal queryFailed(variant error);
    signal modelUpdated();
    signal modelItemAdded();
    signal suggestionCompleted(variant data);
    signal itemUpdated(variant itemObject, bool isFavorite);

    // onQueryFinished will take care of adding
    queryMode: ConnectedList.None
    property string searchTerm

    // Return true if valid search, false if not queried
    function query(options) {
        // Only search if the box is not empty or has more than empty spaces
//        if ((options.searchTerm&&options.searchTerm.search(/[0-9]*[a-z]+[0-9]*/i) != -1) || options.searchCategory || options.type) {
        if ((options.searchTerm && options.searchTerm.search(/^\s*$/i) === -1) || options.searchCategory || options.type) {
            searchTerm = options.searchTerm || "";
            doQuery(options);
        } else {
            modelClear();
            modelUpdated();
            cancelQuery();
        }
        return false;
    }

    function modelClear() {
        // Unfortunately .clear of model does not call destroy() on object. This method will enforce
        // removing items by destroying them!
        for (var i = model.count - 1; i >= 0; i--) {
            var tmp = model.get(i);
            if (!tmp.isFavorite) {
                model.get(i).destroy();
                tmp.destroy();
            } else {
                log.info("will keep the place as it's favorite (PLACES-CONNECTED-LIST)...................     " + tmp)
            }
        }
        model.clear();
    }

    function cancelQuery() {
        log.info("Canceling query");
        doQuery({type:"CANCEL"});
    }

    // see connectedlist.h, gets "error", "results", "model"
    onQueryFinished: {
        if (error) {
            log.error("ERROR in QUERY:"+ JSON.stringify(error));
            queryFailed(error);
        }
        else {
            modelClear();

            for (var i = 0, l = results.length; i < l; i++) {
                try {
                    model.append(results[i]);
                    modelItemAdded();
                } catch(e) {
                    log.error(e)
                }
            }

            log.trace("Data model created.");
        }
        modelUpdated();
    }

}
