var History = function () {
    var dbName = "NokiaMapsDataBase";
    var dbVersion = "1.0";
    var dbComment = "The history of search";
    var dbLimit = 1000000;
    var dbHistoryLength = 10;

    function _getDbConnection() {
        var db = openDatabaseSync(dbName, dbVersion, dbComment, dbLimit);

        _checkIfHistoryTableExistsAndUpToDateAndUpdateIt(db);

        return db;
    }

    function _checkIfHistoryTableExistsAndUpToDateAndUpdateIt(db) {
        try {
            db.transaction(function (tx) {
                tx.executeSql("select entry, time_created, latest_search from history limit 1");
            });
        } catch (e) {
            try {
                db.transaction(function (tx) {
                    tx.executeSql("drop table if exists history");
                    tx.executeSql("create table history (entry text unique, time_created timestamp not null, latest_search timestamp not null, searches int not null default 0)");
                });
            } catch (e) {
                log.error("cannot create history sqllite table, aborting...");
            }
        }
    }

    return {
        addHistoryRecord: function (entry) {
            var result;

            if (entry) {
                entry = entry.replace(/\n/g, "").replace(/\r/g, "").replace(/^\s*|\s*$/g, "").replace(/\s{1,}/g, " ");
                if (entry) {
                    var db = _getDbConnection();

                    try {
                        db.transaction(function (tx) {
                            var ifExists = tx.executeSql("select count(entry) entry_count from history where entry=?", [entry]);
                            if (ifExists.rows.length > 0 && ifExists.rows.item(0).entry_count >= 1) {
                                tx.executeSql("update history set latest_search=datetime('now'), searches=searches+1 where entry=?", [entry]);
                            } else {
                                tx.executeSql("insert into history values (?, datetime('now'), datetime('now'), 1)", [entry]);
                                var totalLength = tx.executeSql("select count(*) count_all from history");
                                if (totalLength.rows.length > 0 && totalLength.rows.item(0).count_all > dbHistoryLength) {
                                    var toBeDeleted = tx.executeSql("select entry from history order by latest_search asc, searches asc limit " + (totalLength.rows.item(0).count_all - dbHistoryLength));
                                    for (var i = 0; i < toBeDeleted.rows.length; i++) {
                                        tx.executeSql("delete from history where entry=?", [toBeDeleted.rows.item(i).entry]);
                                    }
                                }
                            }
                        });

                        result = true;
                    } catch (e) {
                        result = false
                    }
                } else {
                    log.info("will not save it, it's just spaces...............'")
                }
            } else {
                entry = false;
            }

            return result;
        },

        getHistoryRecords: function (callbackSuccess) {
            var result;

            var db = _getDbConnection();

            try {
                db.transaction(function (tx) {
                    callbackSuccess(
                        tx.executeSql("select entry, latest_search, searches from history order by latest_search desc, searches desc limit " + dbHistoryLength)
                    );
                });
                result = true;
            } catch (e) {
                result = false;
            }

            return result;
        }
    };
}()

function showHistory() {
    var result = History.getHistoryRecords(
        function (results) {
            clearHistory();
            var entry;
            for (var i = 0; i < results.rows.length; i++) {
                entry = results.rows.item(i).entry;
                suggestionAndHistoryModel.append({ name: entry, title: entry.replace(/</g,"&lt;").replace(/>/g,"&gt;") });
            }
        }
    );
}

function clearHistory() {
    suggestionAndHistoryModel.clear();
    if (showGPS) {
        suggestionAndHistoryModel.append({
            name: "GPS",
            title: qsTranslate("Maps", "qtn_placesconnector_picker_current_position")
        });
    }
    if (showPickFromMap) {
        suggestionAndHistoryModel.append({
            name: "SHOW_FROM_MAP",
            title: qsTranslate("Maps", "qtn_placesconnector_picker_pick_from_map")
        });
    }
}

function addToHistory(entry) {
    var result = History.addHistoryRecord(entry);
}

function doSearch() {
    if (mainscreen.enableSearchDebugMessages && searchInput.text == "**position-debug") {
        mainscreen.resetAllStacks();
        mainscreen.enableCompletePositioningDebug = !mainscreen.enableCompletePositioningDebug;
    } else if (mainscreen.enableSearchDebugMessages && searchInput.text.indexOf("**-u ") === 0) {
        var geoRunnerURI = searchInput.text.substring(5, searchInput.text.length);
        mainscreen.resetAllStacks();
        appState.setGeoParamsUri(geoRunnerURI);
    } else if (mainscreen.enableSearchDebugMessages && searchInput.text == "**logging-enabled") {
        appState.startFileLogging();
    } else if (mainscreen.enableSearchDebugMessages && searchInput.text == "**logging-disabled") {
        appState.stopFileLogging();
    } else {
        var trimmedNewString = searchInput.text.replace(/^\s*|\s*$/g, "").replace(/\s{1,}/g, " ");
        if (trimmedNewString == "" && popOnCancel) {
            view.cancelMode = "enter";
            pageStack.pop(null,true);
            return;
        }

        if (trimmedNewString != "") {
            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIActivity("Search", { type: "free"} );
            }
            searchInput.searching = true;
            view.model.query({
                searchTerm: searchInput.text,
                location: mapView.center,
                mapView: mapView,
                parent: view,

                enableDidYouMean: true
            });

            addToHistory(searchInput.text);
            listView.forceActiveFocus();
            if (searchInput.platformCloseSoftwareInputPanel) {
                searchInput.platformCloseSoftwareInputPanel();
                if (rootApplicationLevel && rootApplicationLevel.forceGeometryResize) {
                    rootApplicationLevel.forceGeometryResize(true);
                }
            }
            suggestions.opacity = 0;

            mapFocus = true;
        } else {
            handleSearchInputClearedText();
            ensureMapVisible();
        }
    }
}

function clearModel() {
    mapView.model.cleanRemove();
    view.model.clear();
    searchInput.text = "";
}

function closeInfoBubble() {
    mapView.closeInfoBubble(false);
}

function showOnMap(position, extraData) {
    mapView.showInfoBubbleFromMap(position, extraData);
}

function suggestionCompleteCallback(data) {
    var trimmedNewString = searchInput.text.replace(/^\s*|\s*$/g, "").replace(/\s{1,}/g, " ");
    var _aNewString = trimmedNewString.replace(/[-_!@#\$%\^&\*\(\){}\[\]\?`~<>\.,]/g, "");
    searchInput.searching = false;
    clearHistory();

    if (data.length > 0) {
        for (var index = 0, length = data.length; index < length; index++) {
            var value = data[index].replace(/</g,"&lt;").replace(/>/g,"&gt;");
            var title = value.replace(
                new RegExp("(" + _aNewString + ")", "ig"),
                "<u style=\"color: blue;\">$1</u>"
            );

            suggestionAndHistoryModel.append({
                name: value,
                title: title
            });
        }
    }

    model.suggestionCompleted.disconnect(suggestionCompleteCallback);
}

function searchError(err) {
    searchInput.searching = false;
    searchDone(true, view.model, searchInput.text);

    mapFocus = true;
    searchInput.placeholderText = watermark;

    switch(err.status){
        case "400":
        case "404":
        case "408":
            errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_cloud_noconnection"); errorMessage.show();
        break;
        default:
            errorMessage.text = err.reason; errorMessage.show();
    }
}

function searchComplete() {
    searchInput.searching = false;
    log.info("Search complete with results: RESULTS: "+view.model.count);
    searchDone(false, view.model, searchInput.text);

    if (view.model.count <= 0) {  // if search results are empty
        mapFocus = true;
        searchInput.placeholderText = watermark;
        if (didYouMeanPopup.state == "hidden") {
            errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_picker_noresults"); errorMessage.show();
        }
    }
}

function didYouMeanCallback(result) {
    if (result && result.properties && result.properties.title && result.properties.preTitle) {
        errorMessage.hide();
        didYouMeanPopup.didYouMeanPreWord = result.properties.preTitle;
        didYouMeanPopup.didYouMeanWord = result.properties.title;
        searchInput.searching = false;
    }
}

function showPlaceDetails(placeId, position) {
    if (placeId) {
        var props = {
            placesId: placeId,
            showPlaceDetails: true
        }
        myId.placeId = placeId;

        myId.properties = props;
        myId.uri = placeId;
        myId.read({loadDetails:true})

        myId.readFinished.connect(handleShowPlaceDetailsCompleted);

        waiterForPlaceToLoadTimeout.start();
        waiterForPlaceToLoadTimeout.failedMode = false;
    } else if (position) {
        handleShowLocationDetailsCompleted(position);
    }
}

function ensureMapVisible() {
    mapView.forceActiveFocus();
    suggestions.opacity = 0;
    mapFocus = true;
}

function triggerSuggestion(name) {
    if (name == "GPS") {
        view.pickedObject = "";
        view.picked("", null);
    } else if (name == "SHOW_FROM_MAP") {
        view.pickedObject = "";
        view.pickCenter();
    } else {
        doQuery(name);
    }
}

function doQuery(name) {
    if (typeof (analyticsComponent) != "undefined") {
       analyticsComponent.logIMEIActivity("Search", { type: "free"} );
    }
    searchInput.text = name;
    view.model.query({
        searchTerm: name,
        location: mapView.center,
        mapView: mapView,
        parent: view,

        enableDidYouMean: true
    });
    mapView.model.cleanRemove();
    searchInput.searching = true;
    listView.forceActiveFocus();
    addToHistory(searchInput.text);
    suggestions.opacity = 0;
    mapFocus = true;
}

function clearInput() {
    log.trace("--- onCleared text");
    view.model.query({searchTerm:""});
    mapFocus = true;
    searchInput.text = "";
    searchInput.searching = false;
    view.model.searchTerm = "";
    view.__origSearchTerm = "";
    mapView.model.cleanRemove();
}

function focusInput() {
    log.trace(" [places picker]: focusing the search input");
    if (!searchInput.focus) {
        searchInput.forceActiveFocus();
    }
    suggestions.opacity = 0.7;
    //showHistory();
    errorMessage.open && errorMessage.hide();
    didYouMeanPopup.didYouMeanPreWord = "";
    didYouMeanPopup.didYouMeanWord = "";
    searchInput.textChanged();
}

function handleMapViewerLoader() {
    mapView = mapViewerCloner.createObject(view);
    mapView.z = mapViewerReplacer.z;
    __map = mapView;

    handlePostMapViewerLoaderLoaded();
}

function handleWaiterForPlaceToLoadTimeout() {
    waiterForPlaceToLoadTimeout.failedMode = true;
    if (mainscreen.onlineMode) {
        mainscreen.errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_wrong_place");
    } else {
        mainscreen.errorMessage.text = qsTranslate("Maps", "qtn_maps_nearby_offline_recs");
    }

    mainscreen.errorMessage.show();
}

function handleComponentOnCompleted() {
    lazyMapViewerLoader.start();
}

function handlePostMapViewerLoaderLoaded() {
    log.info("Places picker: Completed loading components and connected error handling");
    view.model.queryFailed.connect(searchError);  // connect model error handling

    if (focusedAtStart) {
        searchInput.text = prefilledString;
        focusInput();
        ensureFocusAtStartWillBeApplied.start();
    }
    if (triggerSearch) {
        if (!preventPrefilledStringAutoSearch) {
            searchInput.text = prefilledString;
            doSearch();
        }
    }
}

function handleDidYouMeanPopupClicked() {
    searchInput.text = didYouMeanPopup.didYouMeanWord
    if (typeof (analyticsComponent) != "undefined") {
        analyticsComponent.logUUIDOther("SearchDidYouMeanClick", { searchQueryId: "search ID not available" } );
    }
    view.model.query({
        searchTerm: searchInput.text,
        location: mapView.center,
        mapView: mapView,
        parent: view,

        enableDidYouMean: false
    });
    searchInput.searching = true;

    mapView.model.cleanRemove();
    addToHistory(searchInput.text);
    listView.forceActiveFocus();
    suggestions.opacity = 0;

    mapFocus = true;
}

function handleDidYouMeanPopupClosed() {
    if (typeof(view.model.count) != "undefined" && view.model.count == 0) {
        errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_picker_noresults"); errorMessage.show();
    }
}

function handleListViewDelegateKeyPressed(name, event) {
    if (event.key == Qt.Key_Enter || event.key == Qt.Key_Return) {
        triggerSuggestion(name);
        event.accepted = true;
    }
}

function handleListViewDelegateClicked(name, index) {
    if (typeof (analyticsComponent) != "undefined") {
        analyticsComponent.logUUIDOther("SearchSuggestionClick", { rank: index} );
    }
    triggerSuggestion(name);
}

function handleSuggestionsAndHistoryListWhiteBackgroundClicked() {
    searchInput.text = view.__origSearchTerm
    suggestions.opacity = 0;
    mapView.forceActiveFocus();
    if (popOnCancel) {
        view.cancelMode = "click";
        pageStack.pop(null, true);
    }
}

function handlePlacesCommonListDelegateItemSelected(index, listView) {
    listView.view.currentIndex = index;
    pickedObject = view.model.get(index);
    pickedUri = pickedObject.uri;
    picked(pickedUri, pickedObject);

    if (typeof (analyticsComponent) != "undefined") {
        analyticsComponent.logIMEIOther("SearchResultClick", { rank: listView.view.currentIndex, source: "list" } );
        analyticsComponent.logUUIDOther("SearchResultClick", { rank: listView.view.currentIndex, searchQueryId: "search ID not available", source: "list" } );
    }
}

function handleMapViewerPositionSelected(positionData, currentPositionSelected) {
    if (pickFromMapAllowed) {
        var place = positionSelectionPlace.createObject(view);
        place.uri = currentPositionSelected? -1 : "";
        place.properties = positionData;
        if (!currentPositionSelected && mapSharedData && mapSharedData.currentMapObjectSelected &&
            mapSharedData.currentMapObjectSelected.transitStopInfo &&
            place.properties.geoLatitude === mapSharedData.currentMapObjectSelected.transitStopInfo.position.latitude &&
            place.properties.geoLongitude === mapSharedData.currentMapObjectSelected.transitStopInfo.position.longitude
        ) {
            var placeInfo = PublicTransport.createPtPlaceFromStopInfo(mapSharedData.currentMapObjectSelected.transitStopInfo, mapSharedData.currentCenter);
            place.publictransport = placeInfo.publictransport;
            place.properties = placeInfo.properties;
            place.categories = placeInfo.categories;
            place.favorite = place.getFavorite();
            place.isFavorite = !!place.favorite;
        }

        pickedObject = place;
        picked(place.uri, pickedObject);
    }
}

function handleMapViewerPlaceSelected(uri) {
    var found = false;
    pickedUri = uri;

    for (var i = 0, l = view.model.count; i < l; i++) {
        if (view.model.get(i).uri == uri) {
            if (typeof (analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIOther("SearchResultClick", { rank: i, source: "map" } );
                analyticsComponent.logUUIDOther("SearchResultClick", { rank: i, searchQueryId: "search ID not available", source: "map"  } );
            }
            pickedObject = view.model.get(i);
            picked(pickedUri, pickedObject);

            found = true;
            break;
        }
    }

    if (!found) {
        showPlaceDetails(uri);
    }
}

function handleSearchInputLensClicked() {
    ensureMapVisible();
    lensClickedAndMapVisible();
}

function handleSearchInputTextChanged(text) {
    var trimmedNewString = text.replace(/^\s*|\s*$/g, "").replace(/\s{1,}/g, " ");
    var _aNewString = trimmedNewString.replace(/[^\w]/g,"").replace(/(.)(?!$)/g,"$1\\W{0,}");
    if (trimmedNewString.length > 2) {
        log.trace("Retreiving suggestions");
        searchInput.searching = true;

        model.suggestionCompleted.disconnect(suggestionCompleteCallback);
        model.suggestionCompleted.connect(suggestionCompleteCallback);
        model.query({searchTerm:trimmedNewString ,type:"SUGGESTIONS", location:mapView.center});

    } else {
        model.cloudAdapter.cancel();
        searchInput.searching = false;
        showHistory();
    }
}

function handleSearchInputKeyPressed(text, event) {
    var searchTerm = text, length, index, value,
    key = event.key, part_before, part_after;
    log.trace("PlacesPicker::onKeyPressed");
    if (key == Qt.Key_Enter || key == Qt.Key_Return) {
        if (searchInput.platformCloseSoftwareInputPanel) {
            searchInput.platformCloseSoftwareInputPanel();
            if (rootApplicationLevel && rootApplicationLevel.forceGeometryResize) {
                rootApplicationLevel.forceGeometryResize(true);
            }
        };

        doSearch();
    } else if (key == Qt.Key_Down) {
        suggestionsAndHistoryList.forceActiveFocus();
    }
}

function handleSearchInputSearchCanceled() {
    log.info("--- onSearchCanceled");
    model.cloudAdapter.cancel();
    searchInput.searching = false;
}

function handleSearchInputClearedText() {
    clearInput();
    focusInput();
    if (!searchInput.focus){
        searchInput.forceActiveFocus();
    }
}

function handlePlacesPickerStatusChanged(status) {
    // This line fixes problem with showing empty listView on Device (Qml Bug)
    if (model.count == 0) mapFocus = true;
    if (status === PageStatus.Deactivating) {
        model.suggestionCompleted.disconnect(suggestionCompleteCallback);
        if (searchInput.searching) {
            clearInput();
        }
        if (searchInput.platformCloseSoftwareInputPanel){
            searchInput.platformCloseSoftwareInputPanel();
            if (rootApplicationLevel && rootApplicationLevel.forceGeometryResize) {
                rootApplicationLevel.forceGeometryResize(true);
            }
        }
    }
}

function handleFavoritesConnectedListItemRemoved(object) {
    var obj = object.getPlace();
    if (obj && obj.properties) {
        mapView.lazyAppendIconMode = true;
        for (var i = 0, l = model.count; i < l; i++) {
            if (obj === model.get(i)) {
                var icon = mapView.model.getObjectByPosition({
                    latitude: obj.properties.geoLatitude,
                    longitude: obj.properties.geoLongitude
                });

                mapView.model.remove(icon);
                var icon = mapView.model.appendIcon(
                    obj.properties.geoLatitude,
                    obj.properties.geoLongitude,
                    JsPlaceHelper.getIconSourceFromId(JsPlaceHelper.getCategory(obj.categories).id, obj.properties.typeNumeric, false, "map"),
                    { uri: obj.uri, title: obj.favorite? obj.favorite.text : obj.properties.title, address: JsPlaceHelper.formatAddress(obj.properties), placeId: obj.properties && obj.properties.placesId? obj.properties.placesId : null },
                    {}
                );
                break;
            }
        }
        mapView.lazyAppendIconMode = false;
    }
}

function handlePlacesConnectedListItemUpdated(itemObject, isFavorite) {
    var obj = itemObject;
    if (obj && obj.properties) {
        mapView.lazyAppendIconMode = true;
        for (var i = 0, l = model.count; i < l; i++) {
            if (obj === model.get(i)) {
                var icon = mapView.model.getObjectByPosition({
                    latitude: obj.properties.geoLatitude,
                    longitude: obj.properties.geoLongitude
                });

                mapView.model.remove(icon);
                var icon = mapView.model.appendIcon(
                    obj.properties.geoLatitude,
                    obj.properties.geoLongitude,
                    JsPlaceHelper.getIconSourceFromId(JsPlaceHelper.getCategory(obj.categories).id, obj.properties.typeNumeric, isFavorite, "map"),
                    { uri: obj.uri, title: obj.favorite? obj.favorite.text : obj.properties.title, address: JsPlaceHelper.formatAddress(obj.properties), placeId: obj.properties && obj.properties.placesId? obj.properties.placesId : null },
                    {}
                );
                if (icon) {
                    mapView.showInfoBubble(icon);
                }
                break;
            }
        }
        mapView.lazyAppendIconMode = false;
    }
}

function handlePlacesConnectedListModelUpdated() {
    view.__origSearchTerm = searchInput.text
    mapView.model.cleanRemove();
    if (model.count==0) mapFocus = true;

    var iconToFocus;
    var smallestDistanceSoFar = -1;
    var currentDistanceFromCenter = 0;
    var boundingBox = null;

    mapView.lazyAppendIconMode = true;
    for (var i=0, il = model.count; i < il; i++) {
        var obj = model.get(i);

        if (obj.properties.geoBbxLatitude1 && obj.properties.geoBbxLatitude2 && obj.properties.geoBbxLongitude1 && obj.properties.geoBbxLongitude2) {
            boundingBox = {
                geoBbxLatitude1: Math.max(obj.properties.geoBbxLatitude1, obj.properties.geoBbxLatitude2),
                geoBbxLatitude2: Math.min(obj.properties.geoBbxLatitude1, obj.properties.geoBbxLatitude2),
                geoBbxLongitude1: Math.min(obj.properties.geoBbxLongitude1, obj.properties.geoBbxLongitude2),
                geoBbxLongitude2: Math.max(obj.properties.geoBbxLongitude1, obj.properties.geoBbxLongitude2)
            };
        } else {
            boundingBox = null;
        }

        var icon = mapView.model.appendIcon(
            obj.properties.geoLatitude,
            obj.properties.geoLongitude,
            JsPlaceHelper.getIconSourceFromId(JsPlaceHelper.getCategory(obj.categories).id, obj.properties.typeNumeric, obj.isFavorite, "map"),
            {
                uri: obj.uri,
                title: obj.favorite? obj.favorite.text : obj.properties.title,
                address: JsPlaceHelper.formatAddress(obj.properties),
                placeId: obj.properties && obj.properties.placesId? obj.properties.placesId : null,
                boundingBox: boundingBox
            },
            {}
        );

        /*
        spareGeoCoordinates.latitude = obj.properties.geoLatitude;
        spareGeoCoordinates.longitude = obj.properties.geoLongitude;

        currentDistanceFromCenter = mapView.center.distance(spareGeoCoordinates);
        if (smallestDistanceSoFar == -1 || currentDistanceFromCenter < smallestDistanceSoFar) {
            smallestDistanceSoFar = currentDistanceFromCenter;
            iconToFocus = icon;
        }
        */

        if (i == 0) {
            iconToFocus = icon;
        }
    }
    mapView.lazyAppendIconMode = false;

    if (iconToFocus) {
        mapView.showInfoBubble(iconToFocus);
    }

    mapView.model.setObjectsOwner("places", false);
    searchInput.searching = false;
}

function handlePrefilledStringChanged() {
    if (mapView) {
        if (prefilledString) {
            searchInput.text = prefilledString;
            if (!preventPrefilledStringAutoSearch) {
                doSearch();
            }
        }
    } else {
        //
    }
}

function handleShowPlaceDetailsCompleted() {
    if (!waiterForPlaceToLoadTimeout.failedMode) {
        waiterForPlaceToLoadTimeout.stop();

        var dummy = dummyFactory.createObject(mainscreen);

        dummy.uri = myId.uri;
        dummy.properties = {
            placesId: myId.placeId,
            title: myId.details.names.defaultName.name,

            isPt: false,

            geoLatitude: myId.details.location.geoCoordinates.latitude,
            geoLongitude: myId.details.location.geoCoordinates.longitude,

            addrCityName: myId.details.location.address.city,
            addrCountryCode: myId.details.location.address.countryCode3L,
            addrCountryName: myId.details.location.address.localizedCountryName,
            addrHouseNumber: myId.details.location.address.houseNumber,
            addrPostalCode: myId.details.location.address.zipCode,
            addrStreetName: myId.details.location.address.street
        };
        dummy.details = myId.details;
        dummy.placeId = myId.placeId;
        dummy.categories = myId.categories;
        dummy.iconPath = myId.iconPath;
        dummy.restoreIconPathFromCategories();
        view.pickedObject = dummy;
        view.pageStack.push(view.viewComponent, null, true);
        myId.readFinished.disconnect(handleShowPlaceDetailsCompleted);
    }
}

function handleShowLocationDetailsCompleted(position) {
    var dummy = positionSelectionPlace.createObject(view);

    dummy.properties = position;
    view.pickedObject = dummy;
    view.pageStack.push(view.viewComponent, null, true);
}
