import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0

import "PlaceHelper.js" as JsPlaceHelper
import "PlacesCloudAdapter.js" as Adapter
import "PublicTransport.js" as PublicTransport
import "PlacesPicker.js" as PlacesPickerJS
import "ui"

ConnectorPage {
    id: view

    anchors.fill: parent

    property bool pickFromMapAllowed: true
    property bool disableLongPress: false
    property bool disablePublicTransport: false
    property bool popOnCancel: false
    property bool focusedAtStart: false
    property bool preventPrefilledStringAutoSearch: false
    property bool triggerSearch:false
    property bool showGPS: false
    property bool showPickFromMap: false

    property string pickedUri: ""
    property string watermark: qsTranslate("Maps", "qtn_placesconnector_picker_watermark")
    property string prefilledString
    property string cancelMode: ""
    property string __origSearchTerm: ""

    property variant pickedObject: ""
    property url pathToConnector: "../"

    property alias mapFocus: mapToggle.mapActive
    property alias input: searchInput

    signal searchDone(bool error, variant results, string searchTerm)
    signal picked(string pickedUri, variant pickedObject)
    signal pickCenter()
    signal lensClickedAndMapVisible()

    function clearModel() {
        PlacesPickerJS.clearModel();
    }
    function ensureMapVisible() {
        PlacesPickerJS.ensureMapVisible();
    }
    function showOnMap(position, extraData) {
        PlacesPickerJS.showOnMap(position, extraData);
    }
    function searchComplete() {
        PlacesPickerJS.searchComplete();
    }
    function didYouMeanCallback(result) {
        PlacesPickerJS.didYouMeanCallback(result);
    }
    function showPlaceDetails(placeId, position) {
        PlacesPickerJS.showPlaceDetails(placeId, position);
    }

    model: PlacesConnectedList {
        onModelUpdated: {
            PlacesPickerJS.handlePlacesConnectedListModelUpdated();
        }

        onItemUpdated: {
            PlacesPickerJS.handlePlacesConnectedListItemUpdated(itemObject, isFavorite);
        }
    }

    property MapViewer __map
    property MapViewer mapView
    Rectangle {
        id: mapViewerReplacer

        width: parent.width
        height: parent.height - searchInputBg.height
        anchors.top: searchInputBg.bottom
        //color: "#FBFAF7"
        //color: "#ff00ff"

        Image {
            source: assetsPath + "images/emptyMapPattern150x150.gif"
            anchors.fill: parent

            fillMode: Image.Tile
        }

        z: 1
    }

    Header {
        id: searchInputBg
        dropShadow: mapToggle.mapActive

        MouseArea {
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: searchInput.right
            anchors.rightMargin: 80
            anchors.bottom: parent.bottom

            enabled: searchInput.enabled

            onClicked: {
                PlacesPickerJS.focusInput();
            }
        }

        SearchInput {
            id: searchInput
            objectName: "placesSearchInput"

            width : parent.width - 20 - (mapToggle.visible ? mapToggle.width + 10 : 0)

            anchors.left: parent.left
            anchors.right: mapToggle.left
            anchors.rightMargin: mapToggle.visible? 16 : 0
            anchors.leftMargin:16

            placeholderText: watermark

            enabled: mainscreen.allTabsReady

            onClicked: {
                PlacesPickerJS.focusInput();
            }

            onClearedText: {
                PlacesPickerJS.handleSearchInputClearedText();
            }

            onSearchCanceled: {
                PlacesPickerJS.handleSearchInputSearchCanceled();
            }

            onKeyPressed: {
                PlacesPickerJS.handleSearchInputKeyPressed(text, event);
            }

            onTextChanged: {
                PlacesPickerJS.handleSearchInputTextChanged(text);
            }

            onLensClicked: {
                PlacesPickerJS.handleSearchInputLensClicked();
            }
        }

        MouseArea {
            width: 70
            height: 44
            anchors.right: searchInput.right
            anchors.rightMargin: 4
            anchors.top: searchInput.top
            anchors.topMargin: 4

            enabled: searchInput.enabled

            visible: !searchInput.activeFocus

            onClicked: { searchInput.directlyTryItemClick(); }

        }

        MapListToggle {
            id: mapToggle
            anchors { right: parent.right; rightMargin: 10; verticalCenter: parent.verticalCenter; }
            visible: suggestions.opacity == 0 && listView.count != 0 && mapView.model.count != 0
        }

        z: 2
    }

    Rectangle {
        id: listBg
        objectName: "placesListView"
        color: "#fff"
        width : parent.width
        height : parent.height - searchInputBg.height
        anchors.top : searchInputBg.bottom

        states : [
            State {
                when: mapFocus
                PropertyChanges {
                    target : listBg
                    opacity : 0
                }
            },
            State {
                when: !mapFocus
                PropertyChanges {
                    target : listBg
                    opacity : 1
                }
            }
        ]
        Behavior on opacity { NumberAnimation{} }
        PlacesCommonList {
            id: listView
            anchors.fill:parent
            clip : true
            model : view.model
            itemDelegate : PlacesCommonItemDelegate {
                PlaceListItem {
                    onItemSelected: {
                        PlacesPickerJS.handlePlacesCommonListDelegateItemSelected(index, listView);
                    }
                }
            }
        }

        z: 3
    }

    Rectangle {
        id: suggestions
        width: parent.width
        color: "#000"
        opacity: 0
        anchors.top: searchInputBg.bottom
        anchors.bottom: parent.bottom

        z: 4
    }

    Rectangle {
        id: suggestionsAndHistoryListWhiteBackground
        objectName: "suggestionsAndHistoryListWhiteBackground"

        width: suggestionsAndHistoryList.width
        color: "white"
        anchors.top: suggestionsAndHistoryList.top
        anchors.left: suggestionsAndHistoryList.left
        anchors.bottom: parent.bottom
        visible: suggestionsAndHistoryList.visible
        opacity: suggestionsAndHistoryList.opacity
         MouseArea {
            anchors.fill: parent
            onClicked: {
                PlacesPickerJS.handleSuggestionsAndHistoryListWhiteBackgroundClicked();
            }
        }

         z: 5
    }

    ListView {
        id: suggestionsAndHistoryList
        objectName: "suggestionsAndHistoryList"

        boundsBehavior: ListView.StopAtBounds

        property int delegateItemHeight: 64

        anchors.top : searchInputBg.bottom
        //anchors.bottom: parent.bottom
        //anchors.topMargin: 10
        anchors.horizontalCenter: parent.horizontalCenter
        cacheBuffer: delegateItemHeight * 25 * 10
        height : {
            if (count * delegateItemHeight > suggestionsAndHistoryListWhiteBackground.height) {
                return suggestionsAndHistoryListWhiteBackground.height;
            } else {
                var result = count * delegateItemHeight;
                return result? result : delegateItemHeight;
            }
        }
        width : parent.width

        clip : true
        interactive : true
        opacity: (suggestions.opacity != 0)? 1 : 0
        model: ListModel {
            id: suggestionAndHistoryModel
        }

        delegate: Rectangle {
            id: delegateItem

            width: parent.width
            height: suggestionsAndHistoryList.delegateItemHeight
            color: "#fff"
            border.width: 1
            border.color: "#ccc"

            Image {
                id: _GPSModePositionDot
                source: {
                    if (mapView.currentPosition) {
                        return mapAssetsPath + "images/layout/pos_" + "accurate" + ".png";
                    } else if (mapView.previousPosition) {
                        return mapAssetsPath + "images/layout/pos_" + "lost" + ".png";
                    } else {
                        return mapAssetsPath + "images/layout/pos_" + "not_present" + ".png";
                    }
                }
                anchors.top: parent.top
                anchors.topMargin: 9
                anchors.left: parent.left
                anchors.leftMargin: 8

                visible: name == "GPS"
            }

            Image {
                id: _CurrentPositionModePositionDot
                source: mapAssetsPath + "images/layout/map_pin_location_icon.png";
                anchors.top: parent.top
                anchors.topMargin: 19
                anchors.left: parent.left
                anchors.leftMargin: 14

                visible: name == "SHOW_FROM_MAP"
            }

            Text {
                y: 15
                x: 15
                font.pixelSize: 24
                font.weight: Font.Bold
                text: title
                width:parent.width-30
                anchors.left: {
                    if (name == "GPS") {
                        return _GPSModePositionDot.right;
                    } else if (name == "SHOW_FROM_MAP") {
                        return _CurrentPositionModePositionDot.right;
                    } else {
                        return parent.left;
                    }
                }
                anchors.leftMargin: {
                    if (name == "GPS") {
                        return 10;
                    } else if (name == "SHOW_FROM_MAP") {
                        return 17;
                    } else {
                        return 10;
                    }
                }

                elide: Text.ElideRight
            }
            MouseArea {
                anchors.fill: delegateItem
                onClicked: {
                    PlacesPickerJS.handleListViewDelegateClicked(name, index);
                }
                onPressed: delegateItem.color = "#ccc"
                onReleased: delegateItem.color = "#fff"
                onCanceled: delegateItem.color = "#fff"

            }

            Keys.onPressed: {
                PlacesPickerJS.handleListViewDelegateKeyPressed(name, event);
            }
        }

        z: 6
    }

    InfoPopup {
        id: infoPopup

        anchors.top: searchInputBg.bottom

        z: 7
    }

    InfoBanner {
        id: errorMessage

        timerEnabled: true
        timerShowTime: 3 * 1000

        anchors.top: searchInputBg.bottom
        anchors.topMargin: 10

        z: 8
    }

    DidYouMeanPopup {
        id: didYouMeanPopup
        mapView: view.mapView? view.mapView : mapViewerReplacer

        opacity: !listBg.opacity
        anchors.top: searchInputBg.bottom

        onClicked: {
            PlacesPickerJS.handleDidYouMeanPopupClicked();
        }

        onClosed: {
            PlacesPickerJS.handleDidYouMeanPopupClosed();
        }

        z: 9
    }

    TransitDatabase {
        id: transitDb
    }

    GeoCoordinates {
        id: spareGeoCoordinates
    }

    Component {
        id: dummyFactory

        Place { }
    }

    Place {
        id: myId
    }

    property Component viewComponent: __viewComponent
    Component {
        id: __viewComponent

        PlacesViewer {
            id: viewer
            mapsApp: mainscreen
            anchors.fill: parent
            object: picker.pickedObject
            model: picker.model
            onFavoriteChanged: {
                // it's reversed as it's emited before the change of the state itself
                picker.model.itemUpdated(viewer.object, !isFavorite);
                mainscreen.mapSharedData.setControl("close-infobubble");
            }
        }
    }

    Component {
        id: positionSelectionPlace
        Place {}
    }

    Component {
        id: mapViewerCloner

        MapViewer {
            width: parent.width
            height: parent.height - searchInputBg.height
            anchors.top: searchInputBg.bottom
            visible: mapFocus

            behaviors: {
                var tmpBehaviors = "";

                if (disableLongPress) {
                    tmpBehaviors += "_NO_LONG_PRESS_AT_ALL_:true;";
                }
                if (disablePublicTransport) {
                    tmpBehaviors += "_NO_PUBLIC_TRANSPORT_STOPS_CLICK_:true";
                }

                return tmpBehaviors? tmpBehaviors : behaviors;
            }

            mainScreen: mainscreen
            useGlobalShare: true

            onPlaceSelected: {
                PlacesPickerJS.handleMapViewerPlaceSelected(uri);
            }

            onPositionSelected: {
                PlacesPickerJS.handleMapViewerPositionSelected(positionData, currentPositionSelected);
            }
        }
    }

    onStatusChanged: {
        PlacesPickerJS.handlePlacesPickerStatusChanged(status);
    }

    onPrefilledStringChanged: {
        PlacesPickerJS.handlePrefilledStringChanged();
    }

    Component.onCompleted: {
        PlacesPickerJS.handleComponentOnCompleted();
    }

    Connections {
        target: mainscreen.favoritesConnectedList
        ignoreUnknownSignals: true

        onItemRemoved: {
            PlacesPickerJS.handleFavoritesConnectedListItemRemoved(object);
        }
    }

    Connections {
        target: mainscreen
        ignoreUnknownSignals: true

        onRefreshConnectedLists: {
            PlacesPickerJS.handlePlacesConnectedListModelUpdated();
        }
    }

    Timer {
        id: ensureFocusAtStartWillBeApplied
        interval: 25

        onTriggered: {
            searchInput.forceActiveFocus();
        }
    }

    Timer {
        id: lazyMapViewerLoader
        interval: 25

        onTriggered: {
            PlacesPickerJS.handleMapViewerLoader();
        }
    }

    Timer {
        id: waiterForPlaceToLoadTimeout
        property bool failedMode: false
        interval: 10 * 1000
        onTriggered: {
            PlacesPickerJS.handleWaiterForPlaceToLoadTimeout();
        }
    }
}
