import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.favorites 1.0

import "PlaceHelper.js" as PlaceHelper
import "PlaceViewerLogic.js" as Logic
import "ui"

Rectangle {
    id: containerArea

    property variant model
    property variant object

    property alias mapPreview: mapPreviewMini
    property alias mapImagePrefix: mapPreviewMini.imagePrefix
    property alias mapScreenShotSource: mapPreviewMini.mapScreenShotSource
    property alias guidesModel: guidesModel
    property alias galleryModel: galleryModel
    property alias reviewsModel: reviewsModel
    property alias recommendationsModel: recommendationsModel

    property bool dontPopulateGlobalActiveMap: false

    property bool hasLeft: false
    property bool hasRight: false
    property bool bussyMode: false
    property bool lazyBussyMode: false
    onBussyModeChanged: {
        lazyBussyMode = false;
        if (bussyMode) {
            lazyBussyModeApply.start();
        } else {
            lazyBussyModeApply.stop();
        }
    }

    signal movedLeft()
    signal movedRight()

    function resetToTop() {
        placeDetailsFlickable.contentY = 0;
    }

    color: "#f0f1f2"
    x: parent.x
    y: parent.y
    width: parent.width
    height: parent.height

    Flickable {
        id: placeDetailsFlickable
        objectName: "placeDetailsFlickable"

        clip: true
        anchors.fill: parent
        contentHeight: content.height - swiper.height
        boundsBehavior: Flickable.StopAtBounds

        Column {
            id: content
            width: parent.width

            PlaceTitleArea {
                id: titleArea
                objectName: "titleArea"

                width: parent.width

                placeTitle: containerArea.object? Logic.getPlaceTitle(containerArea.object).substring(0,90) + (containerArea.object && Logic.getPlaceAddress(containerArea.object).length > 90 ? "..." : "") : ""
                placeAddress: containerArea.object && Logic.ready? Logic.getPlaceAddress(containerArea.object).substring(0,90) + (containerArea.object && Logic.getPlaceAddress(containerArea.object).length > 90 ? "..." : "") : ""

                placeRating: containerArea.object && Logic.getPlaceRating(containerArea.object)? Logic.getPlaceRating(containerArea.object) : ""
                placeCategoryName: containerArea.object && Logic.getPlaceCategoryName(containerArea.object)? Logic.getPlaceCategoryName(containerArea.object) : ""
                placeDistance: containerArea.object? PlaceHelper.getReadablePlaceDistance(units, containerArea.object) : ""

                provider: containerArea.object && Logic.getPlaceProvider(containerArea.object)? Logic.getPlaceProvider(containerArea.object) : ""
            }

            Rectangle {
                color: "#ccc"
                width: parent.width
                height: 1
            }

            MapPreviewMini {
                id: mapPreviewMini

                object: containerArea.object
                titleArea: titleArea

                height: 130
                width: parent.width

                z: 1

                Connections {
                    target: view
                    ignoreUnknownSignals: true

                    onFavoriteChanged: {
                        waitSomeBeforeUpdateTheMinimap.start();
                    }
                }

                Timer {
                    id: waitSomeBeforeUpdateTheMinimap
                    interval: 25
                    onTriggered: { mapPreviewMini.update(); }
                }
            }

            Item {
                id: mapAutomaticReverseGeoCoderOnAttachedToLocation

                GeoCoordinates { id: geo }

                ReverseGeoCodeFinder {
                    id: finder

                    mapView: mainscreen.spareMap
                    onFindPureDataComplete: {
                        var data = contextData;
                        var title = data["ADDR_STREET_NAME"] || data["ADDR_CITY_NAME"] || data["ADDR_COUTRY_NAME"];
                        var address = finder.formatAddress(data);
                        if (title) {
                            titleArea.placeTitle = title;
                        }
                        if (address) {
                            titleArea.placeAddress = address;
                        }
                    }
                }

                Timer {
                    interval: 1000 * 30
                    repeat: true
                    running: (mapPreviewMini.currentPositionMode && mainscreen.spareMap.currentPosition)? true : false

                    onTriggered: {
                        finder.find(mainscreen.spareMap.currentPosition.geoCoordinates, false, true);
                    }
                }
            }

            Rectangle {
                color: "#ccc"
                width: parent.width
                height:1

                z: 1
            }

            Rectangle {
                width: parent.width
                height: placesContactContainer.visible ? 16 : 0
                color: "transparent"

                z: 1
            }

            PlacesContact {
                id: placesContactContainer

                placeObject: containerArea.object
                onClicked: {
                    contextMenu.open();
                }

                z: 1
            }

            Rectangle {
                width: parent.width
                height: placesContactContainer.visible ? 16 : 0
                color: "transparent"

                z: 1
            }

            FavoriteDescriptionItem {
                id: favoritesDescriptionContainer

                place: containerArea.object
                visible: !!containerArea.object && containerArea.object.isFavorite && containerArea.object.favorite.description !== ""
                favoriteText: containerArea.object? Logic.getPlaceFavoriteDescription(containerArea.object) : ""

                z: 1
            }

            ThumbnailGallery {
                id: thumbnailGallery

                imagesModel: ListModel { id: galleryModel }
                mainPage: view
                onVisibleChanged: {
                    if (visible == true) detailsShown = true;
                }

                z: 1
            }

            PublicTransportDetails {
                id: ptDetails
                visible: !!containerArea.object && containerArea.object.categories && containerArea.object.categories.name === "public-transport"
                object: containerArea.object
                anchors {left: parent.left; right: parent.right }

                z: 1
            }

            SubHeadline {
                objectName: "placeDetailsSubInformation"
                text: containerArea.object? Logic.getPlaceAdditionalInformation(containerArea.object) : ""
                caption: qsTranslate("Maps", "qtn_placesconnector_viewer_from_owner")

                z: 1
            }

            SubHeadline {
                objectName: "placeDetailsSubOpeningHours"
                text: containerArea.object? Logic.getPlaceOpeningHours(containerArea.object) : ""
                caption: qsTranslate("Maps", "qtn_placesconnector_viewer_openinghours"); // "Opening hours"

                z: 1
            }

            SubHeadline {
                objectName: "placeDetailsSubAnnualClosing"
                text: containerArea.object? Logic.getPlaceAnnualClosing(containerArea.object) : ""
                caption: qsTranslate("Maps", "qtn_placesconnector_viewer_annualclosing"); //"Annual closings notes"

                z: 1
            }

            SubHeadline {
                objectName: "placeDetailsSubPayment"
                text: containerArea.object? Logic.getPlacePaymentMethods(containerArea.object) : ""
                caption: qsTranslate("Maps", "qtn_placesconnector_viewer_payment"); // "Payment methods"

                z: 1
            }

            ReviewGuideList {
                width: parent.width
                objectName: "guidesList";
                headerCaption: qsTranslate("Maps", "qtn_placesconnector_viewer_guides"); // "Guides"
                mainPage: view
                model:  ListModel { id: guidesModel }
                _placeTitleText: placeTitleText
                _placeAddressText: placeAddressText

                z: 1
            }

            ReviewGuideList {
                width: parent.width
                objectName: "reviewsList";
                headerCaption: qsTranslate("Maps", "qtn_placesconnector_viewer_reviews")
                mainPage: view
                model:  ListModel { id: reviewsModel }
                _placeTitleText: placeTitleText
                _placeAddressText: placeAddressText

                z: 1
            }

            ReviewGuideList {
                id: recommendationsList

                width: parent.width
                objectName: "recommendationsList"
                headerCaption: qsTranslate("Maps", "qtn_placesconnector_viewer_recommendations")
                mainPage: view
                model: PlacesRecommendationsList { id: recommendationsModel }
                delegate: PlaceListItem {
                    width: recommendationsList.width
                    objectName: "recommendationsList_" + index
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            var component = Qt.createComponent("PlacesViewer.qml");

                            component = component.createObject(view);
                            component.model = model;
                            component.pageStack = pageStack;
                            pageStack.push(Logic.createSubPage(component, index, recommendationsModel.get(index), view, newPlace));
                        }
                        onPressed: { parent.color = "#d0d1d2" }
                        onReleased: { parent.color = "#f0f1f2" }
                        onCanceled: { parent.color = "#f0f1f2" }
                    }
                }

                z: 1
            }

            Rectangle {
                id: finalSpacerInTheList
                width: parent.width
                height: 40
                color: "transparent"

                z: 1
            }

            BusyIndicator {
                id: bussyIndicator
                objectName: "bussyIndicator"

                platformStyle: BusyIndicatorStyle { inverted: false; size: "large"; }

                running: visible
                visible: containerArea.lazyBussyMode
                Behavior on visible { PropertyAnimation { target: bussyIndicator; property: "opacity"; from: 0; to: 1; duration: 150; } }

                anchors.horizontalCenter: parent.horizontalCenter

                z: 1
            }

            PlacesViewerSequenceSwiper {
                id: swiper

                hasLeft: containerArea.hasLeft
                hasRight: containerArea.hasRight

                onMovedLeft: {
                    containerArea.movedLeft();
                }
                onMovedRight: {
                    containerArea.movedRight();
                }
            }
        }

        PlacesContactPopup {
            id: contextMenu
            objectName: "placesContactPopup"
            placeObject: containerArea.object
            visualParent: mainscreen
        }
    }

    ScrollDecorator {
        flickableItem: placeDetailsFlickable
    }

    Units { id: units }

    Component {
        id: newPlace
        Place { }
    }

    Timer {
        id: lazyBussyModeApply
        interval: 500
        onTriggered: {
            lazyBussyMode = true;
        }
    }

    Component.onCompleted: {
        Logic.setLocalVariables(
            mainscreen,
            object,
            guidesModel,
            galleryModel,
            reviewsModel,
            recommendationsModel,
            viewerInfoPopup,
            qsTranslate,
            view,
            log,
            PageStatus
        );
    }
}
