import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0
import maps.native 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.favorites 1.0

import "PlaceHelper.js" as PlaceHelper
import "PlaceViewerLogic.js" as Logic
import "ui"

Page {
    id: view

    property bool placesViewerComponent: true

    property variant model
    property variant object
    property bool openFromFavorite: false

    property string placeTitleText: object? Logic.getPlaceTitle(object) : ""
    property string placeAddressText: object && Logic.ready? Logic.getPlaceAddress(object) : ""

    signal navigateTo(variant destination)
    signal favoriteChanged(bool isFavorite)
    property int recommendationsLeft: 2

    property bool detailsShown: false
    property bool cleanAllObjectOnLeave: false

    property variant mapsApp

    property variant placesViewerBodyLeft
    property variant placesViewerBodyRight
    property variant placesViewerBodyLeftContainer
    property variant placesViewerBodyRightContainer
    property bool swipingEnabled: false
    function enableSwiping() {
        if (!placesViewerBodyLeftContainer) {
            placesViewerBodyLeftContainer = placesViewerBodyLeftContainerFactory.createObject(view);
            placesViewerBodyLeft = placesViewerBodyLeftContainer.placesViewerBodyLeftItem;
        }
        if (!placesViewerBodyRightContainer) {
            placesViewerBodyRightContainer = placesViewerBodyRightContainerFactory.createObject(view);
            placesViewerBodyRight = placesViewerBodyRightContainer.placesViewerBodyRightItem;
        }

        swipingEnabled = true;
    }

    property variant currentPlace
    Component {
        id: factory

        Place { }
    }

    onNavigateTo: {
        doNavigateTo(destination)
    }
    function _cloneDestination(destination) {
        if (currentPlace) {
            currentPlace.destroy();
        }

        currentPlace = factory.createObject(mainscreen);
        currentPlace.uri = destination.uri;
        currentPlace.name = destination.name;
        currentPlace.properties = JSON.parse(JSON.stringify(destination.properties));
        currentPlace.details = JSON.parse(JSON.stringify(destination.details));
    }

    function doNavigateTo(destination) {
        _cloneDestination(destination);

        while(tabGroup.currentTab.depth > 1) {
            tabGroup.currentTab.pop(null, true);
        }
        while(directionsStack.depth > 1) {
            directionsStack.pop(null, true);
        }

        directionsStack.currentPage.routeTo(currentPlace);
        directionsTabBtn.clicked();
    }

    tools: PlacesDefaultTools {
        pageStack: view.pageStack
        placeObject: view.object
        openFromFavorite: view.openFromFavorite

        onFavoriteChanged: {
            view.favoriteChanged(isFavorite);
            Logic.triggerReloadRecommendations = true;
        }
        onFavoriteOpened: {
            favoritesEditSheet.fillContent();
            favoritesEditSheet.open();
        }
        onAddAsPlace: {
            placesAddAsPlace.doOpen();
        }
        onAddToContacts: {
            appState.saveContactRaw(
                view.object? Logic.getPlaceTitle(view.object).substring(0,90) + (view.object && Logic.getPlaceAddress(view.object).length > 90 ? "..." : "") : "",
                "",
                "",
                view.object.properties.addrStreetName? view.object.properties.addrStreetName : "",
                (view.object.properties.addrCityName? view.object.properties.addrCityName : "") + " " + (view.object.properties.addrHouseNumber? view.object.properties.addrHouseNumber : ""),
                view.object.properties.addrDistrictName? view.object.properties.addrDistrictName : "",
                view.object.properties.addrPostalCode? view.object.properties.addrPostalCode : "",
                view.object.properties.addrCountrytName? view.object.properties.addrCountryName : "",
                "",
                Logic.getPlaceEmail(view.object),
                Logic.getPlacePhone(view.object)
            );
        }
        onBackClicked: {
            view.cleanAllObjectOnLeave = true;
        }
    }

    function setLibrary(obj) {
        Logic.setLocalVariables(
            mainscreen,
            object,
            obj.guidesModel,
            obj.galleryModel,
            obj.reviewsModel,
            obj.recommendationsModel,
            viewerInfoPopup,
            qsTranslate,
            view,
            log,
            PageStatus
        );
    }

    /** To be able to use .pragma library... */
    onStatusChanged: {
        if (view.status === PageStatus.Activating) {
            setLibrary(placesViewerBodyCenter);
            placesViewerBodyCenter.resetToTop();
        } else if (view.status === PageStatus.Deactivating) {
            if (swipingEnabled) {
                placesViewerBodyLeftContainer.anchors.rightMargin = view.width;
                placesViewerBodyRightContainer.anchors.leftMargin = view.width;
            }
        } else if (view.status == PageStatus.Inactive) {
        } else if (view.status === PageStatus.Active) {
            if (swipingEnabled) {
                placesViewerBodyLeftContainer.anchors.rightMargin = 0;
                placesViewerBodyRightContainer.anchors.leftMargin = 0;
            }

            if (Logic.triggerReloadRecommendations) {
                Logic.reloadRecommendations();

                Logic.triggerReloadRecommendations = false;
            }
        }

        Logic.pageStatusChanged();
    }

    onObjectChanged: {
        placesViewerBodyCenter.model = model;
        if (swipingEnabled) {
            placesViewerBodyLeft.model = model;
            placesViewerBodyRight.model = model;
        }

        _populateAllItems();

        setLibrary(placesViewerBodyCenter);
        Logic.preClearEverything();

        lazyUpdateCurrentBody.start();
    }

    Timer {
        id: lazyUpdateCurrentBody
        interval: 1000 * 1.5
        onRunningChanged: {
            if (lazyUpdateCurrentBody.running) {
                placesViewerBodyCenter.bussyMode = true;
            }
        }

        onTriggered: {
            Logic.loadPlaceData(analyticsComponent);
            placesViewerBodyCenter.bussyMode = false;
        }
    }

    function _populateAllItems() {
        placesViewerBodyCenter.object = object;

        if (swipingEnabled) {
            var currentItem = -1;
            var tmp;
            if (view.model) {
                for (var i = 0; i < view.model.count; i++) {
                    tmp = view.model.get(i);
                    if (tmp && (tmp == view.object || (tmp.object && tmp.object == view.object))) {
                        currentItem = i;
                        break;
                    }
                }
            }

            if (currentItem >= 0) {
                var prevIndex;
                var nextIndex;
                if (currentItem > 0) {
                    prevIndex = currentItem - 1;
                } else if (currentItem == 0 && view.model.count > 1) {
                    prevIndex = view.model.count - 1;
                } else {
                    prevIndex = -1;
                }

                if (currentItem < view.model.count - 1) {
                    nextIndex = currentItem + 1;
                } else if (currentItem == view.model.count - 1 && currentItem != 0) {
                    nextIndex = 0;
                } else {
                    nextIndex = -1;
                }

                if (prevIndex >= 0) {
                    placesViewerBodyLeft.visible = true;
                    tmp = view.model.get(prevIndex);
                    placesViewerBodyLeft.object = tmp.object? tmp.object : tmp;

                    placesViewerBodyCenter.hasLeft = true;
                } else {
                    placesViewerBodyLeft.visible = false;
                }

                if (nextIndex >= 0) {
                    placesViewerBodyRight.visible = true;
                    tmp = view.model.get(nextIndex);
                    placesViewerBodyRight.object = tmp.object? tmp.object : tmp;

                    placesViewerBodyCenter.hasRight = true;
                } else {
                    placesViewerBodyRight.visible = false;
                }
            } else {
                placesViewerBodyLeft.visible = false;
                placesViewerBodyRight.visible = false;

                placesViewerBodyCenter.hasLeft = false;
                placesViewerBodyCenter.hasRight = false;
            }
        }
    }

    Component {
        id: placesViewerBodyLeftContainerFactory

        Rectangle {
            property alias placesViewerBodyLeftItem: placesViewerBodyLeftItem

            anchors.top: placesViewerBodyCenter.top
            anchors.right: placesViewerBodyCenter.left
            anchors.rightMargin: view.width
            width: placesViewerBodyCenter.width + 1
            height: placesViewerBodyCenter.height
            color: "#000"

            Rectangle {
                width: parent.width - 1
                height: parent.height
                anchors.left: parent.left

                PlacesViewerBody {
                    id: placesViewerBodyLeftItem

                    mapImagePrefix: 1

                    dontPopulateGlobalActiveMap: true
                }
            }
        }
    }


    Component {
        id: placesViewerBodyRightContainerFactory

        Rectangle {
            property alias placesViewerBodyRightItem: placesViewerBodyRightItem

            anchors.top: placesViewerBodyCenter.top
            anchors.left: placesViewerBodyCenter.right
            anchors.leftMargin: view.width
            width: placesViewerBodyCenter.width + 1
            height: placesViewerBodyCenter.height
            color: "#000"

            Rectangle {
                width: parent.width - 1
                height: parent.height
                anchors.right: parent.right

                PlacesViewerBody {
                    id: placesViewerBodyRightItem

                    mapImagePrefix: 2

                    dontPopulateGlobalActiveMap: true
                }
            }
        }
    }

    PlacesViewerBody {
        id: placesViewerBodyCenter

        mapImagePrefix: 0

        onMovedLeft: {
            lazyUpdateCurrentBody.stop();

            view.object = placesViewerBodyLeft.object;
            view.parent.object = view.object;
            placesViewerBodyCenter.mapScreenShotSource = placesViewerBodyLeft.mapScreenShotSource;

            mainscreen.mapSharedData.setControl("close-infobubble");
        }
        onMovedRight: {
            lazyUpdateCurrentBody.stop();

            view.object = placesViewerBodyRight.object;
            view.parent.object = view.object;
            placesViewerBodyCenter.mapScreenShotSource = placesViewerBodyRight.mapScreenShotSource;

            mainscreen.mapSharedData.setControl("close-infobubble");
        }
    }

    Component {
        id: viewComponent

        PlacesViewer {
        }
    }

    InfoPopup {
        id: viewerInfoPopup
        anchors.top: parent.top
        width: parent.width
    }

    FavoritesEditPage {
        id: favoritesEditSheet
        pickedObject: view.object
        onObjectRemoved: {
            view.object.removeFromStore();

            while (!pageStack.currentPage.placesViewerComponent && pageStack.depth > 3) {
                pageStack.pop(undefined, true);
            }

            pageStack.pop();
            favoriteChanged(false);

            openFromFavorite = false;
        }
    }

    PlacesAddAsPlace {
        id: placesAddAsPlace
        pickedObject: view.object
    }

    Connections {
        target: view
        ignoreUnknownSignals: true

        onStatusChanged: {
            if (status == PageStatus.Inactive && view.cleanAllObjectOnLeave) {
                view.cleanAllObjectOnLeave = false;

                view.object = null;
                placesViewerBodyCenter.object = null;
                placesViewerBodyLeft.object = null;
                placesViewerBodyRight.object = null;
            }
        }
    }
}
