Qt.include("PlaceHelper.js");

function getPtCategoryIconFromTransportStop(transitStop, mode) {
    var type;
    switch (transitStop.linetypes) {
        // this is a bit-mask, don't confuse the constants with TransitLineInfo.TRANSIT_
        case TransitStopInfo.TRANSIT_TYPE_BUS_PUBLIC:
        case TransitStopInfo.TRANSIT_TYPE_BUS_TOURISTIC:
        case TransitStopInfo.TRANSIT_TYPE_BUS_INTERCITY:
        case TransitStopInfo.TRANSIT_TYPE_BUS_EXPRESS:
            type = "bus";
            break;
        case TransitStopInfo.TRANSIT_TYPE_RAIL_METRO:
            type = "rail_citymetro";
            break;
        case TransitStopInfo.TRANSIT_TYPE_RAIL_LIGHT:
            type = "rail_light";
            break;
        case TransitStopInfo.TRANSIT_TYPE_RAIL_REGIONAL:
            type = "rail_regional";
            break;
        case TransitStopInfo.TRANSIT_TYPE_TRAIN_REGIONAL:
        case TransitStopInfo.TRANSIT_TYPE_TRAIN_INTERCITY:
        case TransitStopInfo.TRANSIT_TYPE_TRAIN_HIGH_SPEED :
            type = "train";
            break;
        case TransitStopInfo.TRANSIT_TYPE_MONORAIL:
            type = "rail_regional";
            break;
        case TransitStopInfo.TRANSIT_TYPE_AERIAL:
        case TransitStopInfo.TRANSIT_TYPE_INCLINED:
        case TransitStopInfo.TRANSIT_TYPE_WATER:
            type = "rail_regional";
            break;
    }

    if (type) {
        return placesAssetsPath + "images/categories/mobile_" + mode + "_public_transport_" + type + ".png";
    }
    return null;
}

function getPtCategoryIconFromTransportLine(transitLine, mode) {
    var type;
    switch (transitLine.type) {
        case TransitLineInfo.TRANSIT_TYPE_BUS_PUBLIC:
        case TransitLineInfo.TRANSIT_TYPE_BUS_TOURISTIC:
        case TransitLineInfo.TRANSIT_TYPE_BUS_INTERCITY:
        case TransitLineInfo.TRANSIT_TYPE_BUS_EXPRESS:
            type = "bus";
            break;
        case TransitLineInfo.TRANSIT_TYPE_RAIL_METRO:
            type = "rail_citymetro";
            break;
        case TransitLineInfo.TRANSIT_TYPE_RAIL_LIGHT:
            type = "rail_light";
            break;
        case TransitLineInfo.TRANSIT_TYPE_RAIL_REGIONAL:
            type = "rail_regional";
            break;
        case TransitLineInfo.TRANSIT_TYPE_TRAIN_REGIONAL:
        case TransitLineInfo.TRANSIT_TYPE_TRAIN_INTERCITY:
        case TransitLineInfo.TRANSIT_TYPE_TRAIN_HIGH_SPEED :
            type = "train";
            break;
        case TransitLineInfo.TRANSIT_TYPE_MONORAIL:
            type = "rail_regional";
            break;
        case TransitLineInfo.TRANSIT_TYPE_AERIAL:
        case TransitLineInfo.TRANSIT_TYPE_INCLINED:
        case TransitLineInfo.TRANSIT_TYPE_WATER:
            type = "rail_regional";
            break;
    }

    if (type) {
        return placesAssetsPath + "images/categories/mobile_" + mode + "_public_transport_" + type + ".png";
    }
    return null;
}

function getLineDetails(lines, systems) {
    var results = [], tmp;
    for (var i = 0; i < lines.length; i++) {
        tmp = transitDb.getLineInfo(lines[i]);
        var lineDetails = {
            id: tmp.id.stringId,
            systemId: tmp.systemId.stringId,
            type: tmp.type,
            officialName: tmp.officialName,
            informalName: tmp.informalName,
            shortName: tmp.shortName,
            attributes: tmp.attributes,
            color: "" + tmp.color
        };

        for (var s = 0, len = systems.length; s < len; s++) {
            if (systems[s].id === lineDetails.systemId) {
                if (systems[s].systemLogo) {
                    lineDetails.icon = systems[s].systemLogo.imageUrl;
                }
                break;
            }
        }
        lineDetails.icon = lineDetails.icon || getPtCategoryIconFromTransportLine(lineDetails, "list");

        results.push(lineDetails);
    }
    return results;
}

function getSystemDetails(systems){
    var results = [], tmp, details;
    for (var i = 0; i < systems.length; i++) {
        tmp = transitDb.getSystemInfo(systems[i]);
        details = {
                id: tmp.id.stringId,
                objectName: tmp.objectName,
                systemOfficialName: tmp.systemOfficialName,
                systemInformalName: tmp.systemInformalName,
                systemWebsiteUrl: tmp.systemWebsiteUrl,
                companyOfficialName: tmp.companyOfficialName,
                companyShortName: tmp.companyShortName,
                companyWebsiteUrl: tmp.companyWebsiteUrl,
                companyRoutePlannerUrl: tmp.companyRoutePlannerUrl,
                companyScheduleUrl: tmp.companyScheduleUrl,
                companyPhone: tmp.companyPhone,
                systemLogo: tmp.systemLogo ? {
                     svgContent: tmp.systemLogo.svgContent,
                     localUrl: tmp.systemLogo.localUrl,
                     isValid: tmp.systemLogo.isValid,
                     size: tmp.systemLogo.size,
                     imageUrl: tmp.systemLogo.imageUrl
                } : null,
              systemAccessLogo: tmp.systemAccessLogo ? {
                   svgContent: tmp.systemAccessLogo.svgContent,
                   localUrl: tmp.systemAccessLogo.localUrl,
                   isValid: tmp.systemAccessLogo.isValid,
                   size: tmp.systemAccessLogo.size,
                   imageUrl: tmp.systemAccessLogo.imageUrl
              } : null,
        };
        results.push(details);
    }

    return results;
}

/**
 *  Extract services
 *  \param {Number} attr - the bitwise combination of transit stop attributes
 *  \returns {Array} - services
 */
function getServiceIcons(attr) {
    var icons = [];

    var addIfPresent = function(bit, icon) {
        if ((attr & bit) === bit) {
            icons.push(placesAssetsPath + "images/ptservices/" + icon);
        }
    };

    if (attr) {
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_PAY_CAR_PARKING, "pay_car_parking.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_FREE_CAR_PARKING, "free_car_parking.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_BICYCLE_PARKING, "bicycle_parking.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_SMOKING_ALLOWED, "smoking_allowed.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_TOILETS, "toilets.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_WIRELESS_INTERNET, "wireless_internet.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_CELLULAR_SERVICE, "cellular_service.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_TICKET_MACHINES, "ticket_machines.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_LUGGAGE_LOCKERS, "luggage_lockers.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_LUGGAGE_CHECKS, "luggage_checks.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_ATTENDANT_BOOTH, "attendant_booth.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_SHOPS, "shops.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_OUTDOOR, "outdoor.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_COVERED, "covered.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_PEDESTRIAN_RAMPS, "pedestrian_ramps.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_ELEVATORS, "elevators.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_ESCALATORS, "escalators.png");
        addIfPresent(TransitStopInfo.TRANSIT_STOP_ATTRIBUTE_STAIRS, "stairs.png");
    }

    return icons;
};


function getOperatingHours(operatingHours) {

    var format = function(qtime) {
//        return Qt.formatTime(new Date(1, 0, 1970, qtime.hour(), qtime.minute(), qtime.second()));
        return "" + qtime;
    }

    var addIntervalInfo = function(infoArray, interval, dayOfWeek) {
        if (interval && interval[0]) {
            infoArray.push({
                openFrom: interval[0].startTime,
                openFromText: format(interval[0].startTime),
                openTo: interval[0].endTime,
                openToText: format(interval[0].endTime),
                dayOfWeekId: dayOfWeek
            });
        }
    }

    var info = [];
    addIntervalInfo(info, operatingHours.monday, Qt.Monday);
    addIntervalInfo(info, operatingHours.tuesday, Qt.Tuesday);
    addIntervalInfo(info, operatingHours.wednesday, Qt.Wednesday);
    addIntervalInfo(info, operatingHours.thursday, Qt.Thursday);
    addIntervalInfo(info, operatingHours.friday, Qt.Friday);
    addIntervalInfo(info, operatingHours.saturday, Qt.Saturday);
    addIntervalInfo(info, operatingHours.sunday, Qt.Sunday);
    addIntervalInfo(info, operatingHours.holiday, -1);
    return info;
}

function getDayOfWeekText(dayOfWeekId) {
    var date = new Date(1970, 0, 4 + dayOfWeekId); // Jan 4th 1970 is a sunday, Qt.Monday = 1
    return "" + Qt.formatDate(date, "dddd");
}

function parseOperatingHours(info) {
    if(!info || info.length === 0) return [];
    var isDifferent = false,
        openHours = [],
        holidayHours = [],
        dayStartIndex
    ;

    for (var i=0; i<info.length; i++) {
        var openTime = parseTime(info[i].openFrom);
        var closeTime = parseTime(info[i].openTo);

        // Make sure that if it's open 24 hours, we say 00:00-23:59 instead of 0:00-0:00
        if (closeTime == openTime) {
            var dateTime = new Date(info[i].openTo);
            dateTime.setMinutes(Qt.formatDateTime(closeTime, "mm")-1);
            closeTime = parseTime(dateTime);
        }

        if(info[i].dayOfWeekId === -1) {    // holiday hours
            holidayHours[holidayHours.length] = {
                    dayOfWeekStartId: -1,
                    dayOfWeekEndId: -1,
                    openFromText: openTime,
                    openToText: closeTime
            };
            dayStartIndex = i+1;
        } else if (i === 0) {    // first element on the list, give us something to start with
            dayStartIndex = i;
            openHours.push({
                    dayOfWeekStartId: info[i].dayOfWeekId,
                    dayOfWeekEndId: info[i].dayOfWeekId,
                    openFromText: openTime,
                    openToText: closeTime
            });
        } else if (!(info[i-1].openFromText === info[i].openFromText && info[i-1].openToText === info[i].openToText)) { //If the days are different
            openHours.push({
                    dayOfWeekStartId: info[i].dayOfWeekId,
                    dayOfWeekEndId: info[i].dayOfWeekId,
                    openFromText: openTime,
                    openToText: closeTime
            });
            dayStartIndex = i;
        } else {    // Else keep adding on the current time
            openHours[openHours.length-1] = {
                    dayOfWeekStartId: i - dayStartIndex === 6 ? 100 : info[dayStartIndex].dayOfWeekId,
                    dayOfWeekEndId: i - dayStartIndex === 6 ? 100 : info[i].dayOfWeekId,
                    openFromText: openTime,
                    openToText: closeTime
            }
        }
    }
    return {
        regularHours: openHours,
        holidayHours: holidayHours
    };
}
function parseTime(aTime) {
    //return aTimeStr.substring(aTimeStr.indexOf(":")-2, aTimeStr.lastIndexOf(":"));
    var localFormat = "hh:mm";
    if(appState && appState.locale && appState.locale.measurementSystem === "imperial") {
        localFormat = "h:mm ap";
    }

    return Qt.formatTime(aTime, localFormat);
}

function parseSystems(aSystems) {
    var ops =  [];
    if (!aSystems) return ops;
    for(var i=0; i<aSystems.length; i++) {
        if(!_inArray(ops, aSystems[i].companyOfficialName)) {
            ops[ops.length] = aSystems[i].companyOfficialName;
        }
    }
    return ops;
}

function _inArray(aArray, aString) {
    for(var i=0; i < aArray.length; i++) {
        if(aArray[i] === aString) {
            return true;
        }
    }
    return false;
}

function createPtPlaceFromStopInfo(stopInfo, aFromPosition) {
    var systemDetails = getSystemDetails(stopInfo.systems);
    var pt = {
        systems: systemDetails,
        lines: getLineDetails(stopInfo.lines, systemDetails),
        linetypes: stopInfo.lineTypes,
        operatingHours: getOperatingHours(stopInfo.operatingHours),
        services: getServiceIcons(stopInfo.attributes)
   };
    return {
            properties: {
                isPt: true,
                title: "" + stopInfo.officialName,
                geoLatitude: stopInfo.position.latitude,
                geoLongitude: stopInfo.position.longitude,
                placesId: stopInfo.placesId,
                geoDistance: aFromPosition ? computeDistance({latitude: aFromPosition.latitude, longitude: aFromPosition.longitude},
                                             {latitude: stopInfo.position.latitude, longitude: stopInfo.position.longitude})
                                           : undefined
            },
            publictransport: pt,
            categories: {name: "public-transport"}
    };
}
