import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.places 1.0

import "../PlaceHelper.js" as JsPlaceHelper

Rectangle {
    id: categoriesList;
    anchors.fill: parent;
    color: "#f0f1f2";
    clip: true;

    property variant mainScreen: parent

    property variant currentCategory;
    property string headerName;
    property int offset: 10;
    property bool mapFocus: mapToggle.mapActive;
    property bool showMapListToggle: showResults || loadingInfo.visible;
    property bool showResults: false;
    property bool loading: false;
    property bool iconAlreadyOnMap: false

    signal categoryPlaceSelected(int index);
    signal subCategoryClicked(int index, variant location);
    signal refreshed(variant location);

    ListModel {
        id: currentCatModel
        objectName: "currentCategoryModel"
    }

    property PlacesCommonList __catList: catList
    // The category list
    PlacesCommonList {
        id: catList
        objectName: "catList"
        anchors.top: categoryHeader.bottom
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        model: currentCatModel
        itemDelegate: Rectangle {
            id: fff

            width: categoriesList.width
            height: 70
            color: "#f0f1f2"

            MapsText {
                text: name
                state: "listNormalBold"
                anchors {
                    verticalCenter: parent.verticalCenter
                    left: parent.left
                    leftMargin: 2 * offset
                }
                width: parent.width - anchors.leftMargin/* - arrow.width - arrow.anchors.rightMargin*/
            }
            Image {
                id: arrow;
                source: placesAssetsPath + "images/layout/arrow_right_dark.png";
                anchors.verticalCenter: parent.verticalCenter;
                anchors.right: parent.right;
                anchors.rightMargin: offset;
            }

            MouseArea {
                id: mouse;
                anchors.fill: parent
                onPressed: { parent.color = "#d0d1d2" }
                onReleased: { parent.color = "#f0f1f2" }
                onCanceled: { parent.color = "#f0f1f2" }
                onClicked: { subCategoryClicked(index, mapView.center); }
            }
        }
        visible: false
    }

    property PlacesCommonList __resultList: resultList
    PlacesCommonList {
        id: resultList
        objectName: "resultList"
        anchors.top: categoryHeader.bottom
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        visible: showResults && !mapFocus
        itemDelegate: PlaceListItem {
            onItemSelected: { categoryPlaceSelected(index) }
        }
    }

    // The loading and search result information
    Rectangle {
        id: loadingInfo
        objectName: "loadingInfo"
        visible: !mapFocus && loading || line1.text || line2.text
        width:  categoriesList.width
        height: categoriesList.height
        anchors.top: categoryHeader.bottom
        color: "#FFF"

        BusyIndicator {
            visible: !line1.text && !line2.text
            anchors.centerIn: parent
            anchors.verticalCenterOffset: -40
            objectName: "finderLoadingTextContainer"
            platformStyle: BusyIndicatorStyle { size: "large";  inverted: false; }

            running: visible
        }

        Text {
            id: line1
            visible: !!text
            objectName: "loadingInfoLine1"
            //text: qsTranslate("Maps", "qtn_placesconnector_categorylist_downloading"); // "Getting categories...";
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: -15;
            font.pointSize: 17
            font.family: mainscreen.__labelStyle.fontFamily
            color: "#8C8C8C";
        }

        Text {
            id: line2
            visible: !!text
            objectName: "loadingInfoLine2"
            //text: qsTranslate("Maps", "qtn_placesconnector_categorylist_recdata"); // "receiving data";
            anchors.top: line1.bottom
            anchors.horizontalCenter: parent.horizontalCenter
            font.pointSize: 17
            font.family: mainscreen.__labelStyle.fontFamily
            color: "#8C8C8C";
        }
    }

    // The category header with the title and refresh and map/list toggle for the search results

    Header {
        id: categoryHeader
        text: headerName
        textWidth: parent.width - mapToggle.width - refresh.width
        dropShadow: mapToggle.mapActive

        rightOffset: 32

        // Map button toggle
        MapListToggle {
            id: mapToggle;

            anchors {
                right: parent.right;
                rightMargin: 10;
                leftMargin: 10;
                verticalCenter: parent.verticalCenter;
                verticalCenterOffset: -5
            }

            visible: showMapListToggle
            mapActive: false
            onMapActiveChanged: {
                if (resultList.model.count == 0) {
                    if (mapFocus && errorMessage.scale === 0) {
                        if (errorMessage.text) {
                            errorMessage.show();
                        }
                    } else if (!mapFocus && errorMessage.scale !== 0) {
                        errorMessage.hide();
                    }
                }

                putResultsOnMap();
            }
        }

        // Refresh category button
        Button {
            id: refresh;
            height: 50; width: 50;
            anchors.right: mapToggle.left;
            anchors.rightMargin: 10;
            anchors.leftMargin: 10;
            anchors.verticalCenter: parent.verticalCenter;
            anchors.verticalCenterOffset: -5
            visible: showMapListToggle
            onClicked: {
                removeResultsFromMap();
                refreshResults();
            }
            Image {
                anchors.centerIn: parent;
                visible: refresh.visible && !loading;
                source: placesAssetsPath + "images/layout/icon-refresh" + (refresh.pressed ? "_light.png":"_dark.png");
            }
            BusyIndicator {
                anchors.centerIn: parent
                running: true
                visible: refresh.visible && loading
            }
        }

        z: 1
    }
    // Delegate component for the category list
    Component {
        id: categoryDelegate
        Rectangle {
            width: categoriesList.width
            height: 70
            color: "#f0f1f2"
            MapsText {
                text: name
                state: "listNormalBold"
                anchors {
                    verticalCenter: parent.verticalCenter
                    left: parent.left
                    leftMargin: 2*offset
                }
                width: parent.width - anchors.leftMargin - arrow.width - arrow.anchors.rightMargin
            }
            Image {
                id: arrow
                source: placesAssetsPath + "images/layout/arrow_right_dark.png"
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: parent.right
                anchors.rightMargin: offset
            }

            MouseArea {
                id: mouse
                anchors.fill: parent
                onPressed: { parent.color = "#d0d1d2" }
                onReleased: { parent.color = "#f0f1f2" }
                onCanceled: { parent.color = "#f0f1f2" }
                onClicked: { subCategoryClicked(index, mapView.center); }
            }
        }
    }

    property MapViewer __map: mapView
    MapViewer {
        id: mapView

        width: parent.width
        height: parent.height - categoryHeader.height
        anchors.top: categoryHeader.bottom
        visible: mapFocus

        mainScreen: mainscreen
        useGlobalShare: true
        behaviors: "_NO_LONG_PRESS_AT_ALL_:true;_NO_PUBLIC_TRANSPORT_STOPS_CLICK_:true;"

        onPlaceSelected: {
            var index=0, tmp;
            for (var i = 0; i < resultList.model.count; i++) {
                tmp = resultList.model.get(i);
                if (tmp && tmp.uri == uri) {
                    index = i;
                }
            }
            categoryPlaceSelected(index);
        }

        onPositionSelected: {
            place.uri = "";
            place.properties = positionData;

            var placeViewerObj = placesViewer.createObject(categoriesList);
            tabGroup.currentTab.push(placeViewerObj);
        }
    }

    Component {
        id: placesViewer
        PlacesViewer {
            object : place
        }
    }
    Place {
        id: place
    }

    Timer {
        id: lazyMapLoading

        interval: 100
        repeat: false
        running: false

        onTriggered: {
            refreshed(mapView.center);
        }
    }

    InfoBanner {
        id: errorMessage;
        topMargin: categoryHeader.height + 10
        timerEnabled: true;
        timerShowTime: 3*1000;
    }

    /**
      * Fill the category list with categories.
      * param aCategory {Object} Either an array from parsed tree, or a declarative list model of categories
      *     aCategory can have the form:
      *         {
      *             name: {String} : name of the category. This appears in the categoryHeader.
      *             categoryId: {Number} : the corresponding category id,
      *             childCategories: {Array} : a possibly empty array of Categories
      *             ***OR***
      *             childCategories: {ListModel} : a possibly empty List Model of Categories
      *         }
      *
      */
    function fillCategoryList(aCategory) {
        catList.visible = true;
        var data = aCategory["childCategories"];
        headerName = aCategory.name;
        currentCatModel.clear();
        loading = false;

        // Take care if the children are in the array or if they have been converted to a list model
        if (data.count) {
            for (var i = 0; i < data.count; i++) {
                currentCatModel.append(data.get(i));
            }
        } else {
            for (var prop in data) {
                log.trace("data[" + prop + "]: " + data[prop]);
                if (data[prop]) currentCatModel.append(data[prop]);
            }
        }
    }

    /**
      * Pass signal from refresh button and update visual elements
      */
    function refreshResults() {
//        showResults = false;
        loading = true;
        line1.text = "";
        line2.text = "";
        //loadingInfo.visible = !mapFocus;
        if (mapView.ready) {
            refreshed(mapView.center);
        } else {
            //because map is syncronizing with all other maps it seems this map is requested to get it's center before it's initialized
            //the default value for center is berlin - and on map complete we have our own logick to load all the map details
            //more or less the map is ready when we have ready flag set to true.
            //so in this case if we request center before the map is ready we just wait for it in a timer
            lazyMapLoading.running = true;
        }
    }

    /**
      * Show the loading item and refresh to query a search. Needs to be done from here
      * to prevent the search results from being loaded on the parent category page.
      */
    function showLoadingResults(aTitle) {
        headerName = aTitle;
        showResults = true;
        loading = true;
        catList.visible = false;
        refreshResults();
    }

    function showErrorConnectionText(aTitle, aLine1, aLine2) {
        loading = false;
        showMapListToggle = false;
        headerName = aTitle || "";
        line1.text = aLine1 || qsTranslate("Maps", "qtn_placesconnector_categorylist_noconnection"); //No connection;
        line2.text = aLine2 || "";
    }

    /**
      * Fills the list of search results from the query of the category.
      */
    function fillResultList(aModel, aTitle) {
        loading = false;
        currentCatModel.clear();
        headerName = aTitle;
        resultList.model = aModel;
        showResults = true;

        if (!aModel) {
            if (mapFocus) {
                errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_categorylist_noconnection"); //No connection;
                errorMessage.show();
            }
            line1.text = qsTranslate("Maps", "qtn_placesconnector_categorylist_noconnection"); //No connection;
            line2.text = "";
        } else  if (aModel && aModel.count == 0) {
            if (mapFocus) {
                errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_categorylist_noresult %1").arg(aTitle); //No results found for " + aTitle;
                errorMessage.show();
            }
            line1.text = qsTranslate("Maps", "qtn_placesconnector_categorylist_noresult"); //No results found for
            line2.text = aTitle; // + aTitle
            log.info("NO RESULTS found","CategoryList.qml");
        } else {
            if (typeof (analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIActivity("Search", { type: "Category" } );
            }
            if (mapFocus) {
                //loadingInfo.visible = false;
                putResultsOnMap();
            }
        }
    }

    /**
      * Puts the items in the list result model on the map.
      */
    function putResultsOnMap() {
        if (!iconAlreadyOnMap) {
            var icon, obj;
            mapView.model.clear();

            if (resultList.model && resultList.model.count > 0) {
                mapView.lazyAppendIconMode = true;
                for (var i = 0; i < resultList.model.count; i++) {
                    obj = resultList.model.get(i);
                    mapView.model.appendIcon(
                        obj.properties.geoLatitude,
                        obj.properties.geoLongitude,
                        "file://" + JsPlaceHelper.getIconSourceFromId(JsPlaceHelper.getCategory(obj.categories).id, obj.properties.typeNumeric, (obj.favorite || obj.isFavorite), "map"),
                        { uri: obj.uri, title: obj.properties.title, address: JsPlaceHelper.formatAddress(obj.properties) },
                        {}
                    );
                }
                mapView.lazyAppendIconMode = false;

                iconAlreadyOnMap = true;
                log.info("will put icons on the map...");
                mapView.model.setObjectsOwner("categories", false);
            }
        } else {
            log.info("icons already on the map, skipping...");
        }
    }

    function removeResultsFromMap() {
        mapView.model.clear();
        iconAlreadyOnMap = false;
    }
}
