import QtQuick 1.0

Rectangle {
    id: didYouMeanPopup

    width: parent.width + 1
    x: -1
    color : "#f0f0f0"
    border.color: "#e8e8e8"

    property variant mapView

    property string didYouMeanPreWord
    property string didYouMeanWord

    signal clicked()
    signal closed()

    // [Lukas] unfortunately rich Text doesn't work with elide. E.g. text can't be bold and elide. So 2 text fields are used.
    Text {
        id: textArea
        text: (didYouMeanPreWord && didYouMeanWord)? (didYouMeanPreWord + " ") : ""
        anchors.left: parent.left
        anchors.leftMargin: 31
        anchors.verticalCenter: parent.verticalCenter
        font.pixelSize: 24
        horizontalAlignment: "AlignLeft"
        width: parent.width - 1
        visible: false
    }
    Text {
        id: textArea2
        text: (didYouMeanPreWord && didYouMeanWord) ? (didYouMeanWord) : ""
        anchors.verticalCenter: parent.verticalCenter
        x: textArea.paintedWidth + textArea.x
        font.pixelSize: 24
        width: didYouMeanPopup.width - textArea.x - textArea.paintedWidth - 40
        font.bold: true
        elide: Text.ElideRight
        visible: textArea.visible
    }
    Text {
        id: textArea3
        text: (didYouMeanPreWord && didYouMeanWord) ? ("?") : ""
        anchors.verticalCenter: parent.verticalCenter
        x: textArea2.x + textArea2.paintedWidth
        //anchors.left: textArea2.right
        font.pixelSize: 24
        font.bold: true
        visible: textArea.visible
    }

    MouseArea {
        anchors.fill: parent
        onClicked: {
            didYouMeanPopup.clicked();
            didYouMeanPreWord = "";
            didYouMeanWord = "";
        }
    }

    Timer {
        interval: 8000

        running: didYouMeanPopup.state == "shown"

        onTriggered: {
            if (didYouMeanPreWord || didYouMeanWord) {
                didYouMeanPreWord = "";
                didYouMeanWord = "";
                didYouMeanPopup.closed();
            }
        }
    }

    states: [
        State {
            name: "hidden"
            when: !didYouMeanWord || !didYouMeanPreWord
        },

        State {
            name: "shown"
            when: didYouMeanWord && didYouMeanPreWord
        }
    ]

    transitions: [
        Transition {
            to: "shown"

            SequentialAnimation {
                NumberAnimation { target: didYouMeanPopup; property: "height"; to: 70; duration: 60; }
                PropertyAnimation { target: textArea; property: "visible"; to: true; duration: 1; }
            }
        },

        Transition {
            to: "hidden"

            SequentialAnimation {
                PropertyAnimation { target: textArea; property: "visible"; to: false; duration: 1; }
                NumberAnimation { target: didYouMeanPopup; property: "height"; to: 0; duration: 60; }
            }

        }
    ]

    Connections {
        target: mapView
        ignoreUnknownSignals: true

        onMapClicked: {
            if (didYouMeanPreWord || didYouMeanWord) {
                didYouMeanPreWord = "";
                didYouMeanWord = "";
                didYouMeanPopup.closed();
            }
        }
        onMapDoubleClicked: {
            if (didYouMeanPreWord || didYouMeanWord) {
                didYouMeanPreWord = "";
                didYouMeanWord = "";
                //didYouMeanPopup.closed();
            }
        }
        onMapLongPressed: {
            if (didYouMeanPreWord || didYouMeanWord) {
                didYouMeanPreWord = "";
                didYouMeanWord = "";
                //didYouMeanPopup.closed();
            }
        }
        onMapDragStarted: {
            if (didYouMeanPreWord || didYouMeanWord) {
                didYouMeanPreWord = "";
                didYouMeanWord = "";
                //didYouMeanPopup.closed();
            }
        }
    }
}
