import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0

Rectangle {
    property alias text: title.text
    property alias subText: subTitle.text
    property alias textWidth: textBox.width
    property int leftOffset: 16
    property int rightOffset: 16
    property bool dropShadow: true

    height: header.height
    anchors {
        left: parent.left
        right: parent.right
    }

    Rectangle {
        id: header

        color: "black"
        height: 85
        clip: true
        anchors {
            left: parent.left
            right: parent.right
        }

        Rectangle {
            color: "#E5E6E8"
            height: 85 + 10
            anchors {
                left: parent.left
                right: parent.right
            }

            Item {
                id: textBox

                width: parent.width
                height: parent.height
                anchors {
                    top: parent.top
                    bottom: parent.bottom
                    bottomMargin: 10
                }

                MapsText {
                    id: title

                    anchors {
                        left: parent.left
                        right: parent.right
                        leftMargin: leftOffset
                        rightMargin: rightOffset
                        verticalCenter: parent.verticalCenter
                        verticalCenterOffset: subTitle.visible ? -0.5*subTitle.height : 0
                    }
                    width: parent.width
                    platformStyle: LabelStyle { fontPixelSize: 32 }

                }

                MapsText {
                    id: subTitle

                    text: ""
                    visible: subTitle.text !== ""
                    width: title.width
                    platformStyle: LabelStyle { textColor: subTitle.secondaryColor }
                    anchors {
                        left: parent.left
                        right: parent.right
                        leftMargin: leftOffset
                        rightMargin: rightOffset
                        top: title.bottom
                    }
                }
            }
        }
    }

    Rectangle {
        id: gradient
        visible: dropShadow
        anchors.top: header.bottom

        height: 4
        width: parent.width
        gradient: Gradient {
            GradientStop { position: 0.0; color: "black" }
            GradientStop { position: 1.0; color: "transparent" }
        }
        opacity: 0.5
        z: 1
    }

    Rectangle {
        visible: !dropShadow
        anchors {
            top: header.bottom
            left: parent.left
            right: parent.right
        }
        height: 1
        color: "#CDCECE"
    }
}

