import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector.map 1.0

import "../PlaceHelper.js" as PlaceHelper

Page {
    id: preview

    property variant object
    property variant titleAreaPrototype

    property string title
    property string address
    property variant rating
    property string categoryName
    property string distance
    property string provider
    property string iconPath

    property double startupCenterLatitude
    property double startupCenterLongitude
    property double startupZoomLevel
    property variant initialCenter

    Item {
        id: internals

        function initHeader(params) {
            if (params.placeTitle) {
                preview.title = params.placeTitle;
            } else {
                preview.title = "";
            }

            if (params.placeAddress) {
                preview.address = params.placeAddress;
            } else {
                preview.address = "";
            }

            if (params.placeRating) {
                preview.rating = params.placeRating;
            } else {
                preview.rating = null;
            }

            if (params.placeCategoryName) {
                preview.categoryName = params.placeCategoryName;
            } else {
                preview.categoryName = "";
            }

            if (params.placeDistance) {
                preview.distance = params.placeDistance;
            } else {
                preview.distance = "";
            }

            if (params.provider) {
                preview.provider = params.provider;
            } else {
                preview.provider = "";
            }
        }

        function initMap(object) {
            if (preview.object && mapViewer.model) {
                mapViewer.model.clear();
                var currentPositionMode = (preview.object.uri == -1);

                var iconPath = preview.object.iconPath.replace("_list_", "_map_");

                if (iconPath === "") {
                    var objectCategories = preview.object.details && preview.object.details.categories && preview.object.details.categories.category? preview.object.details.categories.category : preview.object.categories;
                    if (PlaceHelper.getCategory(objectCategories).id) {
                        iconPath = PlaceHelper.getIconSourceFromId(PlaceHelper.getCategory(objectCategories).id, preview.object.properties.typeNumeric, preview.object.isFavorite, "map");
                    } else {
                        iconPath = PlaceHelper.getIconSourceFromCategoryName(PlaceHelper.getCategory(objectCategories).name, preview.object.properties.typeNumeric, preview.object.isFavorite, "map");
                    }

                }

                if (!currentPositionMode) {
                    if (mapViewer.model.count == 0) {
                        mapViewer.model.appendIcon(
                            preview.object.properties.geoLatitude? preview.object.properties.geoLatitude : preview.object.details.location.geoCoordinates.latitude,
                            preview.object.properties.geoLongitude? preview.object.properties.geoLongitude : preview.object.details.location.geoCoordinates.longitude,
                            iconPath,
                            {},
                            {}
                        );
                    }
                    mapViewer.attachedToLocation = false;
                } else {
                    mapViewer.attachedToLocation = true;
                }

                mapViewer.zoomScale = mapViewer.zoomLevels[15];
                mapViewer.center.latitude = (preview.object.properties.geoLatitude? parseFloat(preview.object.properties.geoLatitude) : parseFloat(preview.object.details.location.geoCoordinates.latitude)) + 0.00015;
                mapViewer.center.longitude = (preview.object.properties.geoLongitude? parseFloat(preview.object.properties.geoLongitude) : parseFloat(preview.object.details.location.geoCoordinates.longitude));

                if (preview.initialCenter) {
                    if (preview.initialCenter.destroy) {
                        preview.initialCenter.destroy();
                    } else {
                        preview.initialCenter = null;
                    }
                }
                preview.initialCenter = mapViewer.center.clone(preview);
            }
        }
    }


    tools: view.tools

    PlaceTitleArea {
        id: titleArea
        objectName: "titleArea"

        dropShadow: true

        width: parent.width
        placeTitle: preview.title
        placeAddress: preview.address
        placeRating: preview.rating
        placeCategoryName: preview.categoryName

        placeDistance: preview.distance

        provider: preview.provider

        z: 1

        MouseArea {
            anchors.fill: parent
            onClicked: {
                mapViewer.status = mapViewer._INTERRUPTED_ANIMATION_;
                mapViewer.attachedToLocation = false;
                mapViewer.callMapApi("centerAt", [preview.initialCenter]);
            }
        }
    }

    MapViewer {
        id: mapViewer
        anchors.top: titleArea.bottom
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        behaviors: "_NO_LONG_PRESS_AT_ALL_:true;_NO_AUTO_GPS_INIT_:true;_NO_SETTINGS_:true;_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_:true;_NO_INFO_BUBBLE_CLICK_EFFECT_:true;_NO_INITIAL_SETTINGS_LOAD_:true;"
        mainScreen: mapViewer._NO_MAINSCREEN_
        dontSaveData: true
        startupCenterLatitude: preview.startupCenterLatitude;
        startupCenterLongitude: preview.startupCenterLongitude;
        startupZoomLevel: preview.startupZoomLevel

        Component.onCompleted: {
            preview.initialCenter = mapViewer.center.clone(preview);
        }

        mapSharedData: MapSharedData {
            mode: "preset"
            timeMode: "day"
            mapMode: "pedestrian"
            mode3d: false
            perspective: 0
            orientation: 0
            landmarksVisible: true
            trafficInfoVisible: false
            publicTransportMode: false
            publicTransportModePartial: false
            publicTransportLineHightlighted: false
        }
    }

    Timer {
        id: updateOnCompletedInWhile
        interval: 250
        onTriggered: {
            internals.initMap(preview.object);
            internals.initHeader(preview.titleAreaPrototype);
        }
    }

    Connections {
        target: mapPreviewMini
        ignoreUnknownSignals: true

        onUpdateFlaggerChanged: {
            internals.initMap(preview.object);
        }
    }

    Component.onCompleted: {
        updateOnCompletedInWhile.start();
    }
    onObjectChanged: {
        internals.initMap(preview.object);
    }
    onTitleAreaPrototypeChanged: {
        internals.initHeader(preview.titleAreaPrototype);
    }

    Connections {
        target: preview.titleAreaPrototype
        ignoreUnknownSignals: true

        onPlaceTitleChanged: { internals.initHeader(preview.titleAreaPrototype); }
        onPlaceAddressChanged: { internals.initHeader(preview.titleAreaPrototype); }
    }
}
