import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector.map 1.0

import "../PlaceHelper.js" as PlaceHelper

Rectangle {
    id: mapPreviewMini

    color: "#f0f1f2"

    property variant object
    property variant titleArea

    property bool clickingOnMapEnabled: true

    property int imagePrefix: -1
    property alias mapScreenShotSource: mapScreenShot.source
    property alias currentPositionMode: internals.currentPositionMode

    property bool updateFlagger: false
    function update() {
        internals.doStartUpdateWithObject();
    }

    Item {
        id: internals

        property bool currentPositionMode: false

        function doStartUpdateWithObject() {
            if (mapPreviewMini.object) {
                if (!mainscreen.spareMapBussy) {
                    waitTillSpareMapIsFree.stop();
                    mainscreen.spareMapBussy = true;

                    mainscreen.spareMap.width = mapPreviewMini.width || mainscreen.width;
                    mainscreen.spareMap.height = mapPreviewMini.height;
                    mainscreen.spareMap.mapSharedData = spareMapSharedData;
                    mainscreen.spareMap.model.clear();
                    internals.currentPositionMode = (mapPreviewMini.object.uri == -1);

                    var iconPath = mapPreviewMini.object.iconPath.replace("_list_", "_map_");

                    if (iconPath === "") {
                        var objectCategories =
                            (mapPreviewMini.object.details && mapPreviewMini.object.details.categories && mapPreviewMini.object.details.categories.category)?
                                mapPreviewMini.object.details.categories.category
                                :
                                mapPreviewMini.object.categories
                        ;
                        if (PlaceHelper.getCategory(objectCategories).id) {
                            iconPath = PlaceHelper.getIconSourceFromId(PlaceHelper.getCategory(objectCategories).id, mapPreviewMini.object.properties.typeNumeric, mapPreviewMini.object.isFavorite, "map");
                        } else {
                            iconPath = PlaceHelper.getIconSourceFromCategoryName(PlaceHelper.getCategory(objectCategories).name, mapPreviewMini.object.properties.typeNumeric, mapPreviewMini.object.isFavorite, "map");
                        }

                    }

                    if (!internals.currentPositionMode) {
                        if (mainscreen.spareMap.model.count == 0) {
                            mainscreen.spareMap.model.appendIcon(
                                mapPreviewMini.object.properties.geoLatitude? mapPreviewMini.object.properties.geoLatitude : mapPreviewMini.object.details.location.geoCoordinates.latitude,
                                object.properties.geoLongitude? mapPreviewMini.object.properties.geoLongitude : mapPreviewMini.object.details.location.geoCoordinates.longitude,
                                iconPath,
                                {},
                                {}
                            );
                        }
                        mainscreen.spareMap.attachedToLocation = false;
                    } else {
                        mainscreen.spareMap.attachedToLocation = true;
                    }

                    mainscreen.spareMap.zoomScale = mainscreen.spareMap.zoomLevels[15];
                    mainscreen.spareMap.center.latitude = (mapPreviewMini.object.properties.geoLatitude? parseFloat(mapPreviewMini.object.properties.geoLatitude) : parseFloat(mapPreviewMini.object.details.location.geoCoordinates.latitude)) + 0.00015;
                    mainscreen.spareMap.center.longitude = (mapPreviewMini.object.properties.geoLongitude? parseFloat(mapPreviewMini.object.properties.geoLongitude) : parseFloat(mapPreviewMini.object.details.location.geoCoordinates.longitude));

                    waitSomeTimeBeforeScreenshot.start();
                } else {
                    waitTillSpareMapIsFree.start();
                }
            } else {
                waitTillSpareMapIsFree.stop();
            }
        }

        function doFinishUpdateWithObject() {
            mapScreenShot.source = "";

            var randonNumber = imagePrefix;
            var fileName = appState.getSystemTmpDir() + "/places-viewer-screen-" + randonNumber + ".png";

            mainscreen.spareMap.saveScreenshot(fileName);
            mapScreenShot.source = fileName + "?rand=" + Math.random();

            mainscreen.spareMapBussy = false;

            mapPreviewMini.updateFlagger = true;
            mapPreviewMini.updateFlagger = false;
        }

        Timer {
            id: waitTillSpareMapIsFree

            interval: 25
            repeat: true

            onTriggered: {
                internals.doStartUpdateWithObject();
            }
        }

        Timer {
            id: waitSomeTimeBeforeScreenshot

            interval: 25

            onTriggered: {
                internals.doFinishUpdateWithObject();
            }
        }

        Timer {
            running: internals.currentPositionMode
            repeat: true
            interval: 1000 * 10
            onTriggered: {
                internals.doStartUpdateWithObject();
            }
        }
    }

    BusyIndicator {
        platformStyle: BusyIndicatorStyle { inverted: false; size: "medium"; }

        anchors.centerIn: parent
        running: visible
        visible: waitTillSpareMapIsFree.running || waitSomeTimeBeforeScreenshot.running
    }

    Image {
        id: mapScreenShot

        rotation: 90

        anchors.centerIn: parent
    }

    MapSharedData {
        id: spareMapSharedData

        mode: "preset"
        timeMode: "day"
        mapMode: "pedestrian"
        mode3d: false
        perspective: 0
        orientation: 0
        landmarksVisible: true
        trafficInfoVisible: false
        publicTransportMode: false
        publicTransportModePartial: false
        publicTransportLineHightlighted: false
    }

    MouseArea {
        anchors.fill: parent

        onClicked: {
            if (mapPreviewMini.clickingOnMapEnabled) {
                view.pageStack.push(
                    mapPreviewMaximizedFactory,
                    { object: mapPreviewMini.object }
                );
            }
        }
    }

    Component {
        id: mapPreviewMaximizedFactory

        MapPreviewMaximized {
            titleAreaPrototype: mapPreviewMini.titleArea
            startupCenterLatitude: (mapPreviewMini.object.properties.geoLatitude? parseFloat(mapPreviewMini.object.properties.geoLatitude) : parseFloat(mapPreviewMini.object.details.location.geoCoordinates.latitude)) + 0.00015
            startupCenterLongitude: (mapPreviewMini.object.properties.geoLongitude? parseFloat(mapPreviewMini.object.properties.geoLongitude) : parseFloat(mapPreviewMini.object.details.location.geoCoordinates.longitude))
            startupZoomLevel: 15
        }
    }

    Component.onCompleted: {
        internals.doStartUpdateWithObject();
    }
    onObjectChanged: {
        internals.doStartUpdateWithObject();
    }
    Component.onDestruction: {
    }
}
