import QtQuick 1.0
import ovi.connector 1.0
import ovi.connector.map 1.0

import "../PlaceHelper.js" as PlaceHelper

Rectangle {
    id: placeListItem

    signal itemSelected(int index)
    signal itemLongPressed(int index)

    property variant rating: !!object && PlaceHelper.getRating(object)
    property string icon: getIcon()
    function getIcon() {
        var allowedItems = [
            "sights_museums",
            "shopping",
            "facility",
            "goingout",
            "business_services",
            "automotive",
            "eat_drink",
            "accomodation",
            "leisure",
            "areas_infrastructure",
            "transport",
            "address",
            "nature_geography",
            "default"
        ];
        if (typeof(favorite) != "undefined" && favorite.categories && favorite.categories.length > 0) {
            if (allowedItems.indexOf(favorite.categories[0]) >= 0) {
                return mapAssetsPath + "images/categories/mobile_list_" + favorite.categories[0] + "_fav.png";
            } else {
                return (object && object.iconPath)? object.iconPath :  "";
            }
        } else {
            return (object && object.iconPath)? object.iconPath :  "";
        }
    }

    height: 105
    width: parent.width
    color: "#f0f1f2"

    Item {
        anchors {
            left: parent.left
            right: infoColumn.left
            leftMargin: 8
            topMargin: 16

        }
        height: parent.height
        Image {
            id: img
            source: icon
            height: parent.height
            fillMode: "PreserveAspectFit"
            anchors.centerIn: parent
            onStatusChanged: {
                if (img.status == Image.Error && object && img.source != object.iconPath) {
                    icon = object.iconPath;
                }
            }
        }
    }

    Column {
        id: infoColumn;
        anchors {fill: parent; leftMargin:  89; topMargin: 20; rightMargin: 16 }

        MapsText {
            id: captionText
            state: "listLarge"
            text: {
                if (typeof(favorite) != "undefined" && favorite.text && favorite.text !== "") {
                    return favorite.text;
                } else if (object && object.favorite && object.favorite.text && object.favorite.text !== "") {
                    return object.favorite.text;
                } else {
                    return (object && object.properties && object.properties.title)? object.properties.title : object? (object.name || "") : "";
                }
            }
            width: parent.width - distanceText.width - 16;
        }

        MapsText {
            id: nextLineText
            state: "listSmall"
            text: PlaceHelper.getPlaceSubtitle(object)
            width: parent.width
            elide: Text.ElideRight
        }

        Rating {
            id: stars
            visible: rating && rating.ratingCount > 0
            value: rating && rating.rating
            count: rating && rating.ratingCount
        }
    }

    MapsText {
        id: distanceText
        state: "smallerText"
        text: PlaceHelper.getReadablePlaceDistance(units, object)
        anchors { right: parent.right; rightMargin: 16; top: parent.top; topMargin: infoColumn.anchors.topMargin; }
    }

    Units { id: units }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        onClicked: {
            itemSelected(index);
        }
        onPressAndHold: {
            itemLongPressed(index);
        }
        onPressed: parent.color="#d0d1d2"
        onReleased: parent.color="#f0f1f2"
        onCanceled: parent.color="#f0f1f2"
    }
    Keys.onPressed:{
        if (event.key == Qt.Key_Enter || event.key == Qt.Key_Return) {
            itemSelected(index);
        }
    }
}
