import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0

Rectangle {
    id: placeTitleArea

    property bool dropShadow: false

    width: parent.width
    height: placeCategory.height + ratingsElement.height + 5 + (placeAddress.paintedHeight > 28 ? 85 + placeAddress.paintedHeight - 28 : 85) + placeTitle.paintedHeight -30

    property alias placeTitle: placeTitle.text
    property alias placeAddress: placeAddress.text
    property alias placeDistance: placeDistance.text
    property alias placeCategoryName: placeCategory.text
    property variant placeRating: { value: 0, count: 0 }
    property string provider

    Rectangle {
        id: header

        width: parent.width
        height: placeTitleArea.height
        color: dropShadow? "#E5E6E8" : "#f0f1f2"

        clip: true

        Rectangle {
            width : parent.width
            height: placeTitleArea.height
            color: dropShadow? "#E5E6E8" : "#f0f1f2"

            Column {
                id: headerColumn

                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.leftMargin: 15
                anchors.rightMargin: 15
                anchors.topMargin: 18
                spacing: 2

                Row {
                    spacing: 2
                    width:  parent.width
                    height: placeTitle.paintedHeight
                    MapsText {
                        id: placeTitle
                        width: parent.width - placeDistance.paintedWidth
                        state: "normalWrap"
                        height: placeTitle.paintedHeight
                        platformStyle: LabelStyle { fontPixelSize: 26 }
                        font.weight: Font.Bold
                    }
                    MapsText {
                        id: placeDistance
                        anchors.top: parent.top
                        anchors.topMargin: 5
                        //width: 100
                        height: 28
                        platformStyle: LabelStyle { textColor: placeCategory.secondaryColor; fontPixelSize: 18 }
                        font.weight: Font.Light
                    }
                }
                MapsText {
                    id: placeAddress
                    width: parent.width
                    state: "normalWrap"
                    height: paintedHeight
                    wrapMode: Text.WrapAnywhere
                    platformStyle: LabelStyle { textColor: placeAddress.secondaryColor }
                }

                Rectangle {
                    color: "#f0f1f2"
                    height: 1
                    width: 1
                }

                MapsText {
                    id: placeCategory
                    height: !!placeCategory.text ? 24 : 0
                    platformStyle: LabelStyle { textColor: placeCategory.secondaryColor; fontPixelSize: 20 }
                }

                Item {
                    id: ratingsElement
                    height: (placeProvider && placeProvider.text != "") || ((placeRating && placeRating.count) && (placeRating.count != 0)) ? 25 : 0
                    width: parent.width
                    Rating {
                        id: stars
                        count: (placeRating && placeRating.count) || 0
                        value: (placeRating && placeRating.value) || 0
                        visible: (placeRating && placeRating.count > 0) || false
                    }
                    MapsText {
                        id: placeProvider
                        anchors { top: stars.top; left: stars.right; leftMargin: stars.visible ? 10 : 0}
                        objectName: "providerSourceText"
                        text: provider.length > 0 ? qsTranslate("Maps", "qtn_placesconnector_source") + ": " + provider : ""
                        visible: !!text
                        width: parent.width
                        height: ratingsElement.height
                        platformStyle: LabelStyle { textColor: placeProvider.secondaryColor; fontPixelSize: 20; }
                    }
                }
            }
        }
    }

    Rectangle {
        id: gradient
        visible: dropShadow
        anchors.top: titleArea.bottom

        height: 4
        width: parent.width
        gradient: Gradient {
            GradientStop { position: 0.0; color: "black" }
            GradientStop { position: 1.0; color: "transparent" }
        }
        opacity: 0.5
        z: 1
    }

}
