import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0

import "../PlaceViewerLogic.js" as Logic
import "../PlaceHelper.js" as PlaceHelper

SelectionDialog {
    id: root
    titleText: Logic.getPlaceTitle(placeObject)

    property variant placeObject

    property string dataPhoneNumber: Logic.getPlacePhone(placeObject);
    property string dataEmail: Logic.getPlaceEmail(placeObject);
    property string dataWebsite: Logic.getPlaceWebsite(placeObject);

    property int _phoneIndex: 0
    property int _emailIndex: 1
    property int _webIndex: 2

    anchors {
        left: parent.left
        right: parent.right
    }

    platformStyle: SelectionDialogStyle {
        itemBackground: ""
        //itemBackgroundColor: "#00000000"
        //itemPressedBackground: "image://theme/meegotouch-panel-inverted-background-pressed"
        //itemSelectedBackground: "image://theme/meegotouch-list-fullwidth-background-selected"
    }

    onDataPhoneNumberChanged: {
        contactModel.setProperty(_phoneIndex, "title", qsTranslate("Maps", "qtn_placesconnector_viewer_contact_phone"));
        contactModel.setProperty(_phoneIndex, "info", dataPhoneNumber);
    }
    onDataEmailChanged: {
        contactModel.setProperty(_emailIndex, "title", qsTranslate("Maps", "qtn_placesconnector_viewer_contact_email"));
        contactModel.setProperty(_emailIndex, "info", dataEmail);
    }
    onDataWebsiteChanged: {
        contactModel.setProperty(_webIndex, "title", qsTranslate("Maps", "qtn_placesconnector_viewer_contact_website"));
        contactModel.setProperty(_webIndex, "info", dataWebsite);
    }

    model: ListModel {
        id: contactModel
        ListElement { title: ""; info: ""; prefix: 'tel:'; objName: 'contactMenuPhone' }
        ListElement { title: ""; info: ""; prefix: 'mailto:'; objName: 'contactMenuEmail' }
        ListElement { title: ""; info: ""; prefix: ''; }
        ListElement { title: ""; info: ""; prefix: "padding"; objName: 'contactMenuWeb'}
    }

    onAccepted: {
        var item = model.get(selectedIndex),
            value = item.info
        ;

        if (item.prefix === '') {
            value = PlaceHelper.sanitizeUri(value);
        }
        Qt.openUrlExternally(item.prefix + value);
    }

    delegate: Component {
        id: defaultDelegate

        Item {
            id: delegateItem
            objectName: "objName"
            property bool selected: index === selectedIndex;
            visible: info !== ""
            height: visible ? 70 : 0
            anchors.left: parent.left
            anchors.right: parent.right

            MouseArea {
                id: delegateMouseArea
                enabled: prefix !== "padding"
                anchors.fill: parent;
                onPressed: selectedIndex = index;
                onClicked:  accept();
            }

            Rectangle {
                id: backgroundRect
                anchors.fill: parent
                color: root.platformStyle.itemBackgroundColor? root.platformStyle.itemBackgroundColor : "transparent"
            }

            BorderImage {
                id: background
                anchors.fill: parent
                source: delegateMouseArea.pressed ? root.platformStyle.itemPressedBackground :
                        delegateItem.selected ? root.platformStyle.itemSelectedBackground :
                        root.platformStyle.itemBackground
            }

            MapsText {
                id: fieldText
                text: title
                platformStyle: LabelStyle {
                    textColor: fieldText.secondaryColor
                }
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: root.platformStyle.itemLeftMargin? root.platformStyle.itemLeftMargin : 0
                    rightMargin: root.platformStyle.itemRightMargin? root.platformStyle.itemRightMargin : 0
                    top: parent.top
                    topMargin: 4
                }
            }
            MapsText {
                id: valueText
                text: info
                platformStyle: LabelStyle {
                    fontPixelSize: 24
                    textColor: "white"
                }
                font.weight: Font.Bold
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: root.platformStyle.itemLeftMargi? root.platformStyle.itemLeftMargi : 0
                    rightMargin: root.platformStyle.itemRightMargin? root.platformStyle.itemRightMargin : 0
                    top: fieldText.bottom
                    topMargin: 8
                }
            }
        }
    }
}
