import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector.favorites 1.0

ToolBarLayout {
    id: toolBarLayout

    property variant placeObject
    property variant pageStack
    property bool openFromFavorite: false

    signal favoriteChanged(bool isFavorite);
    signal favoriteOpened()
    signal addAsPlace()
    signal addToContacts()
    signal backClicked()

    ToolIcon {
        id: iconBack
        objectName: "toolBarIconBackButton";
        iconId: "icon-m-toolbar-back";
        onClicked: {
            pageStack.pop();
            backClicked();
        }
    }

    ButtonRow {
        height: parent.height
        width: parent.width - iconBack.width

        Rectangle {
            height: parent.height
            width: parent.width / 4
            color: "transparent"

            ToolIcon {
                id: shareButton

                objectName: "toolBarIconShareButton"
                iconId: "icon-m-toolbar-share"
                anchors.centerIn: parent
                enabled: {
                    if (placeObject) {
                        if (!placeObject.placeId && !placeObject.properties.placesId) {
                            var mapData = mainscreen.pluginMapData.split(".");
                            var isChina = (mapData[3].length === 2);

                            return isChina? false : true;
                        } else {
                            return true;
                        }
                    } else {
                        return false;
                    }
                }
                opacity: enabled? 1 : 0.4

                onClicked: {
                    if (typeof (analyticsComponent) != "undefined") {
                        analyticsComponent.logUUIDOther("POIClick", { Selection: "Share" } );
                    }
                    var url, title, description, image;
                    title = placeObject.properties.title;
                    description = "";
                    if (placeObject.isFavorite) {
                        title = placeObject.favorite.text;
                    }
                    if (placeObject.placeId) {
                        url = shareBaseURI + "/p?pid=" + placeObject.placeId;
                    } else if (placeObject.properties && placeObject.properties.geoLatitude && placeObject.properties.geoLongitude) {
                        url = shareBaseURI + "/?c=%1,%2".arg(placeObject.properties.geoLatitude).arg(placeObject.properties.geoLongitude);
                    }
                    appState.startShareUi(title, description, url);
                }
            }
        }

        Rectangle {
            height: parent.height
            width: parent.width / 4
            color: "transparent"

            ToolIcon {
                property Component objectFactory
                property bool isFav: (placeObject && placeObject.isFavorite)? true : false
                // This strange notation is used to immediately change favorite icon - nothing more :)
                property bool iconFav: !!isFav
                onIsFavChanged: iconFav = !!isFav;

                anchors.centerIn: parent

                enabled: (placeObject || view.object)? true : false
                opacity: enabled? 1 : 0.4

                // syncShareAvailable is "magical" global :>
                visible: syncShareAvailable
                objectName: openFromFavorite ? "toolBarIconEdit" : "toolBarIconFavoriteButton"
                iconId: openFromFavorite ? "" : iconFav ? "icon-m-toolbar-favorite-mark" : "icon-m-toolbar-favorite-unmark"
                iconSource: openFromFavorite ? favoritesAssetsPath + "images/layout/edit.png" : ""
                onClicked: {
                    if (openFromFavorite) {
                        favoriteOpened();
                    } else {
                        if (typeof (analyticsComponent) != "undefined") {
                            analyticsComponent.logUUIDOther("POIClick", { Selection: "FavouriteToggle" } );
                        }
                        iconFav = !iconFav;
                        if (placeObject.isFavorite) {
                            placeObject.removeFromStore();
                        } else {
                            placeObject.saveInStore();
                        }
                        favoriteChanged(isFav);
                    }
                }
            }
        }

        Rectangle {
            height: parent.height
            width: parent.width / 4
            color: "transparent"

            ToolIcon {
                anchors.centerIn: parent
                objectName: "toolBarIconNavigate";
                iconSource: placesAssetsPath + "images/layout/icon_directions.png";
                onClicked: {
                    if (typeof (analyticsComponent) != "undefined"){
                        analyticsComponent.logUUIDOther("POIClick", { Selection: "NavigateTo" } );
                    }

                    navigateTo( view.object );
                }
            }
        }

        Rectangle {
            height: parent.height
            width: parent.width / 4
            color: "transparent"

            ToolIcon {
                anchors.centerIn: parent
                objectName: "toolBarIconMore";
                iconId: "icon-m-toolbar-view-menu"
                onClicked: {
                    var visibleBigOne = false;
                    if (view.object) {
                        if (view.object.uri) {
                            if (view.object.categories && view.object.categories[0] && view.object.categories[0].id == 284) {
                                visibleBigOne = true;
                            } else {
                                visibleBigOne = false;
                            }
                        } else {
                            if (view.object.categories && view.object.categories.name === "public-transport") {
                                visibleBigOne = false;
                            } else {
                                visibleBigOne = true;
                            }
                        }
                    } else {
                        visibleBigOne = false;
                    }

                    if (visibleBigOne) {
                        var countryCode = placeObject && placeObject.properties && placeObject.properties.addrCountryCode?
                            placeObject.properties.addrCountryCode
                            :
                            null
                        ;

                        if (countryCode) {
                            var tmpBlackList = mainscreen.createPlaceCountriesBlackList;
                            if (tmpBlackList && tmpBlackList.length) {
                                visibleBigOne = tmpBlackList.indexOf(countryCode) == -1;
                            }
                        }
                    }


                    if (false && visibleBigOne) {
                        addMenuBig.open();
                    } else {
                        addMenuSmall.open();
                    }
                }
            }
        }
    }


    Menu {
        id: addMenuBig
        visualParent: mainscreen
        parent: toolBarLayout.parent
        MenuLayout {
            MenuItem {
                text: qsTranslate("Maps", "qtn_placesconnector_add_a_place")
                onClicked: {
                    addAsPlace();
                }
            }
            MenuItem {
                text: qsTranslate("Maps", "qtn_placesconnector_add_to_contacts")
                onClicked: {
                    addToContacts();
                }
            }
        }
    }

    Menu {
        id: addMenuSmall
        visualParent: mainscreen
        parent: toolBarLayout.parent
        MenuLayout {
            MenuItem {
                text: qsTranslate("Maps", "qtn_placesconnector_add_to_contacts")
                onClicked: {
                    addToContacts();
                }
            }
        }
    }
}
