import QtQuick 1.0

MouseArea {
    id: self

    anchors.left: parent.left
    anchors.right: parent.right

    property bool hasLeft: false
    property bool hasRight: false

    signal movedLeft()
    signal movedRight()

    Item {
        id: internals

        property int initialContainerAreaX
        property int initialMouseX
        property bool mouseDownMode: false

        property bool possibleMouseDownMode: false
        property int possibleInitialMouseX
        property int possibleInitialMouseY

        property int _MAXIMAL_DELTA_X_TO_START_SWIPING_: 20
        property int _MAXIMAL_DELTA_Y_TO_START_SWIPING_: 20

        function doStartSwiping() {
            internals.mouseDownMode = true;
            placeDetailsFlickable.interactive = false;

            internals.initialContainerAreaX = containerArea.x;
            var obj = internals.mapToItem(mainscreen, mouseX, mouseY);
            internals.initialMouseX = obj.x;

            internals.possibleMouseDownMode = false;
        }
    }

    onPressed: {
        var obj = internals.mapToItem(mainscreen, mouseX, mouseY);

        if (obj.y + placeDetailsFlickable.contentY <= titleArea.height) {
            internals.possibleMouseDownMode = true;

            internals.possibleInitialMouseX = obj.x;
            internals.possibleInitialMouseY = obj.y;

            placeDetailsFlickable.interactive = false;
        } else {
            placeDetailsFlickable.interactive = true;
        }
    }

    onMousePositionChanged: {
        if (internals.mouseDownMode) {
            var obj = internals.mapToItem(mainscreen, mouseX, mouseY);
            var delta = obj.x - internals.initialMouseX;
            if (Math.abs(delta) > 10 && Math.abs(delta) < containerArea.width) {
                if (delta > 0 && hasRight || delta < 0 && hasLeft) {
                    containerArea.x = delta;
                }
            }
        } else if (internals.possibleMouseDownMode) {
            var obj = internals.mapToItem(mainscreen, mouseX, mouseY);

            if (
                Math.abs(obj.x - internals.possibleInitialMouseX) > internals._MAXIMAL_DELTA_X_TO_START_SWIPING_ &&
                Math.abs(obj.y - internals.possibleInitialMouseY) < internals._MAXIMAL_DELTA_Y_TO_START_SWIPING_
            ) {
                internals.doStartSwiping();
            } else if (
                Math.abs(obj.x - internals.possibleInitialMouseX) <= internals._MAXIMAL_DELTA_X_TO_START_SWIPING_ &&
                Math.abs(obj.y - internals.possibleInitialMouseY) > internals._MAXIMAL_DELTA_Y_TO_START_SWIPING_
            ) {
                placeDetailsFlickable.interactive = true;

                internals.possibleMouseDownMode = false;
            }
        }
    }
    onReleased: {
        var obj = internals.mapToItem(mainscreen, mouseX, mouseY);
        var delta = obj.x - internals.initialMouseX;

        if (internals.mouseDownMode) {
            if (Math.abs(delta) > containerArea.width / 4) {
                if (delta > 0 && hasLeft) {
                    smoothAnimation.go(1, containerArea.width);
                } else if (delta < 0 && hasRight) {
                    smoothAnimation.go(-1, containerArea.width);
                }
            } else {
                containerArea.x = internals.initialContainerAreaX;
            }
        } else {
            containerArea.x = internals.initialContainerAreaX;
        }

        internals.mouseDownMode = false;
        internals.possibleMouseDownMode = false;
        internals.possibleInitialMouseX = -1;
        internals.possibleInitialMouseY = -1;

        placeDetailsFlickable.interactive = false;
    }

    Component.onCompleted: {
        self.anchors.top = parent.top;
        self.anchors.bottom = finalSpacerInTheList.top;
    }

    Timer {
        id: smoothAnimation

        interval: 10
        repeat: true

        property int step: 10
        property int direction
        property int destination
        property int loop: 0

        function go(direction, destination) {
            smoothAnimation.loop = 0;
            smoothAnimation.direction = direction;
            smoothAnimation.destination = destination;
            smoothAnimation.start();
        }

        onTriggered: {
            if (Math.abs(containerArea.x) < Math.abs(destination)) {
                var nextStep = step + (++loop * 10) * direction;
                nextStep = (Math.abs(containerArea.x + nextStep) > Math.abs(destination))? direction * (Math.abs(destination) - Math.abs(containerArea.x)) : nextStep;
                containerArea.x += nextStep;
            } else {
                smoothAnimation.stop();

                containerArea.x = 0;

                if (direction > 0) {
                    movedLeft();
                } else {
                    movedRight();
                }
            }
        }
    }
}
