import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0

import "../PublicTransport.js" as PublicTransport
import ".."

Column {
    id: ptDetails
    property variant object

    spacing: 20

    Item {
        anchors.left: parent.left; anchors.right: parent.right
        height: linesTitle.height - parent.spacing
        PlaceCommonListHeader {
            id: linesTitle
            visible: !!lines.model && lines.model.length > 0
            title: qsTranslate("Maps", "qtn_placesconnector_viewer_lines")    // Lines
        }
    }

    Repeater {
        id: lines

        delegate: Item {
            width: ptDetails.width
            height: 50
            anchors {left: parent.left; leftMargin: 20; right: parent.right; rightMargin: 20;}
            Image {
                id: icon
                anchors { left: parent.left; verticalCenter: parent.verticalCenter }
                source: modelData.icon || PublicTransport.getPtCategoryIconFromTransportLine(modelData, "list")
            }

            Rectangle {
                id: colorRect

                anchors {
                    top: parent.top;
                    topMargin: 12;
                    bottom: parent.bottom;
                    bottomMargin: 12;
                    left: icon.right;
                    leftMargin: visible ? 15 : 0;
                }
                color: modelData.color || "#000000"
                visible: color != "#000000"
                width: visible ? height : 0
            }

            MapsText {
                anchors {
                    left: colorRect.right;
                    leftMargin: 15;
                    right: parent.right;
                    verticalCenter: parent.verticalCenter
                }
                text: modelData.officialName || modelData.inofficialName || modelData.shortName || ""
                platformStyle: LabelStyle { fontPixelSize: 20 }
                font.bold: Font.Bold
            }
        }
    }

    Item {
        anchors.left: parent.left; anchors.right: parent.right
        height: serviceTitle.height - parent.spacing
        PlaceCommonListHeader {
            id: serviceTitle
            visible: !!((services.model && services.model.length > 0) || holidayHours.count > 0 || operatingHours.count > 0)
            title: qsTranslate("Maps", "qtn_placesconnector_viewer_services") //"Services";
        }
    }

    Column {
        anchors { left: parent.left; right: parent.right; leftMargin: 20; rightMargin: 20; }

        MapsText {
            text: qsTranslate("Maps", "qtn_placesconnector_viewer_operating_hours") //"Operating Hours";
            visible: !!operatingHours.model && operatingHours.count > 0
        }
        Repeater {
            id: operatingHours
            delegate: hoursDelegate
        }
    }


    Column {
        anchors { left: parent.left; right: parent.right; leftMargin: 20; rightMargin: 20; }

        MapsText {
            text: qsTranslate("Maps", "qtn_placesconnector_viewer_holiday") // "Holiday operating hours"
            visible: !!holidayHours.model && holidayHours.count > 0
        }

        Repeater {
            id: holidayHours
            delegate: hoursDelegate
        }
    }

    Flow {
        anchors {left: parent.left; leftMargin: 20; right: parent.right; rightMargin: 20;}
        spacing: 5

        Repeater {
            id: services
            delegate: Image {
                width: 44
                height: 44
                source: modelData
            }
        }
    }

    Item {
        anchors.left: parent.left; anchors.right: parent.right
        height: operatorTitle.height - parent.spacing
        PlaceCommonListHeader {
            id: operatorTitle
            visible: !!operators.model && operators.model.length > 0
            title: qsTranslate("Maps", "qtn_placesconnector_viewer_operators") //"Operators"
        }
    }

    Repeater {
        id: operators
        delegate: MapsText {
            anchors {left: parent.left; leftMargin: 20; right: parent.right; rightMargin: 20;}
            text: modelData/*.companyOfficialName*/
        }
    }

    Component {
        id: hoursDelegate
        MapsText {
            text: {
                if(modelData.dayOfWeekStartId === 100 || modelData.dayOfWeekStartId === -1) {
                    return "%1 - %2".arg(modelData.openFromText).arg(modelData.openToText);
                } else {
                    var dayStart = PublicTransport.getDayOfWeekText(modelData.dayOfWeekStartId);
                    var dayEnd = PublicTransport.getDayOfWeekText(modelData.dayOfWeekEndId);
                    if(dayStart === dayEnd) {
                        return "%1: %2 - %3".arg(
                                    dayStart).arg(
                                    modelData.openFromText).arg(
                                    modelData.openToText);
                    }

                    return "%1 - %2: %3 - %4".arg(
                                dayStart).arg(
                                dayEnd).arg(
                                modelData.openFromText).arg(
                                modelData.openToText);
                }
            }
        }
    }

    onObjectChanged: {
        if (object && object.publictransport) {
            lines.model = object.publictransport.lines;
            services.model = object.publictransport.services;
            operators.model = PublicTransport.parseSystems(object.publictransport.systems);
            var operatingHoursInfo = PublicTransport.parseOperatingHours(object.publictransport.operatingHours);
            operatingHours.model = operatingHoursInfo.regularHours;
            holidayHours.model = operatingHoursInfo.holidayHours;
            //print(JSON.stringify(object.publictransport))
        }
    }


}
