import QtQuick 1.0

import com.nokia.meego 1.0

TextField {
    id: searchInput

    signal clicked()
    signal keyPressed(variant event)
    signal clearedText()
    signal searchCanceled()
    signal lensClicked()

    property bool searching: false

    inputMethodHints: Qt.ImhNoPredictiveText

    anchors.left: parent.left
    anchors.right: parent.right
    anchors.verticalCenter: parent.verticalCenter
    platformStyle: TextFieldStyle { paddingRight: supplement.width + 24 }
    platformSipAttributes: SipAttributes {
        actionKeyLabel: new String(text).trim() == ""? qsTranslate("Maps", "qtn_placesconnector_picker_watermark_done") : qsTranslate("Maps", "qtn_placesconnector_picker_watermark");
        actionKeyHighlighted: true
    }

    Keys.onPressed: {
        log.trace("a key was pressed...");
        event.accepted = true;
        keyPressed(event);
    }

    Item {
        id: supplement
        width: clearButton.width > loading.width ? clearButton.width : loading.width
        height: parent.height
        anchors.right: parent.right
        anchors.verticalCenter: parent.verticalCenter
        anchors.rightMargin: 15
        Image {
            anchors.centerIn: parent
            id: clearButton
            visible: !searching && text != ""
            source: "image://theme/icon-m-input-clear"
        }

        BusyIndicator {
            id: loading
            visible: searching && text != ""
            anchors.centerIn: parent
            running: true
            platformStyle: BusyIndicatorStyle {size: "small"}
        }

        Image {
            id: searchIcon
            anchors.centerIn: parent
            visible: text === ""
            source: "image://theme/icon-m-common-search"
        }

        MouseArea {
            anchors.fill: parent
            onClicked: { directlyTryItemClick(); }
        }
    }

    function directlyTryItemClick() {
        if (clearButton.visible) {
            log.trace("Clear button visible");

            clearedText();
        } else if (loading.visible) {
            log.info("Search has been cancelled!");

            searchCanceled();
        } else {
            if (searchInput.activeFocus) {
                log.info("Lens clicked...")

                lensClicked();
            } else {
                searchInput.forceActiveFocus();
            }
        }
    }

    onActiveFocusChanged: { activeFocus && clicked(); }
}
