import QtQuick 1.0

import com.nokia.meego 1.0

import "../PlacesCloudAdapter.js" as CloudAdapter

Item {
    height: 135
    width: parent.width
    property QtObject imagesModel

    property Item mainPage

    visible: imagesList.count > 0 

    function selectedItem(index) {
        previewImage.imageIndex = index;
        previewImage.imageUrl = galleryModel.get(index).URL;
        previewImage.provider = galleryModel.get(index).provider || "";
        previewImage.adContent = !!galleryModel.get(index).adContent;
        previewImage.opacity = 1;
    }

    PreviewImage {
        id: previewImage
        parent: mainPage
        anchors.top: parent.top
        anchors.left: parent.left
        width: mainPage.width
        height: mainPage.height + mainPage.tools.height
        opacity: 0

        listModel: imagesModel
    }

    ListView {
        id: imagesList
        anchors { fill: parent; topMargin: 10; bottomMargin: 10; }

        model: imagesModel
        //snapMode:"SnapToItem"
        spacing: 1
        boundsBehavior: Flickable.DragOverBounds
        clip: true


        orientation: "Horizontal"
        delegate:  Item {
            width:image.width?image.width:150
            height:115
            Text {
                id: loadingText
                objectName: "thumbnailImageCaption"
                font { bold:true;  family: mainscreen.__labelStyle.fontFamily; pixelSize: 18 }
                anchors.centerIn: parent
                width: parent.width - 20
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                text: qsTranslate("Maps", "qtn_placesconnector_thumbnail_loadingerror");//"Loading\nimage...":"Error\nloading\nimage"
                visible: image.status == Image.Error
            }

            BusyIndicator {
                id: indicator
                running: (image.status == Image.Ready || image.status == Image.Error)? false : true
                visible: !!running
                anchors.centerIn: parent
            }

            Image {
                id: image

                source: unescape(URL)
                fillMode: Image.PreserveAspectFit
                smooth: true
                clip: true
                opacity:0

                sourceSize.height: 115

                Behavior on opacity {
                    NumberAnimation { }
                }

                states: State {
                    when: image.status == Image.Ready
                    PropertyChanges {
                        target: image
                        opacity: 1.0
                    }
                    PropertyChanges{
                        target: loadingText
                        visible:false
                    }
                }

            }
            MouseArea{
                anchors.fill:parent
                onClicked: selectedItem(index);
            }
        }
    }
}
