import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector.navigation 1.0

RoutePlanner {
    id: directions

    anchors.fill: parent

    mainScreen: mainscreen

    property variant guidanceRoute

    property bool routeToFlow: false
    property bool triggerPostRemoval: typeof(value) == "object"? value : false

    property bool ready: false

    function navigateTo(position, transportationMode) {
        skipToPreview(routeTo(position, transportationMode));
        log.trace("--- transportation mode: " + transportationMode);
    }

    function routeTo(position, transportationMode) {
        pageStack.pop(directions);

        directions.lastSelectedState = "setDestination";
        directions.showInfoBubbleFromDestination = true;
        directions.defaultCurrentPositionIndex = 0;

        if (directions.guidanceRoute) {
            directions.guidanceRoute.waypoints = [null, position];
        }

        var route = newRoute.createObject(directions);
        route.waypoints = [null, position];
        route.autoMode = !transportationMode;
        route.mode = transportationMode || "";
        directions.model = route;
        directions.routeToFlow = true;

        return route;
    }

    function formatAddress(properties) {
        if (!properties) return "";
        var line = (properties.addrStreetName || "") + " "  + (properties.addrHouseNumber || "");
        var line2 = (properties.addrPostalCode || "") + " " + (properties.addrCityName || "");
        if (line != " " && line2 != " ") {
            line += ", " + line2;
        }
        if (line == " ") line = line2;
        return line;
    }

    function _handlePostRemovals() {
        var result = false;
        if (triggerPostRemoval) {
            __map.removeWaypointIcons();
            __map.removeRouteFromMap();

            directions.model.__reset(true);
            //directions.model = newRoute.createObject(directions);

            triggerPostRemoval = false;

            result = true;
        }
        return result;
    }

    function handleStatusChanged(status) {
        var wps = directions.model.waypoints || [];
        if (status == PageStatus.Active) {
            __map.setRouteModeMapScheme();

            if (!routeToFlow && __map.model.selectedObject) {
                __map.mapObjectsSelected(__map.model.selectedObject);
            } else if (!routeToFlow && __map.model.pureSelectedObject) {
                __map.mapObjectsSelected(__map.model.pureSelectedObject);
            } else if (routeToFlow && wps[1]) {
                __map.currentlySelectedIconB = __map.model.getObjectByPosition({
                    latitude: wps[1].latitude || wps[1].properties.geoLatitude,
                    longitude: wps[1].longitude || wps[1].properties.geoLongitude
                });
                if (__map.currentlySelectedIconB) {
                    __map.currentlySelectedIconB.visible = false;
                }
                __map.setControl("emit-force-close-infobubble");
                __map.unhideWaypointIcons(null);
            } else {
                if (__map.currentlySelectedIconA) {
                    __map.currentlySelectedIconAInitialVisibility = __map.currentlySelectedIconA.visible;
                } else {
                    __map.currentlySelectedIconAInitialVisibility = false;
                }
                if (__map.currentlySelectedIconB) {
                    __map.currentlySelectedIconBInitialVisibility = __map.currentlySelectedIconB.visible;
                } else {
                    __map.currentlySelectedIconBInitialVisibility = false;
                }

                __map.showWaypointIcons();
            }
            routeToFlow = false;

            directions.everPickedCompleted = false;
            directions.evetPickedCanceled = false;
        } else if (status == PageStatus.Inactive) {
            if (__map.currentlySelectedIconA) {
                __map.currentlySelectedIconA.visible = __map.currentlySelectedIconAInitialVisibility;
            }
            if (__map.currentlySelectedIconB) {
                __map.currentlySelectedIconB.visible = __map.currentlySelectedIconBInitialVisibility;
            }

            __map.removeWaypointIcons();

            if (directions.everPickedCompleted) {
                directions.__map.model.clear();
            }

            directions.everPickedCompleted = false;
            directions.evetPickedCanceled = false;
            __map.setRouteModeMapScheme("default");

            routeToFlow = false;
        } else if (status = PageStatus.Deactivating) {
            if (directions.everPickedCompleted || directions.evetPickedCanceled) {
                directions.__map.model.setObjectsOwner("__PURE_DELETE__", true);
            }
            __map.setControl("close-infobubble");
        }
    }

    Component {
        id: newRoute
        Route {
            autoMode: true
        }
    }

    Component {
        id: guidanceViewer
        GuidanceViewer {
            onFinished: {
                var completeReset = (result === "done");
                var dest = directions.model.waypoints[1];
                directions.triggerPostRemoval = true;
                pageStack.pop(directions, true);
                directions.resetStartToCurrentPosition(completeReset, dest);
            }

            onStatusChanged: {
                if (status == PageStatus.Activating) {
                    appState.inGuidanceMode = true;
                } else if(status === PageStatus.Deactivating) {
                    appState.inGuidanceMode = false;
                }
            }
        }
    }

    Timer {
        id: lazyPopper
        interval: 750
        onTriggered: {
            triggerPostRemoval = true;
            pageStack.pop(directions, false);
        }
    }

    QueryDialog {
        id: loadingDriveDialog

        function openWithTimeout() {
            loadingDriveDialog.open();
            hideLoadingDialogTimer.start();
        }

        titleText: " "
        message: qsTranslate("Maps", "qtn_maps_directions_starting_drive_navigation") // Starting Drive Navigation...
        buttons: Row {}

        Timer {
            id: hideLoadingDialogTimer
            interval: 10000
            running: false

            onTriggered: {
                loadingDriveDialog.close()
            }
        }
    }

    onStatusChanged: {
        if (!directions.previewJumpToMode) {
            handleStatusChanged(status);
        }

        if (status == PageStatus.Active) {
            var preventRouteReCreation = _handlePostRemovals();

            directions.previewJumpToMode = false;
            if (directions.guidanceRoute) {
                /*
                var layer = _createPluginObject("MapLayer");
                var poly = _createPluginObject("MapPolyline");

                poly.points = directions.guidanceRoute.__calculationResult.routeGeometry;
                //poly.fillColor = "#ff00ff00";
                poly.lineColor = directions.guidanceRoute.color;
                poly.lineWidth = 6;
                poly.dashEnabled = true;
                poly.dashPrimaryLength = 3;
                poly.dashSecondaryLength = 3;

                layer.addMapObject(poly);

                directions.__map.addLayer(layer);
                */
                directions.model.waypoints = directions.guidanceRoute.waypoints;
                directions.__updateRoute(true);
                directions.__map.attachedToLocation = false;
                if (directions.guidanceRoute) {
                    directions.guidanceRoute.destroy();
                }
                directions.hideFlagAndShowInfoBubble(directions.model.waypoints, true);
            } else if (directions.model && directions.model.calculated && !directions.preventAutoRootCreation) {
                directions.hideFlagAndShowInfoBubble(directions.model.waypoints, true);
            }
        } else if (status === PageStatus.Deactivating) {
            directions.__map.closeInfoBubble(false);
        }
    }
    /*
    function _createPluginObject(objectType) {
      return Qt.createQmlObject("import MapsPlugin 1.0; " + objectType + "{}", directions, null);
    }
    */

    onFinished: {
        // (result, error)
        // typeof result == Route!
        if (result.mode === "drive") {
            var source = result.waypoints[0];
            var dest = result.waypoints[result.waypoints.length - 1];
            var title, address;
            if (directions.latestPickedDestinationUri) {
                title = directions.latestPickedDestinationProperties ? directions.latestPickedDestinationProperties.title : dest.name;
                address = formatAddress(directions.latestPickedDestinationProperties);
            } else {
                title = qsTranslate("Maps", "qtn_maps_directions_starting_drive_your_destination");
                address = formatAddress(dest.properties);
            }

            var uri = "geo:%1,%2?action=driveTo&destination-title=%3&destination-address=%4&destination-categoryId=%5&originLatitude=%6&originLongitude=%7".arg(
                    dest.properties ? dest.properties.geoLatitude : dest.latitude
                ).arg(
                    dest.properties ? dest.properties.geoLongitude : dest.longitude
                ).arg(
                    title
                ).arg(
                    address
                ).arg(
                    -1
                ).arg(
                    source.properties ? source.properties.geoLatitude : source.latitude
                ).arg(
                    source.properties ? source.properties.geoLongitude : source.longitude
            );

            nativeLayer.startDrive(uri);
            loadingDriveDialog.openWithTimeout();
            pageStack.pop(directions, true);
        } else {
            var guidanceViewerInstance = guidanceViewer.createObject(guidanceViewer);

            if (directions.guidanceRoute) {
                directions.guidanceRoute.destroy();
            }
            directions.guidanceRoute = newRoute.createObject(directions);
            directions.guidanceRoute.waypoints = result.waypoints;
            directions.guidanceRoute.mode = result.mode;
            guidanceViewerInstance.model = directions.guidanceRoute;

            pageStack.push(guidanceViewerInstance, null, true);
        }
    }

    Component.onCompleted: {
        model = newRoute.createObject(directions);

        ready = true;
    }
}
