import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import ovi.connector.map 1.0
import ovi.connector.places 1.0

Item {
    anchors.fill: parent

    Item {
        id: internals

        property string searchFor
    }

    function _earlyInitialize() {
        var action = appState.geoParamValue("action");
        if (action == "showOnMap" || action == "walkTo" || action == "routeTo" || action == "showPlaceDetails") {
            mainscreen.forcePreventJumpToCurrentPositionOnStartup = true;
        }
    }

    function _handleGeoParametersSignal() {
        log.trace("\n\n\nDISCONNECT FROM GEO PARAMETERS HANDLER TO AVOID MULTIPLE CALLS!!!")
        appState.geoParamsChanged.disconnect(_handleGeoParametersSignal);
        __handleGeoParameters();
        reconnectGeoParameters.start();
    }

    function __getPlaceObject() {
        spareGeoCoordinates.latitude = parseFloat(appState.geoParamValue("latitude"));
        spareGeoCoordinates.longitude = parseFloat(appState.geoParamValue("longitude"));
        // GEO setter implementation in plugin will set different (default) value for invalid latitude or longitude, so we check if value that we set are really there
        var validGeoLatLon = (spareGeoCoordinates.latitude == parseFloat(appState.geoParamValue("latitude"))) && (spareGeoCoordinates.longitude == parseFloat(appState.geoParamValue("longitude")));

        if (appState.geoParams.length && !validGeoLatLon) {
            mainscreen.errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_wrong_geo_lat_lon");
            mainscreen.errorMessage.show();

            return;
        } else {
            return {
                latitude: parseFloat(appState.geoParamValue("latitude")),
                longitude: parseFloat(appState.geoParamValue("longitude")),
                geoLatitude: parseFloat(appState.geoParamValue("latitude")),
                geoLongitude: parseFloat(appState.geoParamValue("longitude"))
            };
        }
    }

    function __handlePlaceIdCase(position) {
        var action = appState.geoParamValue("action");
        var placeId = appState.geoParamValue("placeId");
        var breakAfterCall = false;

        if (placeId) {

            place.uri = "0";
            place.uri = "1";
            place.properties = { placesId: placeId };
            place.read({loadDetails:true});
            waiterForPlaceToLoadTimeout.start();
            waiterForPlaceToLoadTimeout.failedMode = false;

            breakAfterCall = true;
        }

        return breakAfterCall;
    }

    function __handleSearchTermCase(position) {
        var action = appState.geoParamValue("action");
        var searchFor = appState.geoParamValue("searchTerm");
        var breakAfterCall = false;

        if (searchFor) {
            mainscreen.resetAllStacks();

            breakAfterCall = reverseGeoCodeFinder.checkForCache(searchFor);
            if (breakAfterCall) return breakAfterCall;

            if (appState.geoParams.indexOf("searchTermRespectGeo") > -1) {
                mainscreen.mapSharedData.updateCurrentCenter(position);
            }

            if (action == "showOnMap") {
                mainscreen.__placesStack.currentPage.resetSearch(searchFor);
            } else {
                internals.searchFor = searchFor;
                mainscreen.__placesStack.push(placesPicker);
            }

            breakAfterCall = true;
        }

        return breakAfterCall;
    }

    function __handlePureLatitudeLongitudeCase(position) {
        var searchFor = appState.geoParamValue("searchTerm");
        var placeId = appState.geoParamValue("placeId");
        var breakAfterCall = false;

        if (!searchFor && !placeId) {
            reverseGeoCodeFinder.find(position);

            breakAfterCall = true;
        }

        return breakAfterCall;
    }

    function __handleGeoParameters() {
        var position = __getPlaceObject();
        if (!position) { return; }

        if (__handlePureLatitudeLongitudeCase(position)) { return; }
        if (__handlePlaceIdCase(position)) { return; }
        if (__handleSearchTermCase(position)) { return; }

        __handleActions(position);
    }

    function __handleActions(position, placeId) {
        var action = appState.geoParamValue("action");

        switch (action) {
            case "walkTo":
                mainscreen.resetAllStacks();
                mainscreen.forcePrevetAutoModeInRoute = true;

                mainscreen.mapSharedData.state = "walkTo";
                mainscreen.__directionsStack.currentPage.navigateTo(position, "walk");
                mainscreen.__directionsTabBtn.clicked();

                break;
            case "driveTo":
                mainscreen.resetAllStacks();
                mainscreen.forcePrevetAutoModeInRoute = true;

                mainscreen.mapSharedData.state = "driveTo";
                mainscreen.__directionsStack.currentPage.navigateTo(position, "drive");
                mainscreen.__directionsTabBtn.clicked();

                break;
            case "routeTo":
                mainscreen.resetAllStacks();
                mainscreen.forcePrevetAutoModeInRoute = true;

                mainscreen.mapSharedData.state = "routeTo";
                mainscreen.__directionsStack.currentPage.routeTo(position);
                mainscreen.__directionsTabBtn.clicked();

                break;
            case "showOnMap":
                mainscreen.resetAllStacks();
                mainscreen.mapSharedData.state = "showOnMap";

                var extraParameters = {
                    latitude: position.latitude,
                    longitude: position.longitude,
                    zoomLevel: appState.geoParamValue("zoomLevel")? appState.geoParamValue("zoomLevel") : 14,
                    zoomScale: appState.geoParamValue("zoomScale"),
                    title: position.title,
                    address: position.address,
                    placeId: position.placeId,
                    createIcon: appState.geoParamValue("skipIconCreation") == "true"? false : true,
                    skipInfoBubble: appState.geoParamValue("skipInfoBubble") == "true"? true : false
                };
                mainscreen.__placesStack.currentPage.showOnMap(position, extraParameters);

                break;
            case "showPlaceDetails":
                mainscreen.resetAllStacks();

                var extraParameters = {
                    latitude: position.latitude,
                    longitude: position.longitude,
                    zoomLevel: appState.geoParamValue("zoomLevel")? appState.geoParamValue("zoomLevel") : 14,
                    zoomScale: appState.geoParamValue("zoomScale"),
                    title: position.title,
                    address: position.address,
                    placeId: position.placeId,
                    createIcon: true,
                    instantMove: true
                };
                mainscreen.__placesStack.currentPage.showOnMap(position, extraParameters);

                waitForSomeTimeToStopIconCreationBeforePlaceDetailsContainer.placeId = placeId;
                waitForSomeTimeToStopIconCreationBeforePlaceDetailsContainer.position = position;
                waitForSomeTimeToStopIconCreationBeforePlaceDetailsContainer.start();

                break;
            default:
                if (action) {
                    mainscreen.errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_wrong_action");
                    mainscreen.errorMessage.show();
                }

                break;
        }
    }

    Place {
        id: place
        uri: "1"

        onReadFinished: {
            if (place && place.details) {
                if (!waiterForPlaceToLoadTimeout.failedMode) {
                    waiterForPlaceToLoadTimeout.stop();

                    __handleActions(
                        {
                            geoLatitude: parseFloat(place.details.location.geoCoordinates.latitude),
                            geoLongitude: parseFloat(place.details.location.geoCoordinates.longitude),
                            latitude: parseFloat(place.details.location.geoCoordinates.latitude),
                            longitude: parseFloat(place.details.location.geoCoordinates.longitude),
                            title: place.details.names.defaultName.name,
                            address: placeHelper.formatAddress(place.details.location.address),
                            placeId: place.details.a_id
                        },
                        place.details.a_id
                    );
                }
            }
        }
    }

    Component {
        id: placesPicker

        PlacesPicker {
            id: picker

            triggerSearch: true
            disableLongPress: true
            disablePublicTransport: true

            prefilledString: internals.searchFor

            property bool justPicked: false

            tools: ToolBarLayout {
                ToolIcon {
                    iconId: "icon-m-toolbar-back"
                    objectName: "backTabBtn"

                    onClicked: { mainscreen.__placesStack.pop() }
                }
            }

            onPicked: {
                justPicked = true;

                var location = {
                    geoLatitude: parseFloat(picker.pickedObject.properties.geoLatitude),
                    geoLongitude: parseFloat(picker.pickedObject.properties.geoLongitude),
                    latitude: parseFloat(picker.pickedObject.properties.geoLatitude),
                    longitude: parseFloat(picker.pickedObject.properties.geoLongitude),
                    title: picker.pickedObject.properties.title,
                    address: placeHelper.formatAddress(picker.pickedObject.properties),
                    placeId: picker.pickedObject.properties.placesId
                }

                mainscreen.__placesStack.pop();
                picker.__map.model.clear();

                __handleActions(location, picker.pickedObject.properties.placesId);
            }

            onStatusChanged: {
                if (status == PageStatus.Inactive) {
                    if (!justPicked) {
                        picker.__map.model.clear();

                        justPicked = false;
                    }
                }
            }
        }
    }

    PlaceHelper {
        id: placeHelper
    }

    GeoCoordinates { id: spareGeoCoordinates }

    ReverseGeoCodeFinder {
        id: reverseGeoCodeFinder

        mapView: mainscreen.mapSharedData.activeMap

        onFindPureDataComplete: {
            var data = contextData;
            var properties = {
                title: data["ADDR_STREET_NAME"] || data["ADDR_CITY_NAME"] || data["ADDR_COUTRY_NAME"],
                latitude: spareGeoCoordinates.latitude,
                longitude: spareGeoCoordinates.longitude,
                geoLatitude: spareGeoCoordinates.latitude,
                geoLongitude: spareGeoCoordinates.longitude,
                addrCityName: data["ADDR_CITY_NAME"],
                addrCountryCode: data["ADDR_COUNTRY_CODE"],
                addrCountryName: data["ADDR_COUNTRY_NAME"],
                addrDistrictName: data["ADDR_DISTRICT_NAME"],
                addrHouseNumber: data["ADDR_HOUSE_NUMBER"],
                addrPostalCode: data["ADDR_POSTAL_CODE"],
                addrStreetName: data["ADDR_STREET_NAME"],
                placeId: -1
            };
            properties.address = placeHelper.formatAddress(properties);

            __handleActions(properties);
        }
    }

    Timer {
        id: waiterForPlaceToLoadTimeout
        property bool failedMode: false
        interval: 10 * 1000
        onTriggered: {
            waiterForPlaceToLoadTimeout.failedMode = true;
            if (mainscreen.onlineMode) {
                mainscreen.errorMessage.text = qsTranslate("Maps", "qtn_placesconnector_wrong_place");
            } else {
                mainscreen.errorMessage.text = qsTranslate("Maps", "qtn_maps_nearby_offline_recs");
            }

            mainscreen.errorMessage.show();
        }
    }

    Timer {
        id: reconnectGeoParameters
        interval: 1000
        onTriggered: {
            log.trace("\n\n\nRECONNECT AGAIN TO GEO PARAMETERS HANDLER!!!")
            appState.geoParamsChanged.connect(_handleGeoParametersSignal);
        }
    }

    Timer {
        id: waitForAllPagesToLoad

        interval: 1000 * 3
        repeat: true

        onTriggered: {
            if (
                mainscreen.__placesStack.currentPage &&
                mainscreen.__nearbyStack.currentPage &&
                mainscreen.__directionsStack.currentPage &&
                mainscreen.__moreStack.currentPage &&
                mainscreen.placesInstance &&
                mainscreen.__download_manager_allow_go_online_set__
            ) {
                waitForAllPagesToLoad.stop();

                appState.geoParamsChanged.connect(_handleGeoParametersSignal);
                __handleGeoParameters();
            } else {
            }
        }
    }

    Timer {
        id: waitForSomeTimeToStopIconCreationBeforePlaceDetailsContainer
        interval: 500

        property variant placeId
        property variant position

        onTriggered: {
            mainscreen.__placesStack.currentPage.showPlaceDetails(placeId, position);
        }
    }

    Component.onCompleted: {
        _earlyInitialize();

        if (appState.applicationState == "normal") {
            waitForAllPagesToLoad.start();
        }
    }
}

