import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0
import ovi.connector.places 1.0
import ovi.connector.favorites 1.0

import com.nokia.odml 1.0

Rectangle {
    id: mainscreen

    color: "#FBFAF7"

    x: 0
    y: 0
    width: 854
    height: 480

    signal refreshConnectedLists()

    // workaround: adjusting Y position when starting up in portrait mode (just for initial startup)
    //property bool adjustPositionY: true;
    property bool adjustPositionY: false

    property bool appVisible: platformWindow.visible
    property bool appActive: platformWindow.active

    property int __DEFAULT_AUTO_RETRY_VALUE__: 0
    property int autoRetryHopesBeforeGiveUp: mainscreen.__DEFAULT_AUTO_RETRY_VALUE__
    property bool canWeAutoRetryOnlineMode: mainscreen.autoRetryHopesBeforeGiveUp > 0 || mainscreen.autoRetryHopesBeforeGiveUp == -1
    function autoRetryApplied() {
        if (mainscreen.autoRetryHopesBeforeGiveUp > 0) {
            mainscreen.autoRetryHopesBeforeGiveUp--;
        }
    }
    function autoReplyReset() {
        mainscreen.autoRetryHopesBeforeGiveUp = mainscreen.__DEFAULT_AUTO_RETRY_VALUE__;
    }

    property bool onlineMode: false
    property bool __download_manager_allow_go_online_set__: false
    function changeOnlineMode(onlineMode) {
        onlineMode = onlineMode === undefined? appState.isOnline : onlineMode;

        mainscreen.onlineMode = false;
        MapsPlugin.online = false;

        if (onlineMode) {
            console.log("Changing to OnlineMode - will open network session");
            appState.openSession();
        } else {
            console.log("Changing to OfflineMode");
        }

        if (!__download_manager_allow_go_online_set__) {
            downloadManager.allowGoOnline(appState.isOnline);
            __download_manager_allow_go_online_set__ = true;
        }
    }

    property variant createPlaceCountriesBlackList: [
        "ALB", "BLR", "BIH", "CYP", "SRB", "BOL", "ECU", "FLK", "AFG", "ARM", "AZE", "BGD", "BTN", "BRN", "MMR", "KHM", "CHN",
        "GEO", "IRN", "IRQ", "ISR", "JPN", "PRK", "LAO", "NPL", "PAK", "KOR", "LKA", "TJK", "TKM", "DZA", "AGO", "BEN", "BWA",
        "BFA", "BDI", "CMR", "CPV", "CAF", "TCD", "COM", "COG", "COD", "CIV", "DJI", "GNQ", "ERI", "ETH", "GAB", "GMB", "GHA",
        "GIN", "GNB", "LSO", "LBR", "LBY", "MWI", "MDV", "MLI", "MRT", "NAM", "NER", "RWA", "SEN", "SLE", "SOM", "SDN", "SWZ",
        "SYR", "TZA", "TGO", "TUN", "UGA", "ESH", "YEM", "ZMB", "ZWE"
    ]

    property bool zoombarsEnabled: appState.zoombarEnabled
    property bool enableAutomaticRotating: appState.fingerRotationEnabled
    property bool enableMagic3d2d: appState.slide3dEnabled
    property bool applicationMaximized: true

    property bool useKudzoPlacesBackend: false
    property bool acceptBoundingBoxesWhileOpeningInfoBubbles: false

    property bool enableRecommendationsBoundingBox: true
    property bool backlightsAlwaysOn: true
    property bool magicAccelerometer: appState.usingAccelerometer
    property bool magicCompass: appState.usingCompass
    property bool useMagicCompassInGuidanceInstanadOfMOS: false
    property bool useGPSTimeInsteadOfCurrentOne: true
    property bool enableCompletePositioningDebug: false
    property bool enableFeedback24HourRule: true
    property bool enableSearchDebugMessages: false
    property bool enableStaticODMLAddMode: false
    /**
      * @param string The walk guidance update mode
      *
      * position-animation - positionAnimationMode
      * roadview - roadviewAnimationMode
      */
    property string walkGuidanceUpdateMode: "position-animation"
    property bool showAllPositionMarkers: false
    property bool allowUserToDeleteAllMapData: true
    property int mapMatcherMode: 2

    property bool forcePrevetAutoModeInRoute: false
    property bool forcePreventJumpToCurrentPositionOnStartup: false

    property bool appWorkingForMoreThan15SecondsNow: false
    property bool allTabsReady: false
    property bool initialFlyFinished: false

    property variant _justblob
    property string stateRandomizer: "... :: ..."
    function generateRandomState() {
        return (Math.random() * Math.random()) + " :: " + (Math.random() * Math.random());
    }
    function applyRandomState() {
        mainscreen.stateRandomizer = mainscreen.generateRandomState();
    }

    /* Global function to help solve problems with memory handling.
     * Use it to clear ListModel and ConnectedList models
     */
    function clearModel(model) {
        for (var a = model.count - 1; a >= 0; a--) {
            var item = model.get(a);
            model.remove(a);
            item.destroy();
        }
    }
    property MapSharedData mapSharedData: MapSharedData { }
    property MapGlobalData mapGlobalData: MapGlobalData { }
    property PositionProvider positionProvider: PositionProvider { enabled: false; mapMatcherMode: mainscreen.mapMatcherMode; }

    property bool applicationJustStarted: true
    property bool mapCatalogUpdatedAtLeastOnce: false
    property string __lastTab: ""

    property string pluginMapData: MapsPlugin.mapVersion

    property bool spareMapBussy: false
    property MapViewer spareMap: __spareMap
    MapViewer {
        id: __spareMap

        dontSaveData: true

        behaviors: "_NO_LONG_PRESS_AT_ALL_:true;_NO_PUBLIC_TRANSPORT_STOPS_CLICK_:true;_NO_PUBLIC_TRANSPORT_MODE_:true;_NO_INTERACTIVITY_:true;_NO_SETTINGS_:true;_NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_:true;_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_:true;_NO_INFO_BUBBLE_CLICK_EFFECT_:true;_NO_MY_POSITION_DEFAULT_CLICK_ACTION_:true;_NO_MY_POSITION_CLICK_ACTION_AT_ALL_:true;_NO_COMPASS_ALLOWED_:true;_NO_INITIAL_SETTINGS_LOAD_:true;"
    }

    property variant placesInstance: null
    property variant placesInstance1: null
    property variant placesInstance0: null

    property variant placesCloudAdapter: PlacesCloudAdapter { }
    property variant favoritesCloudAdapter: FavoritesCloudAdapter { }
    property variant favoritesConnectedList: FavoritesConnectedList { }
    property Component modelFactory: Component { MapModel { } }
    property Component modelMarkerFactory: Component { MapModelMarker { } }
    property Component modelLayerFactory: Component { MapLayer { } }
    property Component iconFactory: Component {
        MapIcon {
            property variant uri
            property variant data
        }
    }

    property InfoBanner errorMessage: __errorMessage
    InfoBanner {
        id: __errorMessage

        timerEnabled: true
        timerShowTime: 10 * 1000

        anchors.top: parent.top
        anchors.topMargin: 89

        z: 999
    }

    property LabelStyle __labelStyle: labelStyle
    LabelStyle {
        id: labelStyle
    }

    // Page reparenting used for keeping instance in memory rather to create pages every time (PageStack.push) - Component.onCompleted
    function reparentInstance(dummy, instance) {
        instance.parent = dummy;
        instance.state = dummy.state;
        instance.visible = true;
        instance.model = dummy.model;
        instance.openFromFavorite = dummy.openFromFavorite;
        instance.recommendationsLeft = dummy.recommendationsLeft;
        instance.detailsShown = dummy.detailsShown;
        instance.mapsApp = dummy.mapsApp;
        instance.currentPlace = dummy.currentPlace;
        instance.object = dummy.object;

        dummy.tools = instance.tools;
    }
    // Page reparenting used for keeping instance in memory rather to create pages every time (PageStack.push) - onStatusChanged
    function reparentInstanceStatus(dummy, instance, connectedFunction) {
        instance.pageStack = dummy.pageStack;
        if (dummy.status == PageStatus.Activating) {
            reparentInstance(dummy, instance);
            instance.visible = true; instance.parent = dummy; instance.favoriteChanged.connect(connectedFunction);
        }
        if (dummy.status == PageStatus.Inactive) {
            instance.visible = false; instance.parent = mainscreen; instance.favoriteChanged.disconnect(connectedFunction);
        }
        instance.status = dummy.status;
    }

    function changeTab(IMEIActivity) {
        if (typeof (analyticsComponent) !== "undefined") {
            analyticsComponent.logIMEIActivity("TabClick", { type: IMEIActivity });
        }
        if (IMEIActivity === "Search") {
            if (mainscreen.__placesStack.currentPage && mainscreen.__placesStack.currentPage.ensureMapVisible) {
                mainscreen.__placesStack.currentPage.ensureMapVisible();
            }
        }
        if (__lastTab === "More") {
            moreStack.pop(null, true);
        }

        mainscreen.applyRandomState();
        __lastTab = IMEIActivity
    }

    function appVisibilityChangedHandler() {
       /*
        *   NOTES:
        *     visible == true and active == true    - Application is visible and can be interacted with.
        *     visible == true and active == false   - Application is (at least partly) visible but cannot be interacted with. Some dialog on top or application is visible in the task switcher.
        *     visible == false and active == true   - This combination is not allowed.
        *     visible == false and active == false  - Application is not visible and cannot be interacted with.
        */
        if (mainscreen.appVisible && mainscreen.appActive) {
            log.trace("Application window is Fully visible");
            appState.setDisplayUpdatesEnabled(true);
        } else if ( mainscreen.appVisible && !mainscreen.appActive) {
            log.trace("Application window is Partially visible");
            appState.setDisplayUpdatesEnabled(true);
        } else if ( !mainscreen.appVisible && !mainscreen.appActive) {
            log.trace("Application window is Not visible");
            appState.setDisplayUpdatesEnabled(false);
        }
    }

    function ___clearAllStacksAndPrepareToDie() {
        log.trace("\n\n\nCALL TO CLEAR ALL\n\n\n")
        mainscreen.__placesStack.clear();
        appState.processEvents();
        mainscreen.__nearbyStack.clear();
        appState.processEvents();
        mainscreen.__directionsStack.clear();
        appState.processEvents();
        mainscreen.__moreStack.clear();
        appState.processEvents();

        tb.visible = false;
    }

    function ___clearAllStacksAndKeepODML() {
        log.trace("\n\n\nCALL TO CLEAR ALL BUT ODML\n\n\n")
        if (placesStack.currentPage) {
            mainscreen.__placesStack.clear();
            appState.processEvents();
        }
        if (nearbyStack.currentPage) {
            mainscreen.__nearbyStack.clear();
            appState.processEvents();
        }
        if (directionsStack.currentPage) {
            mainscreen.__directionsStack.clear();
            appState.processEvents();
        }
    }

    function resetAllStacks() {
        log.trace("\n\n\nCALL TO RESET ALL\n\n\n")
        mainscreen.__placesStack.pop(mainscreen.__placesComponent, true);
        mainscreen.__nearbyStack.pop(mainscreen.nearbyComponent, true);
        mainscreen.__directionsStack.pop(mainscreen.directionsComponent, true);
        mainscreen.__moreStack.pop(mainscreen.moreComponent, true);

        mainscreen.__placesTabBtn.clicked();
        mainscreen.__placesStack.currentPage.clearModel();
    }

    function ensureAllPageStacks() {
        lazyTabLoader.index = -1;
        lazyTabLoader.start();

        lazyPlacesViewerComponentInitialization.start();
    }

    function handleMapsMainOnCompleted() {
        appVisibilityChangedHandler();
        if (mainscreen.state != "portrait") {
            mainscreen.adjustPositionY = false;
        }
        mainscreen.applicationJustStarted = false;

        if (appState.isEmbarcoVariant && !appSettings.value("isEmbarcoCustomDisclaimerAccepted")) {
            placesStack.push(fakeBackground);
            handleMapsMainEmbarcoFirstStartup();
        } else {
            handleMapsMainRegularStartup();
        }
    }

    function handleMapsMainEmbarcoFirstStartup() {
        __customDisclaimer = customDisclaimer.createObject(mainscreen);
        __customDisclaimer.status = DialogStatus.Open;
    }

    function handleMapsMainRegularStartup() {
        if (appState.applicationState == "update") {
            lazyChangeOnlineModeToAllowMeegoToLoadItself.start();

            var component = Qt.createComponent("More.qml");
            moreComponent = component.createObject(mainscreen);
            moreComponent.anchors.fill = placesComponent.parent;
            moreComponent.tools = commonTools;

            moreStack.push(moreComponent);
            moreTabBtn.clicked();
            moreStack.currentPage.handleClick("offlinemaps");
        } else if (mainscreen.enableStaticODMLAddMode && downloadManager.requests.length != 0) {
            lazyChangeOnlineModeToAllowMeegoToLoadItself.start();

            var component = Qt.createComponent("More.qml");
            moreComponent = component.createObject(mainscreen);
            moreComponent.anchors.fill = placesComponent.parent;
            moreComponent.tools = commonTools;

            moreStack.push(moreComponent);
            moreTabBtn.clicked();
            moreStack.currentPage.handleClick("offlinemaps");
        } else {
            lazyPlacesViewerComponentInitialization.start();

            placesStack.push(placesComponent);
            lazyTabLoader.start();
        }

        var posChange = mainscreen.mapSharedData.currentCenterChanged.connect(function () {
            if (mainscreen.mapSharedData.state === "afterInitialFly") {
                mainscreen.initialFlyFinished = true;
                log.trace("initial map fly finished.")
                mainscreen.mapSharedData.currentCenterChanged.disconnect(posChange);
            }
        });
    }

    // MouseArea added to make sure that you can double click on items when transition is on!
    MouseArea {
        anchors.fill: parent
        enabled: !tabGroup.currentTab.busy
    }
    MouseArea {
        anchors.top: parent.top
        anchors.left: parent.left
        height: parent.height + commonTools.height
        width: 2 * parent.width
        visible: tabGroup.currentTab.busy;
        z: 10000
    }

    ToolBar {
        id: tb
        anchors.bottom: parent.bottom

        enabled: allTabsLoadedIndicator.allTabsReady || appState.applicationState == "update"
    }

    property TabButton __placesTabBtn: placesTabBtn
    property TabButton __nearbyTabBtn: nearbyTabBtn
    property TabButton __directionsTabBtn: directionsTabBtn
    property TabButton __moreTabBtn: moreTabBtn
    property ToolBarLayout __commonTools: commonTools
    ToolBarLayout {
        id: commonTools
        ToolIcon {
            iconId: "icon-m-toolbar-back"
            objectName: "backTabBtn"
            opacity: (tabGroup.currentTab.depth && tabGroup.currentTab.depth > 1)? 1 : 0.4
            onClicked: { tabGroup.currentTab.pop() }
        }
        ButtonRow {
            TabButton {
                id: placesTabBtn
                objectName: "placesTabBtn"
                Image {
                    anchors.centerIn: parent
                    source: "../images/icon_findplaces.png"
                }

                tab: placesStack
                checked: true // akahl20110330 - please don't change me, breaks visuals with older versions of qt components but required/fixes bug in newer ones like on the device
                onClicked: changeTab("Search")
            }
            TabButton {
                id: nearbyTabBtn
                objectName: "nearbyTabBtn"
                Image {
                    anchors.centerIn: parent
                    source: "../images/icon_aroundme.png"
                }
                tab: nearbyStack
                onClicked: changeTab("Nearby")
            }
            TabButton {
                id: directionsTabBtn
                objectName: "directionsTabBtn"
                Image {
                    anchors.centerIn: parent
                    source: "../images/icon_directions.png"
                }
                tab: directionsStack
                onClicked: changeTab("Directions")
            }

            TabButton {
                id: moreTabBtn
                objectName: "moreTabBtn"
                Image {
                    anchors.centerIn: parent
                    source: "../images/icon_more.png"
                }

                tab: moreStack
                onClicked: changeTab("More")
            }
        }
    }

    /* Each tab should have its own stack so we can push scenes in A, switch to tab B, then go to last scene if switching to tab A */
    property PageStack __placesStack: placesStack
    property PageStack __nearbyStack: nearbyStack
    property PageStack __directionsStack: directionsStack
    property PageStack __moreStack: moreStack
    TabGroup {
        id: tabGroup
        anchors.top: parent.top
        anchors.bottom: tb.top
        width: parent.width

        currentTab: placesStack
        PageStack {
            id: placesStack
            anchors.top: parent.top
            toolBar: tb
        }
        PageStack {
            id: nearbyStack
            anchors.top: parent.top
            toolBar: tb
        }
        PageStack {
            id: directionsStack
            anchors.top: parent.top
            toolBar: tb
        }
        PageStack {
            id: moreStack
            anchors.top: parent.top
            toolBar: tb
        }
    }

    property Component __placesComponent: placesComponent
    property Page nearbyComponent
    property Page directionsComponent
    property Page moreComponent
    Component {
        id: placesComponent

        Places {
            anchors.fill: parent
            tools: commonTools

        }
    }

    Component {
        id: fakeBackground

        Page {
            anchors.fill: parent
            tools: commonTools

            Image {
                source: assetsPath + "images/emptyMapPattern150x150.gif"
                anchors.fill: parent

                fillMode: Image.Tile
            }

            z: 1
        }
    }

    RoundedCorners {
        anchors.centerIn: parent
        height: parent.height
        width: parent.width
        z: 1
    }

    property variant __customDisclaimer
    Component {
        id: customDisclaimer

        CustomDisclaimer {
            onAccepted: {
                __customDisclaimer.status = DialogStatus.Closed;
                __customDisclaimer.destroy();

                placesStack.pop();
                appSettings.setValue("isEmbarcoCustomDisclaimerAccepted", true);

                handleMapsMainRegularStartup();
            }

            onRejected: {
                Qt.quit();
            }
        }
    }

    states: [
        State {
            name: "landscape"
            when: main.inPortrait == false;
            PropertyChanges {
                target: mainscreen;
                width: screen.displayWidth
                height: screen.displayHeight - statusBar.height
                x: 0
                y: 0
            }
        },
        State {
            name: "portrait"
            when: main.inPortrait == true;
            PropertyChanges {
                target: mainscreen;
                width: screen.displayHeight
                height: screen.displayWidth - statusBar.height
                // For QApplication
                //x: 0
                //y: -(screen.displayWidth - screen.displayHeight)
                // For MApplication
                x: 0
                //y: 0
                y: mainscreen.adjustPositionY ? -(screen.displayWidth - screen.displayHeight) : 0
            }
        },
        State {
            name: "portraitInverted"
            when: main.state == 'PortraitInverted';
            PropertyChanges {
                target: mainscreen;
                width: screen.displayHeight
                height: screen.displayWidth - statusBar.height
                // For QApplication
                //x: (screen.displayWidth - screen.displayHeight)
                //y: 0
                // For MApplication
                x: 0
                y: 0
            }
        },
        State {
            name: "landscapeInverted"
            when: main.state == "LandscapeInverted";
            PropertyChanges {
                target: mainscreen;
                width: screen.displayWidth
                height: screen.displayHeight - statusBar.height
                x: 0
                y: 0
            }
        }
    ]

    transitions: Transition {
        from: "portrait";
        ScriptAction { script: { mainscreen.adjustPositionY = false; } }
    }

    Analytics {
        id: analyticsComponent
        agentName: "com.nokia.maps.qml"
        agentVersion: appState.mapsVersion
    }

    Connections {
        target: appState
        ignoreUnknownSignals: true

        onMinimized: { }
        onMaximized: { }
        onWindowDeactivated: {
            applicationMaximized = false;
        }
        onWindowActivated: {
            applicationMaximized = true;
        }
    }

    Connections {
        target: mainscreen
        ignoreUnknownSignals: true

        onAppVisibleChanged: {
            appVisibilityChangedHandler();
        }
        onAppActiveChanged: {
            appVisibilityChangedHandler();
        }
    }

    Component.onCompleted: {
        handleMapsMainOnCompleted();
    }

    Timer {
        id: lazyPlacesViewerComponentInitialization
        interval: 250
        repeat: true

        property int currentPass: 0

        onTriggered: {
            if (mainscreen.__placesStack.currentPage && mainscreen.__placesStack.currentPage.__map) {
                if (!mainscreen.placesInstance && currentPass == 0) {
                    appState.processEvents();
                    var component = Qt.createComponent(placesConnectorPath + "PlacesViewerComponent.qml");
                    appState.processEvents();
                    mainscreen.placesInstance = component.createObject(mainscreen);
                    appState.processEvents();
                    mainscreen.placesInstance.enableSwiping();

                    currentPass++;
                } else if (!mainscreen.placesInstance0 && currentPass == 1) {
                    appState.processEvents();
                    var component = Qt.createComponent(placesConnectorPath + "PlacesViewerComponent.qml");
                    appState.processEvents();
                    mainscreen.placesInstance0 = component.createObject(mainscreen);
                    appState.processEvents();

                    currentPass++;
                } else if (!mainscreen.placesInstance1 && currentPass == 2) {
                    appState.processEvents();
                    var component = Qt.createComponent(placesConnectorPath + "PlacesViewerComponent.qml");
                    appState.processEvents();
                    mainscreen.placesInstance1 = component.createObject(mainscreen);
                    appState.processEvents();

                    currentPass++;
                } else {
                    lazyPlacesViewerComponentInitialization.stop();
                }
            } else {
                log.info(".......(PVC) waiting for places map....")
            }
        }
    }

    Timer {
        id: lazyChangeOnlineModeToAllowMeegoToLoadItself
        interval: 25
        onTriggered: {
            changeOnlineMode();
        }
    }

    Timer {
        id: lazyTabLoader

        repeat: true
        property int index: 0

        interval: 25
        onTriggered: {
            if ((index >= 0 && mainscreen.__placesStack.currentPage && mainscreen.__placesStack.currentPage.__map) || (index === -1)) {
                if ((mainscreen.placesInstance/* && mainscreen.placesInstance0 && mainscreen.placesInstance1*/) || (index === -1)) {
                    if (index === -1) {
                        if (!placesStack.currentPage) {
                            log.info("*** [INFO] lazyTabLoader init places stack")
                            placesStack.push(placesComponent);
                        }

                        index++;
                    } else if (index === 0) {
                        if (!nearbyStack.currentPage) {
                            log.info("*** [INFO] lazyTabLoader init nearby stack")
                            appState.processEvents();
                            var component = Qt.createComponent("Nearby.qml");
                            appState.processEvents();
                            nearbyComponent = component.createObject(mainscreen);
                            appState.processEvents();
                            nearbyComponent.anchors.fill = placesComponent.parent;
                            nearbyComponent.tools = commonTools;
                            nearbyStack.push(nearbyComponent);
                            appState.processEvents();
                        }

                        index++;
                    } else if (index === 1 && nearbyStack.currentPage) {
                        if (!directionsStack.currentPage) {
                            log.info("*** [INFO] lazyTabLoader init directions stack")
                            appState.processEvents();
                            var component = Qt.createComponent("Directions.qml");
                            appState.processEvents();
                            directionsComponent = component.createObject(mainscreen);
                            appState.processEvents();
                            directionsComponent.anchors.fill = placesComponent.parent;
                            directionsComponent.tools = commonTools;
                            directionsStack.push(directionsComponent);
                            appState.processEvents();
                        }

                        index++;
                    } else if (index === 2 && directionsStack.currentPage) {
                        if (!moreStack.currentPage) {
                            log.info("*** [INFO] lazyTabLoader init more stack")
                            appState.processEvents();
                            var component = Qt.createComponent("More.qml");
                            appState.processEvents();
                            moreComponent = component.createObject(mainscreen);
                            appState.processEvents();
                            moreComponent.anchors.fill = placesComponent.parent;
                            moreComponent.tools = commonTools;
                            moreStack.push(moreComponent);
                            appState.processEvents();
                        }

                        index++;
                    } else if (index > 2) {
                        lazyTabLoader.stop();
                    } else {
                        log.info(".......(LTL) waiting for previous tab to load at stage " + index + "....")
                    }
                } else {
                    log.info(".......(LTL) waiting for PVC....")
                }
            } else {
                log.info(".......(LTL) waiting for places map....")
            }
        }
    }

    Timer {
        id: allTabsLoadedIndicator

        property bool allTabsReady: false

        running: true
        repeat: true
        interval: 250
        onTriggered: {
            if (__placesStack.currentPage && __nearbyStack.currentPage && __directionsStack.currentPage && __moreStack.currentPage) {
                allTabsLoadedIndicator.stop();

                allTabsLoadedIndicator.allTabsReady = true;
                mainscreen.allTabsReady = true;

                lazyChangeOnlineModeToAllowMeegoToLoadItself.start();
            }
        }
    }

    Timer {
        interval: 4 * 15 * 1000
        running: true
        onTriggered: { appWorkingForMoreThan15SecondsNow = true }
    }

    GeoRunnerMonitor { }
    OnlineStatusMonitor { }
    MapInstallMonitor { }

    /*
    property bool __gps: true
    Rectangle {
        width: 20
        height: 20
        color: mainscreen.__gps? "green" : "red"

        anchors.top: parent.top
        anchors.left: parent.left

        MouseArea {
            anchors.fill: parent
            onClicked: {
                mainscreen.__gps = !mainscreen.__gps;
            }
        }
    }
    */

    /*
    Rectangle {
        width: 200
        height: 100
        color: "red"

        Text {
            text: mainscreen.onlineMode
            anchors.fill: parent
            font.pixelSize: 22
            color: "black"
        }
    }
    */
}

