import QtQuick 1.0

import com.nokia.meego 1.0 as QtComponents

import MapsPlugin 1.0

import "benchmark.js" as Benchmark

Item {
    id: rootApplicationLevel

    width: screen.displayWidth
    height: screen.displayHeight

    signal signInComplete(bool success)
    property bool mapLocked: overlay.visible

    property bool isSynchronizing: false
    property variant synchronizationProgress: 0.0

    property int timesClicked: 0
    property int __fakeToolbarHeightPortrait: 72
    property int __fakeToolbarHeightLandscape: 56

    property bool applicationFullyReady: mapsMain.allTabsReady

    property variant share: SyncShareService {}

    property variant sso: SsoManager {
        onSignInComplete: {
            if (success) {
                overlay.visible = false;

                log.info("Will disable SSO, so it's not shown every time...'")
                appState.ssoEnabled = false;
            }

            rootApplicationLevel.signInComplete(success);
        }
    }

    function forceGeometryResize(forceNoKeyboard) {
        if (inputContext.simulateSip) {
            return;
        }

        if (main.inPortrait) {
            if (forceNoKeyboard) {
                windowContent.heightDelta = 0;
            } else {
                windowContent.heightDelta = inputContext.softwareInputPanelRect.width - (inputContext.softwareInputPanelVisible? __fakeToolbarHeightPortrait : 0);
            }
        } else {
            if (forceNoKeyboard) {
                windowContent.heightDelta = 0;
            } else {
                windowContent.heightDelta = inputContext.softwareInputPanelRect.height - (inputContext.softwareInputPanelVisible? __fakeToolbarHeightLandscape : 0);
            }
        }
    }

    Component.onCompleted: {
        if (!appState.orientationSwitchEnabled) {
            screen.allowedOrientations = QtComponents.Screen.Portrait;
        }

        if (appState.ssoEnabled) {
            sso.initialize();
        }

        if (syncShareAvailable) {
            share.initialize();
        }

        main.updateGeometry();
    }

    Component.onDestruction: {
        if (appState.ssoEnabled) {
        }
    }

    function benchmark(name, fun) {
        if (typeof fun !== 'function') {
            log.error('benchmark for ' + name + ': fun is not a function');
            return;
        }

        Benchmark.start (name);
        fun();
        Benchmark.stop (name);
    }

    Item {
        id: main
        rotation: 270//screen.rotation;
        property bool inPortrait: true/*(screen.currentOrientation == QtComponents.Screen.Portrait) ||
        (screen.currentOrientation == QtComponents.Screen.PortraitInverted)*/

        // Just the same signals like in MeeGo Window component
        signal orientationChangeAboutToStart
        signal orientationChangeStarted
        signal orientationChangeFinished

        function updateGeometry () {
            main.x = inPortrait ? (screen.displayWidth - screen.displayHeight) / 2 : 0
            main.y = -main.x;
            main.width = inPortrait ? screen.displayHeight : screen.displayWidth;
            main.height = inPortrait ? screen.displayWidth : screen.displayHeight;

            statusBar.orientation = QtComponents.Screen.Portrait;//screen.currentOrientation;
            screen.updatePlatformStatusBarRect(statusBar)

            orientationChangeAboutToStart();
            orientationChangeStarted();
            orientationChangeFinished();
        }

        Item {
            id: windowContent
            objectName: "windowContent"
            width: parent.width
            height: parent.height - heightDelta

            Connections {
                target: screen
                onCurrentOrientationChanged: {
                    // added orientation change to support Virtual Keyboard
                    if (!appState.orientationSwitchEnabled){
                        screen.allowedOrientations = QtComponents.Screen.Portrait;
                    }
                    platformWindow.startSipOrientationChange(main.rotation);
                    main.inPortrait = true;/*(screen.currentOrientation == QtComponents.Screen.Portrait) ||
                    (screen.currentOrientation == QtComponents.Screen.PortraitInverted);*/

                    main.updateGeometry();
                }
            }

            // Used for resizing windowContent when virtual keyboard appears
            property int heightDelta: 0

            Connections {
                target: inputContext
                ignoreUnknownSignals: true

                onSoftwareInputPanelVisibleChanged: { rootApplicationLevel.forceGeometryResize(); }
                onSoftwareInputPanelRectChanged: { rootApplicationLevel.forceGeometryResize(); }
            }


            Component.onCompleted:{
            }


            QtComponents.StatusBar {
                id: statusBar
                anchors.top: parent.top
                Component.onCompleted: {
                    statusBar.orientation = QtComponents.Screen.Portrait;//screen.currentOrientation;
                    screen.updatePlatformStatusBarRect(statusBar);
                }
            }

            /*
            Timer {
                id: loadingTimer
                running: true
                repeat: true
                interval: 1//250
                onTriggered: {
                    if (!appState.ssoEnabled) {
                        loader.source = "MapsMain.qml";
                        loadingTimer.running = false;
                    }
                }
            }
            */

            // Before the user is logged into SSO, the overlay is supposed to block the application
            Rectangle {
                id: overlay

                anchors { top: statusBar.bottom; bottom: parent.bottom }
                width: main.width
                height: main.height

                visible: appState.ssoEnabled
                color: "black"
                opacity: 0.5
                z: 2

                MouseArea {
                    anchors.fill: parent

                    onClicked: {
                        if (false) {
                            if (++timesClicked == 10) {
                                rootApplicationLevel.signInComplete(true);
                                overlay.visible = false;
                                log.trace("+++++++++++ debug remove SSO screen ++++++++++++++++")
                            }
                        }
                    }
                }
            }

            MapsMain {
                id: mapsMain
                anchors { top: statusBar.bottom; bottom: parent.bottom }
            }

            /*
            Loader {
                id: loader
                // Setting bigger z index that whole page will break Popup meego component!
                //z: 1
                anchors { top: statusBar.bottom; bottom: parent.bottom }
                //source: "MapsMain.qml"   // comment this out if u want old loading concept back
            }
            */

            /*
            Rectangle {
                // JUST a dummy component (instead of real preloader)
                id: preloader
                color: "black"

                visible: loader.status != Loader.Ready

                anchors.fill: parent
            }
            */

            /*
            Preloader {
                id: preloader

                visible: loader.status != Loader.Ready

                anchors.top: statusBar.bottom
                anchors.bottom: parent.bottom
                width: main.width
                height: main.height
            }
            */
        }
    }
}
