import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0
import ovi.connector.places 1.0
import ovi.connector.favorites 1.0
import ovi.connector.navigation 1.0

import com.nokia.odml 1.0

import "More"

Page {
    id: moreScene
    anchors.fill: parent
    clip: true

    property variant pickedObject
    property variant model
    property variant favInstance // We keep instance of favorites all the time so we dont need to recreate it again and again!
    property int offset: 10
    property int __feedbackIndex: 3

    property bool ready: false


    onStatusChanged: {
        if (status === PageStatus.Active) {
            if (shouldAddFeedback()) {
                addFeedbackItem(__feedbackIndex);
            } else {
                removeFeedbackItem();
            }
        }
    }

    Rectangle {
        anchors.fill: parent
        color: "#f0f1f2"
        z: 0
    }

    ListModel {
        id: contactModel
     }

    ToolBarLayout {
        id: backButtonTools
        visible: false
        ToolIcon {
            anchors.left: parent.left
            objectName: "toolBarIconBackButton";
            iconId: "icon-m-toolbar-back";
            onClicked: {
                pageStack.pop();
            }
        }
    }

    Component {
        id: contactDelegate
        Rectangle {
            id: listElement
            width:  parent.width
            height: 75
            color: "#f0f1f2"

            objectName: tag
            property string tagName: tag

            Column {
                anchors.fill: parent;
                Item {
                    id:listElementHelper
                    height: 75
                    width: parent.width
                    MapsText {
                        state: "listNormalBold"
                        x: 20
                        anchors.verticalCenter: parent.verticalCenter
                        text: title
                    }
                    Image {
                        id: arrow;
                        source: placesAssetsPath + "images/layout/arrow_right_dark.png";
                        anchors.verticalCenter: parent.verticalCenter;
                        anchors.right: parent.right;
                        anchors.rightMargin: offset;
                    }
                }
            }
            MouseArea {
                id: mouse
                anchors.fill: parent
                onPressed: parent.color = "#d0d1d2"
                onReleased: parent.color = "#f0f1f2"
                onCanceled: parent.color = "#f0f1f2"
                onClicked: { handleClick(tag) }
            }
        }
    }

    ListView {
        id: moreList
        //anchors.top: header.bottom
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        anchors.left: parent.left
        model: contactModel
        delegate: contactDelegate
        highlight: Rectangle { color: "#c0c1c2" }
        focus: true
    }

//    ScrollDecorator {
//        flickableItem: moreList
//    }
    property variant localizedStrings;

    property variant alreadyCreatedPages

    Component {
        id: feedbackItem
        Feedback {
            anchors.fill: parent;
            onReportSubmitted: {
                if (mainscreen.enableFeedback24HourRule) {
                    var today = new Date();
                    db.set("nps_nextValidFeedbackDate", new Date().setDate(today.getDate() + 1));
                    removeFeedbackItem();
                }
            }
        }
    }

    Component {
        id: settingsItem
        Settings {
            tools: backButtonTools
        }
    }

    Component {
        id: debugItem
        Debug { tools: moreScene.tools }
    }

    Component {
        id: aboutItem
        Page {
            tools: backButtonTools
            About {}
        }
    }

    Component {
        id: offlineMapsItem
        OfflineMaps { }
    }

    Component {
        id: categoryItem

        CategoryTree {
            id: categoryTree

            mainScreen: mainscreen

            tools: moreScene.tools
            anchors.fill: parent
            Component.onCompleted: {
                categoryTree.baseDepth = moreScene.pageStack.depth + 1;
            }
            onFinished: {
                moreScene.fromFavorite = false;
                moreScene.pickedObject = categoryTree.pickedObject;
                moreScene.model = categoryTree.model;
                moreScene.pageStack.push(mapsAppPlacesViewer);
            }
        }
    }

    Component {
        id: favoritesPicker
        Page {
            id: dummy  // This dummy page is used for reparenting a favoritesPickerPage so we keep instance in memory

            Component.onCompleted: {
                favInstance.parent = dummy;
                favInstance.status = dummy.status;
                favInstance.state = dummy.state;
                dummy.tools = favInstance.tools;
                favInstance.visible = true;
            }

            onStatusChanged: {
                if (favInstance) favInstance.status = status;
                if (status == PageStatus.Activating) { favInstance.visible = true; favInstance.parent = dummy; }
                if (status == PageStatus.Inactive) { favInstance.visible = false; favInstance.parent = moreScene; }
            } 
        }
    }

    Component {
        id: favPickerPage
        FavoritesPickerPage {
            tools: moreScene.tools

            onFavoritePicked: {
                moreScene.fromFavorite = true;
                moreScene.pickedObject = object;
                moreScene.model = listModel;
                moreScene.pageStack.push(mapsAppPlacesViewer);
            }
        }
    }
    property bool fromFavorite: false;
    // Create the viewer
    Component {
        id: mapsAppPlacesViewer
        PlacesViewer {
            mapsApp: mainscreen;
            anchors.fill: parent;
            object: moreScene.pickedObject
            model: moreScene.model
            openFromFavorite: moreScene.fromFavorite
        }
    }

    SettingsPersistence {
        id: db;
    }

    function handleClick(tag) {
        if (tag) {
            var item;
            switch(tag) {
            case "feedback":
                item = feedbackItem;
                break;
            case "debug":
                item = debugItem;
                break;
            case "about":
                item = aboutItem;
                break;
            case "categories":
                item = categoryItem;
                if (typeof(analyticsComponent) !== "undefined"){
                    analyticsComponent.logIMEIActivity("TabClick", { type: "Categories" } );
                }
                break;
            case "favorites":
                item = favoritesPicker;
                break;
            case "offlinemaps":
                item = offlineMapsItem;
                if (typeof(analyticsComponent) !== "undefined"){
                    analyticsComponent.logIMEIActivity("TabClick", { type: "Manage Maps" } );
                }
                break;
            case "settings":
                item = settingsItem;
                break;
            }

            var tmp = alreadyCreatedPages;
            if (!tmp) {
                tmp = {};
            }
            if (!tmp[tag]) {
                tmp[tag] = item.createObject(moreScene);
            }
            alreadyCreatedPages = tmp;

            pageStack.push(tmp[tag]);

            delete tmp;
        }
    }
    Connections {
        target: mainscreen
        onEnableFeedback24HourRuleChanged: {
            if(shouldAddFeedback()) {
                addFeedbackItem(__feedbackIndex);
            } else {
                removeFeedbackItem();
            }
        }
    }

    Component.onCompleted: {
        favInstance = favPickerPage.createObject(moreScene);

        contactModel.append({
            title: qsTranslate("Maps", "qtn_maps_more_categories"),
            hasNext: true,
            tag: "categories"
        });
        if (syncShareAvailable || true) {
            contactModel.append({
                title: qsTranslate("Maps", "qtn_maps_more_favourites"),
                hasNext: false,
                tag: "favorites"
            });
        }
        contactModel.append({
             title: qsTranslate("Maps", "qtn_maps_more_offlinemaps"),
             hasNext: true,
             tag: "offlinemaps"
        });
        contactModel.append({
            title: qsTranslate("Maps Settings", "qtn_maps_deviceSettings_title"),
            hasNext: true,
            tag: "settings"
        });
        contactModel.append({
            title: qsTranslate("Maps", "qtn_maps_more_about"),
            hasNext: false,
            tag: "about"
        });

        ready = true;
    }

    function shouldAddFeedback() {
        var shouldAdd = true;
        if (mainscreen.enableFeedback24HourRule) {
            // Check if can send feedback
            db.get("nps_nextValidFeedbackDate", function(value) {
            var nextValidDate = value;
            var today = new Date();
            log.trace(" Next valid feedback submit date: " + nextValidDate);
            today = new Date().setDate(today.getDate()+0);
            log.trace(" Today: " + today);
            if (nextValidDate > today) {
                shouldAdd = false;
            }
            }, function(err) {
                       log.error(" error getting nextValidFeedbackDate");
            });
        }
        log.info(" [more]: Adding feedback item: " + shouldAdd);
        return shouldAdd;
    }

    function getFeedbackIndex() {
        var item = contactModel.get(__feedbackIndex);

        if (item.tag === "feedback") {
            return __feedbackIndex;
        } else {
            for (var i = 0; i < contactModel.count; i++) {
                item = contactModel.get(i);
                if ( item.title === "feedback") {
                    return i;
                }
            }
        }
        return -1;
    }

    function addFeedbackItem(atIndex) {
        if(getFeedbackIndex() !== -1) { return; }
        var feedbackObj = {
                        title: qsTranslate("Maps", "qtn_maps_more_feedback"),
                        hasNext: false,
                        tag: "feedback"
                    };
        if(typeof atIndex === "number") {
            contactModel.insert(atIndex, feedbackObj);
        } else {
            contactModel.append(feedbackObj);
        }
    }

    function removeFeedbackItem() {
        var index = getFeedbackIndex();
        if(index !== -1) {
            contactModel.remove(index);
        }
    }
}
