import QtQuick 1.0

import ovi.connector 1.0
import com.nokia.meego 1.0

import MapsPlugin 1.0

Rectangle {
    id: page
    anchors.fill: parent

    color: "#f0f1f2"

    property int linkFontSize: 16

    ScrollDecorator {
        flickableItem: aboutContent
    }

    Flickable {
        id: aboutContent
        anchors.fill: parent

        contentWidth: parent.width;
        contentHeight: column.childrenRect.height + column.anchors.topMargin + column.anchors.bottomMargin + 10
        clip: true

        Column {
            id: column
            anchors {
                left: parent.left
                right: parent.right
                top: parent.top
                bottom: parent.bottom
                topMargin: 20
                leftMargin: 20
                rightMargin: 10
            }

            width: aboutContent.width
            spacing: 25

            Item { // Start of first section header
                anchors {
                    left: parent.left
                    right: parent.right
                }
                height: Math.max(oviMap.height, versionSection.height)

                Image {
                    id: oviMap
                    source: "../../images/icon-l-maps.png"
                    anchors.left: parent.left
                }
                MapsText {
                    text: qsTranslate("Maps", "qtn_maps_maps");
                    platformStyle: LabelStyle { fontPixelSize: 32 }
                    anchors {
                        top: oviMap.bottom
                        topMargin: 10
                        horizontalCenter: oviMap.horizontalCenter
                    }
                }

                Column {  // Start of section version
                    id: versionSection
                    spacing: 20
                    anchors {
                        right: parent.right
                        left: parent.left
                        leftMargin: Math.max(page.width/2, oviMap.width)
                    }

                    AboutVersionInfo {
                        heading: qsTranslate('Maps', 'qtn_maps_about_version')
                        value: appState.mapsVersion
                    }
                    AboutVersionInfo {
                        heading: qsTranslate('Maps', 'qtn_maps_about_mapdata')
                        value: {
                            var tmp, mapdata = MapsPlugin.mapVersion;
                            tmp = mapdata.split(".");
                            if(!tmp || tmp[0] > 10) {
                                mapdata = "";
                            }
                            return mapdata;
                        }
                    }
                    AboutVersionInfo {
                        heading: "MOS-QML: "
                        value: MapsPlugin.pluginVersion
                        state: "small"
                    }
                    AboutVersionInfo {
                        heading: "MOS Id: "
                        value: MapsPlugin.clientId
                        state: "small"
                    }

                }// End of section version
            }   // End of first section header

            Button {
                text: qsTranslate("Maps", "qtn_maps_about_support")
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: parent.width*.1
                    rightMargin: parent.width*.1
                }
                onClicked: Qt.openUrlExternally("http://nokia.mobi/apps/maps/meego/support")
            }

            OviListHeader { // Start of section Copyright Notice
                headerText: qsTranslate("Maps", "qtn_maps_about_copyright_notice");
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: 0
                }
            }

            Row {
                spacing: 10
                Image {
                    source: "../../images/navteq-maps-logo.png"
                }
                Image {
                    source: "../../images/nav2-navinfo-logo.png"
                }

            }// End of section Copyright Notice


            AboutListElement {
                firstLineText: "2010 NAVTEQ,"
                secondLineText: qsTranslate("Maps", "qtn_maps_about_all_rights_reserved_navteq")
            }

            AboutListElement {
                firstLineText: "2010 NavInfo Co Ltd,"
                secondLineText: qsTranslate("Maps", "qtn_maps_about_all_rights_reserved_navinfo")
            }

            AboutListElement {
                firstLineText: "2002 Planetary Visions Ltd"
                secondLineText: "planetaryvisions.com"
            }

            AboutListElement {
                firstLineText: "2010 DigitalGlobe, Inc."
                secondLineText: ""
            }

            AboutListElement {
                firstLineText: "Crown Copyright 100048088"
                secondLineText: "Aprobado por el Instituto Geográfico\nMilitar.\nNúmero de expediente\nGG081531/Sept 5 2008\nSRTM V4, 2008, CIAT"
            }

            AboutListElement {
                firstLineText: "Royal Jordanian Geographic Centre"
                secondLineText: ""
            }

            AboutListElement {
                firstLineText: "Iran Maps provided by THTC"
                secondLineText: ""
            }

            AboutListElement {
                firstLineText: "NASA's Earth Observatory"
                secondLineText: "http://visibleearth.nasa.gov"
            }

            OviListHeader { // Start of section Legal Disclaimer
                headerText: qsTranslate("Maps", "qtn_maps_about_all_legal_disclaimer");
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: 0
                }
            }

            MapsText {
                text: qsTranslate("Maps", "qtn_maps_about_all_legal_disclaimer_text")
                width: parent.width
                wrapMode: Text.Wrap;
                elide: "ElideNone"
            }

            Button {
                text: qsTranslate("Maps", "qtn_maps_terms_button");
                anchors.horizontalCenter: parent.horizontalCenter
                onClicked: Qt.openUrlExternally("http://nokia.mobi/privacy/services/terms/nokia-service")
            } // End of section Legal Disclaimer
        }
    }
}
