import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import ovi.connector.places 1.0

Page {
    id: debugScene
    anchors.fill: parent
    clip: true

    Rectangle {
        anchors.fill: parent
        color: "#f0f1f2";
        z: 0
    }

    ListModel {
        id: contactModel
        ListElement {
            title: "Enable zoom bar"
            tag: "zoomEnabled"
            cb: true
        }
        ListElement {
            title: "Enable finger rotation"
            tag: "enableAutomaticRotating"
            cb: true
        }
        ListElement {
            title: "Enable magic 3d / 2d"
            tag: "enableMagic3d2d"
            cb: true
        }

        ListElement {
            title: "Recommendations Bounding Box"
            tag: "enableRecommendationsBoundingBox"
            cb: true
        }

        ListElement {
            title: "Enable orientation switch"
            tag: "orientationSwitch"
            cb: true
        }

        ListElement {
            title: "SSO Enabled"
            tag: "ssoEnabled"
            cb: true
        }

        ListElement {
            title: "Anti-Aliasing Enabled"
            tag: "antialiasingEnabled"
            cb: true
        }

        ListElement {
            title: "Use Roadview Guidance Update Mode"
            tag: "roadviewUpdateMode"
            cb: true
        }

        ListElement {
            title: "Show All Positions Markers"
            tag: "showAllPositionMarkers"
            cb: true
        }

        ListElement {
            title: "Delete All Maps"
            tag: "allowUserToDeleteAllMapData"
            cb: true
        }

        ListElement {
            title: "MapMathcher mode"
            tag: "mapMatcherMode"
            sl: true
            minValue: 1
            maxValue: 2
            optionsWrapper: "none,Car,Pedestrian"
        }

        ListElement {
            title: "Server configuration"
            tag: "serverConfig"
            cb: false
        }
        ListElement {
            title: "Logging"
            tag: "logging"
            cb: false
        }
        ListElement {
            title: "FPS: List"
            tag: "listFPS"
            cb: false
        }
        ListElement {
            title: "FPS: Map"
            tag: "mapsFPS"
            cb: false
        }
        ListElement {
            title: "Positioning data"
            tag: "positioningData"
            cb: false
        }
        ListElement {
            title: "Language: "
            tag: "language"
            cb: false
        }
        ListElement {
            title: "Enable Feedback 24-hour submit rule"
            tag: "feedback"
            cb: true
        }
     }

    Component {
        id: contactDelegate
        Rectangle {
            id: listElement
            width:  parent.width
            height: 75
            color: "#f0f1f2"
            Column {
                anchors.fill: parent;
                Item {
                    id:listElementHelper
                    height: 75
                    width: parent.width
                    Text {
                        font.pixelSize: 24
                        color: "#282828";
                        font.bold: true
                        x: 20
                        font.family: mainscreen.__labelStyle.fontFamily
                        anchors.verticalCenter: parent.verticalCenter
                        text: {
                            if (tag === "language") {
                                return title + locale;
                            }
                            if (optionsWrapper) {

                                var optionsWrapperArray = optionsWrapper.split(",");
                                var objectToGetValueFrom;
                                if (sl === true) {
                                    objectToGetValueFrom = sliderObject;
                                } else if (cb === true) {
                                    objectToGetValueFrom = checkboxObject;
                                }

                                if (objectToGetValueFrom && objectToGetValueFrom.value !== undefined && optionsWrapperArray[objectToGetValueFrom.value] !== undefined) {
                                    return qsTranslate("Maps", title) + " (" + optionsWrapperArray[objectToGetValueFrom.value] + ")";
                                } else {
                                    return qsTranslate("Maps", title);
                                }
                            } else {
                                return qsTranslate("Maps", title);
                            }
                        }
                    }
                    Rectangle {
                        id: bottomLine
                        anchors.bottom: parent.bottom
                        width: parent.width
                        height: 1
                        color: "#e4e5e6"
                    }
                }

            }
            MouseArea {
                id: mouse;
                anchors.fill: parent
                onPressed: parent.color="#d0d1d2"
                onReleased: parent.color="#f0f1f2"
                onCanceled: parent.color="#f0f1f2"
                onClicked: {
                    log.trace("onClicked Signal on: "+ title);
                    if(tag) {
                        var item = -1;
                        switch(tag) {
                        case "listFPS":
                            item = listFPSItem;
                            break;
                        case "serverConfig":
                            item = serverConfigItem;
                            break;
                        case "logging":
                            item = loggingItem;
                            break;
                        case "mapsFPS":
                            item = mapsFPSItem;
                            break;
                        case "positioningData":
                            item = positioningData;
                            break;
                        }
                    }
                    if(item !== -1) {
                        pageStack.push(item);
                    }
                }
            }

            Slider {
                id: sliderObject

                visible: sl === true
                anchors {
                    right: parent.right
                    rightMargin: 20
                    verticalCenter: parent.verticalCenter
                }

                value: {
                    if (sl) {
                        if (tag === "mapMatcherMode") return mainscreen.mapMatcherMode;
                    }
                    return "";
                }

                minimumValue: minValue? minValue : 0
                maximumValue: maxValue? maxValue : 100
                stepSize: 1
                valueIndicatorVisible: true

                onValueChanged: {
                    if (tag === "mapMatcherMode")
                        mainscreen.mapMatcherMode = value;
                }
            }

            CheckBox {
                id: checkboxObject

                visible: cb === true
                anchors {
                    right: parent.right
                    rightMargin: 20
                    verticalCenter: parent.verticalCenter
                }

                checked: {
                    if (cb) {
                        if (tag === "zoomEnabled") return mainscreen.zoombarsEnabled;
                        else if (tag === "enableAutomaticRotating") return mainscreen.enableAutomaticRotating;
                        else if (tag === "enableMagic3d2d") return mainscreen.enableMagic3d2d;
                        else if (tag === "ssoEnabled") return appState.ssoEnabled && appState.isDevice;
                        else if (tag === "orientationSwitch") return appState.orientationSwitchEnabled && appState.isDevice;
                        else if (tag === "enableRecommendationsBoundingBox") return mainscreen.enableRecommendationsBoundingBox;
                        else if (tag === "antialiasingEnabled") return appState.antialiasingEnabled;
                        else if (tag === "roadviewUpdateMode") return mainscreen.walkGuidanceUpdateMode === "roadview";
                        else if (tag === "showAllPositionMarkers") return mainscreen.showAllPositionMarkers;
                        else if (tag === "allowUserToDeleteAllMapData") return mainscreen.allowUserToDeleteAllMapData;
                        else if (tag === "feedback") return mainscreen.enableFeedback24HourRule;
                    }
                    return false;
                }

                onCheckedChanged: {
                    if (tag === "zoomEnabled") mainscreen.zoombarsEnabled = checked;
                    else if (tag === "enableAutomaticRotating") mainscreen.enableAutomaticRotating = checked;
                    else if (tag === "enableMagic3d2d") mainscreen.enableMagic3d2d = checked;
                    else if (tag === "ssoEnabled") { if (appState.isDevice) { appState.ssoEnabled = checked; } }
                    else if (tag === "orientationSwitch") { if (appState.isDevice) { appState.orientationSwitchEnabled = checked; } }
                    else if (tag === "enableRecommendationsBoundingBox") mainscreen.enableRecommendationsBoundingBox = checked;
                    else if (tag === "feedback") mainscreen.enableFeedback24HourRule = checked;
                }

                onClicked: {
                    if (tag === "antialiasingEnabled") {
                        queryDialog.titleText = "Enable AntiAliasing?"
                        queryDialog.message = "Anti-Aliasing settings can be applied only after application is restarted. Do you want to restart now?"
                        queryDialog.rejectButtonText = "later";
                        queryDialog.acceptButtonText = "restart";
                        queryDialog.open();

                        appState.antialiasingEnabled = checked;
                    } else if (tag === "roadviewUpdateMode") {
                        mainscreen.walkGuidanceUpdateMode = (checked)? "roadview" : "position-animation";
                    } else if (tag === "showAllPositionMarkers") {
                        mainscreen.showAllPositionMarkers = checked;
                    } else if (tag === "allowUserToDeleteAllMapData") {
                        mainscreen.allowUserToDeleteAllMapData = checked;
                    } else if (tag === "feedback") {
                        mainscreen.enableFeedback24HourRule = checked;
                    }
                }
            }
            property bool forceUnCheck: false
        }
    }

    ListView {
        id: debugList
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        anchors.right: parent.right;
        anchors.left: parent.left;
        model: contactModel
        delegate: contactDelegate
        highlight: Rectangle { color: "#c0c1c2" }
        focus: true
    }


    ScrollDecorator {
        flickableItem: debugList
    }

    Component {
        id: serverConfigItem;
        Page {
            tools: debugScene.tools;
            ServerConfig {}
        }
    }

    Component {
        id: loggingItem;
        Page {
            tools: debugScene.tools;
            Logging {}
        }
    }

    Component {
        id: mapsFPSItem;
        Page {
            tools: debugScene.tools;
            MapsFPS {}
        }
    }
    Component {
        id: listFPSItem;
        Page {
            tools: debugScene.tools;
            ListFPS {}
        }
    }
    Component {
        id: positioningData;
        Page {
            tools: debugScene.tools;
            PositioningData {}
        }
    }

    QueryDialog {
        id: queryDialog
        onAccepted: {
            Qt.quit();
        }
    }
}
