import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import ovi.connector 1.0

import "Feedback.js" as Logic

Page {
    id: feedbackScene

    signal reportSubmitted

    property bool successfulSubmit: false;
    property bool animationEnabled: true;
    property bool showRetryBtn: false

    property int state_idle: 0;
    property int state_submitSuccess: 1;
    property int state_submitFail: 2;
    property int state_submitting: 3;
    property int feedbackState: feedbackScene.state_idle;

    property variant devInfo
    property variant genInfo
    property variant netInfo

    anchors.fill: parent
    tools: ToolBarLayout {
        ToolIcon { objectName: "toolBarIconBackButton"; iconId: "icon-m-toolbar-back"; onClicked: { pageStack.pop(); } anchors.left: parent.left}
    }

    Rectangle {
        anchors.fill: parent
        color: "#f0f1f2"
        z: 0
    }

    ScrollDecorator {
        flickableItem: npsComponent
    }

    Flickable {
        id: npsComponent
        anchors.fill: parent
        contentWidth: width
        contentHeight: nps.height
        clip: true

        NetPromoterScore {
            id: nps
            width: parent.width
            color: "#00000000"
            uiString: {
                "header": qsTranslate("NPS", "qtn_nps_tellus"),
                "question": qsTranslate("NPS", "qtn_nps_question").arg(qsTranslate("Maps", "qtn_maps_maps")),
                "notLikely": qsTranslate("NPS", "qtn_nps_not_likely"),
                "extremelyLikely": qsTranslate("NPS", "qtn_nps_extremely_likely"),
                "selectScore": qsTranslate("NPS", "qtn_nps_ratingerror"),
                "commentsInputLabel": qsTranslate("NPS", "qtn_nps_why"),
                "contactAgreement": qsTranslate("NPS", "qtn_nps_contactconfirm"),
                "emailInputLabel":qsTranslate("NPS", "qtn_nps_contactdetails"),
                "emailPlaceholderText": qsTranslate("NPS", "qtn_nps_emailaddress"),
                "emailSipActionKeyLabel": qsTranslate("Maps", "qtn_placesconnector_picker_watermark_done"),
                "invalidEmail": qsTranslate("NPS", "qtn_nps_invalid_email_text"),
                "legalText": qsTranslate("NPS", "qtn_nps_nokia_policy"),
                "submit": qsTranslate("NPS", (showRetryBtn ? "qtn_nps_feedback_retry" : "qtn_nps_submit_btn_text"))
            }
            onSubmit: {
                log.trace("Score=" + score + ", comments=" + comments +
                                  ", email=" + emailAddress + ", useEmail=" + useEmail);
                feedbackScene.feedbackState = feedbackScene.state_submitting
                Logic.submitReport(feedbackEnv, nps, devInfo, netInfo, genInfo);
            }
        }
    }

    QueryDialog {
        id: statusDialog
        //titleText: " " // this is a small hack because if we don't set it, the title would be "Query Dialog"

        title: BusyIndicator {
            visible: feedbackScene.feedbackState === feedbackScene.state_submitting
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            height: 60

            running: true
            platformStyle: BusyIndicatorStyle {inverted: true}
        }

        onAccepted: {
            feedbackScene.feedbackState = feedbackScene.state_idle;
        }

        onRejected: {
            if(feedbackScene.feedbackState === state_submitSuccess) {
                pageStack.pop();
            } else {
                Logic.cancel();
            }
        }
    }

    states: [
        State {
            name: "sendingFeedback"
            when: feedbackScene.feedbackState === feedbackScene.state_submitting
            StateChangeScript {
                name: "sending"
                script: {
                    statusDialog.titleText = " ";
                    statusDialog.rejectButtonText = qsTranslate("NPS", "qtn_nps_feedback_cancel");
                    statusDialog.message = qsTranslate("NPS", "qtn_nps_feedback_sending");
                    statusDialog.open();
                }
            }
        },
        State {
            name: "successful"
            when: feedbackScene.feedbackState === feedbackScene.state_submitSuccess
            StateChangeScript {
                name: "fbSuccessful"
                script: {
                    feedbackScene.showRetryBtn = false;
                    statusDialog.titleText = qsTranslate("NPS", "qtn_nps_thanks")
                    statusDialog.rejectButtonText = "";
                    statusDialog.message = qsTranslate("NPS", "qtn_nps_feedback_success");
                    statusDialog.open();
                }
            }
        },
        State {
            name: "failure"
            when: feedbackScene.feedbackState === feedbackScene.state_submitFail
            StateChangeScript {
                script: {
                    feedbackScene.showRetryBtn = true;
                    statusDialog.titleText = qsTranslate("NPS", "qtn_nps_feedback_submissionerror");
                    statusDialog.message = qsTranslate("NPS", "qtn_nps_feedback_fail")
                    statusDialog.rejectButtonText = ""
                    statusDialog.open();
                }
            }
        }
    ]

    Component.onCompleted: {
        if(appState.isDevice) {
            devInfo = Qt.createQmlObject("import MapsPlugin 1.0; import QtMobility.systeminfo 1.1; DeviceInfo{}", feedbackScene, null);
            genInfo = Qt.createQmlObject("import MapsPlugin 1.0; import QtMobility.systeminfo 1.1; GeneralInfo{}", feedbackScene, null);
            netInfo = Qt.createQmlObject("import MapsPlugin 1.0; import QtMobility.systeminfo 1.1; NetworkInfo{}", feedbackScene, null);
        }
    }
}

