import QtQuick 1.0

Item {

    id: fpsCounter;

    property bool limit: false
    property bool isRunning: false;
    property real fps : 0.0
    property real smoothedfps : 0.0
    property real lastredraw : 0
    property real smoothedmax : 0.0
    property real smoothedmin : 0.0
    property real smoothtime: 0
    property real measureCount: 0;
    property real totalFPS: 0;
    property real averageFPS: 0;


    function updateFps() {

        var currentTime =  new Date()
        var newtime = currentTime.getTime();
        var delta = newtime - lastredraw;

        // Smooth decay
        smoothtime *= 0.9;
        smoothedmax = smoothedfps + (0.9 * (smoothedmax - smoothedfps));
        smoothedmin = smoothedfps - (0.9 * (smoothedfps - smoothedmin));

        fps = 1000.0/delta;
        smoothtime += delta;
        smoothedfps = 10000.0/smoothtime

        if (limit) {
            if (fps>60.0) fps=60.0
            if (smoothedfps>60.0) smoothedfps=60.0
        }

        if (fps>smoothedmax)
            smoothedmax = fps;
        if (fps<smoothedmin)
            smoothedmin = fps;

        if(fps != 0) {
            totalFPS += fps;
            measureCount++;
            var newAvg = totalFPS/measureCount;
            averageFPS = (newAvg == Number.NaN || newAvg == Number.NEGATIVE_INFINITY || newAvg == Number.POSITIVE_INFINITY) ? averageFPS : newAvg;
        }

        //log.info(newtime,lastredraw,fps)
        lastredraw = newtime;
    }

    SequentialAnimation {
        loops: Animation.Infinite
        running: fpsCounter.isRunning
        ScriptAction { script: fpsCounter.updateFps(); }
        PauseAnimation { duration: 10 }
    }

    function reset() {
        fpsCounter.averageFPS = 0;
        fpsCounter.measureCount = 0;
        fpsCounter.fps = 0;
        fpsCounter.totalFPS = 0;
    }
}
