import QtQuick 1.0

Rectangle {
    id: fpsScene;
    anchors.fill: parent;
    property bool showImages: true;
    ListModel {
        id: fruitModel;
    }
    ListView {
        id: list;
        height: parent.height*.9;
        width: parent.width;
        model: fruitModel
        delegate: Rectangle {
        color: "#FFF";
        border.color: "#AAA";
        border.width: 3;
        width: 180; height: 60
        Image { id: icon; source: picture; visible: fpsScene.showImages }
            Column {
             anchors.left: icon.right;
             Text { text: '<b>fruit:</b> ' + name }
             Text { text: '<b>cost: $</b> ' + cost }
            }
        }
        highlight: Rectangle { color: "lightsteelblue"; radius: 5 }
        focus: true
        highlightFollowsCurrentItem: true;
        currentIndex: 4;
     }


    Rectangle {
        id: info;
        width: 200;
        height: 200;
        anchors.right: parent.right;
        anchors.top: parent.top;
        color: "lightgreen";
        Column {
            spacing: 10;
            Text {
                id: infoText;
                font.pointSize: 13;
                text: "Information"
                font.bold: true;
                font.underline: true;
            }
            Text {
                text: "Last measured FPS: \n" + fpsCounter.fps;
                wrapMode: Text.Wrap;
                font.pointSize: 13;
                color: "green";
            }
            Text {
                text: "Average measured FPS: \n" + fpsCounter.averageFPS
                color: "blue"
                font.pointSize: 13;
            }
            Text {
                text: "# items:\t" + fruitModel.count +
                      "\n60 FPS limit:\t" + fpsCounter.limit +
                      "\nImages:\t" + fpsScene.showImages
                font.pointSize: 11;
            }
        }
    }

    Rectangle {
        width: 200;
        height: 200;
        anchors.right: parent.right;
        anchors.top: info.bottom;
        color: "lightblue";

        Rectangle {
            height: 25;
            width: 50;
            color: "#FFF";
            border.color: "#AAA";
            border.width: 5;
            //anchors.top: directions.bottom;
            //anchors.topMargin: 10;
            TextInput {
                id: textInput;
                text: "30";
                cursorVisible: textInput.focus;
                anchors.centerIn: parent;
                width: parent.width - 10;
                height: parent.height - 10;
                validator: IntValidator{bottom: 1;}
//                focus: true;
                Keys.onEnterPressed: { focus = false; }
            }
            Rectangle {
                id: button;
                anchors.left: textInput.right;
                anchors.leftMargin: 20;
                border.color: "#AAA"
                border.width: 5;
                height: textInput.height + 20;
                width: 120;
                color: "lightsteelblue"
                Text {
                    anchors.centerIn: parent;
                    text: "Create 3x List"
                    font.pointSize: 11;
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: { buildList(textInput.text); }
                    onPressed: {button.color = "blue"}
                    onReleased: {button.color = "lightsteelblue"}
                }
            }
            Rectangle {
                id: imageButton;
                anchors.top: button.bottom;
                anchors.left: textInput.right;
                anchors.leftMargin: 20;
                border.color: "#AAA"
                border.width: 5;
                height: textInput.height + 20;
                width: 120;
                color: "lightsteelblue"
                Text {
                    anchors.centerIn: parent;
                    text: "Turn Images " + (fpsScene.showImages ? "off" : "on");
                    font.pointSize: 11;
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: { fpsScene.showImages = !fpsScene.showImages; }
                    onPressed: {imageButton.color = "blue"}
                    onReleased: {imageButton.color = "lightsteelblue"}
                }
            }

            Rectangle {
                id: limitButton;
                anchors.top: imageButton.bottom;
                anchors.left: textInput.right;
                anchors.leftMargin: 20;
                border.color: "#AAA"
                border.width: 5;
                height: textInput.height + 20;
                width: 120;
                color: "lightsteelblue"
                Text {
                    anchors.centerIn: parent;
                    text: "Turn FPS Limit " + (fpsCounter.limit ? "Off" : "On");
                    font.pointSize: 11;
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: { fpsCounter.limit = !fpsCounter.limit; reset(); }
                    onPressed: {button.color = "blue"}
                    onReleased: {button.color = "lightsteelblue"}
                }
            }

            Text {
                id: directions;
                anchors.top: limitButton.bottom;
                anchors.topMargin: 10;
                text: "Enter a number to create a list\nwith 3 times the number entered.\nThen click the button to show the list."
                wrapMode: Text.Wrap;
            }
        }
    }
    function reset() {
        fpsCounter.averageFPS = 0;
        fpsCounter.measureCount = 0;
        fpsCounter.fps = 0;
        fpsCounter.totalFPS = 0;
    }

    function buildList(iterations) {
        reset();
        fruitModel.clear();
        for(var i=0; i<iterations; i++) {
            fruitModel.append({"name": "apple " + i, "cost": 2.45, "picture": "../../images/icon_debug_settings.png"});
            fruitModel.append({"name": "orange " + i, "cost": 3.25, "picture": "../../images/icon_about.png"});
            fruitModel.append({"name": "banana " + i, "cost": 1.95, "picture": "../../images/icon_feedback.png"});
        }
    }

    FpsCounter {
        id: fpsCounter;
        isRunning: list.moving;
    }

    Component.onCompleted: {buildList(textInput.text);}
}
