import QtQuick 1.0
import MapsPlugin 1.0
import ovi.connector.map 1.0

Rectangle {
    id: mapsFps;
    color: "#FFF";
    anchors.fill: parent;

    property bool pannable: true;
    property bool toPoint2: true;
    property bool testMapCon: true;
    property bool usePluginFps: false;
    property int animation: Map.ANIMATION_LINEAR;
    property int btnSize: 40;
    property bool showOptions: false;


    Map{
        id: mapPlugin;
        width:parent.width*.8;
        height:parent.height;
        visible: !mapsFps.testMapCon;
        property int iterations;
    }

    MapViewer {
        id: mapConnector;
        width: parent.width*.8;
        height: parent.height;
        mainScreen: mainscreen;
        useGlobalShare: true
        visible: mapsFps.testMapCon;
        property int iterations;
    }

    Connections {
        target: mapsFps.testMapCon ? mapConnector : mapPlugin;
        ignoreUnknownSignals: true
        onAnimationDone: {
            fpsCounter.isRunning = false;
            mapsFps.pannable = true;
            var mapType = mapsFps.testMapCon ? mapConnector : mapPlugin;
            mapType.iterations--;
            if(mapType.iterations > 0) {
                testFps();
            }
        }
        onMouseDown: {if(mapsFps.pannable) fpsCounter.isRunning = false; }
        onCenterChanged: {if(mapsFps.pannable) fpsCounter.isRunning = true; }
    }

    Rectangle {
        id: infoRect;
        color: "lightgreen";
        height: col.height+10;
        width: parent.width*.2;

        anchors.right: parent.right;
        anchors.top: parent.top;

        Column {
            id: col;
            spacing: 10;
            x: 10;
            Text {
                id: infoText;
                font.pointSize: 10;
                text: "Information"
                font.bold: true;
                font.underline: true;
            }
            Text {
                text: "Last FPS:\n" + (fpsCounter.fps).toFixed(4);
                wrapMode: Text.Wrap;
                font.pointSize: 10;
                color: "green";
            }
            Text {
                text: "Avg FPS:\n" + (fpsCounter.averageFPS).toFixed(4);
                color: "blue"
                font.pointSize: 10;
            }
            Text {
                text: "Map:\n" + (mapsFps.testMapCon ? "Connector" : "Plugin")
                color: "purple"
                font.pointSize: 10;
            }
        }
    }
    Rectangle {
        color: "lightsteelblue";
        width: parent.width*.2;
        height: optCol.height+10;
        anchors.right: parent.right;
        anchors.top: infoRect.bottom;
        x: 10;
        Column {
            visible: !showOptions;
            id: actCol;
            width: parent.width;
            spacing: 5;
            Text {
                font.pointSize: 10;
                font.bold: true;
                font.underline: true;
                text: "Actions";
            }
            Rectangle {
                color: mouseArea.enabled ? "purple" : "#9370db";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "#FFF";
                    anchors.centerIn: parent;
                    text: "test moveTo 10x";
                }
                MouseArea {
                    id: mouseArea;
                    anchors.fill: parent;
                    enabled: mapsFps.testMapCon;
                    onClicked: {
                        var mapType = (mapsFps.testMapCon ? mapConnector : mapPlugin);
                        mapType.iterations = 10;
                        testFps();
                    }
                    onPressed: {parent.color = "#9370db";}
                    onReleased: {parent.color = "purple";}
                }
            }
            Rectangle {
                color: "green";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "#FFF";
                    anchors.centerIn: parent;
                    text: "test moveTo 1x";
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        var mapType = (mapsFps.testMapCon ? mapConnector : mapPlugin);
                        mapType.iterations = 1;
                        testFps()
                    }
                    onPressed: {parent.color = "lightgreen";}
                    onReleased: {parent.color = "green";}
                }
            }
            Rectangle {
                color: "#FC0";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "#FFF";
                    anchors.centerIn: parent;
                    text: "<b>reset stats</b>";
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {fpsCounter.reset()}
                    onPressed: {parent.color = "yellow";}
                    onReleased: {parent.color = "#FC0";}
                }
            }
            Rectangle {
                color: "red";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "#FFF";
                    anchors.centerIn: parent;
                    text: "Show Options";
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        showOptions = true;
                    }
                    onPressed: {parent.color = "#F0C";}
                    onReleased: {parent.color = "red";}
                }
            }
        }
        Column {
            id: optCol;
            visible: showOptions;
            width: parent.width;
            spacing: 5;
            Text {
                font.pointSize: 10;
                font.bold: true;
                font.underline: true;
                text: "Options";
            }
            Rectangle {
                color: "red";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "#FFF";
                    anchors.centerIn: parent;
                    text: "Hide Options";
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        showOptions = false;
                    }
                    onPressed: {parent.color = "#F0C";}
                    onReleased: {parent.color = "red";}
                }
            }
            Rectangle {
                color: "orange";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "#FFF";
                    anchors.centerIn: parent;
                    text: "toggle " + (mapsFps.testMapCon ? "plugin" : "MapCon");
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        fpsCounter.reset();
                        mapsFps.testMapCon = !mapsFps.testMapCon;
                        fpsCounter.isRunning = false;
                    }
                    onPressed: {parent.color = "#ff7f50";}
                    onReleased: {parent.color = "orange";}
                }
            }
            // TODO: enable this when available in plugin
            /*
            Rectangle {
                color: "lightgray";
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: "darkgray";
                    anchors.centerIn: parent;
                    text: "(disabled)" + (mapsFps.usePluginFps ? "plugin FPS" : "qml FPS");
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        // todo: enable when ready
                        //mapsFps.usePluginFps = !mapsFps.usePluginFps;
                        //fpsCounter.reset();
                    }
                    onPressed: {parent.color = "lightgray";} // change to #AAA when ready
                    onReleased: {parent.color = "lightgray";}
                }
            }
            */
            Text {
                font.bold: true;
                text: "Map Animation:";
            }
            Rectangle {
                color: (mapsFps.animation == Map.ANIMATION_BOW ? "#00F" : "lightgray");
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    color: mapsFps.animation == Map.ANIMATION_BOW ? "#FFF" : "#000";
                    anchors.centerIn: parent;
                    text: "Bow";
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        mapsFps.animation = Map.ANIMATION_BOW;
                    }
                }
            }
            Rectangle {
                color: (mapsFps.animation == Map.ANIMATION_LINEAR ? "#00F" : "lightgray");
                border.color: "#AAA"; border.width: 5;
                height: mapsFps.btnSize; width: parent.width;
                Text {
                    anchors.centerIn: parent;
                    text: "Linear";
                    color: mapsFps.animation == Map.ANIMATION_LINEAR ? "#FFF" : "#000";
                }
                MouseArea {
                    anchors.fill: parent;
                    onClicked: {
                        mapsFps.animation = Map.ANIMATION_LINEAR;
                    }
                }
            }
        }

    }

    GeoCoordinates {
        id: geo;
        property bool pointa: true;
        latitude: pointa ? 52.506472 : 52.523405
        longitude: pointa ? 13.332371 : 13.4114
    }
    function testFps() {
        fpsCounter.isRunning = false;
        if(mapsFps.usePluginFps) {
            // todo: implement when available from plugin
        } else {
            var mapChoice = mapsFps.testMapCon ? mapConnector : mapPlugin;

            mapsFps.pannable = false;
            fpsCounter.isRunning = true;

            mapChoice.moveTo(geo,
               mapsFps.animation,
               Map.PRESERVE_ZOOM_SCALE,
               Map.PRESERVE_ORIENTATION,
               Map.PRESERVE_PERSPECTIVE
            );
            geo.pointa = !geo.pointa;
        }
    }

    FpsCounter{
        id: fpsCounter;
    }

}
