import QtQuick 1.0
import MapsPlugin 1.0


Rectangle {
    id: positioningDataScene;
    color: "#f0f1f2";
    anchors.fill: parent;

    PositionProvider {
        id: positionProvider;
        enabled: false;

        onPositionUpdated: {
            log.info("Position provider :: position updated in PositioningData in MORE");
        }
    }

    property string no_data: "unknown";

    property variant geoPosition: positionProvider.hasValidPosition ? positionProvider.position : null;
    property variant mapMatchedPosition: positionProvider.mapMatchedPosition;
    property variant devicePosition: positionProvider.devicePosition;
    property variant positionMethod: positionProvider.positionMethod;
    property int invalidValue: Position.UNKNOWN_VALUE;

//    Component.onCompleted: { positionProvider.enabled = true; }
    //Component.onDestruction: { positionProvider.enabled = false; }

    Column {
        width: parent.width;
        spacing: 4;
        anchors.centerIn: parent

        DebugListItem {
            description: "Valid position: " + positionProvider.hasValidPosition;
        }
        DebugListItem {
            description: "Latitude: " + (geoPosition ? geoPosition.geoCoordinates.latitude + "°" : no_data);
        }
        DebugListItem {
            description: "Longitude: " + (geoPosition ? geoPosition.geoCoordinates.longitude + "°" : no_data);
        }
        DebugListItem {
            description: "Map matched position " + (mapMatchedPosition ? ("lat: " + (mapMatchedPosition.geoCoordinates.latitude) + ", long: " + (mapMatchedPosition.geoCoordinates.longitude)) : no_data );
        }
        DebugListItem {
            description: "Device position " + (devicePosition ? ("lat: " + (devicePosition.geoCoordinates.latitude) + ", long: " + (devicePosition.geoCoordinates.latitude)) : no_data );
        }
        DebugListItem {
            description: "Accuracy: " + (geoPosition && geoPosition.accuracy != invalidValue ? Math.round(geoPosition.accuracy) + "m" : no_data)
                       + "," + (geoPosition && geoPosition.altitudeAccuracy != invalidValue ? Math.round(geoPosition.altitudeAccuracy) + "m" : no_data);
        }
        DebugListItem {
            description: "Speed: " + (geoPosition && geoPosition.speed != invalidValue ? Math.round(geoPosition.speed*3.6) +"km/h" : no_data);
        }
        DebugListItem {
            description: "Average speed: " + Math.round(positionProvider.averageSpeed*3.6) + "km/h";
        }
        DebugListItem {
            description: "Direction: " + (geoPosition && geoPosition.direction != invalidValue ? Math.round(geoPosition.direction) + "°" : no_data);
        }
        DebugListItem {
            description: "Positioning method: " + posMethodIdToStr(positionMethod);
        }
        DebugListItem {
            description: "MapMatcherMode: " + mapMatcherModeToStr(positionProvider.mapMatcherMode);
        }
    }

    function posMethodIdToStr(positioningId) {
        switch(positionProvider.positionMethod) {
                case PositionProvider.METHOD_GPS: return "GPS";
                case PositionProvider.METHOD_NETWORK: return "WIFI/CELL";
                default: return "OFF";
         } // switch
    }

    function mapMatcherModeToStr(mapMatcherMode) {
        switch(positionProvider.mapMatcherMode) {
                case PositionProvider.MAPMATCH_PEDESTRIAN: return "Pedestrian";
                case PositionProvider.MAPMATCH_CAR: return "Car";
                default: return no_data;
         } // switch
    }

}



