import QtQuick 1.0

import com.nokia.meego 1.0

Rectangle {
    id: serverConfigurationScreen

    width: parent.width;
    height: parent.height

    ScrollDecorator {
        flickableItem: flickArea
    }

    Flickable {
        id: flickArea

        anchors.fill: parent
        contentWidth: parent.width;
        contentHeight: dataColumn.childrenRect.height + 4*dataColumn.anchors.margins
        clip: true

        Column {
            id: dataColumn

            spacing: 10
            anchors.fill: parent
            anchors.leftMargin: 10

            Label {
                text: "<h2>Server configuration</h2>"
            }

            // PLACES
            Column {
                spacing: 10;

                Label {
                    text: "<h3>Places</h3>"
                }

                Label {
                    text: "searchURI"
                }
                ButtonColumn {
                    id: placesSearch

                    onCheckedButtonChanged: {
                        log.info("Changed settings for places/searchURI to "+checkedButton.identifier)
                        appSettings.setValue("places/searchURI",checkedButton.identifier)
                    }

                    RadioButton {
                        id: searchProduction
                        property string identifier: "http://where.desktop.mos.svc.ovi.com/NOSe/json"
                        Text { text: "production (http://where.desktop.mos.svc.ovi.com/NOSe/json)"; anchors.left: parent.right }
                    }
                }

                Label {
                    text: "suggestionsURI"
                }
                ButtonColumn {
                    id: placesSuggestions

                    onCheckedButtonChanged: {
                        log.info("Changed settings for places/suggestionsURI to "+checkedButton.identifier)
                        appSettings.setValue("places/suggestionsURI",checkedButton.identifier)
                    }

                    RadioButton {
                        id: suggProduction
                        property string identifier: "http://where.desktop.mos.svc.ovi.com/NOSe/json"
                        Text { text: "production (http://where.desktop.mos.svc.ovi.com/NOSe/json)"; anchors.left: parent.right; }
                    }
                }

                Label {
                    text: "placesURI"
                }
                ButtonColumn {
                    id: places

                    onCheckedButtonChanged: {
                        log.info("Changed settings for places/placesURI to "+checkedButton.identifier)
                        appSettings.setValue("places/placesURI",checkedButton.identifier)
                    }

                    RadioButton {
                        id: placesQA
                        property string identifier: "http://uc-stable-qa.places.public.devbln.europe.nokia.com/rest/v1/places/"
                        Text { text: "stable QA (http://uc-stable-qa.places.public.devbln.europe.nokia.com/rest/v1/places/)"; anchors.left: parent.right; }
                    }
                    RadioButton {
                        id: placesProduction
                        property string identifier: "http://places.maps.ovi.com/rest/v1/places/"
                        Text { text: "production (http://places.maps.ovi.com/rest/v1/places/)"; anchors.left: parent.right; }
                    }
                }

                Label {
                    text: "recommendationsURI"
                }
                ButtonColumn {
                    id: placesRecomendations

                    onCheckedButtonChanged: {
                        log.info("Changed settings for places/recommendationsURI to "+checkedButton.identifier)
                        appSettings.setValue("places/recommendationsURI",checkedButton.identifier)
                    }

                    RadioButton {
                         id: recoDev
                         property string identifier: "http://ci-new-data.search.devbln.europe.nokia.com/rest/v1/recommendations/nearby/"
                         Text { text: "dev (http://ci-new-data.search.devbln.europe.nokia.com/rest/v1/recommendations/nearby/)"; anchors.left: parent.right; }
                    }
                    RadioButton {
                        id: recoQA
                        property string identifier: "http://uc-stable-qa.places.public.devbln.europe.nokia.com/rest/v1/recommendations/nearby/"
                        Text { text: "stable QA (http://uc-stable-qa.places.public.devbln.europe.nokia.com/rest/v1/recommendations/nearby/)"; anchors.left: parent.right; }
                    }
                    RadioButton {
                        id: recoProduction
                        property string identifier: "http://avalon.mobile.recommendations.maps.svc.ovi.com/rest/v1/recommendations/nearby/"
                        Text { text: "production (http://avalon.mobile.recommendations.maps.svc.ovi.com/rest/v1/recommendations/nearby/)"; anchors.left: parent.right }
                    }
                }

                Label {
                    text: "shareBaseURI"
                }
                ButtonColumn {
                    id: placesShare

                    onCheckedButtonChanged: {
                        log.info("Changed settings for places/shareBaseURI to "+checkedButton.identifier)
                        appSettings.setValue("places/shareBaseURI",checkedButton.identifier)
                    }
                    RadioButton {
                        id: shareProduction
                        property string identifier: "http://m.ovi.me"
                        Text { text: "production (http://m.ovi.me)"; anchors.left: parent.right }
                    }
                }

            }

            // the rest
            Column {
                spacing: 10;
                Item {
                    anchors {
                        left: parent.left
                        right: parent.right
                    }
                    height: npsLabel.height + Math.max(nps.height, npsValue.height)

                    Label {
                        id: npsLabel
                        text: "<h3>NPS feedback</h3>"
                    }
                    ButtonColumn {
                        id: nps
                        anchors.top: npsLabel.bottom
                        anchors.left: parent.left
                        width: parent.width*.3
                        anchors.topMargin: 10
                        onCheckedButtonChanged: {
                            log.info("Changed settings for nps/feedbackEnv to "+checkedButton.identifier)
                            appSettings.setValue("nps/feedbackEnv",checkedButton.identifier)
                        }
                        RadioButton {
                            id: it
                            property string identifier: "it"
                            property string uri: "http://cpqit.nokia.com"
                            property string name: "Ovi Maps QML"
                            property string npsid: "6031"
                            property string version: "PR1.0"
                            property string sourceId: "2972826290b8a5a7db841f480d6cb32d"
                            Text { text: "IT"; anchors.left: parent.right }
                        }
                        RadioButton {
                            id: qa
                            property string identifier: "qa"
                            property string uri: "http://cpqqa.nokia.com"
                            property string name: "Ovi Maps QML"
                            property string npsid: "2306"
                            property string version: "PR1.0"
                            property string sourceId: "2972826290b8a5a7db841f480d6cb32d"

                            Text { text: "QA"; anchors.left: parent.right }
                        }
                        RadioButton {
                            id: prod
                            property string identifier: "prd"
                            property string uri: "http://cpq.nokia.com"
                            property string name: "Ovi Maps QML"
                            property string npsid: "2008"
                            property string version: "PR1.0"
                            property string sourceId: "2972826290b8a5a7db841f480d6cb32d"

                            Text { text: "Production"; anchors.left: parent.right }
                        }
                    }
                    Column {
                        id: npsValue
                        anchors.left:  nps.right
                        anchors.leftMargin: 20
                        anchors.top: nps.top
                        anchors.topMargin: 10
                        Text { text: "Configuration for " + nps.checkedButton.identifier; font.weight: Font.Bold}
                        Text { text: "<b>Uri</b>: " + nps.checkedButton.uri; elide: Text.ElideRight }
                        Text { text: "<b>Name</b>: " + nps.checkedButton.name; elide: Text.ElideRight }
                        Text { text: "<b>ID</b>: " + nps.checkedButton.npsid; elide: Text.ElideRight }
                        Text { text: "<b>Version</b>: " + nps.checkedButton.version; elide: Text.ElideRight }
                        Text { text: "<b>Source Id</b>: " + nps.checkedButton.sourceId; elide: Text.ElideRight }
                    }
                }

                Label {
                    text: "<h3>Syncshare/SSO server environment</h3>"
                }
                ButtonColumn {
                    id: sso

                    onCheckedButtonChanged: {
                                           log.info("Changed settings for sso/ssoType to "+checkedButton.identifier)
                                           appSettings.setValue("sso/ssoType",checkedButton.identifier)
                    }

                    RadioButton {
                        id: ssoDevelopment
                        property string identifier: "dev"
                        Text { text: "development"; anchors.left: parent.right; }
                    }
                    RadioButton {
                        id: ssoTesting
                        property string identifier: "staging"
                        Text {
                            text: "testing";
                            anchors.left: parent.right;
                            id: ssoTestingText
                        }
                    }
                    RadioButton {
                        id: ssoProduction
                        property string identifier: "production"
                        Text { text: "production"; anchors.left: parent.right; }
                    }
                }

                Label {
                    text: "<h3>ODML</h3>"
                }
                ButtonColumn {
                    id: odml

                    onCheckedButtonChanged: {
                                           log.info("Changed settings for odml/odmlBaseURI to "+checkedButton.identifier)
                                           appSettings.setValue("odml/odmlBaseURI",checkedButton.identifier)
                    }

                    RadioButton {
                        id: odmlProduction
                        property string identifier: "http://static.maploader.maps.svc.ovi.com/map5/"
                        Text { text: "production (http://static.maploader.maps.svc.ovi.com/map5)"; anchors.left: parent.right; }
                    }
                }

            }

            Label {
                text: "<h5>Note: Values are saved directly.<br>Application needs to restart to take effect.</h5>"
            }

        }
    }

    Component.onCompleted: {
        log.info("loading debug server settings done")

        // set radiobuttons to current values

        log.info("searchURI: "+ searchURI);
        if(searchURI == searchProduction.identifier)
            placesSearch.checkedButton = searchProduction

        log.info("suggestionsURI: "+ suggestionsURI);
        if(suggestionsURI == suggProduction.identifier)
            placesSuggestions.checkedButton = suggProduction;

        log.info("placesURI: "+ placesURI);
        if(placesURI == placesQA.identifier)
            places.checkedButton = placesQA
        else if(placesURI == placesProduction.identifier)
            places.checkedButton = placesProduction

        log.info("recommendationsURI: "+ recommendationsURI);
        if(recommendationsURI == recoQA.identifier)
            placesRecomendations.checkedButton = recoQA
        else if(recommendationsURI == recoDev.identifier)
            placesRecomendations.checkedButton = recoDev
        else if(recommendationsURI == recoProduction.identifier)
            placesRecomendations.checkedButton = recoProduction

        log.info("shareBaseURI: "+ shareBaseURI);
        if(shareBaseURI == shareProduction.identifier)
            placesShare.checkedButton = shareProduction

        log.info("odmlBaseURI: "+ odmlBaseURI);
        if(odmlBaseURI == odmlProduction.identifier)
            odml.checkedButton = odmlProduction

        log.info("ssoType: "+ ssoType);
        if(ssoType == ssoDevelopment.identifier)
            sso.checkedButton = ssoDevelopment
        else if(ssoType == ssoTesting.identifier)
            sso.checkedButton = ssoTesting
        else if(ssoType == ssoProduction.identifier)
            sso.checkedButton = ssoProduction

        log.info("feedbackEnv: " + feedbackEnv);
        if(feedbackEnv === it.identifier)
            nps.checkedButton = it;
        else if(feedbackEnv === qa.identifier)
            nps.checkedButton = qa;
        else if(feedbackEnv === prod.identifier)
            nps.checkedButton = prod;
    }
}
