import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0
import ovi.connector.places 1.0

Page {
    id: settingsPage

    PlacesCommonList {
        id: settingsList
        itemDelegate: settingDelegate
        model: settingsModel
        anchors {
            top: header.bottom
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
    }

    Component {
        id: settingDelegate
        Item {
            anchors {
                left: parent.left
                right: parent.right
                leftMargin: 20
                rightMargin: 10
            }
            height: 75
            MouseArea {
                id: mouse
                anchors.fill: parent
            }
            Item {
                anchors.fill: parent
                MapsText {
                    id: settingTitle
                    text: settingName
                    state: "listNormalBold"
                    anchors {
                        left: parent.left
                        right: parent.right
                        rightMargin: toggle.visible ? toggle.width : 0
                        verticalCenter: parent.verticalCenter
                    }
                }
                Switch {
                    id: toggle
                    visible: toggleChecked !== undefined
                    anchors {
                        right: parent.right
                        verticalCenter: parent.verticalCenter
                    }
                    checked: !!toggleChecked
                    property bool toggleChecked: (tag == "showZoomBar")?
                        mainscreen.zoombarsEnabled :
                        (tag == "enableAutomaticRotating")?
                            mainscreen.enableAutomaticRotating :
                            (tag == "enableMagic3d2d")?
                                mainscreen.enableMagic3d2d :
                                (tag == "onlineOfflineToggle")?
                                    mainscreen.onlineMode : undefined
                    onToggleCheckedChanged: { checked = !!toggleChecked; }

                    onCheckedChanged: {
                        if (tag === "showZoomBar") {
                            appState.zoombarEnabled = checked;
                        } else if (tag === "enableAutomaticRotating") {
                            appState.fingerRotationEnabled = checked;
                        } else if (tag === "enableMagic3d2d") {
                            appState.slide3dEnabled = checked;
                        } else if (tag === "onlineOfflineToggle") {
                            appState.isOnline = checked;
                            mainscreen.changeOnlineMode(checked);
                            checked = !!toggleChecked;
                            if (settingsPage.visible) {
                                if (!checked) {
                                    console.log("Switching to Offline mode - will close the network session");
                                    appState.closeSession();
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    Header {
        id: header
        text: qsTranslate("Maps Settings", "qtn_maps_deviceSettings_title");
        dropShadow: false
    }

    ListModel {
        id: settingsModel
    }
    Component.onCompleted: {
        settingsModel.append({
            settingName: qsTranslate("Maps Settings", "qtn_maps_deviceSettings_connection_online"),
            tag: "onlineOfflineToggle"
        });
        settingsModel.append({
            settingName: qsTranslate("Maps Settings", "qtn_maps_deviceSettings_zoom_bar"),
            tag: "showZoomBar"
        });
        settingsModel.append({
            settingName: qsTranslate("Maps Settings", "qtn_maps_deviceSettings_zoom_gesture"),
            tag: "enableAutomaticRotating"
        });
//        settingsModel.append({
//            settingName: qsTranslate("Maps Settings", "qtn_maps_settings_slide3d"),
//            tag: "enableMagic3d2d"
//        });
    }

}
