import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector.places 1.0
import ovi.connector.favorites 1.0

import "Nearby"
import "Nearby/NearbyLogic.js" as Logic

//Nearby.qml
// This is the launcher for the Nearby Scene
Page {
    id: nearbyScene;

    property variant nearbyModel: ListModel {}  // The model that contains all the entries shown on the map or list
    property variant viewObject;                // used to pass the object from the map or list to the places viewer
    property variant model;
    property variant allSearches: [favoritesModel, recModel];// Note: if a favorite is the same as a recommendation, the icon on the map will be the first listed in this array
    property variant lastSearchPosition;        // The last position searched

    property int numResultsShown: 2;            // The number of results to show for each section. Does not include "More" item.
    property int numMaxResultsShown: 25;            // The number of results to show for each section - max

    property int refreshDistance: 3000;         // The distance to refresh if the center of the map has moved since the last refresh
    property int validFavoriteType: 4;          // Type should be a FavoriteListItem value for setting the type in NearbyFlexibleDelegate
    property int reco_search_radius: 3;
    property int reco_max_results: 25;

    property bool mapShowErr: false
    property bool loadingDone: false;
    property bool showLoadedResults: true;
    property bool favoriteOpened: false

    property string mapErrMessage: "";

    property bool ready: false

    function clearMap(){
        nearbyStart.nearbyMap.clearPlacesOnMap();
        nearbyStart.nearbyMap.map.model.clear();
    }

    onStatusChanged: {
        if (status == PageStatus.Activating) {
            showLoadedResults = true;
            var centerPos = mainscreen.mapSharedData.currentCenter.clone(mainscreen);
            if (Logic.shouldRefresh(centerPos)) {
                lastSearchPosition = {
                    latitude: centerPos.latitude,
                    longitude: centerPos.longitude
                };

                clearMap();
                Logic.prepareModelsForQuery();
                Logic.refreshNearbyData(centerPos);
            }
            nearbyStart.nearbyMap.map.model.setObjectsOwner("nearby", false);
        } else if (status == PageStatus.Deactivating && !loadingDone) {
            showLoadedResults = false;
        }
    }

    PlacesRecommendationsList {
        id: recModel;

        property string type: "_recs"
        property string categoryName: qsTranslate("Maps", "qtn_maps_nearby_heading_recommendations");
        property string noResults: qsTranslate("Maps", "qtn_maps_nearby_noPlacesAround");
        property string moreString: qsTranslate("Maps", "qtn_maps_nearby_more");
        property string queryErrorString: qsTranslate("Maps", "qtn_maps_nearby_error");
        property string hideString: qsTranslate("Maps", "qtn_maps_nearby_hide");
        property string queryServerErrorString: qsTranslate("Maps", "qtn_maps_nearby_server_error");
        property string errorOffline: qsTranslate("Maps", "qtn_maps_nearby_offline_recs");
        property string message
        property bool error: false;
        property bool ready: false;
        property variant visualProperties: {
                                                showOnMap: true,
                                                showNoResults: true,
                                                showNoResultsBubble: true,
                                                showLoading: true
                                            }
        onModelUpdated: {
            Logic.updateNearbyModels(recModel, numResultsShown, err);
            nearbyStart.addModelToMap(recModel);
        }
    }

    PlacesRecommendationsList {
        id: ptModel;

        property string type: "_pt"
        property string categoryName: qsTranslate("Maps", "qtn_maps_nearby_heading_publictransport");
        property string noResults: qsTranslate("Maps", "qtn_maps_nearby_noresult_pt");
        property string moreString: qsTranslate("Maps", "qtn_maps_nearby_more");
        property string hideString: qsTranslate("Maps", "qtn_maps_nearby_hide");
        property string queryErrorString: qsTranslate("Maps", "qtn_maps_nearby_error");
        property bool error: false;
        property bool ready: false;
        property string message
        property variant visualProperties: {
            showOnMap: false,
            showNoResults: false,
            showNoResultsBubble: false,
            showLoading: false
        }
        onQueryFinished: {
            Logic.updateNearbyModels(ptModel, numResultsShown, false);
        }
    }

    PlacesConnectedList {
        id: favoritesModel

        property string type: "_favorites"
        property string categoryName: qsTranslate("Maps", "qtn_maps_nearby_heading_favourites");
        property string noResults: qsTranslate("Maps", "qtn_maps_nearby_noresults_favs");
        property string moreString: qsTranslate("Maps", "qtn_maps_nearby_more");
        property string hideString: qsTranslate("Maps", "qtn_maps_nearby_hide");
        property string queryErrorString: qsTranslate("Maps", "qtn_maps_general_notimplemented");
        property bool error: false;
        property bool ready: false;
        property string message
        property variant visualProperties: {
            showOnMap: true,
            showNoResults: false,
            showNoResultsBubble: false,
            showLoading: false
        }

        onModelUpdated: {
            Logic.updateNearbyModels(favoritesModel, numResultsShown, false);
            nearbyStart.addModelToMap(favoritesModel);
        }
    }

    FavoritesConnectedList {
        id: actualFavoritesModel
        onModelUpdated: {
            favoritesModel.clear();
            var item, addToNearbyModel;
            for (var i = 0; i < actualFavoritesModel.count; i++) {
                item = actualFavoritesModel.get(i);
                if (item.storeObjectProperties && item.storeObjectProperties.position) {
                    addToNearbyModel = Logic.isPointWithinRadius(item.storeObjectProperties.position, refreshDistance, {
                        latitude: mainscreen.mapSharedData.currentCenter.latitude,
                        longitude: mainscreen.mapSharedData.currentCenter.longitude
                    });
                    if (addToNearbyModel) {
                        favoritesModel.append(item.getPlace());
                    }
                }
            }
            favoritesModel.modelUpdated();
        }
    }

    NearbyMain {
        id: nearbyStart;
        showMap: true;

        ptModelData: ptModel
        recModelData: recModel
        favModelData: favoritesModel

        showResultsWhenLoaded: showLoadedResults;
        onRefreshClicked: {
            clearMap();
            Logic.refreshNearbyData(mainscreen.mapSharedData.currentCenter);
        }
        onViewPlace: {
            viewObject = object;
            model = modelData;
            nearbyScene.favoriteOpened = false;
            if (viewObject.isFavorite && !!viewObject.favorite) {
                nearbyScene.favoriteOpened = !!viewObject.favorite.isFavorite;
            }
            pageStack.push(viewer);

            var centerPos = mainscreen.mapSharedData.currentCenter;
            lastSearchPosition = {
                latitude: centerPos.latitude,
                longitude: centerPos.longitude
            };
        }

        Component.onCompleted: {
            Logic.refreshNearbyData();
        }

    }

    Component {
        id: viewer

        PlacesViewer {
            mapsApp: mainscreen
            anchors.fill: parent
            object: nearbyScene.viewObject
            openFromFavorite: nearbyScene.favoriteOpened
            model: nearbyScene.model
        }
    }

    Component.onCompleted: {
        ready = true;
    }
}
