import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector.places 1.0
import ovi.connector.favorites 1.0

PlacesCommonItemDelegate {
    id: flexDelegate

    signal selectNearbyItem(int index);
    signal selectMore(int index);
    signal selectHide();
    signal longPressedItem(int index);

    property int offset: 10;

    // types:
    // 0: No data
    // 1: Loading
    // 2: PlaceListItem
    // 3: More button
    // 4: FavoriteListItem
    // 5: Hide Button
    function getDelegateOfType(typeIndex) {
        if(typeIndex == 0) {
            return noDataItem;
        } else if(typeIndex == 1) {
            return loadingItem;
        } else if(typeIndex == 2) {
            return validPlaceItem;
        } else if(typeIndex == 3) {
            return moreItem;
        } else if(typeIndex == 5) {
            return hideItem;
        }
    }

    Component {
        id: noDataItem
        NearbyItem {
            Text {
                text: message ? message : "...";
                anchors { left: parent.left; leftMargin: offset; verticalCenter: parent.verticalCenter; }
                font { pixelSize: 22; weight: Font.Light; letterSpacing: -1 }
                color: "#505050"
            }
        }
     }

    Component {
        id: loadingItem
        NearbyItem {
            BusyIndicator {
                id: busy;
                running: true;
                anchors { left: parent.left; leftMargin: offset }
                y: flexDelegate.height/3;
            }
        }
    }

    Component {
        id: validPlaceItem;
        PlaceListItem {
            color: "#f0f1f2"
            onItemSelected: { selectNearbyItem(index); }
            onItemLongPressed: { longPressedItem(index); }
        }
        // note: can get place info from object.properties
        // object is the same as in PlaceListItem, or from the model as model.get(index).object.properties.addrStreetName or whatever
    }

    Component {
        id: moreItem
        NearbyItem {
            Button {
                anchors.centerIn: parent;
                text: message;
                onClicked: selectMore(index)
            }
        }
    }

    Component {
        id: hideItem
        NearbyItem {
            Button {
                anchors.centerIn: parent;
                text: message
                onClicked: selectHide()
            }
        }
    }

    Loader {
        id: delItem;
        anchors.fill: parent;
        sourceComponent: getDelegateOfType(type);
    }
}
