import QtQuick 1.0

import com.nokia.meego 1.0

Flickable {
    id: root
    signal nearbyPlacePicked(variant object, variant modelData)

    property variant recModel
    property variant ptModel
    property variant favModel

    property int _recIndex: 0
    property int _ptIndex: 1
    property int _favIndex: 2
    property int _speed: 300

    contentWidth: parent.width
    contentHeight: colContent.height

    flickableDirection: Flickable.VerticalFlick

    clip: true

    Column {
        id: colContent
        anchors { left: parent.left; right: parent.right}

        Repeater {
            id: sectionRepeater
            model: [recModel, ptModel, favModel]
            delegate: NearbyCategoryComponent {
                id: section
                model: modelData
                onSelectNearbyPlace: nearbyPlacePicked(object, modelData)
                onZoomToTop: {
                    root.interactive = false;
                    boundsTimer.stop();
                    _speed = modelData.count > 0? modelData.count * 80 : 100;
                    root.contentY = section.y;
                    boundsTimer.start();
                }
            }
        }
    }

    function modelUpdated(aIndex, aModel) {
        sectionRepeater.model[aIndex] = aModel;
    }

    function closeAllSections() {
        var del;
        for (var i = 0; i<sectionRepeater.count; i++) {
            del = sectionRepeater.itemAt(i);
            del.showFullList = false;
            del.state = "";
        }
    }

    Behavior on contentY {
        NumberAnimation { duration: _speed }
    }

    Timer {
        id: boundsTimer;
        interval: _speed+20
        onTriggered: { root.returnToBounds(); root.interactive = true; }
    }

    onRecModelChanged: modelUpdated(_recIndex, recModel);
    onPtModelChanged: modelUpdated(_ptIndex, ptModel);
    onFavModelChanged: modelUpdated(_favIndex, favModel);
}
