function updateNearbyModels(aSubModel, aNumResults, aError) {
    aSubModel.ready = true;
    aSubModel.error = !!aError;
    if (aSubModel.count == 0) {
        log.warning("EMPTY submodel: "+aError+" with: "+aSubModel.queryErrorString);

        var errorMessage = aSubModel.noResults;

        if (aError) switch (aError.status) {
            case "408": errorMessage = aSubModel.queryServerErrorString; break;
            case "503": errorMessage = aSubModel.errorOffline; break;
            default: errorMessage = aSubModel.queryErrorString;
        }
        aSubModel.message = errorMessage;
        if (aSubModel.visualProperties.showNoResultsBubble) {
            nearbyScene.mapErrMessage = errorMessage;
        }
    }
    nearbyScene.loadingDone = /*favoritesModel.ready &&*/ ptModel.ready && recModel.ready;
}

function isPointWithinRadius(pos, aRadius, aPoint) {
    var dist = computeDistance(
            aPoint,
            {latitude: pos.latitude, longitude: pos.longitude}
        );
    return (dist <= aRadius);
}

function clearAllModels() {
//    clearModel(favoritesModel);
    clearModel(ptModel);
    clearModel(recModel);
}
function clearModel(model){
    // Ensure that all objects are deleted
    for (var i = model.count - 1; i >= 0; i--) {
        var tmp = model.get(i);
        if (!tmp.isFavorite) {
            model.get(i).destroy();
            tmp.destroy();
        } else {
            log.info("will keep the place as it's favorite (NEARBY-LOGIC)...................     " + tmp)
        }
    }
    model.clear();
}
function setModelsReady(aReady) {
    favoritesModel.ready = aReady;
    ptModel.ready = aReady;
    recModel.ready = aReady;
}

function prepareModelsForQuery() {
    clearAllModels();

    recModel.error = false;
    ptModel.error = false;
    favoritesModel.error = false;
    nearbyScene.mapErrMessage = "";

    setModelsReady(false);
    nearbyScene.loadingDone = false;
}

function refreshNearbyData(centerPos) {
    nearbyStart.updateHeaderAddress();
    prepareModelsForQuery();

    if(!centerPos){
        log.warning("Could not get current center position for recomendations request! Ignoring...");
    }

    // TODO: Memory leak for connecting function - please fix it if u see it :)
    // This doesnt work at all anyway... removing
    /*recModel.queryFailed.connect(function (error) {
        updateNearbyModels(recModel, numResultsShown, true);
    });*/

    var boundingBox = mainscreen.mapSharedData.activeMap?
        mainscreen.mapSharedData.activeMap.getMapBoundingBox():
        {
             edgeTopLeft: { latitude: 0, longitude: 0 },
             edgeTopRight: { latitude: 0, longitude: 0 },
             edgeBottomRight: { latitude: 0, longitude: 0 },
             edgeBottomLeft: { latitude: 0, longitude: 0 }
         }
    ;

    recModel.query({
        type: "RECO",
        latitude: centerPos? centerPos.latitude : null,
        longitude: centerPos? centerPos.longitude : null,
        radius: reco_search_radius,
        total: reco_max_results,
        boundingBox: boundingBox ? {
            topLeft: { latitude: boundingBox.edgeTopLeft.latitude, longitude: boundingBox.edgeTopLeft.longitude },
            bottomRight: { latitude: boundingBox.edgeBottomRight.latitude, longitude: boundingBox.edgeBottomRight.longitude}
        } : undefined,
        useBoundingBox: mainscreen.enableRecommendationsBoundingBox && mainscreen.mapSharedData.activeMap
    });


    ptModel.query({
        type: "PT",
        latitude: centerPos.latitude,
        longitude: centerPos.longitude,
        radius: reco_search_radius,
        total: reco_max_results
    });

    actualFavoritesModel.query({
        latitude: centerPos.latitude,
        longitude: centerPos.longitude
    });
}



/**
  * Determines if the nearby scene needs to be refreshed.
  * It is refreshed if:
  *     - It is the first time Nearby has loaded
  *     - The center of the map has changed since last time
  *     - Someone else owns the map model content
  */
function shouldRefresh(aCurrentPos) {
    return ((nearbyStart.nearbyMap.map.model.getObjectsOwners() != "nearby") ||
           !lastSearchPosition ||
           (lastSearchPosition.latitude !== aCurrentPos.latitude) ||
           (lastSearchPosition.longitude !== aCurrentPos.longitude));
}


/**
     * Computes the distance between two geo locations in meters.
     *
     * @param {Object} from the first geo location
     * @param {Number} from.longitude the longitude of the geo location.
     * @param {Object} to the second geo location
     * @param {Number} to.latitude the latitude of the geo location.
     * @param {Number} to.longitude the longitude of the geo location.
     * @return {Number} the distance in meters
     */
function computeDistance(from, to) {
    var PI_OVER_180 = Math.PI / 180,
        thetaA = from.latitude * PI_OVER_180,
        phiA = from.longitude * PI_OVER_180,
        cosThetaA = Math.cos(thetaA),
        ax = cosThetaA * Math.sin(phiA),
        ay = cosThetaA * Math.cos(phiA),
        az = Math.sin(thetaA),
        thetaB = to.latitude * PI_OVER_180,
        phiB = to.longitude * PI_OVER_180,
        cosThetaB = Math.cos(thetaB),
        bx = cosThetaB * Math.sin(phiB),
        by = cosThetaB * Math.cos(phiB),
        bz = Math.sin(thetaB);
    return 6371e3 * Math.acos(ax*bx + ay*by + az*bz);
}
