import QtQuick 1.0

import com.nokia.meego 1.0
import ovi.connector.map 1.0

import ovi.connector.places 1.0

Rectangle {
    id: nearby;
    anchors.fill: parent;

    signal viewPlace(variant object, variant modelData);
    signal refreshClicked();
    signal mapPositionChanged
    signal back();

    property bool showResultsWhenLoaded: true;
    property alias showMap: mapBtn.mapActive                      // Toggle list and map view
    property alias nearbyMap: nearbyMapView;
    property variant modelsToUpdate:[];

    property variant ptModelData
    property variant recModelData
    property variant favModelData

    function updateHeaderAddress() {
        nearbyMap.updateHeaderAddress();
    }

    function addModelToMap(aModel) {
//        // This strange conctruct is to workaround problem with not storing data in variant properties

//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- ");
//        log.info(" ---------------------------- NearbyMain::addModelToMap: " + aModel.type);
//        var found = false;
//        for (var i=0; !found && i<modelsToUpdate.length; i++) {
//            log.info(" ------------------------ modelsToUpdate["+i+"].type: " + modelsToUpdate[i].type);
//            if (modelsToUpdate[i].type === aModel.type) {
//                modelsToUpdate[i] = aModel;
//                found = true;
//            }
//        }
//        if(!found) {
//            modelsToUpdate = modelsToUpdate.concat([aModel]);
//        }
//        log.info(" now putting resuts on map");
//        nearbyMap.putResultsOnMap(modelsToUpdate);

        if (aModel !== favModelData || nearbyMap.map.model.checkForOwner("nearby")) {
            nearbyMap.putResultsOnMap([favModelData, recModelData]);
        }
    }

    Rectangle {
        id: rootBackground
        anchors.fill: parent
        color: "#f0f1f2"
    }

    Header {
        id: header
        text: qsTranslate("Maps", "qtn_maps_nearby_title")          // Nearby
        subText: qsTranslate("Maps", "qtn_maps_nearby_loading");    // Loading, please wait...
        textWidth: parent.width - buttons.width
        dropShadow: mapBtn.mapActive

        Item {
            id: buttons;
            height: parent.height;
            width: mapBtn.width + refreshBtn.width + 16;
            anchors.right: parent.right;
            anchors.rightMargin: 16;
            anchors.verticalCenter: parent.text.verticalCenter
            MapListToggle {
                id: mapBtn;
                anchors.right: parent.right;
                anchors.verticalCenter: parent.verticalCenter;
                anchors.verticalCenterOffset: -1//-10 / 2
            }

            Button {
                id: refreshBtn;
                height: 50; width: 50;
                anchors.right: mapBtn.left;
                anchors.rightMargin: 6;
                anchors.verticalCenter: parent.verticalCenter;
                anchors.verticalCenterOffset: -1//-10 / 2
                enabled: nearbyScene.loadingDone;
                onClicked: {
                    nearbyListView.closeAllSections();
                    refreshClicked();
                    header.subText = qsTranslate("Maps", "qtn_maps_nearby_loading");
                    nearbyMap.updateHeaderAddress();
                }
                Image {
                    anchors.centerIn: parent;
                    source: parent.enabled ? "../../images/icon-refresh" + (parent.pressed ?"_light.png":"_dark.png") : "";
                }

                BusyIndicator {
                    id: refreshingIndicator;
                    anchors.centerIn: parent;
                    visible: !refreshBtn.enabled;
                    running: true;
                    height: 32; width: 32;
                }
            }
        }

        z: 1
    }

    NearbyMap {
        id: nearbyMapView;
        visible: showMap;
        showResultsWhenLoaded: nearby.showResultsWhenLoaded

        anchors { top: header.bottom; bottom: parent.bottom; left: parent.left; right: parent.right }

        onSelectNearbyPlace: {
            viewPlace(object, nearbyListView.recModel);
        }
        onMapReverseGeoCode: {
            if (nearbyMap.__map.mapSharedData.state == "default" && !nearbyMap.__map.currentPosition && !nearbyMap.__map.previousPosition) {
                header.subText = "";//qsTranslate("Maps", "qtn_maps_nearby_first_time_welcome_message");
            } else {
                if (contextData) {
                    header.subText = placeHelper.formatAddress({addrCountryCode: contextData["ADDR_COUNTRY_CODE"], addrCountryName: contextData["ADDR_COUNTRY_NAME"], addrCityName: contextData["ADDR_CITY_NAME"], addrStreetName: contextData["ADDR_STREET_NAME"], addrHouseNumber: contextData["ADDR_HOUSE_NUMBER"]});
                }
            }
        }
    }

    NearbyList {
        id: nearbyListView
        anchors {
            top: header.bottom
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }
        visible: !showMap
        favModel: favModelData
        ptModel: ptModelData
        recModel: recModelData

        onNearbyPlacePicked: viewPlace(object, modelData)
    }

    ScrollDecorator {
        flickableItem: nearbyListView
    }

    PlaceHelper { id: placeHelper }
}
