import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

import ovi.connector.map 1.0
import ovi.connector.places 1.0

import "NearbyMap.js" as JS

Rectangle {
    id: nearbyMapScene;
    signal selectNearbyPlace(variant object);
    signal mapReverseGeoCode(variant contextData);
    property variant models;
    property bool showmapShowErr;
    property variant aModelWaiting
    property alias map: mapView
    property bool showResultsWhenLoaded: true

//    ListModel {id: placesOnMap}

    function clearPlacesOnMap(){
        JS.placesOnMap = [];
    }

    property MapViewer __map: mapView
    MapViewer {

        id: mapView;
        mainScreen: mainscreen;
        useGlobalShare: true
        anchors.fill: parent;

        property string currentUriSelected

        onPlaceSelected: {
            mapView.currentUriSelected = uri;

            var found = false;
            for(var i=0; !found && i<JS.placesOnMap.length; i++) {
                found = JS.placesOnMap[i].place.uri == uri;
            }

            found && selectNearbyPlace(JS.placesOnMap[i-1].place);
        }

        onMouseDown: {
            mapView.currentUriSelected = "";
        }
        onCenterChanged: {
            mapView.currentUriSelected = "";
        }


        property int farDragPixelDistance: 200;

        onMapDraggedFar: {
            reverseGeoCode.find(mapView.center);
        }

        onReadyChanged: {
            if (mapView.ready && nearbyMapScene.aModelWaiting) {
                doTheMagic(nearbyMapScene.aModelWaiting);
                nearbyMapScene.aModelWaiting = null;
            }
        }
        onPositionSelected: {
            var place = placeFactory.createObject(nearbyMapScene);
            place.uri = currentPositionSelected? -1 : "";
            place.properties = positionData;

            if (!currentPositionSelected && mapSharedData && mapSharedData.currentMapObjectSelected &&
                    mapSharedData.currentMapObjectSelected.transitStopInfo &&
                    place.properties.geoLatitude === mapSharedData.currentMapObjectSelected.transitStopInfo.position.latitude &&
                    place.properties.geoLongitude === mapSharedData.currentMapObjectSelected.transitStopInfo.position.longitude
            ) {
                var placeInfo = mainscreen.placesCloudAdapter.getPtInfoFromStop(mapSharedData.currentMapObjectSelected.transitStopInfo, mapSharedData.currentCenter);
                place.publictransport = placeInfo.publictransport;
                place.properties = placeInfo.properties;
                place.categories = placeInfo.categories;
                place.favorite = place.getFavorite();
                place.isFavorite = !!place.favorite;
            }

            if (!__placesViewer) {
                __placesViewer = placesViewer.createObject(nearbyMapScene);
            }
            __placesViewer.object = place;
            tabGroup.currentTab.push(__placesViewer);
        }

    }

    property variant __placesViewer
    Component {
        id: placesViewer
        PlacesViewer {
        }
    }
    Component {
        id: placeFactory
        Place {
        }
    }

    ReverseGeoCodeFinder {
        id: reverseGeoCode

        mapView: mapView

        onFindPureDataComplete: {
            if (contextData) {
                mapReverseGeoCode(contextData);
            }
        }
    }

    TransitDatabase {
        id: transitDb
    }

    InfoBanner {
        id: errorMessage;
        text: nearbyScene.mapErrMessage;
        timerEnabled: true;
        timerShowTime: 3*1000;
    }

    Connections {
        target: nearbyScene
        ignoreUnknownSignals: true

        onMapErrMessageChanged: {
            if (nearbyScene.mapErrMessage && nearbyScene.mapErrMessage !== "") {
                errorMessage.show();
            }
        }
    }

    Component.onCompleted: {
        updateHeaderAddress();
    }

    function updateHeaderAddress() {
        reverseGeoCode.find(mainscreen.mapSharedData.currentCenter);
    }

    function doTheMagic(aModels) {
        if (!nearbyMapScene.showResultsWhenLoaded) { mapView.model.clear(); return; }

        var icon, item, tmpModel;

        mapView.model.clear();
        models = aModels;
        if (!aModels || aModels.length == 0) {
            errorMessage.scale === 0 && errorMessage.show();
            return;
        }

        mapView.lazyAppendIconMode = true;
        for (var m=0; m<aModels.length; m++) {
            tmpModel = aModels[m];

            for (var i=0; tmpModel && i< tmpModel.count; i++) {
                item = tmpModel.get(i);

                if (item && item.properties && item.properties.geoLatitude && item.properties.geoLongitude) {
                    if (!isAlreadyOnMap(item.uri, item.properties)) {
                        icon = mapView.model.appendIcon(
                            item.properties.geoLatitude,
                            item.properties.geoLongitude,
                            item.iconPath.replace("_list_", "_map_"),
                            {
                                uri: item.uri,
                                title: (item.favorite && item.favorite.text && item.favorite.text !== "") ? item.favorite.text : item.properties.title,
                                address: formatAddress(item.properties)
                            },
                            {}
                        );
                        if (item.uri == mapView.currentUriSelected) {
                            mapView.showInfoBubble(icon);
                        }
                        if (!item.uri || item.uri === "") {
                            item.uri = icon.data.uri;
                        }

                        JS.placesOnMap.push({"place": item});

                        if (!nearbyMapScene.showResultsWhenLoaded) { mapView.lazyAppendIconMode = false; mapView.model.clear(); return; }
                    }
                }

                //appState.processEvents();
            }
        }
        mapView.lazyAppendIconMode = false;
        mapView.model.setObjectsOwner("nearby", false);
    }

    function isAlreadyOnMap(uri, props) {
        var data;

        for(var i=0; i<mapView.model.count; i++) {
            data = mapView.model.get(i).data;
            if(data.uri === uri || (data.latitude === props.geoLatitude && data.longitude === props.geoLongitude)) {
                return true;
            }
        }
        return false;
    }

    /**
      * Puts the items in the list result model on the map.
      */
    function putResultsOnMap(aModels) {        
        if (mapView.ready) {
            doTheMagic(aModels);
        } else {
            nearbyMapScene.aModelWaiting = aModels;
        }
    }

    function formatAddress(properties) {
        var line = (properties.addrStreetName || "") + " "  + (properties.addrHouseNumber || "");
        var line2 = (properties.addrPostalCode || "") + " " + (properties.addrCityName || "");
        if (line != " " && line2 != " ") {
            line += ", " + line2;
        }
        if (line == " ") line = line2;
        return line
    }
}
