import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0

Item {
    id: onlineStatusMonitor

    anchors.fill: parent

    Connections {
        target: MapsPlugin
        ignoreUnknownSignals: true

        onDiskcacheUnmounted: {
            mainscreen.___clearAllStacksAndPrepareToDie();
            var objectFactory = Qt.createComponent("StartupErrorScreenContent.qml");
            var object = objectFactory.createObject(mainscreen);
        }
    }

    Connections {
        target: appState
        ignoreUnknownSignals: true

        onSessionOpened: {
            log.info("OnlineStatusMonitor opened")
            mainscreen.onlineMode = true;
            MapsPlugin.online = true;

            mainscreen.autoReplyReset();

            queryDialog.titleText = "connection opened"
            queryDialog.message = "connection opened, going to online mode"
            queryDialog.rejectButtonText = "close"
            //queryDialog.open();

            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIOther("SessionStart", {});
                analyticsComponent.logUUIDOther("SessionStart", {});
            }
        }
        onSessionClosed: {
            log.info("OnlineStatusMonitor closed")
            mainscreen.onlineMode = false;
            MapsPlugin.online = false;

            queryDialog.titleText = "connection closed"
            queryDialog.message = "connection closed, going to offline mode"
            queryDialog.rejectButtonText = "close"
            //queryDialog.open();

            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIOther("SessionEnd", {});
                analyticsComponent.logUUIDOther("SessionEnd", {});
            }
        }
        onSessionError: {
            log.error("OnlineStatusMonitor error")
            mainscreen.onlineMode = false;
            MapsPlugin.online = false;

            queryDialog.titleText = "connection error"
            queryDialog.message = "cannot connect, moving back to offline mode"
            queryDialog.rejectButtonText = "close"
            //queryDialog.open();
        }
        onSessionStateChanged: {
            log.trace("OnlineStatusMonitor state changed to " + state)

            // 0 - invalid
            // 1 - not available
            // 2 - connecting
            // 3 - connected
            // 4 - disconnecting
            // 5 - disconnected
            // 6 - operator roaming
            if (state == 5) {
                //mainscreen.changeOnlineMode(false);
                if (appState.isOnline && mainscreen.canWeAutoRetryOnlineMode) {
                    mainscreen.changeOnlineMode(true);
                    mainscreen.autoRetryApplied();
                }
            } else if (state == 3) {
                if (appState.isOnline) {
                    log.info("OnlineStatusMonitor connected...")
                    mainscreen.onlineMode = true;
                    MapsPlugin.online = true;
                }
            }
        }

        //NAC Analytics Tracking
        onMinimized: {
            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIActivity("Pause / Resume", { type: "minimized" } );
            }
        }
        onMaximized: {
            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIActivity("Pause / Resume", { type: "maximized" } );
            }
        }
        onWindowDeactivated: {
            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIActivity("Pause / Resume", { type: "windowDeactivated" } );
            }
        }
        onWindowActivated: {
            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.logIMEIActivity("Pause / Resume", { type: "windowActivated" } );
            }
        }
        onAboutToQuit: {
            if (typeof(analyticsComponent) != "undefined") {
                analyticsComponent.destruct();
            }
        }
    }

    QueryDialog {
        id: queryDialog
    }
}
