import Qt 4.7
import noasso 1.0

import "sso.js" as Logic

Item {
    id: sso

    state: ssoType

    property int server
    property string key
    property string secret
    property string syncShareUri
    property string serviceId
    property bool isUiShown: false
    property bool signedIn: false
    property string emailAddress
    property string oviAccount
    property string token
    property string noaAccountId

    function initialize () { Logic.initialize(noa); }
    function close () { noa.close(); }

    signal signInComplete(bool success)

    NoaInterface {
        id: noa

        signal signInComplete(bool success)

        function checkLogin () {
            log.info("Initialize NOA with public-key: " + key+ " and server: "+server);
            noa.initialize(key, secret, server, serviceId, {});
        }

        onInitializeResponse: Logic.onInitializeResponse(noa, status);
        onSignInResponse: Logic.onSignInResponse(noa, status, signInResponseData);
        onSignInComplete: sso.signInComplete(success)
    }

    Connections {
        target: noa
    }

    /*
     * 0 => TestServer is       https://efsun.itlase.com/rest/1.0/          // dev credentials don't work
     * 1 => QaServer is         https://nacci.nrln.net/rest/1.0/            // Not existing anymore! QA-Instance is now st-account.
     * 2 => ProductionServer is https://nokia.account.com/rest/1.0/         // hostname wrong (WTF)
     * 3 => StagingServer is    https://st-account.nokia.com/rest/1.0/ x    // staging server
     * 4 => BlrHaServer is      https://noaha.blrops.com/rest/1.0/ x
     * 5 => NabbiServer is      https://nabbi.noklab.com/rest/1.0/ x        // dev server
     */

    states: [
        State { 
            name: "dev"
            PropertyChanges {
                target: sso
                server: 5
                key: ssoKeyManager.getKey(server, false)
                secret: ssoKeyManager.getKey(server, true)
                syncShareUri: "http://dev-app.syncshare.public.devbln.europe.nokia.com:80/sync"
                serviceId: "Ovi"
            }
        },
        State { 
            name: "staging"
            PropertyChanges {
                target: sso
                server: 3
                key: ssoKeyManager.getKey(server, false)
                secret:  ssoKeyManager.getKey(server, true)
                syncShareUri: "https://mobile.stg.syncshare.svc.ovi.com:443/sync"
                serviceId: "Ovi"
            }
        },
        State {
            name: "production"
            PropertyChanges {
                target: sso
                server: 2
                key: ssoKeyManager.getKey(server, false)
                secret:  ssoKeyManager.getKey(server, true)
                syncShareUri: "https://mobile.syncshare.svc.ovi.com:443/sync"
                serviceId: "MapsforMeego"
            }
        }
    ]

    Timer {
        id: gentleQuitter
        interval: 250 * 2
        repeat: true
        onTriggered: {
            if (rootApplicationLevel.applicationFullyReady) {
                gentleQuitter.stop();
                Qt.quit();
            }
        }
    }
}
