import QtQuick 1.0

Rectangle {
    id: tabs;
    width: mainscreen.width
    height: parent.height;

    property bool showBack: false;
    property QtObject currentTabLoader;
    property string tabSceneSource: "TabScene.qml";

    gradient: Gradient {
        GradientStop {
            position: 0
            color: "#ffffff"
        }

        GradientStop {
            position: 1
            color: "#000000"
        }
    }

    ListModel {
        id: tabModel;
        ListElement {
            tabName: "places"
            name: qsTranslate("Maps", "qtn_maps_tabbar_places_title");
            link: "Places.qml"
        }
        ListElement {
            tabName: "nearby"
            name: qsTranslate("Maps", "qtn_maps_tabbar_nearby_title");
            link: "Nearby.qml";
        }
        ListElement {
            tabName: "directions"
            name: qsTranslate("Maps", "qtn_maps_tabbar_directions_title");
            link: "Directions.qml";
        }
        ListElement {
            tabName: "more"
            name: qsTranslate("Maps", "qtn_maps_tabbar_more_title");
            link: "More.qml";
        }
    }

    Component {
        id: tabComponent;
        BorderImage {
            source: "../images/bt_time.png"
            width: parent.width/tabModel.count - (tabs.showBack ? backButton.width/tabModel.count : 0)
            height: parent.height
            border.left: 5; border.top: 5
            border.right: 5; border.bottom: 5
            anchors.left: parent.left;
            anchors.leftMargin: width*index + (tabs.showBack ? backButton.width : 0);
            Text {
                id: label
                text: qsTranslate("Maps", name);
                font.pointSize: 22
                color: "black"
                width: parent.width
                elide: Text.ElideRight;
                horizontalAlignment: Text.AlignHCenter
                anchors.centerIn: parent;
                smooth: true;
            }
            MouseArea {
                anchors.fill: parent;
                onClicked: {
                    changeTab(tabName, link);
                }
                 onPressed: {
                    label.color = "blue";
                 }
                 onReleased: {
                      label.color = "black";
                 }
            }
        }
    }

    GridView {
        id: tabView;
        width: parent.width - (tabs.showBack ? backButton.width : 0);
        height: parent.height;
        model: tabModel;
        delegate: tabComponent;
        anchors.fill: parent;
    }
    BorderImage {
        id: backButton;
        objectName: "backButton";
        source: "../images/bt_time.png"
        width: 50;
        height: parent.height
        border.left: 5; border.top: 5
        border.right: 5; border.bottom: 5
        anchors.left: parent.left;
        visible: false;
        Text {
            id: label
            text: "back";
            color: "black"
            width: parent.width
            elide: Text.ElideRight;
            horizontalAlignment: Text.AlignHCenter
            anchors.centerIn: parent;
            smooth: true;
        }
        MouseArea {
            anchors.fill: parent;
            onClicked: {
                tabs.currentTabLoader.item.popScene();
            }
        }
    }

    Loader {
        id: placesLoader;
        height: mainscreen.height - tabs.height;
        width: mainscreen.width;
        anchors.bottom: tabs.top;
    }
    Loader {
        id: directionsLoader;
        height: mainscreen.height - tabs.height;
        width: mainscreen.width;
        anchors.bottom: tabs.top;
    }
    Loader {
        id: nearbyLoader;
        height: mainscreen.height - tabs.height;
        width: mainscreen.width;
        anchors.bottom: tabs.top;
    }
    Loader {
        id: moreLoader;
        height: mainscreen.height - tabs.height;
        width: mainscreen.width;
        anchors.bottom: tabs.top;
    }

    // Function to deactivate the current scene and activate the new one
    function changeTab(newTab, pageLink) {
        var valid = false;
        if(tabs.currentTabLoader && tabs.currentTabLoader.item) {
            tabs.currentTabLoader.item.deactivate();
        }
        switch(newTab) {
            case "more":
                tabs.currentTabLoader = moreLoader;
                valid = true;
                break;
            case "nearby":
                tabs.currentTabLoader = nearbyLoader;
                valid = true;
                break;
            case "directions":
                tabs.currentTabLoader = directionsLoader;
                valid = true;
                break;
            case "places":
                tabs.currentTabLoader = placesLoader;
                valid = true;
                break;
        }
        if(valid){
            tabs.currentTabLoader.source = tabs.tabSceneSource;
            tabs.currentTabLoader.item.activate(pageLink);
        }
    }
    states: [
        State {
            name: "showBackButton"
            when: tabs.showBack == true
            PropertyChanges {
                target: backButton
                visible: true;
            }
        }
    ]
}
