import QtQuick 1.0

Rectangle {
    id: scene;
    width: mainscreen.width;

    property bool active: false;
    property QtObject loader;

    property variant sceneModel:
        ListModel {
            id: sceneStack;
        }

    // So the background will be transparent to show the map
    color: "#00000000"

    Loader {
        id: pageLoader;
        height: parent.height;
        width: parent.width;
    }

    function onActivated() {}
    function onDeactivated() {}

    /**
      * To add a scene to the tab stack, from the scene, call addScene()
      * @params: page - {String} the url of the page to load
      */
    function addScene(page, aTag) {
        pageLoader.source = page;
        sceneStack.append({sourcePage: page, tag: aTag})
        tabbar.showBack = sceneStack.count > 1;
    }

    /**
      * popScene
      * goes back to the previous scene on the stack
      */
    function popScene() {
        sceneStack.remove(sceneStack.count-1);

        if(sceneStack.count > 0) {
            pageLoader.source = sceneStack.get(sceneStack.count-1).sourcePage;
        }
        tabbar.showBack = sceneStack.count > 1;
    }

    function popTo(aTag) {
        var targetIndex = -1, i = sceneStack.count-1;
        for(i = 0; i<sceneStack.count && targetIndex == -1; i++) {
            if(aTag == sceneStack.get(i).tag) {
                targetIndex = i;
            }
        }
        if(targetIndex != -1) {
            i = sceneStack.count;
            while(i-- && i > targetIndex+1) {
                sceneStack.remove(i);
            }
            popScene();
        }
    }

    /**
      * Activates
      */
    function activate(aPageSource) {
        if(sceneStack.count == 0) {
            addScene(aPageSource);
        }
        pageLoader.item.visible = true;
        scene.onActivated();
        tabbar.showBack = sceneStack.count > 1;
    }
    function deactivate() {
        if(pageLoader.item) {
            pageLoader.item.visible = false;
        }
        scene.onDeactivated();
    }
}
