// -*- mode: js; js-indent-level: 4; -*-

function _log (message, trace) {
    message = '[sso] ' + message;

    if (trace) {
        log.trace(message);
    } else {
        log.info(message);
    }
}

function initialize (noa) {
    var component, manager;

    _log('Initialization of Noa/SSO');

    noa.signInComplete.connect(onSignInCompleted);
    noa.checkLogin();

    return true;
}

function onSignInCompleted (success) {
    _log('sign-in completed, success is ' + (success ? 'true' : 'false'));

    noa.signInComplete.disconnect(onSignInCompleted);
    
    signedIn = success;

    if (!success && appState.ssoEnabled /* ^= never signed in before */) {
        gentleQuitter.start();
    }
}

function onInitializeResponse (noa, status) {
    _log('Initialization successful, sign-in status: ' + status);

    if (status === 0 || status === 1000) {
        noa.signIn();
    }
}

function onSignInResponse (noa, status, response) {
    var index;

    _log("sign-in operation completed, status: " + status);

    if (status === 0) {
        emailAddress = response.EmailAddress;
        oviAccount = response.UserId;
        token = response.Token;
        noaAccountId = response.NoaAccountId;

        for (index in response) {
            if (response.hasOwnProperty(index)) {
                _log('response ['+index+']: '+response[index]);
            }
        }
        // result e.g.
        // 1303132782109 [sso] response [EmailAddress]: alexander.kahl@nokia.com
        // 1303132782110 [sso] response [EmailAddressVerified]: false
        // 1303132782111 [sso] response [LoginId]: alexander.kahl@nokia.com
        // 1303132782111 [sso] response [MobilePhoneNumber]: +4915155155834
        // 1303132782112 [sso] response [MobilePhoneNumberVerified]: false
        // 1303132782113 [sso] response [NoaAccountId]: 6022cf3d-b126-01cd-595e-ee457a1e8b11
        // 1303132782113 [sso] response [ServerUTCTime]: Mon Apr 18 2011 15:18:32 GMT+0200 (CEST)
        // 1303132782114 [sso] response [Token]: 8O6gQcWqvjyc3RdkL88rLOJ0BbBQ7fag3S4CuT8S8dxv923Z
        // 1303132782115 [sso] response [UserId]:

        sso.isUiShown = false;
        noa.signInComplete(true);
    } else if (status === 703) {
        sso.isUiShown = false;
        noa.signInComplete(false);
    } else {
        if (sso.isUiShown) {
            noa.signInComplete(false);

            if (!appState.ssoEnabled) {
                sso.isUiShown = false;
                return;
            }
        }

        sso.isUiShown = true;
        noa.launchUi(0);
    }
}
